#define TRACE
using System.Diagnostics;
using System.Net.Mail;
using System.Runtime.InteropServices;
using System.Security;
using System.Security.Cryptography.X509Certificates;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	internal static class CollaborationHelperFunctions
	{
		private const short c_CollabVersion = 1;

		private static bool s_Initialized;

		private static object s_LockInitialized = new object();

		[SecurityCritical]
		internal static void Initialize()
		{
			if (s_Initialized)
			{
				return;
			}
			lock (s_LockInitialized)
			{
				if (!s_Initialized)
				{
					if (!PeerToPeerOSHelper.SupportsP2P)
					{
						throw new PlatformNotSupportedException(SR.GetString("P2P_NotAvailable"));
					}
					int num = UnsafeCollabNativeMethods.PeerCollabStartup(1);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabStartup returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_StartupFailed"), num);
					}
					s_Initialized = true;
				}
			}
		}

		internal static GUID ConvertGuidToGUID(Guid guid)
		{
			GUID result = default(GUID);
			bool flag = 1 == 0;
			byte[] array = guid.ToByteArray();
			string text = guid.ToString();
			int num = 0;
			int num2 = text.IndexOf('-');
			result.data1 = Convert.ToUInt32(text.Substring(num, num2 - num), 16);
			num = num2 + 1;
			num2 = text.IndexOf('-', num2 + 1);
			result.data2 = Convert.ToUInt16(text.Substring(num, num2 - num), 16);
			num = num2 + 1;
			num2 = text.IndexOf('-', num2 + 1);
			result.data3 = Convert.ToUInt16(text.Substring(num, num2 - num), 16);
			result.data4 = array[8];
			result.data5 = array[9];
			result.data6 = array[10];
			result.data7 = array[11];
			result.data8 = array[12];
			result.data9 = array[13];
			result.data10 = array[14];
			result.data11 = array[15];
			return result;
		}

		internal static Guid ConvertGUIDToGuid(GUID guid)
		{
			return new Guid(d: new byte[8]
			{
				guid.data4,
				guid.data5,
				guid.data6,
				guid.data7,
				guid.data8,
				guid.data9,
				guid.data10,
				guid.data11
			}, a: (int)guid.data1, b: (short)guid.data2, c: (short)guid.data3);
		}

		[SecurityCritical]
		internal static PeerContact ConvertPEER_CONTACTToPeerContact(PEER_CONTACT pc)
		{
			return ConvertPEER_CONTACTToPeerContact(pc, isMyContact: false);
		}

		[SecurityCritical]
		internal static PeerContact ConvertPEER_CONTACTToPeerContact(PEER_CONTACT pc, bool isMyContact)
		{
			PeerContact peerContact = isMyContact ? new MyContact() : new PeerContact();
			peerContact.PeerName = new PeerName(pc.pwzPeerName);
			peerContact.DisplayName = pc.pwzDisplayName;
			peerContact.Nickname = pc.pwzNickname;
			peerContact.EmailAddress = ((pc.pwzEmailAddress != null) ? new MailAddress(pc.pwzEmailAddress) : null);
			if (!isMyContact)
			{
				peerContact.SubscribeAllowed = pc.WatcherPermissions;
			}
			peerContact.IsSubscribed = (isMyContact || pc.fWatch);
			byte[] array = null;
			if (pc.credentials.cbData != 0)
			{
				array = new byte[pc.credentials.cbData];
				Marshal.Copy(pc.credentials.pbData, array, 0, (int)pc.credentials.cbData);
			}
			if (array != null)
			{
				SafeCertStore safeCertStore = UnsafeCollabNativeMethods.CertOpenStore(new IntPtr(5), 65537u, IntPtr.Zero, 1u, ref pc.credentials);
				if (safeCertStore == null || safeCertStore.IsInvalid)
				{
					int lastWin32Error = Marshal.GetLastWin32Error();
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_CredentialsError"), lastWin32Error);
				}
				try
				{
					X509Store x509Store = new X509Store(safeCertStore.DangerousGetHandle());
					peerContact.Credentials = new X509Certificate2(x509Store.Certificates[0]);
				}
				finally
				{
					safeCertStore?.Dispose();
				}
			}
			return peerContact;
		}

		[SecurityCritical]
		internal static PEER_CONTACT ConvertPeerContactToPEER_CONTACT(PeerContact peerContact, ref SafeCollabMemory safeCredentials)
		{
			PEER_CONTACT result = default(PEER_CONTACT);
			result.pwzDisplayName = peerContact.DisplayName;
			result.pwzEmailAddress = ((peerContact.EmailAddress == null) ? null : peerContact.EmailAddress.ToString());
			result.pwzNickname = peerContact.Nickname;
			result.pwzPeerName = peerContact.PeerName.ToString();
			result.fWatch = peerContact.IsSubscribed;
			result.WatcherPermissions = peerContact.SubscribeAllowed;
			PEER_DATA pvSafeToPara = default(PEER_DATA);
			if (peerContact.Credentials != null)
			{
				SafeCertStore safeCertStore = UnsafeCollabNativeMethods.CertOpenStore(new IntPtr(2), 0u, IntPtr.Zero, 8193u, IntPtr.Zero);
				if (safeCertStore == null || safeCertStore.IsInvalid)
				{
					int lastWin32Error = Marshal.GetLastWin32Error();
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_CredentialsError"), lastWin32Error);
				}
				try
				{
					X509Store x509Store = new X509Store(safeCertStore.DangerousGetHandle());
					x509Store.Add(peerContact.Credentials);
					bool flag = UnsafeCollabNativeMethods.CertSaveStore(safeCertStore, 65537u, 2u, 2u, ref pvSafeToPara, 0u);
					if (pvSafeToPara.cbData != 0 && flag)
					{
						safeCredentials = new SafeCollabMemory((int)pvSafeToPara.cbData);
						pvSafeToPara.pbData = safeCredentials.DangerousGetHandle();
						flag = UnsafeCollabNativeMethods.CertSaveStore(safeCertStore, 65537u, 2u, 2u, ref pvSafeToPara, 0u);
					}
					else
					{
						pvSafeToPara.cbData = 0u;
						pvSafeToPara.pbData = IntPtr.Zero;
					}
				}
				finally
				{
					safeCertStore?.Dispose();
				}
			}
			else
			{
				pvSafeToPara.cbData = 0u;
				pvSafeToPara.pbData = IntPtr.Zero;
			}
			result.credentials = pvSafeToPara;
			return result;
		}

		internal static void ByteArrayToSin6Addr(byte[] addrBytes, ref SOCKADDR_IN6 sin6)
		{
			sin6.sin6_addr0 = addrBytes[0];
			sin6.sin6_addr1 = addrBytes[1];
			sin6.sin6_addr2 = addrBytes[2];
			sin6.sin6_addr3 = addrBytes[3];
			sin6.sin6_addr4 = addrBytes[4];
			sin6.sin6_addr5 = addrBytes[5];
			sin6.sin6_addr6 = addrBytes[6];
			sin6.sin6_addr7 = addrBytes[7];
			sin6.sin6_addr8 = addrBytes[8];
			sin6.sin6_addr9 = addrBytes[9];
			sin6.sin6_addr10 = addrBytes[10];
			sin6.sin6_addr11 = addrBytes[11];
			sin6.sin6_addr12 = addrBytes[12];
			sin6.sin6_addr13 = addrBytes[13];
			sin6.sin6_addr14 = addrBytes[14];
			sin6.sin6_addr15 = addrBytes[15];
		}

		[SecurityCritical]
		internal static PeerNearMe PEER_PEOPLE_NEAR_METoPeerNearMe(PEER_PEOPLE_NEAR_ME ppnm)
		{
			PeerNearMe peerNearMe = new PeerNearMe();
			peerNearMe.Id = ConvertGUIDToGuid(ppnm.id);
			peerNearMe.Nickname = Marshal.PtrToStringUni(ppnm.pwzNickname);
			PEER_ENDPOINT endpoint = ppnm.endpoint;
			PeerEndPoint item = ConvertPEER_ENDPOINTToPeerEndPoint(endpoint);
			peerNearMe.PeerEndPoints.Add(item);
			return peerNearMe;
		}

		[SecurityCritical]
		internal static PeerObject ConvertPEER_OBJECTToPeerObject(PEER_OBJECT po)
		{
			byte[] array = null;
			if (po.data.cbData != 0)
			{
				array = new byte[po.data.cbData];
				Marshal.Copy(po.data.pbData, array, 0, (int)po.data.cbData);
			}
			return new PeerObject(ConvertGUIDToGuid(po.guid), array, (PeerScope)po.dwPublicationScope);
		}

		[SecurityCritical]
		internal static PeerApplication ConvertPEER_APPLICATIONToPeerApplication(PEER_APPLICATION pa)
		{
			byte[] array = null;
			if (pa.data.cbData != 0)
			{
				array = new byte[pa.data.cbData];
				Marshal.Copy(pa.data.pbData, array, 0, (int)pa.data.cbData);
			}
			return new PeerApplication(ConvertGUIDToGuid(pa.guid), Marshal.PtrToStringUni(pa.pwzDescription), array, null, null, PeerScope.None);
		}

		[SecurityCritical]
		internal static PeerEndPoint ConvertPEER_ENDPOINTToPeerEndPoint(PEER_ENDPOINT pe)
		{
			byte[] address = new byte[16]
			{
				pe.peerAddress.sin6.sin6_addr0,
				pe.peerAddress.sin6.sin6_addr1,
				pe.peerAddress.sin6.sin6_addr2,
				pe.peerAddress.sin6.sin6_addr3,
				pe.peerAddress.sin6.sin6_addr4,
				pe.peerAddress.sin6.sin6_addr5,
				pe.peerAddress.sin6.sin6_addr6,
				pe.peerAddress.sin6.sin6_addr7,
				pe.peerAddress.sin6.sin6_addr8,
				pe.peerAddress.sin6.sin6_addr9,
				pe.peerAddress.sin6.sin6_addr10,
				pe.peerAddress.sin6.sin6_addr11,
				pe.peerAddress.sin6.sin6_addr12,
				pe.peerAddress.sin6.sin6_addr13,
				pe.peerAddress.sin6.sin6_addr14,
				pe.peerAddress.sin6.sin6_addr15
			};
			IPAddress address2 = new IPAddress(address, pe.peerAddress.sin6.sin6_scope_id);
			ushort port = (ushort)IPAddress.NetworkToHostOrder((short)pe.peerAddress.sin6.sin6_port);
			IPEndPoint endPoint = new IPEndPoint(address2, port);
			return new PeerEndPoint(endPoint, Marshal.PtrToStringUni(pe.pwzEndpointName));
		}

		internal static PEER_ADDRESS ConvertIPEndpointToPEER_ADDRESS(IPEndPoint endPoint)
		{
			PEER_ADDRESS result = default(PEER_ADDRESS);
			SOCKADDR_IN6 sin = default(SOCKADDR_IN6);
			sin.sin6_family = (ushort)endPoint.AddressFamily;
			sin.sin6_flowinfo = 0u;
			sin.sin6_port = (ushort)IPAddress.HostToNetworkOrder((short)endPoint.Port);
			sin.sin6_scope_id = (uint)endPoint.Address.ScopeId;
			ByteArrayToSin6Addr(endPoint.Address.GetAddressBytes(), ref sin);
			result.dwSize = 32u;
			result.sin6 = sin;
			return result;
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, UnmanagedCode = true)]
		internal static void CleanEventVars(ref RegisteredWaitHandle waitHandle, ref SafeCollabEvent safeEvent, ref AutoResetEvent firedEvent)
		{
			if (waitHandle != null)
			{
				waitHandle.Unregister(null);
				waitHandle = null;
			}
			if (safeEvent != null && !safeEvent.IsInvalid)
			{
				safeEvent.Dispose();
			}
			if (firedEvent != null)
			{
				firedEvent.Close();
				firedEvent = null;
			}
		}

		[SecurityCritical]
		internal static void AddMyPresenceChanged(EventHandler<PresenceChangedEventArgs> callback, ref EventHandler<PresenceChangedEventArgs> presenceChanged, object lockPresenceChangedEvent, ref RegisteredWaitHandle regPresenceChangedWaitHandle, ref AutoResetEvent presenceChangedEvent, ref SafeCollabEvent safePresenceChangedEvent, WaitOrTimerCallback PresenceChangedCallback)
		{
			lock (lockPresenceChangedEvent)
			{
				if (presenceChanged == null)
				{
					presenceChangedEvent = new AutoResetEvent(initialState: false);
					regPresenceChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(presenceChangedEvent, PresenceChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.MyPresenceChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(presenceChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out safePresenceChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_PresenceChangedRegFailed"), num);
					}
				}
				presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Combine(presenceChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddMyPresenceChanged() successful.");
		}

		[SecurityCritical]
		internal static void AddMyApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback, ref EventHandler<ApplicationChangedEventArgs> applicationChanged, object lockAppChangedEvent, ref RegisteredWaitHandle regAppChangedWaitHandle, ref AutoResetEvent appChangedEvent, ref SafeCollabEvent safeAppChangedEvent, WaitOrTimerCallback ApplicationChangedCallback)
		{
			lock (lockAppChangedEvent)
			{
				if (applicationChanged == null)
				{
					appChangedEvent = new AutoResetEvent(initialState: false);
					regAppChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(appChangedEvent, ApplicationChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.MyApplicationChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(appChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out safeAppChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ApplicationChangedRegFailed"), num);
					}
				}
				applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Combine(applicationChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() successful.");
		}

		[SecurityCritical]
		internal static void AddMyObjectChanged(EventHandler<ObjectChangedEventArgs> callback, ref EventHandler<ObjectChangedEventArgs> objectChanged, object lockObjChangedEvent, ref RegisteredWaitHandle regObjChangedWaitHandle, ref AutoResetEvent objChangedEvent, ref SafeCollabEvent safeObjChangedEvent, WaitOrTimerCallback ObjectChangedCallback)
		{
			lock (lockObjChangedEvent)
			{
				if (objectChanged == null)
				{
					objChangedEvent = new AutoResetEvent(initialState: false);
					regObjChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(objChangedEvent, ObjectChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.MyObjectChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(objChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out safeObjChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectChangedRegFailed"), num);
					}
				}
				objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Combine(objectChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChanged() successful.");
		}

		internal static void ThrowIfInvitationResponseInvalid(PeerInvitationResponse response)
		{
			if (response.PeerInvitationResponseType < PeerInvitationResponseType.Declined || response.PeerInvitationResponseType > PeerInvitationResponseType.Expired)
			{
				throw new PeerToPeerException(SR.GetString("Collab_InviteFailed"));
			}
		}
	}
}
