#define TRACE
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	public sealed class ContactManager : IDisposable
	{
		private class CreateContactAsyncState
		{
			private PeerNearMe m_peerNearMe;

			private object m_userToken;

			internal PeerNearMe PeerNearMe => m_peerNearMe;

			internal object UserToken => m_userToken;

			internal CreateContactAsyncState(PeerNearMe peerNearMe, object userToken)
			{
				m_peerNearMe = peerNearMe;
				m_userToken = userToken;
			}
		}

		private ISynchronizeInvoke m_synchronizingObject;

		private object m_lockSubLstChangedEvent;

		private RegisteredWaitHandle m_regSubLstChangedWaitHandle;

		private AutoResetEvent m_subLstChangedEvent;

		private SafeCollabEvent m_safeSubLstChangedEvent;

		private object m_lockNameChangedEvent;

		private RegisteredWaitHandle m_regNameChangedWaitHandle;

		private AutoResetEvent m_nameChangedEvent;

		private SafeCollabEvent m_safeNameChangedEvent;

		private object m_lockPresenceChangedEvent;

		private RegisteredWaitHandle m_regPresenceChangedWaitHandle;

		private AutoResetEvent m_presenceChangedEvent;

		private SafeCollabEvent m_safePresenceChangedEvent;

		private object m_lockAppChangedEvent;

		private RegisteredWaitHandle m_regAppChangedWaitHandle;

		private AutoResetEvent m_appChangedEvent;

		private SafeCollabEvent m_safeAppChangedEvent;

		private object m_lockObjChangedEvent;

		private RegisteredWaitHandle m_regObjChangedWaitHandle;

		private AutoResetEvent m_objChangedEvent;

		private SafeCollabEvent m_safeObjChangedEvent;

		private SendOrPostCallback OnCreateContactCompletedDelegate;

		private object m_createContactAsyncListLock;

		private Dictionary<object, AsyncOperation> m_createContactAsyncList = new Dictionary<object, AsyncOperation>();

		private bool m_Disposed;

		public static PeerContact LocalContact
		{
			[SecurityCritical]
			get
			{
				SafeCollabData ppwzContactData = null;
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Get MyContact called.");
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				MyContact result;
				try
				{
					int num = UnsafeCollabNativeMethods.PeerCollabGetContact(null, out ppwzContactData);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetContact returned with errorcode {0}", num);
						return null;
					}
					PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(ppwzContactData.DangerousGetHandle(), typeof(PEER_CONTACT));
					result = (MyContact)CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc, isMyContact: true);
				}
				finally
				{
					ppwzContactData?.Dispose();
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Get MyContact successful.");
				return result;
			}
		}

		[Description("SynchronizingObject")]
		[DefaultValue(null)]
		[Browsable(false)]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_synchronizingObject;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_synchronizingObject = value;
			}
		}

		private object LockSubLstChangedEvent
		{
			get
			{
				if (m_lockSubLstChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockSubLstChangedEvent, value, null);
				}
				return m_lockSubLstChangedEvent;
			}
		}

		private object LockNameChangedEvent
		{
			get
			{
				if (m_lockNameChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockNameChangedEvent, value, null);
				}
				return m_lockNameChangedEvent;
			}
		}

		private object LockPresenceChangedEvent
		{
			get
			{
				if (m_lockPresenceChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockPresenceChangedEvent, value, null);
				}
				return m_lockPresenceChangedEvent;
			}
		}

		private object LockAppChangedEvent
		{
			get
			{
				if (m_lockAppChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockAppChangedEvent, value, null);
				}
				return m_lockAppChangedEvent;
			}
		}

		private object LockObjChangedEvent
		{
			get
			{
				if (m_lockObjChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockObjChangedEvent, value, null);
				}
				return m_lockObjChangedEvent;
			}
		}

		private object CreateContactAsyncListLock
		{
			get
			{
				if (m_createContactAsyncListLock == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_createContactAsyncListLock, value, null);
				}
				return m_createContactAsyncListLock;
			}
		}

		private event EventHandler<SubscriptionListChangedEventArgs> m_subscriptionListChanged;

		public event EventHandler<SubscriptionListChangedEventArgs> SubscriptionListChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddSubscriptionListChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemoveSubscriptionListChanged(value);
			}
		}

		private event EventHandler<NameChangedEventArgs> m_nameChanged;

		public event EventHandler<NameChangedEventArgs> NameChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddNameChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemoveNameChanged(value);
			}
		}

		private event EventHandler<PresenceChangedEventArgs> m_presenceChanged;

		public event EventHandler<PresenceChangedEventArgs> PresenceChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddPresenceChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemovePresenceChanged(value);
			}
		}

		private event EventHandler<ApplicationChangedEventArgs> m_applicationChanged;

		public event EventHandler<ApplicationChangedEventArgs> ApplicationChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddApplicationChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemoveApplicationChanged(value);
			}
		}

		private event EventHandler<ObjectChangedEventArgs> m_objectChanged;

		public event EventHandler<ObjectChangedEventArgs> ObjectChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddObjectChangedEvent(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemoveObjectChangedEvent(value);
			}
		}

		private event EventHandler<CreateContactCompletedEventArgs> m_createContactCompleted;

		public event EventHandler<CreateContactCompletedEventArgs> CreateContactCompleted
		{
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_createContactCompleted = (EventHandler<CreateContactCompletedEventArgs>)Delegate.Combine(this.m_createContactCompleted, value);
			}
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_createContactCompleted = (EventHandler<CreateContactCompletedEventArgs>)Delegate.Remove(this.m_createContactCompleted, value);
			}
		}

		[SecurityCritical]
		internal ContactManager()
		{
			CollaborationHelperFunctions.Initialize();
			OnCreateContactCompletedDelegate = CreateContactCompletedWaitCallback;
		}

		[SecurityCritical]
		private void AddSubscriptionListChanged(EventHandler<SubscriptionListChangedEventArgs> callback)
		{
			lock (LockSubLstChangedEvent)
			{
				if (this.m_subscriptionListChanged == null)
				{
					m_subLstChangedEvent = new AutoResetEvent(initialState: false);
					m_regSubLstChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_subLstChangedEvent, SubListChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.WatchListChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_subLstChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safeSubLstChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_SubListChangedRegFailed"), num);
					}
				}
				this.m_subscriptionListChanged = (EventHandler<SubscriptionListChangedEventArgs>)Delegate.Combine(this.m_subscriptionListChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddSubscriptionListChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveSubscriptionListChanged(EventHandler<SubscriptionListChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveSubscriptionListChanged() called.");
			lock (LockSubLstChangedEvent)
			{
				this.m_subscriptionListChanged = (EventHandler<SubscriptionListChangedEventArgs>)Delegate.Remove(this.m_subscriptionListChanged, callback);
				if (this.m_subscriptionListChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref m_regSubLstChangedWaitHandle, ref m_safeSubLstChangedEvent, ref m_subLstChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean SubscriptionListChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveSubscriptionListChanged() successful.");
		}

		[SecurityCritical]
		private void SubListChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "SubListChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				SubscriptionListChangedEventArgs subscriptionListChangedEventArgs = null;
				try
				{
					lock (LockSubLstChangedEvent)
					{
						if (m_safeSubLstChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safeSubLstChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetSubListChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.WatchListChanged)
						{
							PEER_EVENT_WATCHLIST_CHANGED_DATA watchListChangedData = pEER_COLLAB_EVENT_DATA.watchListChangedData;
							PeerContact peerContact = null;
							if (watchListChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(watchListChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							subscriptionListChangedEventArgs = new SubscriptionListChangedEventArgs(null, peerContact, watchListChangedData.changeType);
						}
						goto IL_0161;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_0161:
				EventHandler<SubscriptionListChangedEventArgs> subscriptionListChanged = this.m_subscriptionListChanged;
				if (subscriptionListChangedEventArgs != null && subscriptionListChanged != null)
				{
					if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
					{
						SynchronizingObject.BeginInvoke(subscriptionListChanged, new object[2]
						{
							this,
							subscriptionListChangedEventArgs
						});
					}
					else
					{
						subscriptionListChanged(this, subscriptionListChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the subscription list changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving SubListChangedCallback().");
		}

		[SecurityCritical]
		private void AddNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			lock (LockNameChangedEvent)
			{
				if (this.m_nameChanged == null)
				{
					m_nameChangedEvent = new AutoResetEvent(initialState: false);
					m_regNameChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_nameChangedEvent, NameChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.EndPointChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_nameChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safeNameChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_NameChangedRegFailed"), num);
					}
				}
				this.m_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Combine(this.m_nameChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddNameChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() called.");
			lock (LockNameChangedEvent)
			{
				this.m_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Remove(this.m_nameChanged, callback);
				if (this.m_nameChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref m_regNameChangedWaitHandle, ref m_safeNameChangedEvent, ref m_nameChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean NameChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() successful.");
		}

		[SecurityCritical]
		private void NameChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "NameChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				NameChangedEventArgs nameChangedEventArgs = null;
				try
				{
					lock (LockNameChangedEvent)
					{
						if (m_safeNameChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safeNameChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetNameChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointChanged)
						{
							PEER_EVENT_ENDPOINT_CHANGED_DATA endpointChangedData = pEER_COLLAB_EVENT_DATA.endpointChangedData;
							PeerContact peerContact = null;
							PeerEndPoint peerEndPoint = null;
							string name = null;
							if (endpointChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(endpointChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (endpointChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(endpointChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
								name = peerEndPoint.Name;
							}
							nameChangedEventArgs = new NameChangedEventArgs(peerEndPoint, peerContact, name);
						}
						goto IL_01b1;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_01b1:
				EventHandler<NameChangedEventArgs> nameChanged = this.m_nameChanged;
				if (nameChangedEventArgs != null && nameChanged != null)
				{
					if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
					{
						SynchronizingObject.BeginInvoke(nameChanged, new object[2]
						{
							this,
							nameChangedEventArgs
						});
					}
					else
					{
						nameChanged(this, nameChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the name changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving NameChangedCallback().");
		}

		[SecurityCritical]
		private void AddPresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			lock (LockPresenceChangedEvent)
			{
				if (this.m_presenceChanged == null)
				{
					m_presenceChangedEvent = new AutoResetEvent(initialState: false);
					m_regPresenceChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_presenceChangedEvent, PresenceChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.EndPointPresenceChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_presenceChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safePresenceChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_PresenceChangedRegFailed"), num);
					}
				}
				this.m_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Combine(this.m_presenceChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddPresenceChanged() successful.");
		}

		[SecurityCritical]
		private void RemovePresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() called.");
			lock (LockPresenceChangedEvent)
			{
				this.m_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Remove(this.m_presenceChanged, callback);
				if (this.m_presenceChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref m_regPresenceChangedWaitHandle, ref m_safePresenceChangedEvent, ref m_presenceChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean PresenceChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() successful.");
		}

		[SecurityCritical]
		private void PresenceChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PresenceChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				PresenceChangedEventArgs presenceChangedEventArgs = null;
				try
				{
					lock (LockPresenceChangedEvent)
					{
						if (m_safePresenceChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safePresenceChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPresenceChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointPresenceChanged)
						{
							PEER_EVENT_PRESENCE_CHANGED_DATA presenceChangedData = pEER_COLLAB_EVENT_DATA.presenceChangedData;
							PeerPresenceInfo peerPresenceInfo = null;
							if (presenceChangedData.pPresenceInfo != IntPtr.Zero)
							{
								PEER_PRESENCE_INFO pEER_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(presenceChangedData.pPresenceInfo, typeof(PEER_PRESENCE_INFO));
								peerPresenceInfo = new PeerPresenceInfo();
								peerPresenceInfo.PresenceStatus = pEER_PRESENCE_INFO.status;
								peerPresenceInfo.DescriptiveText = pEER_PRESENCE_INFO.descText;
							}
							PeerContact peerContact = null;
							PeerEndPoint peerEndPoint = null;
							if (presenceChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(presenceChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (presenceChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(presenceChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							presenceChangedEventArgs = new PresenceChangedEventArgs(peerEndPoint, peerContact, presenceChangedData.changeType, peerPresenceInfo);
						}
						goto IL_020d;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_020d:
				EventHandler<PresenceChangedEventArgs> presenceChanged = this.m_presenceChanged;
				if (presenceChangedEventArgs != null && presenceChanged != null)
				{
					if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
					{
						SynchronizingObject.BeginInvoke(presenceChanged, new object[2]
						{
							this,
							presenceChangedEventArgs
						});
					}
					else
					{
						presenceChanged(this, presenceChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the presence changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving PresenceChangedCallback().");
		}

		[SecurityCritical]
		private void AddApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() called.");
			lock (LockAppChangedEvent)
			{
				if (this.m_applicationChanged == null)
				{
					m_appChangedEvent = new AutoResetEvent(initialState: false);
					m_regAppChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_appChangedEvent, ApplicationChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.EndPointApplicationChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_appChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safeAppChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ApplicationChangedRegFailed"), num);
					}
				}
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Combine(this.m_applicationChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() called.");
			lock (LockAppChangedEvent)
			{
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Remove(this.m_applicationChanged, callback);
				if (this.m_applicationChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref m_regAppChangedWaitHandle, ref m_safeAppChangedEvent, ref m_appChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() successful.");
		}

		[SecurityCritical]
		private void ApplicationChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ApplicationChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				ApplicationChangedEventArgs applicationChangedEventArgs = null;
				try
				{
					lock (LockAppChangedEvent)
					{
						if (m_safeAppChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safeAppChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetApplicationChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointApplicationChanged)
						{
							PEER_EVENT_APPLICATION_CHANGED_DATA applicationChangedData = pEER_COLLAB_EVENT_DATA.applicationChangedData;
							PEER_APPLICATION pa = (PEER_APPLICATION)Marshal.PtrToStructure(applicationChangedData.pApplication, typeof(PEER_APPLICATION));
							PeerApplication peerApplication = CollaborationHelperFunctions.ConvertPEER_APPLICATIONToPeerApplication(pa);
							PeerContact peerContact = null;
							PeerEndPoint peerEndPoint = null;
							if (applicationChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(applicationChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (applicationChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(applicationChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							applicationChangedEventArgs = new ApplicationChangedEventArgs(peerEndPoint, peerContact, applicationChangedData.changeType, peerApplication);
						}
						goto IL_01d2;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_01d2:
				EventHandler<ApplicationChangedEventArgs> applicationChanged = this.m_applicationChanged;
				if (applicationChangedEventArgs != null && applicationChanged != null)
				{
					if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
					{
						SynchronizingObject.BeginInvoke(applicationChanged, new object[2]
						{
							this,
							applicationChangedEventArgs
						});
					}
					else
					{
						applicationChanged(this, applicationChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the application changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ApplicationChangedCallback().");
		}

		[SecurityCritical]
		private void AddObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChangedEvent() called.");
			lock (LockObjChangedEvent)
			{
				if (this.m_objectChanged == null)
				{
					m_objChangedEvent = new AutoResetEvent(initialState: false);
					m_regObjChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_objChangedEvent, ObjectChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.EndPointObjectChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_objChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safeObjChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectChangedRegFailed"), num);
					}
				}
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Combine(this.m_objectChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() called.");
			lock (LockObjChangedEvent)
			{
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Remove(this.m_objectChanged, callback);
				if (this.m_objectChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref m_regObjChangedWaitHandle, ref m_safeObjChangedEvent, ref m_objChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() successful.");
		}

		[SecurityCritical]
		private void ObjectChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ObjectChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				ObjectChangedEventArgs objectChangedEventArgs = null;
				try
				{
					lock (LockObjChangedEvent)
					{
						if (m_safeObjChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safeObjChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointObjectChanged)
						{
							PEER_EVENT_OBJECT_CHANGED_DATA objectChangedData = pEER_COLLAB_EVENT_DATA.objectChangedData;
							PEER_OBJECT po = (PEER_OBJECT)Marshal.PtrToStructure(objectChangedData.pObject, typeof(PEER_OBJECT));
							PeerObject peerObject = CollaborationHelperFunctions.ConvertPEER_OBJECTToPeerObject(po);
							PeerContact peerContact = null;
							PeerEndPoint peerEndPoint = null;
							if (objectChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(objectChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (objectChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(objectChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							objectChangedEventArgs = new ObjectChangedEventArgs(peerEndPoint, peerContact, objectChangedData.changeType, peerObject);
						}
						goto IL_01d2;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_01d2:
				EventHandler<ObjectChangedEventArgs> objectChanged = this.m_objectChanged;
				if (objectChangedEventArgs != null && objectChanged != null)
				{
					if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
					{
						SynchronizingObject.BeginInvoke(objectChanged, new object[2]
						{
							this,
							objectChangedEventArgs
						});
					}
					else
					{
						objectChanged(this, objectChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the object changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ObjectChangedCallback().");
		}

		[SecurityCritical]
		public unsafe PeerContactCollection GetContacts()
		{
			PeerContactCollection peerContactCollection = new PeerContactCollection();
			SafeCollabEnum phPeerEnum = null;
			uint pCount = 0u;
			int num = 0;
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetContacts()");
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabEnumContacts(out phPeerEnum);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumContacts returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetContactsFailed"), num);
				}
				num = UnsafeCollabNativeMethods.PeerGetItemCount(phPeerEnum, ref pCount);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetContactsFailed"), num);
				}
				if (pCount == 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "No contacts found. \nLeaving GetContacts()");
					return peerContactCollection;
				}
				SafeCollabData pppvItems = null;
				try
				{
					num = UnsafeCollabNativeMethods.PeerGetNextItem(phPeerEnum, ref pCount, out pppvItems);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetNextItem returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetContactsFailed"), num);
					}
					IntPtr value = pppvItems.DangerousGetHandle();
					IntPtr* ptr = (IntPtr*)(void*)value;
					for (ulong num2 = 0uL; num2 < pCount; num2++)
					{
						IntPtr ptr2 = ptr[num2];
						PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(ptr2, typeof(PEER_CONTACT));
						PeerContact item = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
						peerContactCollection.Add(item);
					}
				}
				finally
				{
					pppvItems.Dispose();
				}
			}
			finally
			{
				phPeerEnum.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Returning collections with {0} contacts. \nLeaving GetContacts()");
			return peerContactCollection;
		}

		[SecurityCritical]
		public PeerContact GetContact(PeerName peerName)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerName == null)
			{
				throw new ArgumentNullException("peerName");
			}
			int num = 0;
			SafeCollabData ppwzContactData = null;
			PeerContact result = null;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetContact() with peername {0}", peerName.ToString());
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabGetContact(peerName.ToString(), out ppwzContactData);
				switch (num)
				{
				case -2140971007:
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Contact not found in Contact Manager");
					throw new PeerToPeerException(SR.GetString("Collab_ContactNotFound"));
				default:
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetContact returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetContactFailed"), num);
				case 0:
					if (!ppwzContactData.DangerousGetHandle().Equals(IntPtr.Zero))
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Found contact.");
						PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(ppwzContactData.DangerousGetHandle(), typeof(PEER_CONTACT));
						result = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
					}
					else
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "No contact found.");
					}
					break;
				}
			}
			finally
			{
				ppwzContactData.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving GetContact()");
			return result;
		}

		[SecurityCritical]
		public PeerContact CreateContact(PeerNearMe peerNearMe)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerNearMe == null)
			{
				throw new ArgumentNullException("peerNearMe");
			}
			if (peerNearMe.PeerEndPoints == null || peerNearMe.PeerEndPoints.Count == 0 || peerNearMe.PeerEndPoints[0].EndPoint == null)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering CreateContact() with peernearme", peerNearMe.ToString());
			PeerEndPoint peerEndPoint = peerNearMe.PeerEndPoints[0];
			PEER_ENDPOINT pEER_ENDPOINT = default(PEER_ENDPOINT);
			pEER_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gCHandle = default(GCHandle);
			if (peerEndPoint.Name != null)
			{
				gCHandle = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
				pEER_ENDPOINT.pwzEndpointName = gCHandle.AddrOfPinnedObject();
			}
			GCHandle gCHandle2 = GCHandle.Alloc(pEER_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gCHandle2.AddrOfPinnedObject();
			string ppwzContactData = null;
			int num = 0;
			peerNearMe.RefreshData();
			num = UnsafeCollabNativeMethods.PeerCollabQueryContactData(pcEndpoint, ref ppwzContactData);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetContact returned with errorcode {0}", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_CreateContactFailed"), num);
			}
			SafeCollabData ppContactData = null;
			PeerContact peerContact = null;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabParseContact(ppwzContactData, out ppContactData);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetContact returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_CreateContactFailed"), num);
				}
				PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(ppContactData.DangerousGetHandle(), typeof(PEER_CONTACT));
				peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
				peerContact.JustCreated = true;
				peerContact.ContactXml = ppwzContactData;
			}
			finally
			{
				ppContactData?.Dispose();
				gCHandle.Free();
				gCHandle2.Free();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving CreateContact().");
			return peerContact;
		}

		public void CreateContactAsync(PeerNearMe peerNearMe, object userToken)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerNearMe == null)
			{
				throw new ArgumentNullException("PeerNearMe");
			}
			if (peerNearMe.PeerEndPoints == null || peerNearMe.PeerEndPoints.Count == 0 || peerNearMe.PeerEndPoints[0].EndPoint == null)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering CreateContactAsync() with token {0} and following peernearme", userToken);
				peerNearMe.TracePeerNearMe();
			}
			lock (CreateContactAsyncListLock)
			{
				if (m_createContactAsyncList.ContainsKey(userToken))
				{
					throw new ArgumentException(SR.GetString("DuplicateUserToken"));
				}
				AsyncOperation value = AsyncOperationManager.CreateOperation(userToken);
				m_createContactAsyncList[userToken] = value;
			}
			ThreadPool.QueueUserWorkItem(CreateContactAsyncHelper, new CreateContactAsyncState(peerNearMe, userToken));
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving CreateContactAsync().");
		}

		private void CreateContactAsyncHelper(object state)
		{
			Exception ex = null;
			PeerContact peerContact = null;
			CreateContactAsyncState createContactAsyncState = state as CreateContactAsyncState;
			PeerNearMe peerNearMe = createContactAsyncState.PeerNearMe;
			object userToken = createContactAsyncState.UserToken;
			try
			{
				peerContact = CreateContact(peerNearMe);
			}
			catch (ObjectDisposedException ex2)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "CreateContactAsyncHelper caught error {0}", ex2.Message);
				ex = ex2;
			}
			catch (PeerToPeerException ex3)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "CreateContactAsyncHelper caught error {0}", ex3.Message);
				ex = ex3;
			}
			PrepareToRaiseCreateContactCompletedEvent(args: (ex != null) ? new CreateContactCompletedEventArgs(peerContact, ex, cancelled: false, userToken) : new CreateContactCompletedEventArgs(peerContact, null, cancelled: false, userToken), asyncOP: m_createContactAsyncList[userToken]);
		}

		private void OnCreateContactCompleted(CreateContactCompletedEventArgs e)
		{
			EventHandler<CreateContactCompletedEventArgs> createContactCompleted = this.m_createContactCompleted;
			if (createContactCompleted != null)
			{
				createContactCompleted(this, e);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the create contact completed event callback.");
			}
		}

		private void CreateContactCompletedWaitCallback(object operationState)
		{
			CreateContactCompletedEventArgs createContactCompletedEventArgs = (CreateContactCompletedEventArgs)operationState;
			m_createContactAsyncList.Remove(createContactCompletedEventArgs.UserState);
			OnCreateContactCompleted(createContactCompletedEventArgs);
		}

		internal void PrepareToRaiseCreateContactCompletedEvent(AsyncOperation asyncOP, CreateContactCompletedEventArgs args)
		{
			asyncOP.PostOperationCompleted(OnCreateContactCompletedDelegate, args);
		}

		[SecurityCritical]
		public void AddContact(PeerContact peerContact)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerContact == null)
			{
				throw new ArgumentNullException("peerContact");
			}
			string text = null;
			if (peerContact.ContactXml == null)
			{
				try
				{
					text = peerContact.ToXml();
				}
				catch (PeerToPeerException ex)
				{
					throw new PeerToPeerException(SR.GetString("Collab_AddContactFailedNoXml"), ex.InnerException);
				}
			}
			else
			{
				text = peerContact.ContactXml;
			}
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddContact() with following peercontact");
				peerContact.TracePeerContact();
			}
			SafeCollabData ppContact = null;
			int num = 0;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabAddContact(text, out ppContact);
			}
			finally
			{
				ppContact?.Dispose();
			}
			switch (num)
			{
			case -2147024713:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabAddContact returned with errorcode {0}. Contact already exists.", num);
				throw new ArgumentException(SR.GetString("Collab_AddContactFailed") + " " + SR.GetString("Collab_ContactExists"));
			default:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabAddContact returned with errorcode {0}", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_AddContactFailed"), num);
			case 0:
				peerContact.JustCreated = false;
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving AddContact() successfully.");
				break;
			}
		}

		public void DeleteContact(PeerContact peerContact)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering DeleteContact().");
			if (peerContact == null)
			{
				throw new ArgumentNullException("peerContact");
			}
			DeleteContact(peerContact.PeerName);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving DeleteContact() successfully.");
		}

		[SecurityCritical]
		public void DeleteContact(PeerName peerName)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering DeleteContact().");
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerName == null)
			{
				throw new ArgumentNullException("peerName");
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Peername is {0}", peerName.ToString());
			int num = UnsafeCollabNativeMethods.PeerCollabDeleteContact(peerName.ToString());
			switch (num)
			{
			case -2140971007:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Contact not found in Contact Manager");
				throw new ArgumentException(SR.GetString("Collab_ContactNotFound"), "peerName");
			default:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabDeleteContact returned with errorcode {0}", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_DeleteContactFailed"), num);
			case 0:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving DeleteContact() successfully.");
				break;
			}
		}

		[SecurityCritical]
		public void UpdateContact(PeerContact peerContact)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerContact == null)
			{
				throw new ArgumentNullException("peerContact");
			}
			if (peerContact.PeerName == null)
			{
				throw new ArgumentException(SR.GetString("Collab_NoPeerNameInContact"));
			}
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering UpdateContact() with following peercontact");
				peerContact.TracePeerContact();
			}
			SafeCollabMemory safeCredentials = null;
			int num = 0;
			try
			{
				PEER_CONTACT pc = CollaborationHelperFunctions.ConvertPeerContactToPEER_CONTACT(peerContact, ref safeCredentials);
				num = UnsafeCollabNativeMethods.PeerCollabUpdateContact(ref pc);
			}
			finally
			{
				safeCredentials?.Dispose();
			}
			switch (num)
			{
			case -2140971007:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Contact not found in Contact Manager");
				throw new ArgumentException(SR.GetString("Collab_ContactNotFound"), "peerContact");
			default:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabUpdateContact returned with errorcode {0}", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_UpdateContactFailed"), num);
			case 0:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving UpdateContact() successfully.");
				break;
			}
		}

		[SecurityCritical]
		public void Dispose()
		{
			Dispose(disposing: true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		private void Dispose(bool disposing)
		{
			if (!m_Disposed)
			{
				CollaborationHelperFunctions.CleanEventVars(ref m_regSubLstChangedWaitHandle, ref m_safeSubLstChangedEvent, ref m_subLstChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean SubscriptionListChanged variables successful.");
				CollaborationHelperFunctions.CleanEventVars(ref m_regNameChangedWaitHandle, ref m_safeNameChangedEvent, ref m_nameChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean NameChanged variables successful.");
				CollaborationHelperFunctions.CleanEventVars(ref m_regPresenceChangedWaitHandle, ref m_safePresenceChangedEvent, ref m_presenceChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean PresenceChanged variables successful.");
				CollaborationHelperFunctions.CleanEventVars(ref m_regAppChangedWaitHandle, ref m_safeAppChangedEvent, ref m_appChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChanged variables successful.");
				CollaborationHelperFunctions.CleanEventVars(ref m_regObjChangedWaitHandle, ref m_safeObjChangedEvent, ref m_objChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChanged variables successful.");
				m_Disposed = true;
			}
		}
	}
}
