#define TRACE
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	internal class InviteAsyncHelper : IDisposable
	{
		internal object m_userState;

		internal SafeCollabInvite m_SafeCollabInvite;

		internal AutoResetEvent m_InviteEvent = new AutoResetEvent(initialState: false);

		internal RegisteredWaitHandle m_RegisteredWaitHandle;

		internal bool m_Disposed;

		internal bool m_Cancelled;

		private AsyncOperation m_AsyncOp;

		private WeakReference m_peerWeakReference;

		private object m_Lock = new object();

		private int m_TraceEventId;

		private Exception m_latestException;

		private Collection<PeerInvitationResponseType> m_responses = new Collection<PeerInvitationResponseType>();

		private bool m_Completed;

		private bool m_aboutToFireCallback;

		private object m_aboutToFireCallbackLock = new object();

		private int m_numberOfResponses;

		private PeerContact m_peerContact;

		private PeerEndPointCollection m_peerEndPoints;

		private Guid m_applicationId;

		private string m_message;

		private byte[] m_inviteData;

		internal int TraceEventId => m_TraceEventId;

		internal InviteAsyncHelper(PeerContact peerContact, Peer parentPeer, PeerEndPointCollection peerEndPoints, Guid applicationId, string message, byte[] inviteData, object userState, int NewTraceEventId)
		{
			m_userState = userState;
			m_peerContact = peerContact;
			m_applicationId = applicationId;
			m_message = message;
			m_inviteData = inviteData;
			m_peerEndPoints = peerEndPoints;
			m_TraceEventId = NewTraceEventId;
			m_peerWeakReference = new WeakReference(parentPeer);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "New InviteAsyncHelper created with TraceEventID {0}", m_TraceEventId);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "\tPeerContact: {0}, App Guid: {1}, userState {2}, ParentReference {3}", (m_peerContact != null) ? m_peerContact.ToString() : "null", applicationId.ToString(), userState.GetHashCode(), m_peerWeakReference.Target.GetHashCode());
		}

		[SecurityCritical]
		internal void InviteAsync()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "InviteAsync called");
			m_RegisteredWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_InviteEvent, InviteCallback, null, -1, executeOnlyOnce: false);
			PEER_INVITATION pi = default(PEER_INVITATION);
			pi.applicationId = CollaborationHelperFunctions.ConvertGuidToGUID(m_applicationId);
			pi.pwzMessage = m_message;
			SafeCollabMemory safeCollabMemory = null;
			pi.applicationData.cbData = (uint)((m_inviteData != null) ? m_inviteData.Length : 0);
			if (m_inviteData != null && m_inviteData.Length > 0)
			{
				safeCollabMemory = new SafeCollabMemory(m_inviteData.Length);
				pi.applicationData.pbData = safeCollabMemory.DangerousGetHandle();
				Marshal.Copy(m_inviteData, 0, pi.applicationData.pbData, m_inviteData.Length);
			}
			else
			{
				pi.applicationData.pbData = IntPtr.Zero;
			}
			foreach (PeerEndPoint peerEndPoint in m_peerEndPoints)
			{
				try
				{
					InviteAsyncEndPoint(peerEndPoint, pi);
				}
				catch (PeerToPeerException)
				{
					if (!m_SafeCollabInvite.IsInvalid && !m_SafeCollabInvite.IsClosed)
					{
						m_SafeCollabInvite.Dispose();
					}
					m_RegisteredWaitHandle.Unregister(null);
					m_RegisteredWaitHandle = null;
					throw;
				}
			}
			m_AsyncOp = AsyncOperationManager.CreateOperation(m_userState);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Leaving InviteAsync.");
		}

		[SecurityCritical]
		internal void InviteAsyncEndPoint(PeerEndPoint peerEndPoint, PEER_INVITATION pi)
		{
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "InviteAsyncEndPoint() is called with the following Info");
				peerEndPoint.TracePeerEndPoint();
			}
			PEER_ENDPOINT pEER_ENDPOINT = default(PEER_ENDPOINT);
			pEER_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gCHandle = default(GCHandle);
			if (peerEndPoint.Name != null)
			{
				gCHandle = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
				pEER_ENDPOINT.pwzEndpointName = gCHandle.AddrOfPinnedObject();
			}
			GCHandle gCHandle2 = GCHandle.Alloc(pEER_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gCHandle2.AddrOfPinnedObject();
			int num;
			try
			{
				if (m_peerContact != null)
				{
					SafeCollabMemory safeCredentials = null;
					PEER_CONTACT pcContact = CollaborationHelperFunctions.ConvertPeerContactToPEER_CONTACT(m_peerContact, ref safeCredentials);
					try
					{
						num = UnsafeCollabNativeMethods.PeerCollabAsyncInviteContact(ref pcContact, pcEndpoint, ref pi, m_InviteEvent.SafeWaitHandle, out m_SafeCollabInvite);
					}
					finally
					{
						safeCredentials?.Dispose();
					}
				}
				else
				{
					num = UnsafeCollabNativeMethods.PeerCollabAsyncInviteEndpoint(pcEndpoint, ref pi, m_InviteEvent.SafeWaitHandle, out m_SafeCollabInvite);
				}
			}
			finally
			{
				if (gCHandle.IsAllocated)
				{
					gCHandle.Free();
				}
				if (gCHandle2.IsAllocated)
				{
					gCHandle2.Free();
				}
			}
			if (num != 0)
			{
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_AsyncInviteFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Leaving InviteAsyncEndPoint.");
		}

		[SecurityCritical]
		internal void InviteCallback(object state, bool timedOut)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Entering InviteCallback.");
			SafeCollabData ppInvitationResponse = null;
			int num = 0;
			InviteCompletedEventArgs inviteCompletedEventArgs = null;
			Peer peer = null;
			bool flag = false;
			PEER_INVITATION_RESPONSE pEER_INVITATION_RESPONSE = default(PEER_INVITATION_RESPONSE);
			try
			{
				lock (m_Lock)
				{
					if (m_Cancelled || m_Completed)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Invite cancelled({0}) or completed({1}). Returning without doing anything.", m_Cancelled, m_Completed);
						return;
					}
					num = UnsafeCollabNativeMethods.PeerCollabGetInvitationResponse(m_SafeCollabInvite, out ppInvitationResponse);
				}
				if (num != 0 && num != -2140966907)
				{
					m_latestException = PeerToPeerException.CreateFromHr(SR.GetString("Collab_AsyncInviteException"), num);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Got an exception {0}. Storing it in latest exception.", m_latestException);
				}
				else
				{
					pEER_INVITATION_RESPONSE = (PEER_INVITATION_RESPONSE)Marshal.PtrToStructure(ppInvitationResponse.DangerousGetHandle(), typeof(PEER_INVITATION_RESPONSE));
					lock (m_responses)
					{
						m_responses.Add(pEER_INVITATION_RESPONSE.action);
					}
					if (pEER_INVITATION_RESPONSE.action == PeerInvitationResponseType.Accepted)
					{
						inviteCompletedEventArgs = new InviteCompletedEventArgs(new PeerInvitationResponse(pEER_INVITATION_RESPONSE.action), null, cancelled: false, m_AsyncOp.UserSuppliedState);
						flag = true;
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Found an accepted. About to fire callback.");
						m_RegisteredWaitHandle.Unregister(null);
					}
				}
				Interlocked.Increment(ref m_numberOfResponses);
				if (!flag && m_numberOfResponses == m_peerEndPoints.Count && !m_aboutToFireCallback)
				{
					lock (m_aboutToFireCallbackLock)
					{
						if (m_aboutToFireCallback)
						{
							return;
						}
						m_aboutToFireCallback = true;
						bool flag2 = false;
						bool flag3 = false;
						flag = true;
						using (IEnumerator<PeerInvitationResponseType> enumerator = m_responses.GetEnumerator())
						{
							while (enumerator.MoveNext())
							{
								switch (enumerator.Current)
								{
								case PeerInvitationResponseType.Expired:
									flag3 = true;
									break;
								case PeerInvitationResponseType.Declined:
									flag2 = true;
									goto end_IL_0231;
								}
							}
							end_IL_0231:;
						}
						if (flag2)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Got a declined invite response.");
							inviteCompletedEventArgs = new InviteCompletedEventArgs(new PeerInvitationResponse(PeerInvitationResponseType.Declined), null, cancelled: false, m_AsyncOp.UserSuppliedState);
						}
						else if (flag3)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Got an expired invite response but no accepted or declined.");
							inviteCompletedEventArgs = new InviteCompletedEventArgs(new PeerInvitationResponse(PeerInvitationResponseType.Expired), null, cancelled: false, m_AsyncOp.UserSuppliedState);
						}
						else
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Got all error responses");
							inviteCompletedEventArgs = new InviteCompletedEventArgs(null, (m_latestException != null) ? m_latestException : new PeerToPeerException("InviteAsync failure."), cancelled: false, m_AsyncOp.UserSuppliedState);
						}
					}
				}
				if (flag)
				{
					peer = (m_peerWeakReference.Target as Peer);
					if (!m_Completed && peer != null)
					{
						lock (m_Lock)
						{
							if (!m_Completed && !m_Cancelled)
							{
								m_Completed = true;
								Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Firing callback with response type {0}.", inviteCompletedEventArgs.InviteResponse);
								peer.PrepareToRaiseInviteCompletedEvent(m_AsyncOp, inviteCompletedEventArgs);
							}
						}
					}
				}
			}
			finally
			{
				ppInvitationResponse?.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Leaving InviteCallback.");
		}

		[SecurityCritical]
		public void ContinueCancelCallback(object state)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Entering ContineCancelCallback.");
			try
			{
				lock (m_Lock)
				{
					if (m_Completed)
					{
						return;
					}
					m_Cancelled = true;
					int num = UnsafeCollabNativeMethods.PeerCollabCancelInvitation(m_SafeCollabInvite);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabCancelInvitation returned with errorcode {0}", num);
					}
					m_SafeCollabInvite.Dispose();
				}
				Peer peer = m_peerWeakReference.Target as Peer;
				if (peer != null)
				{
					InviteCompletedEventArgs args = new InviteCompletedEventArgs(null, null, cancelled: true, m_AsyncOp.UserSuppliedState);
					peer.PrepareToRaiseInviteCompletedEvent(m_AsyncOp, args);
				}
			}
			catch (ObjectDisposedException ex)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Critical, 0, "Exception while cancelling the call {0}", ex);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Leaving ContineCancelCallback.");
		}

		[SecurityCritical]
		public void CancelAsync(object state)
		{
			ThreadPool.QueueUserWorkItem(ContinueCancelCallback, state);
		}

		[SecurityCritical]
		public void Dispose()
		{
			Dispose(disposing: true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		public void Dispose(bool disposing)
		{
			if (!m_Disposed)
			{
				if (!m_SafeCollabInvite.IsInvalid)
				{
					m_SafeCollabInvite.Dispose();
				}
				if (m_RegisteredWaitHandle != null)
				{
					m_RegisteredWaitHandle.Unregister(null);
					m_RegisteredWaitHandle = null;
				}
				if (m_InviteEvent != null)
				{
					m_InviteEvent.Close();
				}
			}
			m_Disposed = true;
		}

		public override int GetHashCode()
		{
			return m_TraceEventId;
		}
	}
}
