#define TRACE
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Text;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public abstract class Peer : IDisposable, IEquatable<Peer>, ISerializable
	{
		private PeerEndPointCollection m_peerEndPoints = new PeerEndPointCollection();

		private ISynchronizeInvoke m_synchronizingObject;

		private SendOrPostCallback OnInviteCompletedDelegate;

		internal Dictionary<object, InviteAsyncHelper> m_inviteAsyncHelperList = new Dictionary<object, InviteAsyncHelper>();

		private static int s_TraceEventId = 1;

		private bool m_Disposed;

		public virtual PeerEndPointCollection PeerEndPoints
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_peerEndPoints;
			}
		}

		public bool IsOnline
		{
			get
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Get Isonline called.");
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				bool flag = false;
				foreach (PeerEndPoint peerEndPoint in PeerEndPoints)
				{
					PeerPresenceInfo peerPresenceInfo = null;
					try
					{
						peerPresenceInfo = GetPresenceInfo(peerEndPoint);
					}
					catch (Exception ex)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Exception thrown {0}", ex.Message);
					}
					if (peerPresenceInfo != null && peerPresenceInfo.PresenceStatus == PeerPresenceStatus.Online)
					{
						flag = true;
						break;
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving Isonline called with {0}.", flag);
				return flag;
			}
		}

		[DefaultValue(null)]
		[Description("SynchronizingObject")]
		[Browsable(false)]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_synchronizingObject;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_synchronizingObject = value;
			}
		}

		internal static Guid CurrentApplicationGuid
		{
			[SecurityCritical]
			get
			{
				Guid empty = Guid.Empty;
				string b = Path.Combine(Environment.CurrentDirectory, Process.GetCurrentProcess().ProcessName + ".exe");
				string b2 = null;
				string[] commandLineArgs = Environment.GetCommandLineArgs();
				int num = commandLineArgs.Length;
				if (num > 1)
				{
					StringBuilder stringBuilder = new StringBuilder();
					for (int i = 1; i < num; i++)
					{
						stringBuilder.Append(commandLineArgs[i]);
						if (i != num - 1)
						{
							stringBuilder.Append(' ');
						}
					}
					b2 = stringBuilder.ToString();
				}
				PeerApplicationCollection localRegisteredApplications = PeerCollaboration.GetLocalRegisteredApplications();
				foreach (PeerApplication item in localRegisteredApplications)
				{
					if (item.CommandLineArgs == b2 && item.Path == b)
					{
						return item.Id;
					}
				}
				return empty;
			}
		}

		internal static int NewTraceEventId
		{
			get
			{
				Interlocked.CompareExchange(ref s_TraceEventId, 0, int.MaxValue);
				Interlocked.Increment(ref s_TraceEventId);
				return s_TraceEventId;
			}
		}

		private event EventHandler<InviteCompletedEventArgs> m_inviteCompleted;

		public event EventHandler<InviteCompletedEventArgs> InviteCompleted
		{
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_inviteCompleted = (EventHandler<InviteCompletedEventArgs>)Delegate.Combine(this.m_inviteCompleted, value);
			}
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_inviteCompleted = (EventHandler<InviteCompletedEventArgs>)Delegate.Remove(this.m_inviteCompleted, value);
			}
		}

		internal Peer()
		{
			OnInviteCompletedDelegate = InviteCompletedWaitCallback;
		}

		protected Peer(SerializationInfo serializationInfo, StreamingContext streamingContext)
			: this()
		{
			m_peerEndPoints = (PeerEndPointCollection)serializationInfo.GetValue("_PeerEndPoints", typeof(PeerEndPointCollection));
		}

		[SecurityCritical]
		public PeerPresenceInfo GetPresenceInfo(PeerEndPoint peerEndPoint)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "GetPresenceInfo()called.");
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new ArgumentException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Getting presence for the following endpoint.");
				peerEndPoint.TracePeerEndPoint();
			}
			SafeCollabData pPresenceInfo = null;
			PeerPresenceInfo peerPresenceInfo = null;
			PEER_ENDPOINT pEER_ENDPOINT = default(PEER_ENDPOINT);
			pEER_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gCHandle = default(GCHandle);
			if (peerEndPoint.Name != null)
			{
				gCHandle = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
				pEER_ENDPOINT.pwzEndpointName = gCHandle.AddrOfPinnedObject();
			}
			GCHandle gCHandle2 = GCHandle.Alloc(pEER_ENDPOINT, GCHandleType.Pinned);
			IntPtr endpoint = gCHandle2.AddrOfPinnedObject();
			RefreshIfNeeded();
			try
			{
				int num = UnsafeCollabNativeMethods.PeerCollabGetPresenceInfo(endpoint, out pPresenceInfo);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetPresenceInfo returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPresenceFailed"), num);
				}
				IntPtr ptr = pPresenceInfo.DangerousGetHandle();
				PEER_PRESENCE_INFO pEER_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(ptr, typeof(PEER_PRESENCE_INFO));
				peerPresenceInfo = new PeerPresenceInfo();
				peerPresenceInfo.PresenceStatus = pEER_PRESENCE_INFO.status;
				peerPresenceInfo.DescriptiveText = pEER_PRESENCE_INFO.descText;
			}
			finally
			{
				if (gCHandle.IsAllocated)
				{
					gCHandle.Free();
				}
				if (gCHandle2.IsAllocated)
				{
					gCHandle2.Free();
				}
				pPresenceInfo?.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving GetPresenceInfo().");
			return peerPresenceInfo;
		}

		[SecurityCritical]
		public PeerObjectCollection GetObjects()
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			return InternalGetAllObjects(Guid.Empty, guidSupplied: false);
		}

		[SecurityCritical]
		public PeerObjectCollection GetObjects(Guid objectId)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			return InternalGetAllObjects(objectId, guidSupplied: true);
		}

		internal abstract void RefreshIfNeeded();

		[SecurityCritical]
		private PeerObjectCollection InternalGetAllObjects(Guid objectId, bool guidSupplied)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering InternalGetAllObjects() with ObjectId {0}.", objectId);
			Dictionary<Guid, PeerObject> dictionary = new Dictionary<Guid, PeerObject>();
			RefreshIfNeeded();
			PeerObjectCollection peerObjectCollection;
			foreach (PeerEndPoint peerEndPoint in PeerEndPoints)
			{
				peerObjectCollection = InternalGetObjects(objectId, guidSupplied, peerEndPoint);
				if (guidSupplied && peerObjectCollection.Count != 0)
				{
					return peerObjectCollection;
				}
				foreach (PeerObject item in peerObjectCollection)
				{
					dictionary[item.Id] = item;
				}
			}
			Dictionary<Guid, PeerObject>.ValueCollection values = dictionary.Values;
			peerObjectCollection = new PeerObjectCollection();
			foreach (PeerObject item2 in values)
			{
				peerObjectCollection.Add(item2);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving InternalGetAllObjects(). Returning collection with {0} objects.", peerObjectCollection.Count);
			return peerObjectCollection;
		}

		[SecurityCritical]
		internal unsafe static PeerObjectCollection InternalGetObjects(Guid objectId, bool guidSupplied, PeerEndPoint peerEndPoint)
		{
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering InternalGetObjects() with the following PeerEndPoint");
				peerEndPoint.TracePeerEndPoint();
			}
			PeerObjectCollection peerObjectCollection = new PeerObjectCollection();
			SafeCollabEnum phPeerEnum = null;
			uint pCount = 0u;
			int num = 0;
			GCHandle gCHandle = default(GCHandle);
			IntPtr pObjectId = IntPtr.Zero;
			if (guidSupplied)
			{
				GUID gUID = CollaborationHelperFunctions.ConvertGuidToGUID(objectId);
				gCHandle = GCHandle.Alloc(gUID, GCHandleType.Pinned);
				pObjectId = gCHandle.AddrOfPinnedObject();
			}
			PEER_ENDPOINT pEER_ENDPOINT = default(PEER_ENDPOINT);
			pEER_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gCHandle2 = default(GCHandle);
			if (peerEndPoint.Name != null)
			{
				gCHandle2 = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
				pEER_ENDPOINT.pwzEndpointName = gCHandle2.AddrOfPinnedObject();
			}
			GCHandle gCHandle3 = GCHandle.Alloc(pEER_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gCHandle3.AddrOfPinnedObject();
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabEnumObjects(pcEndpoint, pObjectId, out phPeerEnum);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumObjects returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectsFailed"), num);
				}
				num = UnsafeCollabNativeMethods.PeerGetItemCount(phPeerEnum, ref pCount);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectsFailed"), num);
				}
				if (pCount == 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "No PeerObjects found.");
					return peerObjectCollection;
				}
				num = UnsafeCollabNativeMethods.PeerGetNextItem(phPeerEnum, ref pCount, out SafeCollabData pppvItems);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetNextItem returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectsFailed"), num);
				}
				IntPtr value = pppvItems.DangerousGetHandle();
				IntPtr* ptr = (IntPtr*)(void*)value;
				for (ulong num2 = 0uL; num2 < pCount; num2++)
				{
					PEER_OBJECT* ptr2 = (PEER_OBJECT*)(void*)ptr[num2];
					byte[] array = null;
					if (ptr2->data.cbData != 0)
					{
						array = new byte[ptr2->data.cbData];
						Marshal.Copy(ptr2->data.pbData, array, 0, (int)ptr2->data.cbData);
					}
					PeerObject peerObject = new PeerObject(CollaborationHelperFunctions.ConvertGUIDToGuid(ptr2->guid), array, (PeerScope)ptr2->dwPublicationScope);
					if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Retrieved following Object");
						peerObject.TracePeerObject();
					}
					peerObjectCollection.Add(peerObject);
				}
			}
			finally
			{
				if (gCHandle.IsAllocated)
				{
					gCHandle.Free();
				}
				if (gCHandle2.IsAllocated)
				{
					gCHandle2.Free();
				}
				if (gCHandle3.IsAllocated)
				{
					gCHandle3.Free();
				}
				phPeerEnum?.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving InternalGetApplications(). Returning collection with {0} objects.", peerObjectCollection.Count);
			return peerObjectCollection;
		}

		public abstract PeerInvitationResponse Invite();

		public abstract PeerInvitationResponse Invite(PeerApplication applicationToInvite, string message, byte[] invitationData);

		[SecurityCritical]
		internal static PeerInvitationResponse InternalInviteEndPoint(Guid applicationToInviteGuid, string message, byte[] invitationData, PeerEndPoint peerEndPoint, PeerContact peerContact)
		{
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering InternalInviteEndPoint() with the following information.");
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Invitation Message: ", message);
				if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose) && invitationData != null)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tInvitation data:");
					Logging.DumpData(Logging.P2PTraceSource, TraceEventType.Verbose, Logging.P2PTraceSource.MaxDataSize, invitationData, 0, invitationData.Length);
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Invitation Data length: ", (invitationData != null) ? invitationData.Length : 0);
				}
				peerEndPoint?.TracePeerEndPoint();
			}
			PEER_INVITATION pcInvitation = default(PEER_INVITATION);
			pcInvitation.applicationId = CollaborationHelperFunctions.ConvertGuidToGUID(applicationToInviteGuid);
			pcInvitation.pwzMessage = message;
			SafeCollabMemory safeCollabMemory = null;
			pcInvitation.applicationData.cbData = (uint)((invitationData != null) ? invitationData.Length : 0);
			if (invitationData != null && invitationData.Length > 0)
			{
				safeCollabMemory = new SafeCollabMemory(invitationData.Length);
				pcInvitation.applicationData.pbData = safeCollabMemory.DangerousGetHandle();
				Marshal.Copy(invitationData, 0, pcInvitation.applicationData.pbData, invitationData.Length);
			}
			else
			{
				pcInvitation.applicationData.pbData = IntPtr.Zero;
			}
			PEER_ENDPOINT pEER_ENDPOINT = default(PEER_ENDPOINT);
			pEER_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gCHandle = default(GCHandle);
			if (peerEndPoint.Name != null)
			{
				gCHandle = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
				pEER_ENDPOINT.pwzEndpointName = gCHandle.AddrOfPinnedObject();
			}
			GCHandle gCHandle2 = GCHandle.Alloc(pEER_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gCHandle2.AddrOfPinnedObject();
			SafeCollabData ppResponse = null;
			PeerInvitationResponse peerInvitationResponse = null;
			try
			{
				int num;
				if (peerContact != null)
				{
					SafeCollabMemory safeCredentials = null;
					PEER_CONTACT pcContact = CollaborationHelperFunctions.ConvertPeerContactToPEER_CONTACT(peerContact, ref safeCredentials);
					try
					{
						num = UnsafeCollabNativeMethods.PeerCollabInviteContact(ref pcContact, pcEndpoint, ref pcInvitation, out ppResponse);
					}
					finally
					{
						safeCredentials?.Dispose();
					}
				}
				else
				{
					num = UnsafeCollabNativeMethods.PeerCollabInviteEndpoint(pcEndpoint, ref pcInvitation, out ppResponse);
				}
				if (num != 0)
				{
					if (num == -2140966907 || num == -2147023436)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Timed out. Leaving InternalInviteEndPoint() with InvitationResponseType expired.");
						return new PeerInvitationResponse(PeerInvitationResponseType.Expired);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, ((peerContact != null) ? "PeerCollabInviteContact" : "PeerCollabInviteEndpoint") + " returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_InviteFailed"), num);
				}
				if (!ppResponse.IsInvalid)
				{
					PEER_INVITATION_RESPONSE pEER_INVITATION_RESPONSE = (PEER_INVITATION_RESPONSE)Marshal.PtrToStructure(ppResponse.DangerousGetHandle(), typeof(PEER_INVITATION_RESPONSE));
					peerInvitationResponse = new PeerInvitationResponse(pEER_INVITATION_RESPONSE.action);
				}
			}
			finally
			{
				ppResponse?.Dispose();
				if (gCHandle.IsAllocated)
				{
					gCHandle.Free();
				}
				if (gCHandle2.IsAllocated)
				{
					gCHandle2.Free();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving InternalInviteEndPoint() with InvitationResponse {0}.", peerInvitationResponse);
			return peerInvitationResponse;
		}

		public abstract void InviteAsync(object userToken);

		public abstract void InviteAsync(PeerApplication applicationToInvite, string message, byte[] invitationData, object userToken);

		[SecurityCritical]
		internal void InternalInviteAsync(Guid applicationToInviteGuid, string message, byte[] invitationData, PeerEndPointCollection peerEndPoints, PeerContact peerContact, object userToken)
		{
			InviteAsyncHelper inviteAsyncHelper = null;
			int newTraceEventId = NewTraceEventId;
			lock (m_inviteAsyncHelperList)
			{
				if (m_inviteAsyncHelperList.ContainsKey(userToken))
				{
					throw new ArgumentException(SR.GetString("DuplicateUserToken"));
				}
				inviteAsyncHelper = new InviteAsyncHelper(peerContact, this, peerEndPoints, applicationToInviteGuid, message, invitationData, userToken, newTraceEventId);
				m_inviteAsyncHelperList[userToken] = inviteAsyncHelper;
			}
			try
			{
				inviteAsyncHelper.InviteAsync();
			}
			catch
			{
				lock (m_inviteAsyncHelperList)
				{
					m_inviteAsyncHelperList.Remove(userToken);
				}
				throw;
			}
		}

		protected virtual void OnInviteCompleted(InviteCompletedEventArgs e)
		{
			EventHandler<InviteCompletedEventArgs> inviteCompleted = this.m_inviteCompleted;
			if (inviteCompleted != null)
			{
				inviteCompleted(this, e);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the invite completed event callback.");
			}
		}

		private void InviteCompletedWaitCallback(object operationState)
		{
			OnInviteCompleted((InviteCompletedEventArgs)operationState);
		}

		internal void PrepareToRaiseInviteCompletedEvent(AsyncOperation asyncOP, InviteCompletedEventArgs args)
		{
			lock (m_inviteAsyncHelperList)
			{
				InviteAsyncHelper inviteAsyncHelper = m_inviteAsyncHelperList[args.UserState];
				if (inviteAsyncHelper == null)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Critical, 0, "userState for which we are about to call Completed event does not exist in the pending async list");
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Critical, inviteAsyncHelper.TraceEventId, "userState {0} is being removed from the pending async list", args.UserState.GetHashCode());
					m_inviteAsyncHelperList.Remove(args.UserState);
				}
			}
			asyncOP.PostOperationCompleted(OnInviteCompletedDelegate, args);
		}

		[SecurityCritical]
		public void InviteAsyncCancel(object userToken)
		{
			if (userToken == null)
			{
				throw new ArgumentNullException("userToken");
			}
			InviteAsyncHelper value;
			lock (m_inviteAsyncHelperList)
			{
				if (!m_inviteAsyncHelperList.TryGetValue(userToken, out value))
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Warning, 0, "InviteAsyncCancel called with a userState token that is not in the pending async list - returning");
					return;
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, value.TraceEventId, "Proceeding to cancel the pending async");
			value.CancelAsync(userToken);
		}

		public bool Equals(Peer other)
		{
			if (other != null && other.PeerEndPoints != null)
			{
				return other.PeerEndPoints.Equals(PeerEndPoints);
			}
			return false;
		}

		public override string ToString()
		{
			return PeerEndPoints.ToString();
		}

		public void Dispose()
		{
			Dispose(disposing: true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_PeerEndPoints", PeerEndPoints);
		}

		protected virtual void Dispose(bool disposing)
		{
			if (!m_Disposed)
			{
				m_Disposed = true;
			}
		}
	}
}
