#define TRACE
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerApplication : IDisposable, IEquatable<PeerApplication>, ISerializable
	{
		private const int c_16K = 16384;

		private Guid m_id;

		private byte[] m_data;

		private string m_description;

		private string m_path;

		private string m_commandLineArgs;

		private PeerScope m_peerScope;

		private ISynchronizeInvoke m_synchronizingObject;

		private object m_lockAppChangedEvent;

		private RegisteredWaitHandle m_regAppChangedWaitHandle;

		private AutoResetEvent m_appChangedEvent;

		private SafeCollabEvent m_safeAppChangedEvent;

		private bool m_Disposed;

		public Guid Id
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_id;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_id = value;
			}
		}

		public byte[] Data
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_data;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				if (value != null && value.Length > 16384)
				{
					throw new ArgumentException(SR.GetString("Collab_ApplicationDataSizeFailed"));
				}
				m_data = value;
			}
		}

		public string Description
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_description;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_description = value;
			}
		}

		public string Path
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_path;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_path = value;
			}
		}

		public string CommandLineArgs
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_commandLineArgs;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_commandLineArgs = value;
			}
		}

		public PeerScope PeerScope
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_peerScope;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_peerScope = value;
			}
		}

		[DefaultValue(null)]
		[Browsable(false)]
		[Description("SynchronizingObject")]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_synchronizingObject;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_synchronizingObject = value;
			}
		}

		private object LockAppChangedEvent
		{
			get
			{
				if (m_lockAppChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockAppChangedEvent, value, null);
				}
				return m_lockAppChangedEvent;
			}
		}

		private event EventHandler<ApplicationChangedEventArgs> m_applicationChanged;

		public event EventHandler<ApplicationChangedEventArgs> ApplicationChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddApplicationChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemoveApplicationChanged(value);
			}
		}

		[SecurityCritical]
		static PeerApplication()
		{
			CollaborationHelperFunctions.Initialize();
		}

		public PeerApplication()
		{
		}

		public PeerApplication(Guid id, string description, byte[] data, string path, string commandLineArgs, PeerScope peerScope)
		{
			if (data != null && data.Length > 16384)
			{
				throw new ArgumentException(SR.GetString("Collab_ApplicationDataSizeFailed"), "data");
			}
			m_id = id;
			m_description = description;
			m_data = data;
			m_path = path;
			m_commandLineArgs = commandLineArgs;
			m_peerScope = peerScope;
		}

		protected PeerApplication(SerializationInfo serializationInfo, StreamingContext streamingContext)
		{
			m_id = (Guid)serializationInfo.GetValue("_Id", typeof(Guid));
			m_data = (byte[])serializationInfo.GetValue("_Data", typeof(byte[]));
			m_description = serializationInfo.GetString("_Description");
			m_path = serializationInfo.GetString("_Path");
			m_commandLineArgs = serializationInfo.GetString("_CommandLineArgs");
			m_peerScope = (PeerScope)serializationInfo.GetInt32("_Scope");
		}

		[SecurityCritical]
		private void AddApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() called.");
			lock (LockAppChangedEvent)
			{
				if (this.m_applicationChanged == null)
				{
					if (m_id.Equals(Guid.Empty))
					{
						throw new PeerToPeerException("No application guid defined");
					}
					m_appChangedEvent = new AutoResetEvent(initialState: false);
					m_regAppChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_appChangedEvent, ApplicationChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.EndPointApplicationChanged;
					GUID gUID = CollaborationHelperFunctions.ConvertGuidToGUID(m_id);
					GCHandle gCHandle = GCHandle.Alloc(gUID, GCHandleType.Pinned);
					pEventRegistrations.pInstance = gCHandle.AddrOfPinnedObject();
					try
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Registering event with App ID {0}", m_id.ToString());
						int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_appChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safeAppChangedEvent);
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ApplicationChangedRegFailed"), num);
						}
					}
					finally
					{
						if (gCHandle.IsAllocated)
						{
							gCHandle.Free();
						}
					}
				}
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Combine(this.m_applicationChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() called.");
			lock (LockAppChangedEvent)
			{
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Remove(this.m_applicationChanged, callback);
				if (this.m_applicationChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref m_regAppChangedWaitHandle, ref m_safeAppChangedEvent, ref m_appChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() successful.");
		}

		protected virtual void OnApplicationChanged(ApplicationChangedEventArgs appChangedArgs)
		{
			EventHandler<ApplicationChangedEventArgs> applicationChanged = this.m_applicationChanged;
			if (applicationChanged != null)
			{
				if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
				{
					SynchronizingObject.BeginInvoke(applicationChanged, new object[2]
					{
						this,
						appChangedArgs
					});
				}
				else
				{
					applicationChanged(this, appChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the application changed event callback.");
			}
		}

		[SecurityCritical]
		private void ApplicationChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ApplicationChangedCallback() called.");
			if (m_Disposed)
			{
				return;
			}
			while (true)
			{
				bool flag = true;
				ApplicationChangedEventArgs applicationChangedEventArgs = null;
				try
				{
					lock (LockAppChangedEvent)
					{
						if (m_safeAppChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safeAppChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetApplicationChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointApplicationChanged)
						{
							PEER_EVENT_APPLICATION_CHANGED_DATA applicationChangedData = pEER_COLLAB_EVENT_DATA.applicationChangedData;
							PEER_APPLICATION pa = (PEER_APPLICATION)Marshal.PtrToStructure(applicationChangedData.pApplication, typeof(PEER_APPLICATION));
							PeerApplication peerApplication = CollaborationHelperFunctions.ConvertPEER_APPLICATIONToPeerApplication(pa);
							if (object.Equals(m_id, peerApplication.Id))
							{
								PeerContact peerContact = null;
								PeerEndPoint peerEndPoint = null;
								if (applicationChangedData.pContact != IntPtr.Zero)
								{
									PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(applicationChangedData.pContact, typeof(PEER_CONTACT));
									peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
								}
								if (applicationChangedData.pEndPoint != IntPtr.Zero)
								{
									PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(applicationChangedData.pEndPoint, typeof(PEER_ENDPOINT));
									peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
								}
								applicationChangedEventArgs = new ApplicationChangedEventArgs(peerEndPoint, peerContact, applicationChangedData.changeType, peerApplication);
							}
						}
						goto IL_0210;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_0210:
				if (applicationChangedEventArgs != null)
				{
					OnApplicationChanged(applicationChangedEventArgs);
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ApplicationChangedCallback().");
		}

		public bool Equals(PeerApplication other)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			if (other != null)
			{
				return object.Equals(other.Id, m_id);
			}
			return false;
		}

		public override bool Equals(object obj)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerApplication peerApplication = obj as PeerApplication;
			if (peerApplication != null)
			{
				return object.Equals(peerApplication.Id, Id);
			}
			return false;
		}

		public new static bool Equals(object objA, object objB)
		{
			PeerApplication peerApplication = objA as PeerApplication;
			PeerApplication peerApplication2 = objB as PeerApplication;
			if (peerApplication != null && peerApplication2 != null)
			{
				return object.Equals(peerApplication.Id, peerApplication2.Id);
			}
			return false;
		}

		public override int GetHashCode()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return Id.GetHashCode();
		}

		public override string ToString()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return Id.ToString() + " " + Description;
		}

		[SecurityCritical]
		public void Dispose()
		{
			Dispose(disposing: true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		protected virtual void Dispose(bool disposing)
		{
			if (!m_Disposed)
			{
				CollaborationHelperFunctions.CleanEventVars(ref m_regAppChangedWaitHandle, ref m_safeAppChangedEvent, ref m_appChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChanged variables successful.");
				m_Disposed = true;
			}
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_Id", m_id);
			info.AddValue("_Data", m_data);
			info.AddValue("_Description", m_description);
			info.AddValue("_Path", m_path);
			info.AddValue("_CommandLineArgs", m_commandLineArgs);
			info.AddValue("_Scope", m_peerScope);
		}

		internal void TracePeerApplication()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerApplication");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tGuid: {0}", Id);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tDescription: {0}", Description);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tPath: {0}", Path);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tCommandLineArgs: {0}", CommandLineArgs);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tPeerScope: {0}", PeerScope);
			if (Data != null)
			{
				if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose))
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tApplication data:");
					Logging.DumpData(Logging.P2PTraceSource, TraceEventType.Verbose, Logging.P2PTraceSource.MaxDataSize, Data, 0, Data.Length);
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tApplication data length {0}", Data.Length);
				}
			}
		}
	}
}
