#define TRACE
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	public static class PeerCollaboration
	{
		private static object s_contactManagerSyncObject;

		private static ContactManager s_contactManager;

		private static ISynchronizeInvoke s_synchronizingObject;

		private static object s_lockNameChangedEvent;

		private static RegisteredWaitHandle s_regNameChangedWaitHandle;

		private static AutoResetEvent s_nameChangedEvent;

		private static SafeCollabEvent s_safeNameChangedEvent;

		private static object s_lockPresenceChangedEvent;

		private static RegisteredWaitHandle s_regPresenceChangedWaitHandle;

		private static AutoResetEvent s_presenceChangedEvent;

		private static SafeCollabEvent s_safePresenceChangedEvent;

		private static EventHandler<ObjectChangedEventArgs> s_objectChanged;

		private static object s_lockObjChangedEvent;

		private static RegisteredWaitHandle s_regObjChangedWaitHandle;

		private static AutoResetEvent s_objChangedEvent;

		private static SafeCollabEvent s_safeObjChangedEvent;

		private static object s_lockAppChangedEvent;

		private static RegisteredWaitHandle s_regAppChangedWaitHandle;

		private static AutoResetEvent s_appChangedEvent;

		private static SafeCollabEvent s_safeAppChangedEvent;

		public static PeerPresenceInfo LocalPresenceInfo
		{
			[SecurityCritical]
			get
			{
				SafeCollabData pPresenceInfo = null;
				PeerPresenceInfo peerPresenceInfo = null;
				try
				{
					CollaborationHelperFunctions.Initialize();
					int num = UnsafeCollabNativeMethods.PeerCollabGetPresenceInfo(IntPtr.Zero, out pPresenceInfo);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetPresenceInfo returned with errorcode {0}", num);
						return null;
					}
					IntPtr ptr = pPresenceInfo.DangerousGetHandle();
					PEER_PRESENCE_INFO pEER_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(ptr, typeof(PEER_PRESENCE_INFO));
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Retrieved a PeerPresenceInfo with PresenceStatus {0}, Desc {1}", pEER_PRESENCE_INFO.status, pEER_PRESENCE_INFO.descText);
					peerPresenceInfo = new PeerPresenceInfo();
					peerPresenceInfo.PresenceStatus = pEER_PRESENCE_INFO.status;
					peerPresenceInfo.DescriptiveText = pEER_PRESENCE_INFO.descText;
				}
				catch (Exception ex)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetPresenceInfo had exception {0}", ex.Message);
					throw;
				}
				finally
				{
					pPresenceInfo?.Dispose();
				}
				return peerPresenceInfo;
			}
			[SecurityCritical]
			set
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				if (value == null)
				{
					throw new ArgumentNullException("value");
				}
				if (value.PresenceStatus == PeerPresenceStatus.Offline)
				{
					throw new PeerToPeerException(SR.GetString("Collab_SetPresenceOffline"));
				}
				PEER_PRESENCE_INFO ppi = default(PEER_PRESENCE_INFO);
				ppi.status = value.PresenceStatus;
				ppi.descText = value.DescriptiveText;
				int num = UnsafeCollabNativeMethods.PeerCollabSetPresenceInfo(ref ppi);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSetPresenceInfo returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_SetLocalPresenceFailed"), num);
				}
			}
		}

		public static string LocalEndPointName
		{
			[SecurityCritical]
			get
			{
				string ppwzEndpointName = null;
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "LocalEndPointName get called.");
				try
				{
					int num = UnsafeCollabNativeMethods.PeerCollabGetEndpointName(ref ppwzEndpointName);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEndpointName returned with errorcode {0}", num);
					}
				}
				catch (Exception ex)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "LocalEndPointName threw exception {0}", ex.ToString());
					throw;
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "LocalEndPointName get returning {0}.", ppwzEndpointName);
				return ppwzEndpointName;
			}
			[SecurityCritical]
			set
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "LocalEndPointName set called with {0}.", value);
				int num = UnsafeCollabNativeMethods.PeerCollabSetEndpointName(value);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSetEndpointName returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_SetLocalEndPointNameFailed"), num);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving LocalEndPointName set.");
			}
		}

		public static PeerScope SignInScope
		{
			[SecurityCritical]
			get
			{
				PeerScope dwSignInOptions = PeerScope.None;
				CollaborationHelperFunctions.Initialize();
				int num = UnsafeCollabNativeMethods.PeerCollabGetSigninOptions(ref dwSignInOptions);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetSigninOptions returned with errorcode {0}", num);
				}
				return dwSignInOptions;
			}
		}

		internal static object ContactManagerSyncObject
		{
			get
			{
				if (s_contactManagerSyncObject == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref s_contactManagerSyncObject, value, null);
				}
				return s_contactManagerSyncObject;
			}
		}

		public static ContactManager ContactManager
		{
			[SecurityCritical]
			get
			{
				if (s_contactManager == null)
				{
					lock (ContactManagerSyncObject)
					{
						if (s_contactManager == null)
						{
							s_contactManager = new ContactManager();
						}
					}
				}
				return s_contactManager;
			}
		}

		public static PeerApplicationLaunchInfo ApplicationLaunchInfo
		{
			[SecurityCritical]
			get
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetApplicationLaunchInfo.");
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerApplicationLaunchInfo peerApplicationLaunchInfo = null;
				SafeCollabData ppLaunchInfo = null;
				int num = UnsafeCollabNativeMethods.PeerCollabGetAppLaunchInfo(out ppLaunchInfo);
				switch (num)
				{
				case -2147023728:
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "No Application launch info available.");
					return peerApplicationLaunchInfo;
				default:
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetAppLaunchInfo returned with errorcode {0}", num);
					return peerApplicationLaunchInfo;
				case 0:
					try
					{
						PEER_APP_LAUNCH_INFO pEER_APP_LAUNCH_INFO = (PEER_APP_LAUNCH_INFO)Marshal.PtrToStructure(ppLaunchInfo.DangerousGetHandle(), typeof(PEER_APP_LAUNCH_INFO));
						peerApplicationLaunchInfo = new PeerApplicationLaunchInfo();
						if (pEER_APP_LAUNCH_INFO.pContact != IntPtr.Zero)
						{
							PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(pEER_APP_LAUNCH_INFO.pContact, typeof(PEER_CONTACT));
							peerApplicationLaunchInfo.PeerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
						}
						PEER_INVITATION pEER_INVITATION = (PEER_INVITATION)Marshal.PtrToStructure(pEER_APP_LAUNCH_INFO.pInvitation, typeof(PEER_INVITATION));
						peerApplicationLaunchInfo.PeerApplication = new PeerApplication(CollaborationHelperFunctions.ConvertGUIDToGuid(pEER_INVITATION.applicationId), null, null, null, null, PeerScope.None);
						peerApplicationLaunchInfo.Message = pEER_INVITATION.pwzMessage;
						byte[] array = null;
						if (pEER_INVITATION.applicationData.cbData != 0)
						{
							array = new byte[pEER_INVITATION.applicationData.cbData];
							Marshal.Copy(pEER_INVITATION.applicationData.pbData, array, 0, (int)pEER_INVITATION.applicationData.cbData);
						}
						peerApplicationLaunchInfo.Data = array;
						PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(pEER_APP_LAUNCH_INFO.pEndpoint, typeof(PEER_ENDPOINT));
						peerApplicationLaunchInfo.PeerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "GetApplicationLaunchInfo successful.");
					}
					catch (Exception ex)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "GetApplicationLaunchInfo threw error {0}.", ex.ToString());
						throw;
					}
					finally
					{
						ppLaunchInfo?.Dispose();
					}
					return peerApplicationLaunchInfo;
				}
			}
		}

		[Browsable(false)]
		[DefaultValue(null)]
		[Description("SynchronizingObject")]
		public static ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				return s_synchronizingObject;
			}
			set
			{
				s_synchronizingObject = value;
			}
		}

		private static object LockNameChangedEvent
		{
			get
			{
				if (s_lockNameChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref s_lockNameChangedEvent, value, null);
				}
				return s_lockNameChangedEvent;
			}
		}

		private static object LockPresenceChangedEvent
		{
			get
			{
				if (s_lockPresenceChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref s_lockPresenceChangedEvent, value, null);
				}
				return s_lockPresenceChangedEvent;
			}
		}

		private static object LockObjChangedEvent
		{
			get
			{
				if (s_lockObjChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref s_lockObjChangedEvent, value, null);
				}
				return s_lockObjChangedEvent;
			}
		}

		private static object LockAppChangedEvent
		{
			get
			{
				if (s_lockAppChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref s_lockAppChangedEvent, value, null);
				}
				return s_lockAppChangedEvent;
			}
		}

		private static event EventHandler<NameChangedEventArgs> s_nameChanged;

		public static event EventHandler<NameChangedEventArgs> LocalNameChanged
		{
			[SecurityCritical]
			add
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				AddNameChanged(value);
			}
			[SecurityCritical]
			remove
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				RemoveNameChanged(value);
			}
		}

		private static event EventHandler<PresenceChangedEventArgs> s_presenceChanged;

		public static event EventHandler<PresenceChangedEventArgs> LocalPresenceChanged
		{
			[SecurityCritical]
			add
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				AddPresenceChanged(value);
			}
			[SecurityCritical]
			remove
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				RemovePresenceChanged(value);
			}
		}

		public static event EventHandler<ObjectChangedEventArgs> LocalObjectChanged
		{
			[SecurityCritical]
			add
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				AddObjectChangedEvent(value);
			}
			[SecurityCritical]
			remove
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				RemoveObjectChangedEvent(value);
			}
		}

		private static event EventHandler<ApplicationChangedEventArgs> s_applicationChanged;

		public static event EventHandler<ApplicationChangedEventArgs> LocalApplicationChanged
		{
			[SecurityCritical]
			add
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				AddApplicationChanged(value);
			}
			[SecurityCritical]
			remove
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				RemoveApplicationChanged(value);
			}
		}

		[SecurityCritical]
		public static void SignIn(PeerScope peerScope)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering SignIn with peerscope {0}.", peerScope);
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerScope < PeerScope.None || peerScope > PeerScope.All)
			{
				throw new ArgumentOutOfRangeException("peerScope");
			}
			if (peerScope == PeerScope.None)
			{
				throw new ArgumentException(SR.GetString("Collab_SignInWithNone"), "peerScope");
			}
			CollaborationHelperFunctions.Initialize();
			int num = UnsafeCollabNativeMethods.PeerCollabSignin(IntPtr.Zero, peerScope);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSignin returned with errorcode {0}", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_SignInFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Signed into {0}.", peerScope);
		}

		[SecurityCritical]
		public static void SignOut(PeerScope peerScope)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering SignOut with peerscope {0}.", peerScope);
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			CollaborationHelperFunctions.Initialize();
			if (peerScope < PeerScope.None || peerScope > PeerScope.All)
			{
				throw new ArgumentOutOfRangeException("peerScope");
			}
			if (peerScope != 0)
			{
				int num = UnsafeCollabNativeMethods.PeerCollabSignout(peerScope);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSignout returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_SignOutFailed"), num);
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Signed out of {0}.", peerScope);
		}

		[SecurityCritical]
		public unsafe static PeerNearMeCollection GetPeersNearMe()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetPeersNearMeGetPeersNearMe.");
			CollaborationHelperFunctions.Initialize();
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			PeerNearMeCollection peerNearMeCollection = new PeerNearMeCollection();
			SafeCollabEnum phPeerEnum = null;
			uint pCount = 0u;
			int num = 0;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabEnumPeopleNearMe(out phPeerEnum);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumPeopleNearMe returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPeersNearMeFailed"), num);
				}
				num = UnsafeCollabNativeMethods.PeerGetItemCount(phPeerEnum, ref pCount);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPeersNearMeFailed"), num);
				}
				if (pCount == 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "No peers found. \nLeaving GetPeersNearMe()");
					return peerNearMeCollection;
				}
				num = UnsafeCollabNativeMethods.PeerGetNextItem(phPeerEnum, ref pCount, out SafeCollabData pppvItems);
				IntPtr value = pppvItems.DangerousGetHandle();
				IntPtr* ptr = (IntPtr*)(void*)value;
				for (ulong num2 = 0uL; num2 < pCount; num2++)
				{
					IntPtr ptr2 = ptr[num2];
					PEER_PEOPLE_NEAR_ME pEER_PEOPLE_NEAR_ME = (PEER_PEOPLE_NEAR_ME)Marshal.PtrToStructure(ptr2, typeof(PEER_PEOPLE_NEAR_ME));
					PeerNearMe peerNearMe = new PeerNearMe();
					peerNearMe.Id = CollaborationHelperFunctions.ConvertGUIDToGuid(pEER_PEOPLE_NEAR_ME.id);
					peerNearMe.Nickname = Marshal.PtrToStringUni(pEER_PEOPLE_NEAR_ME.pwzNickname);
					PEER_ENDPOINT endpoint = pEER_PEOPLE_NEAR_ME.endpoint;
					peerNearMe.PeerEndPoints.Add(CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(endpoint));
					peerNearMeCollection.Add(peerNearMe);
				}
			}
			finally
			{
				phPeerEnum?.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving GetPeersNearMeGetPeersNearMe with {0} peers.", peerNearMeCollection.Count);
			return peerNearMeCollection;
		}

		[SecurityCritical]
		public static void RegisterApplication(PeerApplication application, PeerApplicationRegistrationType type)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (application == null)
			{
				throw new ArgumentNullException("application");
			}
			if (application.Path == null)
			{
				throw new ArgumentException(SR.GetString("Collab_AppRegNoPathError"));
			}
			if (type < PeerApplicationRegistrationType.CurrentUser || type > PeerApplicationRegistrationType.AllUsers)
			{
				throw new ArgumentOutOfRangeException("type");
			}
			CollaborationHelperFunctions.Initialize();
			int num = 0;
			PEER_APPLICATION_REGISTRATION_INFO appRegInfo = default(PEER_APPLICATION_REGISTRATION_INFO);
			appRegInfo.application.guid = CollaborationHelperFunctions.ConvertGuidToGUID(application.Id);
			appRegInfo.pwzApplicationArguments = application.CommandLineArgs;
			appRegInfo.pwzApplicationToLaunch = application.Path;
			appRegInfo.dwPublicationScope = (uint)application.PeerScope;
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RegisterApplication() is called with the following Info");
				application.TracePeerApplication();
			}
			SafeCollabMemory safeCollabMemory = null;
			appRegInfo.application.data.cbData = (uint)((application.Data != null) ? application.Data.Length : 0);
			GCHandle gCHandle = default(GCHandle);
			try
			{
				if (application.Data != null && application.Data.Length > 0)
				{
					safeCollabMemory = new SafeCollabMemory(application.Data.Length);
					appRegInfo.application.data.pbData = safeCollabMemory.DangerousGetHandle();
					Marshal.Copy(application.Data, 0, appRegInfo.application.data.pbData, application.Data.Length);
				}
				else
				{
					appRegInfo.application.data.pbData = IntPtr.Zero;
				}
				gCHandle = GCHandle.Alloc(application.Description, GCHandleType.Pinned);
				appRegInfo.application.pwzDescription = gCHandle.AddrOfPinnedObject();
				num = UnsafeCollabNativeMethods.PeerCollabRegisterApplication(ref appRegInfo, type);
			}
			finally
			{
				if (gCHandle.IsAllocated)
				{
					gCHandle.Free();
				}
				safeCollabMemory?.Dispose();
			}
			switch (num)
			{
			case -2147024713:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterApplication returned with errorcode {0}. Application already registered.", num);
				throw new ArgumentException(SR.GetString("Collab_AppRegFailed") + " " + SR.GetString("Collab_AppExists"));
			default:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterApplication returned with errorcode {0}", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_AppRegFailed"), num);
			case 0:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "RegisterApplication successful");
				break;
			}
		}

		[SecurityCritical]
		public static void UnregisterApplication(PeerApplication application, PeerApplicationRegistrationType type)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (application == null)
			{
				throw new ArgumentNullException("application");
			}
			if (application.Id.Equals(Guid.Empty))
			{
				throw new ArgumentException(SR.GetString("Collab_EmptyGuidError"));
			}
			if (type < PeerApplicationRegistrationType.CurrentUser || type > PeerApplicationRegistrationType.AllUsers)
			{
				throw new ArgumentOutOfRangeException("type");
			}
			CollaborationHelperFunctions.Initialize();
			GUID pApplicationId = CollaborationHelperFunctions.ConvertGuidToGUID(application.Id);
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "UnregisterApplication() is called with the following Info");
				application.TracePeerApplication();
			}
			int num = UnsafeCollabNativeMethods.PeerCollabUnregisterApplication(ref pApplicationId, type);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabUnregisterApplication returned with errorcode {0}", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_AppUnregFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "UnregisterApplication successful");
		}

		public static PeerApplicationCollection GetLocalRegisteredApplications()
		{
			PeerApplicationCollection localRegisteredApplications = GetLocalRegisteredApplications(PeerApplicationRegistrationType.AllUsers);
			PeerApplicationCollection localRegisteredApplications2 = GetLocalRegisteredApplications(PeerApplicationRegistrationType.CurrentUser);
			if (localRegisteredApplications == null || localRegisteredApplications.Count == 0)
			{
				return localRegisteredApplications2;
			}
			if (localRegisteredApplications2 == null || localRegisteredApplications2.Count == 0)
			{
				return localRegisteredApplications;
			}
			foreach (PeerApplication item in localRegisteredApplications2)
			{
				PeerApplication peerApplication = null;
				foreach (PeerApplication item2 in localRegisteredApplications)
				{
					if (item2.Id.Equals(item.Id))
					{
						peerApplication = item2;
					}
				}
				if (peerApplication != null)
				{
					localRegisteredApplications.Remove(peerApplication);
				}
			}
			foreach (PeerApplication item3 in localRegisteredApplications)
			{
				localRegisteredApplications2.Add(item3);
			}
			return localRegisteredApplications2;
		}

		[SecurityCritical]
		public unsafe static PeerApplicationCollection GetLocalRegisteredApplications(PeerApplicationRegistrationType type)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetLocalRegisteredApplications.");
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (type < PeerApplicationRegistrationType.CurrentUser || type > PeerApplicationRegistrationType.AllUsers)
			{
				throw new ArgumentOutOfRangeException("type");
			}
			CollaborationHelperFunctions.Initialize();
			PeerApplicationCollection peerApplicationCollection = new PeerApplicationCollection();
			SafeCollabEnum phPeerEnum = new SafeCollabEnum();
			uint pCount = 0u;
			int num = 0;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabEnumApplications(IntPtr.Zero, IntPtr.Zero, out phPeerEnum);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumApplications returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalAppsFailed"), num);
				}
				num = UnsafeCollabNativeMethods.PeerGetItemCount(phPeerEnum, ref pCount);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalAppsFailed"), num);
				}
				if (pCount == 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "No local registered PeerApplications found.");
					return peerApplicationCollection;
				}
				SafeCollabData pppvItems = null;
				try
				{
					num = UnsafeCollabNativeMethods.PeerGetNextItem(phPeerEnum, ref pCount, out pppvItems);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetNextItem returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalAppsFailed"), num);
					}
					IntPtr value = pppvItems.DangerousGetHandle();
					IntPtr* ptr = (IntPtr*)(void*)value;
					for (ulong num2 = 0uL; num2 < pCount; num2++)
					{
						PEER_APPLICATION* ptr2 = (PEER_APPLICATION*)(void*)ptr[num2];
						string description = Marshal.PtrToStringUni(ptr2->pwzDescription);
						byte[] array = null;
						if (ptr2->data.cbData != 0)
						{
							array = new byte[ptr2->data.cbData];
							Marshal.Copy(ptr2->data.pbData, array, 0, (int)ptr2->data.cbData);
						}
						PeerApplication item = new PeerApplication(CollaborationHelperFunctions.ConvertGUIDToGuid(ptr2->guid), description, array, null, null, PeerScope.None);
						peerApplicationCollection.Add(item);
					}
				}
				finally
				{
					pppvItems.Dispose();
				}
			}
			finally
			{
				phPeerEnum.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Got all local registered Applications. Start filtering");
			PeerApplicationCollection peerApplicationCollection2 = new PeerApplicationCollection();
			foreach (PeerApplication item2 in peerApplicationCollection)
			{
				GUID pApplicationId = CollaborationHelperFunctions.ConvertGuidToGUID(item2.Id);
				SafeCollabData ppApplication = new SafeCollabData();
				try
				{
					num = UnsafeCollabNativeMethods.PeerCollabGetApplicationRegistrationInfo(ref pApplicationId, type, out ppApplication);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetApplicationRegistrationInfo returned with errorcode {0}", num);
					}
					if (!ppApplication.IsInvalid)
					{
						PEER_APPLICATION_REGISTRATION_INFO pEER_APPLICATION_REGISTRATION_INFO = (PEER_APPLICATION_REGISTRATION_INFO)Marshal.PtrToStructure(ppApplication.DangerousGetHandle(), typeof(PEER_APPLICATION_REGISTRATION_INFO));
						item2.Path = pEER_APPLICATION_REGISTRATION_INFO.pwzApplicationToLaunch;
						item2.CommandLineArgs = pEER_APPLICATION_REGISTRATION_INFO.pwzApplicationArguments;
						item2.PeerScope = (PeerScope)pEER_APPLICATION_REGISTRATION_INFO.dwPublicationScope;
						peerApplicationCollection2.Add(item2);
					}
				}
				finally
				{
					ppApplication.Dispose();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Filtering successful. Returning collection with {0} applications", peerApplicationCollection2.Count);
			return peerApplicationCollection2;
		}

		[SecurityCritical]
		public static void SetObject(PeerObject peerObject)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerObject == null)
			{
				throw new ArgumentNullException("peerObject");
			}
			CollaborationHelperFunctions.Initialize();
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "SetObject() is called with the following info");
				peerObject.TracePeerObject();
			}
			PEER_OBJECT pcObject = default(PEER_OBJECT);
			int num = 0;
			pcObject.guid = CollaborationHelperFunctions.ConvertGuidToGUID(peerObject.Id);
			pcObject.dwPublicationScope = (uint)peerObject.PeerScope;
			SafeCollabMemory safeCollabMemory = null;
			try
			{
				if (peerObject.Data != null && peerObject.Data.Length > 0)
				{
					safeCollabMemory = new SafeCollabMemory(peerObject.Data.Length);
					pcObject.data.pbData = safeCollabMemory.DangerousGetHandle();
					pcObject.data.cbData = (uint)peerObject.Data.Length;
					Marshal.Copy(peerObject.Data, 0, pcObject.data.pbData, peerObject.Data.Length);
				}
				else
				{
					pcObject.data.pbData = IntPtr.Zero;
					pcObject.data.cbData = 0u;
				}
				num = UnsafeCollabNativeMethods.PeerCollabSetObject(ref pcObject);
			}
			finally
			{
				safeCollabMemory?.Dispose();
			}
			switch (num)
			{
			case -2147024713:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSetObject returned with errorcode {0}. Object already set.", num);
				throw new ArgumentException(SR.GetString("Collab_ObjectSetFailed") + " " + SR.GetString("Collab_ObjectExists"));
			default:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSetObject returned with errorcode {0}", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectSetFailed"), num);
			case 0:
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Set Object successful.");
				break;
			}
		}

		[SecurityCritical]
		public static void DeleteObject(PeerObject peerObject)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerObject == null)
			{
				throw new ArgumentNullException("peerObject");
			}
			CollaborationHelperFunctions.Initialize();
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "DeleteObject() is called with the following info");
				peerObject.TracePeerObject();
			}
			GUID pObjectId = CollaborationHelperFunctions.ConvertGuidToGUID(peerObject.Id);
			int num = UnsafeCollabNativeMethods.PeerCollabDeleteObject(ref pObjectId);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabDeleteObject returned with errorcode {0}", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectDeleteFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Delete Object successful.");
		}

		[SecurityCritical]
		public unsafe static PeerObjectCollection GetLocalSetObjects()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetLocalSetObjects.");
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			CollaborationHelperFunctions.Initialize();
			PeerObjectCollection peerObjectCollection = new PeerObjectCollection();
			SafeCollabEnum phPeerEnum = null;
			uint pCount = 0u;
			int num = 0;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabEnumObjects(IntPtr.Zero, IntPtr.Zero, out phPeerEnum);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumObjects returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalObjectsFailed"), num);
				}
				num = UnsafeCollabNativeMethods.PeerGetItemCount(phPeerEnum, ref pCount);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalObjectsFailed"), num);
				}
				if (pCount == 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "No local PeerObjects found.");
					return peerObjectCollection;
				}
				SafeCollabData pppvItems = null;
				try
				{
					num = UnsafeCollabNativeMethods.PeerGetNextItem(phPeerEnum, ref pCount, out pppvItems);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetNextItem returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalObjectsFailed"), num);
					}
					IntPtr value = pppvItems.DangerousGetHandle();
					IntPtr* ptr = (IntPtr*)(void*)value;
					for (ulong num2 = 0uL; num2 < pCount; num2++)
					{
						PEER_OBJECT* ptr2 = (PEER_OBJECT*)(void*)ptr[num2];
						byte[] array = null;
						if (ptr2->data.cbData != 0)
						{
							array = new byte[ptr2->data.cbData];
							Marshal.Copy(ptr2->data.pbData, array, 0, (int)ptr2->data.cbData);
						}
						PeerObject item = new PeerObject(CollaborationHelperFunctions.ConvertGUIDToGuid(ptr2->guid), array, (PeerScope)ptr2->dwPublicationScope);
						peerObjectCollection.Add(item);
					}
				}
				finally
				{
					pppvItems.Dispose();
				}
			}
			finally
			{
				phPeerEnum.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Returning collection with {0} objects", peerObjectCollection.Count);
			return peerObjectCollection;
		}

		[SecurityCritical]
		private static void AddNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddNameChanged().");
			lock (LockNameChangedEvent)
			{
				if (PeerCollaboration.s_nameChanged == null)
				{
					s_nameChangedEvent = new AutoResetEvent(initialState: false);
					s_regNameChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(s_nameChangedEvent, NameChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.MyEndPointChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(s_nameChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out s_safeNameChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_NameChangedRegFailed"), num);
					}
				}
				PeerCollaboration.s_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Combine(PeerCollaboration.s_nameChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddNameChanged() successful.");
		}

		[SecurityCritical]
		private static void RemoveNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() called.");
			lock (LockNameChangedEvent)
			{
				PeerCollaboration.s_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Remove(PeerCollaboration.s_nameChanged, callback);
				if (PeerCollaboration.s_nameChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref s_regNameChangedWaitHandle, ref s_safeNameChangedEvent, ref s_nameChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean NameChangedEvent variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() successful.");
		}

		[SecurityCritical]
		private static void NameChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "NameChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				NameChangedEventArgs nameChangedEventArgs = null;
				try
				{
					lock (LockNameChangedEvent)
					{
						if (s_safeNameChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(s_safeNameChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetNameChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyEndPointChanged)
						{
							if (pEER_COLLAB_EVENT_DATA.endpointChangedData.pEndPoint != IntPtr.Zero)
							{
								return;
							}
							string localEndPointName = LocalEndPointName;
							nameChangedEventArgs = new NameChangedEventArgs(null, null, localEndPointName);
						}
						goto IL_0136;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_0136:
				EventHandler<NameChangedEventArgs> eventHandler = PeerCollaboration.s_nameChanged;
				if (nameChangedEventArgs != null && eventHandler != null)
				{
					if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
					{
						SynchronizingObject.BeginInvoke(eventHandler, new object[2]
						{
							null,
							nameChangedEventArgs
						});
					}
					else
					{
						eventHandler(null, nameChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the name changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving NameChangedCallback().");
		}

		[SecurityCritical]
		private static void AddPresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			CollaborationHelperFunctions.AddMyPresenceChanged(callback, ref PeerCollaboration.s_presenceChanged, LockPresenceChangedEvent, ref s_regPresenceChangedWaitHandle, ref s_presenceChangedEvent, ref s_safePresenceChangedEvent, PresenceChangedCallback);
		}

		[SecurityCritical]
		private static void RemovePresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() called.");
			lock (LockPresenceChangedEvent)
			{
				PeerCollaboration.s_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Remove(PeerCollaboration.s_presenceChanged, callback);
				if (PeerCollaboration.s_presenceChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref s_regPresenceChangedWaitHandle, ref s_safePresenceChangedEvent, ref s_presenceChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean PresenceChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() successful.");
		}

		[SecurityCritical]
		private static void PresenceChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PresenceChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				PresenceChangedEventArgs presenceChangedEventArgs = null;
				try
				{
					lock (LockPresenceChangedEvent)
					{
						if (s_safePresenceChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(s_safePresenceChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPresenceChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyPresenceChanged)
						{
							PEER_EVENT_PRESENCE_CHANGED_DATA presenceChangedData = pEER_COLLAB_EVENT_DATA.presenceChangedData;
							PeerPresenceInfo peerPresenceInfo = null;
							if (presenceChangedData.pPresenceInfo != IntPtr.Zero)
							{
								PEER_PRESENCE_INFO pEER_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(presenceChangedData.pPresenceInfo, typeof(PEER_PRESENCE_INFO));
								peerPresenceInfo = new PeerPresenceInfo();
								peerPresenceInfo.PresenceStatus = pEER_PRESENCE_INFO.status;
								peerPresenceInfo.DescriptiveText = pEER_PRESENCE_INFO.descText;
							}
							if (presenceChangedData.pEndPoint != IntPtr.Zero)
							{
								return;
							}
							presenceChangedEventArgs = new PresenceChangedEventArgs(null, null, presenceChangedData.changeType, peerPresenceInfo);
						}
						goto IL_019e;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_019e:
				EventHandler<PresenceChangedEventArgs> eventHandler = PeerCollaboration.s_presenceChanged;
				if (presenceChangedEventArgs != null && eventHandler != null)
				{
					if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
					{
						SynchronizingObject.BeginInvoke(eventHandler, new object[2]
						{
							null,
							presenceChangedEventArgs
						});
					}
					else
					{
						eventHandler(null, presenceChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the presence changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving PresenceChangedCallback().");
		}

		[SecurityCritical]
		private static void AddObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			CollaborationHelperFunctions.AddMyObjectChanged(callback, ref s_objectChanged, LockObjChangedEvent, ref s_regObjChangedWaitHandle, ref s_objChangedEvent, ref s_safeObjChangedEvent, ObjectChangedCallback);
		}

		[SecurityCritical]
		private static void RemoveObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() called.");
			lock (LockObjChangedEvent)
			{
				s_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Remove(s_objectChanged, callback);
				if (s_objectChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref s_regObjChangedWaitHandle, ref s_safeObjChangedEvent, ref s_objChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChangedEvent variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() successful.");
		}

		[SecurityCritical]
		private static void ObjectChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ObjectChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				ObjectChangedEventArgs objectChangedEventArgs = null;
				try
				{
					lock (LockObjChangedEvent)
					{
						if (s_safeObjChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(s_safeObjChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyObjectChanged)
						{
							PEER_EVENT_OBJECT_CHANGED_DATA objectChangedData = pEER_COLLAB_EVENT_DATA.objectChangedData;
							PEER_OBJECT po = (PEER_OBJECT)Marshal.PtrToStructure(objectChangedData.pObject, typeof(PEER_OBJECT));
							PeerObject peerObject = CollaborationHelperFunctions.ConvertPEER_OBJECTToPeerObject(po);
							PeerEndPoint peerEndPoint = null;
							if (objectChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(objectChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							objectChangedEventArgs = new ObjectChangedEventArgs(peerEndPoint, null, objectChangedData.changeType, peerObject);
						}
						goto IL_018a;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_018a:
				EventHandler<ObjectChangedEventArgs> eventHandler = s_objectChanged;
				if (objectChangedEventArgs != null && eventHandler != null)
				{
					if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
					{
						SynchronizingObject.BeginInvoke(eventHandler, new object[2]
						{
							null,
							objectChangedEventArgs
						});
					}
					else
					{
						eventHandler(null, objectChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the object changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ObjectChangedCallback().");
		}

		[SecurityCritical]
		private static void AddApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			CollaborationHelperFunctions.AddMyApplicationChanged(callback, ref PeerCollaboration.s_applicationChanged, LockAppChangedEvent, ref s_regAppChangedWaitHandle, ref s_appChangedEvent, ref s_safeAppChangedEvent, ApplicationChangedCallback);
		}

		[SecurityCritical]
		private static void RemoveApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() called.");
			lock (LockAppChangedEvent)
			{
				PeerCollaboration.s_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Remove(PeerCollaboration.s_applicationChanged, callback);
				if (PeerCollaboration.s_applicationChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref s_regAppChangedWaitHandle, ref s_safeAppChangedEvent, ref s_appChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChangEvent variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() successful.");
		}

		[SecurityCritical]
		private static void ApplicationChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ApplicationChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				ApplicationChangedEventArgs applicationChangedEventArgs = null;
				try
				{
					lock (LockAppChangedEvent)
					{
						if (s_safeAppChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(s_safeAppChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetApplicationChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyApplicationChanged)
						{
							PEER_EVENT_APPLICATION_CHANGED_DATA applicationChangedData = pEER_COLLAB_EVENT_DATA.applicationChangedData;
							PEER_APPLICATION pa = (PEER_APPLICATION)Marshal.PtrToStructure(applicationChangedData.pApplication, typeof(PEER_APPLICATION));
							PeerApplication peerApplication = CollaborationHelperFunctions.ConvertPEER_APPLICATIONToPeerApplication(pa);
							if (applicationChangedData.pEndPoint != IntPtr.Zero)
							{
								return;
							}
							applicationChangedEventArgs = new ApplicationChangedEventArgs(null, null, applicationChangedData.changeType, peerApplication);
						}
						goto IL_0160;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_0160:
				EventHandler<ApplicationChangedEventArgs> eventHandler = PeerCollaboration.s_applicationChanged;
				if (applicationChangedEventArgs != null && eventHandler != null)
				{
					if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
					{
						SynchronizingObject.BeginInvoke(eventHandler, new object[2]
						{
							null,
							applicationChangedEventArgs
						});
					}
					else
					{
						eventHandler(null, applicationChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the application changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ApplicationChangedCallback().");
		}
	}
}
