using System.Security;
using System.Security.Permissions;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public sealed class PeerCollaborationPermission : CodeAccessPermission, IUnrestrictedPermission
	{
		private bool m_noRestriction;

		internal static readonly PeerCollaborationPermission UnrestrictedPeerCollaborationPermission = new PeerCollaborationPermission(PermissionState.Unrestricted);

		public PeerCollaborationPermission(PermissionState state)
		{
			m_noRestriction = (state == PermissionState.Unrestricted);
		}

		internal PeerCollaborationPermission(bool free)
		{
			m_noRestriction = free;
		}

		public bool IsUnrestricted()
		{
			return m_noRestriction;
		}

		public override IPermission Copy()
		{
			if (m_noRestriction)
			{
				return new PeerCollaborationPermission(free: true);
			}
			return new PeerCollaborationPermission(free: false);
		}

		public override IPermission Union(IPermission target)
		{
			if (target == null)
			{
				return Copy();
			}
			PeerCollaborationPermission peerCollaborationPermission = target as PeerCollaborationPermission;
			if (peerCollaborationPermission == null)
			{
				throw new ArgumentException(SR.GetString("Collab_PermissionUnionError"), "target");
			}
			return new PeerCollaborationPermission(m_noRestriction || peerCollaborationPermission.m_noRestriction);
		}

		public override IPermission Intersect(IPermission target)
		{
			if (target == null)
			{
				return null;
			}
			PeerCollaborationPermission peerCollaborationPermission = target as PeerCollaborationPermission;
			if (peerCollaborationPermission == null)
			{
				throw new ArgumentException(SR.GetString("Collab_PermissionIntersectError"), "target");
			}
			if (m_noRestriction && peerCollaborationPermission.m_noRestriction)
			{
				return new PeerCollaborationPermission(free: true);
			}
			return null;
		}

		public override bool IsSubsetOf(IPermission target)
		{
			if (target == null)
			{
				return !m_noRestriction;
			}
			PeerCollaborationPermission peerCollaborationPermission = target as PeerCollaborationPermission;
			if (peerCollaborationPermission == null)
			{
				throw new ArgumentException(SR.GetString("Collab_BadPermissionTarget"), "target");
			}
			return !m_noRestriction || peerCollaborationPermission.m_noRestriction;
		}

		public override void FromXml(SecurityElement e)
		{
			if (e == null)
			{
				throw new ArgumentNullException("e");
			}
			if (!e.Tag.Equals("IPermission"))
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElem"), "e");
			}
			string text = e.Attribute("class");
			if (text == null)
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElemNoClass"), "e");
			}
			if (text.IndexOf(GetType().FullName, StringComparison.Ordinal) < 0)
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElemNoType"), "e");
			}
			string text2 = e.Attribute("Unrestricted");
			m_noRestriction = (text2 != null && 0 == string.Compare(text2, "true", StringComparison.OrdinalIgnoreCase));
		}

		public override SecurityElement ToXml()
		{
			SecurityElement securityElement = new SecurityElement("IPermission");
			securityElement.AddAttribute("class", GetType().FullName + ", " + GetType().Module.Assembly.FullName.Replace('"', '\''));
			securityElement.AddAttribute("version", "1");
			if (m_noRestriction)
			{
				securityElement.AddAttribute("Unrestricted", "true");
			}
			return securityElement;
		}
	}
}
