#define TRACE
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Net.Mail;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Cryptography.X509Certificates;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerContact : Peer, IEquatable<PeerContact>, ISerializable
	{
		private PeerName m_peerName;

		private string m_nickname;

		private string m_displayName;

		private MailAddress m_emailAddress;

		private SubscriptionType m_subscribeAllowed;

		private bool m_isSubscribed;

		private X509Certificate2 m_credentials;

		private string m_contactXml;

		private bool m_justCreated;

		private object m_isSubscribeLock;

		private AsyncOperation m_AsyncOp;

		private object m_asyncOpLock;

		private SendOrPostCallback OnSubscribeCompletedDelegate;

		private object m_lockAppChangedEvent;

		private RegisteredWaitHandle m_regAppChangedWaitHandle;

		private AutoResetEvent m_appChangedEvent;

		internal SafeCollabEvent m_safeAppChangedEvent;

		private object m_lockObjChangedEvent;

		private RegisteredWaitHandle m_regObjChangedWaitHandle;

		private AutoResetEvent m_objChangedEvent;

		internal SafeCollabEvent m_safeObjChangedEvent;

		private object m_lockPresenceChangedEvent;

		private RegisteredWaitHandle m_regPresenceChangedWaitHandle;

		private AutoResetEvent m_presenceChangedEvent;

		internal SafeCollabEvent m_safePresenceChangedEvent;

		private bool m_Disposed;

		public PeerName PeerName
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_peerName;
			}
			internal set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_peerName = value;
			}
		}

		public string Nickname
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_nickname;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_nickname = value;
			}
		}

		public string DisplayName
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_displayName;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_displayName = value;
			}
		}

		public MailAddress EmailAddress
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_emailAddress;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_emailAddress = value;
			}
		}

		public SubscriptionType SubscribeAllowed
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return InternalSubscribeAllowedGet();
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				InternalSubscribeAllowedSet(value);
			}
		}

		public bool IsSubscribed
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return InternalIsSubscribed();
			}
			internal set
			{
				m_isSubscribed = value;
			}
		}

		internal object IsSubscribeLock
		{
			get
			{
				if (m_isSubscribeLock == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_isSubscribeLock, value, null);
				}
				return m_isSubscribeLock;
			}
		}

		public X509Certificate2 Credentials
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				return m_credentials;
			}
			internal set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_credentials = value;
			}
		}

		internal bool JustCreated
		{
			get
			{
				return m_justCreated;
			}
			set
			{
				m_justCreated = value;
			}
		}

		internal string ContactXml
		{
			get
			{
				return m_contactXml;
			}
			set
			{
				m_contactXml = value;
			}
		}

		public unsafe override PeerEndPointCollection PeerEndPoints
		{
			[SecurityCritical]
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Get PeerEndPoints called.");
				PeerEndPointCollection peerEndPointCollection = new PeerEndPointCollection();
				SafeCollabEnum phPeerEnum = null;
				uint pCount = 0u;
				int num = 0;
				try
				{
					SafeCollabMemory safeCredentials = null;
					try
					{
						PEER_CONTACT pcContact = CollaborationHelperFunctions.ConvertPeerContactToPEER_CONTACT(this, ref safeCredentials);
						num = UnsafeCollabNativeMethods.PeerCollabEnumEndpoints(ref pcContact, out phPeerEnum);
					}
					finally
					{
						safeCredentials?.Dispose();
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumEndpoints returned with errorcode {0}", num);
						return peerEndPointCollection;
					}
					num = UnsafeCollabNativeMethods.PeerGetItemCount(phPeerEnum, ref pCount);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", num);
						return peerEndPointCollection;
					}
					if (pCount == 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "No endpoints. Get PeerEndPoints returning.");
						return peerEndPointCollection;
					}
					num = UnsafeCollabNativeMethods.PeerGetNextItem(phPeerEnum, ref pCount, out SafeCollabData pppvItems);
					IntPtr value = pppvItems.DangerousGetHandle();
					IntPtr* ptr = (IntPtr*)(void*)value;
					for (ulong num2 = 0uL; num2 < pCount; num2++)
					{
						IntPtr ptr2 = ptr[num2];
						PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(ptr2, typeof(PEER_ENDPOINT));
						PeerEndPoint item = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
						peerEndPointCollection.Add(item);
					}
				}
				finally
				{
					phPeerEnum?.Dispose();
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving Get PeerEndPoints with {0} endpoints.", peerEndPointCollection.Count);
				return peerEndPointCollection;
			}
		}

		internal AsyncOperation AsyncOp
		{
			get
			{
				return m_AsyncOp;
			}
			set
			{
				m_AsyncOp = value;
			}
		}

		internal object AsyncLock
		{
			get
			{
				if (m_asyncOpLock == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_asyncOpLock, value, null);
				}
				return m_asyncOpLock;
			}
		}

		internal object LockAppChangedEvent
		{
			get
			{
				if (m_lockAppChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockAppChangedEvent, value, null);
				}
				return m_lockAppChangedEvent;
			}
		}

		internal RegisteredWaitHandle AppChangedWaitHandle
		{
			set
			{
				m_regAppChangedWaitHandle = value;
			}
		}

		internal AutoResetEvent AppChangedEvent
		{
			get
			{
				return m_appChangedEvent;
			}
			set
			{
				m_appChangedEvent = value;
			}
		}

		internal object LockObjChangedEvent
		{
			get
			{
				if (m_lockObjChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockObjChangedEvent, value, null);
				}
				return m_lockObjChangedEvent;
			}
		}

		internal RegisteredWaitHandle ObjChangedWaitHandle
		{
			set
			{
				m_regObjChangedWaitHandle = value;
			}
		}

		internal AutoResetEvent ObjChangedEvent
		{
			get
			{
				return m_objChangedEvent;
			}
			set
			{
				m_objChangedEvent = value;
			}
		}

		internal object LockPresenceChangedEvent
		{
			get
			{
				if (m_lockPresenceChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockPresenceChangedEvent, value, null);
				}
				return m_lockPresenceChangedEvent;
			}
		}

		internal RegisteredWaitHandle PresenceChangedWaitHandle
		{
			set
			{
				m_regPresenceChangedWaitHandle = value;
			}
		}

		internal AutoResetEvent PresenceChangedEvent
		{
			get
			{
				return m_presenceChangedEvent;
			}
			set
			{
				m_presenceChangedEvent = value;
			}
		}

		private event EventHandler<SubscribeCompletedEventArgs> m_subscribeCompleted;

		public event EventHandler<SubscribeCompletedEventArgs> SubscribeCompleted
		{
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_subscribeCompleted = (EventHandler<SubscribeCompletedEventArgs>)Delegate.Combine(this.m_subscribeCompleted, value);
			}
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_subscribeCompleted = (EventHandler<SubscribeCompletedEventArgs>)Delegate.Remove(this.m_subscribeCompleted, value);
			}
		}

		private event EventHandler<ApplicationChangedEventArgs> m_applicationChanged;

		public event EventHandler<ApplicationChangedEventArgs> ApplicationChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddApplicationChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemoveApplicationChanged(value);
			}
		}

		private event EventHandler<ObjectChangedEventArgs> m_objectChanged;

		public event EventHandler<ObjectChangedEventArgs> ObjectChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddObjectChangedEvent(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemoveObjectChangedEvent(value);
			}
		}

		private event EventHandler<PresenceChangedEventArgs> m_presenceChanged;

		public event EventHandler<PresenceChangedEventArgs> PresenceChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddPresenceChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemovePresenceChanged(value);
			}
		}

		internal virtual SubscriptionType InternalSubscribeAllowedGet()
		{
			return m_subscribeAllowed;
		}

		internal virtual void InternalSubscribeAllowedSet(SubscriptionType value)
		{
			m_subscribeAllowed = value;
		}

		internal virtual bool InternalIsSubscribed()
		{
			lock (IsSubscribeLock)
			{
				return m_isSubscribed;
			}
		}

		[SecurityCritical]
		static PeerContact()
		{
			CollaborationHelperFunctions.Initialize();
		}

		[SecurityCritical]
		internal PeerContact()
		{
			OnSubscribeCompletedDelegate = SubscribeCompletedWaitCallback;
		}

		[SecurityCritical]
		protected PeerContact(SerializationInfo serializationInfo, StreamingContext streamingContext)
			: this()
		{
			m_peerName = (PeerName)serializationInfo.GetValue("_PeerName", typeof(PeerName));
			m_nickname = serializationInfo.GetString("_NickName");
			m_displayName = serializationInfo.GetString("_DisplayName");
			try
			{
				m_emailAddress = new MailAddress(serializationInfo.GetString("_EmailAddress"));
			}
			catch (SerializationException)
			{
			}
			m_subscribeAllowed = (SubscriptionType)serializationInfo.GetValue("_SubscribeAllowed", typeof(SubscriptionType));
			byte[] rawData = (byte[])serializationInfo.GetValue("_Credentials", typeof(byte[]));
			m_credentials = new X509Certificate2(rawData);
			m_contactXml = serializationInfo.GetString("_ContactXml");
			m_justCreated = serializationInfo.GetBoolean("_JustCreated");
		}

		[SecurityCritical]
		public string ToXml()
		{
			string ppwzContactData = null;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering ToXml().");
			if (JustCreated)
			{
				return ContactXml;
			}
			int num = UnsafeCollabNativeMethods.PeerCollabExportContact(PeerName.ToString(), ref ppwzContactData);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabExportContact returned with errorcode {0}.", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ContactToXmlFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ToXml() with XML string: {0}", ppwzContactData);
			return ppwzContactData;
		}

		[SecurityCritical]
		public static PeerContact FromXml(string peerContactXml)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering FromXml() with XML string: {0}", peerContactXml);
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			CollaborationHelperFunctions.Initialize();
			if (peerContactXml == null)
			{
				throw new ArgumentNullException("peerContactXml");
			}
			SafeCollabData ppContactData = null;
			PeerContact peerContact = null;
			int num;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabParseContact(peerContactXml, out ppContactData);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabParseContact returned with errorcode {0}. Contact already exists.", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ContactFromXmlFailed"), num);
				}
				PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(ppContactData.DangerousGetHandle(), typeof(PEER_CONTACT));
				peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
			}
			finally
			{
				ppContactData?.Dispose();
			}
			try
			{
				ppContactData = null;
				num = UnsafeCollabNativeMethods.PeerCollabGetContact(peerContact.PeerName.ToString(), out ppContactData);
			}
			finally
			{
				ppContactData?.Dispose();
			}
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Error or contact not found in Contact Manager. ErrorCode {0}", num);
				peerContact.JustCreated = true;
				peerContact.ContactXml = peerContactXml;
			}
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving FromXml() with following peercontact");
				peerContact.TracePeerContact();
			}
			return peerContact;
		}

		public virtual void Subscribe()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering Subscribe().");
			if (IsSubscribed)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Already subscribed. Leaving Subscribe().");
				return;
			}
			InternalSubscribe(false);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving Subscribe().");
		}

		private void InternalSubscribe(object isAsyncObj)
		{
			bool flag = (bool)isAsyncObj;
			Exception ex = null;
			lock (IsSubscribeLock)
			{
				IsSubscribed = true;
				try
				{
					PeerCollaboration.ContactManager.UpdateContact(this);
				}
				catch (Exception ex2)
				{
					IsSubscribed = false;
					if (!flag)
					{
						throw;
					}
					ex = ex2;
				}
			}
			if (flag)
			{
				PrepareToRaiseSubscribeCompletedEvent(args: (ex != null) ? new SubscribeCompletedEventArgs(null, null, ex, cancelled: false, AsyncOp.UserSuppliedState) : new SubscribeCompletedEventArgs(null, this, null, cancelled: false, AsyncOp.UserSuppliedState), asyncOP: AsyncOp);
			}
		}

		public virtual void SubscribeAsync(object userToken)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			if (userToken == null)
			{
				throw new ArgumentNullException("userToken");
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering SubscribeAsync() with user token {0}.", userToken);
			lock (AsyncLock)
			{
				if (AsyncOp != null)
				{
					throw new PeerToPeerException(SR.GetString("Collab_DuplicateSubscribeAsync"));
				}
				AsyncOp = AsyncOperationManager.CreateOperation(userToken);
			}
			ThreadPool.QueueUserWorkItem(InternalSubscribe, true);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving SubscribeAsync().");
		}

		protected void OnSubscribeCompleted(SubscribeCompletedEventArgs e)
		{
			EventHandler<SubscribeCompletedEventArgs> subscribeCompleted = this.m_subscribeCompleted;
			if (subscribeCompleted != null)
			{
				subscribeCompleted(this, e);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the subscribe completed event callback.");
			}
		}

		private void SubscribeCompletedWaitCallback(object operationState)
		{
			AsyncOp = null;
			OnSubscribeCompleted((SubscribeCompletedEventArgs)operationState);
		}

		internal void PrepareToRaiseSubscribeCompletedEvent(AsyncOperation asyncOP, SubscribeCompletedEventArgs args)
		{
			asyncOP.PostOperationCompleted(OnSubscribeCompletedDelegate, args);
		}

		public virtual void Unsubscribe()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			if (!IsSubscribed)
			{
				return;
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering UnSubscribe().");
			lock (IsSubscribeLock)
			{
				IsSubscribed = false;
				try
				{
					PeerCollaboration.ContactManager.UpdateContact(this);
				}
				catch (Exception)
				{
					IsSubscribed = true;
					throw;
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Laaving UnSubscribe().");
		}

		[SecurityCritical]
		public override PeerInvitationResponse Invite()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			return InternalInviteAllEndPoints(PeerEndPoints, null, null, currentApplicationGuid);
		}

		[SecurityCritical]
		public override PeerInvitationResponse Invite(PeerApplication applicationToInvite, string message, byte[] invitationData)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			return InternalInviteAllEndPoints(PeerEndPoints, message, invitationData, applicationToInvite.Id);
		}

		[SecurityCritical]
		private PeerInvitationResponse InternalInviteAllEndPoints(PeerEndPointCollection peerEndPoints, string message, byte[] invitationData, Guid applicationId)
		{
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			bool flag = false;
			bool flag2 = false;
			foreach (PeerEndPoint peerEndPoint in peerEndPoints)
			{
				PeerInvitationResponse peerInvitationResponse = Peer.InternalInviteEndPoint(applicationId, message, invitationData, peerEndPoint, this);
				if (peerInvitationResponse.PeerInvitationResponseType == PeerInvitationResponseType.Accepted)
				{
					return peerInvitationResponse;
				}
				if (peerInvitationResponse.PeerInvitationResponseType == PeerInvitationResponseType.Declined)
				{
					flag = true;
				}
				else if (peerInvitationResponse.PeerInvitationResponseType == PeerInvitationResponseType.Expired)
				{
					flag2 = true;
				}
			}
			if (flag)
			{
				return new PeerInvitationResponse(PeerInvitationResponseType.Declined);
			}
			if (flag2)
			{
				return new PeerInvitationResponse(PeerInvitationResponseType.Expired);
			}
			throw new PeerToPeerException(SR.GetString("Collab_InviteFailed"));
		}

		[SecurityCritical]
		public PeerInvitationResponse Invite(PeerEndPoint peerEndPoint)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			PeerInvitationResponse peerInvitationResponse = Peer.InternalInviteEndPoint(currentApplicationGuid, null, null, peerEndPoint, this);
			CollaborationHelperFunctions.ThrowIfInvitationResponseInvalid(peerInvitationResponse);
			return peerInvitationResponse;
		}

		[SecurityCritical]
		public PeerInvitationResponse Invite(PeerEndPoint peerEndPoint, PeerApplication applicationToInvite, string message, byte[] invitationData)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new ArgumentException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			PeerInvitationResponse peerInvitationResponse = Peer.InternalInviteEndPoint(applicationToInvite.Id, message, invitationData, peerEndPoint, this);
			CollaborationHelperFunctions.ThrowIfInvitationResponseInvalid(peerInvitationResponse);
			return peerInvitationResponse;
		}

		[SecurityCritical]
		public override void InviteAsync(object userToken)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			PeerEndPointCollection peerEndPoints = PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			InternalInviteAsync(currentApplicationGuid, null, null, peerEndPoints, this, userToken);
		}

		[SecurityCritical]
		public override void InviteAsync(PeerApplication applicationToInvite, string message, byte[] invitationData, object userToken)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			PeerEndPointCollection peerEndPoints = PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			InternalInviteAsync(applicationToInvite.Id, null, null, peerEndPoints, this, userToken);
		}

		[SecurityCritical]
		public void InviteAsync(PeerEndPoint peerEndPoint, object userToken)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new ArgumentException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			PeerEndPointCollection peerEndPointCollection = new PeerEndPointCollection();
			peerEndPointCollection.Add(peerEndPoint);
			InternalInviteAsync(currentApplicationGuid, null, null, peerEndPointCollection, this, userToken);
		}

		[SecurityCritical]
		public void InviteAsync(PeerEndPoint peerEndPoint, string message, byte[] invitationData, PeerApplication applicationToInvite, object userToken)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new ArgumentException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			PeerEndPointCollection peerEndPointCollection = new PeerEndPointCollection();
			peerEndPointCollection.Add(peerEndPoint);
			InternalInviteAsync(applicationToInvite.Id, message, invitationData, peerEndPointCollection, this, userToken);
		}

		[SecurityCritical]
		public PeerApplicationCollection GetApplications()
		{
			return InternalGetAllApplications(Guid.Empty, guidSupplied: false);
		}

		[SecurityCritical]
		public PeerApplicationCollection GetApplications(Guid applicationId)
		{
			return InternalGetAllApplications(applicationId, guidSupplied: true);
		}

		[SecurityCritical]
		public PeerApplicationCollection GetApplications(PeerEndPoint peerEndPoint)
		{
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			return InternalGetApplications(Guid.Empty, guidSupplied: false, peerEndPoint);
		}

		[SecurityCritical]
		public PeerApplicationCollection GetApplications(PeerEndPoint peerEndPoint, Guid applicationId)
		{
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			return InternalGetApplications(applicationId, guidSupplied: true, peerEndPoint);
		}

		[SecurityCritical]
		private PeerApplicationCollection InternalGetAllApplications(Guid applicationId, bool guidSupplied)
		{
			Dictionary<Guid, PeerApplication> dictionary = new Dictionary<Guid, PeerApplication>();
			PeerEndPointCollection peerEndPoints = PeerEndPoints;
			PeerApplicationCollection peerApplicationCollection;
			foreach (PeerEndPoint item in peerEndPoints)
			{
				peerApplicationCollection = InternalGetApplications(applicationId, guidSupplied, item);
				foreach (PeerApplication item2 in peerApplicationCollection)
				{
					dictionary[item2.Id] = item2;
				}
			}
			Dictionary<Guid, PeerApplication>.ValueCollection values = dictionary.Values;
			peerApplicationCollection = new PeerApplicationCollection();
			foreach (PeerApplication item3 in values)
			{
				peerApplicationCollection.Add(item3);
			}
			return peerApplicationCollection;
		}

		[SecurityCritical]
		private unsafe static PeerApplicationCollection InternalGetApplications(Guid applicationId, bool guidSupplied, PeerEndPoint peerEndPoint)
		{
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering InternalGetApplications() with the following PeerEndPoint");
				peerEndPoint.TracePeerEndPoint();
			}
			PeerApplicationCollection peerApplicationCollection = new PeerApplicationCollection();
			SafeCollabEnum phPeerEnum = null;
			uint pCount = 0u;
			int num = 0;
			GCHandle gCHandle = default(GCHandle);
			IntPtr pObjectId = IntPtr.Zero;
			if (guidSupplied)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Guid supplied is {0}.", applicationId.ToString());
				GUID gUID = CollaborationHelperFunctions.ConvertGuidToGUID(applicationId);
				gCHandle = GCHandle.Alloc(gUID, GCHandleType.Pinned);
				pObjectId = gCHandle.AddrOfPinnedObject();
			}
			PEER_ENDPOINT pEER_ENDPOINT = default(PEER_ENDPOINT);
			pEER_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gCHandle2 = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
			pEER_ENDPOINT.pwzEndpointName = gCHandle2.AddrOfPinnedObject();
			GCHandle gCHandle3 = GCHandle.Alloc(pEER_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gCHandle3.AddrOfPinnedObject();
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabEnumApplications(pcEndpoint, pObjectId, out phPeerEnum);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumApplications returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetAppsFailed"), num);
				}
				num = UnsafeCollabNativeMethods.PeerGetItemCount(phPeerEnum, ref pCount);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetAppsFailed"), num);
				}
				if (pCount == 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "No PeerApplications found.");
					return peerApplicationCollection;
				}
				num = UnsafeCollabNativeMethods.PeerGetNextItem(phPeerEnum, ref pCount, out SafeCollabData pppvItems);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetAppsFailed"), num);
				}
				IntPtr value = pppvItems.DangerousGetHandle();
				IntPtr* ptr = (IntPtr*)(void*)value;
				for (ulong num2 = 0uL; num2 < pCount; num2++)
				{
					PEER_APPLICATION* ptr2 = (PEER_APPLICATION*)(void*)ptr[num2];
					string description = Marshal.PtrToStringUni(ptr2->pwzDescription);
					byte[] array = null;
					if (ptr2->data.cbData != 0)
					{
						array = new byte[ptr2->data.cbData];
						Marshal.Copy(ptr2->data.pbData, array, 0, (int)ptr2->data.cbData);
					}
					PeerApplication peerApplication = new PeerApplication(CollaborationHelperFunctions.ConvertGUIDToGuid(ptr2->guid), description, array, null, null, PeerScope.None);
					if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Retrieved following Application");
						peerApplication.TracePeerApplication();
					}
					peerApplicationCollection.Add(peerApplication);
				}
			}
			finally
			{
				if (gCHandle.IsAllocated)
				{
					gCHandle.Free();
				}
				if (gCHandle2.IsAllocated)
				{
					gCHandle2.Free();
				}
				if (gCHandle3.IsAllocated)
				{
					gCHandle3.Free();
				}
				phPeerEnum?.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving InternalGetApplications(). Returning collection with {0} applications.", peerApplicationCollection.Count);
			return peerApplicationCollection;
		}

		[SecurityCritical]
		public PeerObjectCollection GetObjects(PeerEndPoint peerEndPoint)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			return Peer.InternalGetObjects(Guid.Empty, guidSupplied: false, peerEndPoint);
		}

		[SecurityCritical]
		public PeerObjectCollection GetObjects(PeerEndPoint peerEndPoint, Guid objectId)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			return Peer.InternalGetObjects(objectId, guidSupplied: true, peerEndPoint);
		}

		internal override void RefreshIfNeeded()
		{
		}

		[SecurityCritical]
		internal virtual void AddApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddApplicationChanged().");
			lock (LockAppChangedEvent)
			{
				if (this.m_applicationChanged == null)
				{
					m_appChangedEvent = new AutoResetEvent(initialState: false);
					AppChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_appChangedEvent, ApplicationChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.EndPointApplicationChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_appChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safeAppChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ApplicationChangedRegFailed"), num);
					}
				}
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Combine(this.m_applicationChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() successful.");
		}

		[SecurityCritical]
		internal virtual void RemoveApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() called.");
			lock (LockAppChangedEvent)
			{
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Remove(this.m_applicationChanged, callback);
				if (this.m_applicationChanged == null)
				{
					CleanContactAppEventVars();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() successful.");
		}

		[SecurityCritical]
		internal void CleanContactAppEventVars()
		{
			CollaborationHelperFunctions.CleanEventVars(ref m_regAppChangedWaitHandle, ref m_safeAppChangedEvent, ref m_appChangedEvent);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChanged variables successful.");
		}

		protected virtual void OnApplicationChanged(ApplicationChangedEventArgs appChangedArgs)
		{
			EventHandler<ApplicationChangedEventArgs> applicationChanged = this.m_applicationChanged;
			if (applicationChanged != null)
			{
				if (base.SynchronizingObject != null && base.SynchronizingObject.InvokeRequired)
				{
					base.SynchronizingObject.BeginInvoke(applicationChanged, new object[2]
					{
						this,
						appChangedArgs
					});
				}
				else
				{
					applicationChanged(this, appChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the application changed event callback.");
			}
		}

		[SecurityCritical]
		internal virtual void ApplicationChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ApplicationChangedCallback() called.");
			if (m_Disposed)
			{
				return;
			}
			while (true)
			{
				bool flag = true;
				ApplicationChangedEventArgs applicationChangedEventArgs = null;
				try
				{
					lock (LockAppChangedEvent)
					{
						if (m_safeAppChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safeAppChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetApplicationChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointApplicationChanged)
						{
							PEER_EVENT_APPLICATION_CHANGED_DATA applicationChangedData = pEER_COLLAB_EVENT_DATA.applicationChangedData;
							PeerContact peerContact = null;
							if (applicationChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(applicationChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (peerContact != null && Equals(peerContact))
							{
								PEER_APPLICATION pa = (PEER_APPLICATION)Marshal.PtrToStructure(applicationChangedData.pApplication, typeof(PEER_APPLICATION));
								PeerApplication peerApplication = CollaborationHelperFunctions.ConvertPEER_APPLICATIONToPeerApplication(pa);
								PeerEndPoint peerEndPoint = null;
								if (applicationChangedData.pEndPoint != IntPtr.Zero)
								{
									PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(applicationChangedData.pEndPoint, typeof(PEER_ENDPOINT));
									peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
								}
								applicationChangedEventArgs = new ApplicationChangedEventArgs(peerEndPoint, peerContact, applicationChangedData.changeType, peerApplication);
							}
						}
						goto IL_0203;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_0203:
				if (applicationChangedEventArgs != null)
				{
					OnApplicationChanged(applicationChangedEventArgs);
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ApplicationChangedCallback().");
		}

		[SecurityCritical]
		internal virtual void AddObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddObjectChangedEvent().");
			lock (LockObjChangedEvent)
			{
				if (this.m_objectChanged == null)
				{
					m_objChangedEvent = new AutoResetEvent(initialState: false);
					ObjChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_objChangedEvent, ObjectChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.EndPointObjectChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_objChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safeObjChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectChangedRegFailed"), num);
					}
				}
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Combine(this.m_objectChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChanged() successful.");
		}

		[SecurityCritical]
		internal virtual void RemoveObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() called.");
			lock (LockObjChangedEvent)
			{
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Remove(this.m_objectChanged, callback);
				if (this.m_objectChanged == null)
				{
					CleanContactObjEventVars();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() successful.");
		}

		[SecurityCritical]
		internal void CleanContactObjEventVars()
		{
			CollaborationHelperFunctions.CleanEventVars(ref m_regObjChangedWaitHandle, ref m_safeObjChangedEvent, ref m_objChangedEvent);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChangedEvent variables successful.");
		}

		protected virtual void OnObjectChanged(ObjectChangedEventArgs objChangedArgs)
		{
			EventHandler<ObjectChangedEventArgs> objectChanged = this.m_objectChanged;
			if (objectChanged != null)
			{
				if (base.SynchronizingObject != null && base.SynchronizingObject.InvokeRequired)
				{
					base.SynchronizingObject.BeginInvoke(objectChanged, new object[2]
					{
						this,
						objChangedArgs
					});
				}
				else
				{
					objectChanged(this, objChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the object changed event callback.");
			}
		}

		[SecurityCritical]
		internal virtual void ObjectChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ObjectChangedCallback() called.");
			if (m_Disposed)
			{
				return;
			}
			while (true)
			{
				bool flag = true;
				ObjectChangedEventArgs objectChangedEventArgs = null;
				try
				{
					lock (LockObjChangedEvent)
					{
						if (m_safeObjChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safeObjChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointObjectChanged)
						{
							PEER_EVENT_OBJECT_CHANGED_DATA objectChangedData = pEER_COLLAB_EVENT_DATA.objectChangedData;
							PeerContact peerContact = null;
							if (objectChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(objectChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (peerContact != null && Equals(peerContact))
							{
								PeerEndPoint peerEndPoint = null;
								if (objectChangedData.pEndPoint != IntPtr.Zero)
								{
									PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(objectChangedData.pEndPoint, typeof(PEER_ENDPOINT));
									peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
								}
								PEER_OBJECT po = (PEER_OBJECT)Marshal.PtrToStructure(objectChangedData.pObject, typeof(PEER_OBJECT));
								PeerObject peerObject = CollaborationHelperFunctions.ConvertPEER_OBJECTToPeerObject(po);
								objectChangedEventArgs = new ObjectChangedEventArgs(peerEndPoint, peerContact, objectChangedData.changeType, peerObject);
							}
						}
						goto IL_0203;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_0203:
				if (objectChangedEventArgs != null)
				{
					OnObjectChanged(objectChangedEventArgs);
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ObjectChangedCallback().");
		}

		[SecurityCritical]
		internal virtual void AddPresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddPresenceChanged().");
			lock (LockPresenceChangedEvent)
			{
				if (this.m_presenceChanged == null)
				{
					m_presenceChangedEvent = new AutoResetEvent(initialState: false);
					PresenceChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_presenceChangedEvent, PresenceChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.EndPointPresenceChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_presenceChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safePresenceChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_PresenceChangedRegFailed"), num);
					}
				}
				this.m_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Combine(this.m_presenceChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddPresenceChanged() successful.");
		}

		[SecurityCritical]
		internal virtual void RemovePresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() called.");
			lock (LockPresenceChangedEvent)
			{
				this.m_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Remove(this.m_presenceChanged, callback);
				if (this.m_presenceChanged == null)
				{
					CleanContactPresenceEventVars();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() successful.");
		}

		[SecurityCritical]
		internal void CleanContactPresenceEventVars()
		{
			CollaborationHelperFunctions.CleanEventVars(ref m_regPresenceChangedWaitHandle, ref m_safePresenceChangedEvent, ref m_presenceChangedEvent);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean PresenceChanged variables successful.");
		}

		protected virtual void OnPresenceChanged(PresenceChangedEventArgs presenceChangedArgs)
		{
			EventHandler<PresenceChangedEventArgs> presenceChanged = this.m_presenceChanged;
			if (presenceChanged != null)
			{
				if (base.SynchronizingObject != null && base.SynchronizingObject.InvokeRequired)
				{
					base.SynchronizingObject.BeginInvoke(presenceChanged, new object[2]
					{
						this,
						presenceChangedArgs
					});
				}
				else
				{
					presenceChanged(this, presenceChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the presence changed event callback.");
			}
		}

		[SecurityCritical]
		internal virtual void PresenceChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PresenceChangedCallback() called.");
			if (m_Disposed)
			{
				return;
			}
			while (true)
			{
				bool flag = true;
				PresenceChangedEventArgs presenceChangedEventArgs = null;
				try
				{
					lock (LockPresenceChangedEvent)
					{
						if (m_safePresenceChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safePresenceChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPresenceChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointPresenceChanged)
						{
							PEER_EVENT_PRESENCE_CHANGED_DATA presenceChangedData = pEER_COLLAB_EVENT_DATA.presenceChangedData;
							PeerContact peerContact = null;
							if (presenceChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(presenceChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (peerContact != null && Equals(peerContact))
							{
								PeerEndPoint peerEndPoint = null;
								if (presenceChangedData.pEndPoint != IntPtr.Zero)
								{
									PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(presenceChangedData.pEndPoint, typeof(PEER_ENDPOINT));
									peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
								}
								PeerPresenceInfo peerPresenceInfo = null;
								if (presenceChangedData.pPresenceInfo != IntPtr.Zero)
								{
									PEER_PRESENCE_INFO pEER_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(presenceChangedData.pPresenceInfo, typeof(PEER_PRESENCE_INFO));
									peerPresenceInfo = new PeerPresenceInfo();
									peerPresenceInfo.PresenceStatus = pEER_PRESENCE_INFO.status;
									peerPresenceInfo.DescriptiveText = pEER_PRESENCE_INFO.descText;
								}
								presenceChangedEventArgs = new PresenceChangedEventArgs(peerEndPoint, peerContact, presenceChangedData.changeType, peerPresenceInfo);
							}
						}
						goto IL_023e;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_023e:
				if (presenceChangedEventArgs != null)
				{
					OnPresenceChanged(presenceChangedEventArgs);
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving PresenceChangedCallback().");
		}

		public bool Equals(PeerContact other)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			if (other != null)
			{
				if (other.PeerName != null)
				{
					return other.PeerName.Equals(PeerName);
				}
				if (PeerName == null)
				{
					return true;
				}
			}
			return false;
		}

		public override bool Equals(object obj)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerContact peerContact = obj as PeerContact;
			if (peerContact != null)
			{
				return Equals(peerContact);
			}
			return false;
		}

		public new static bool Equals(object objA, object objB)
		{
			PeerContact peerContact = objA as PeerContact;
			PeerContact peerContact2 = objB as PeerContact;
			if (peerContact != null && peerContact2 != null)
			{
				if (peerContact.PeerName != null)
				{
					return peerContact.PeerName.Equals(peerContact2.PeerName);
				}
				if (peerContact2.PeerName == null)
				{
					return true;
				}
			}
			return false;
		}

		public override int GetHashCode()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return (PeerName != null) ? PeerName.GetHashCode() : 0;
		}

		public override string ToString()
		{
			return m_displayName;
		}

		[SecurityCritical]
		protected override void Dispose(bool disposing)
		{
			if (!m_Disposed)
			{
				try
				{
					CleanContactAppEventVars();
					CleanContactObjEventVars();
					CleanContactPresenceEventVars();
					m_Disposed = true;
				}
				finally
				{
					base.Dispose(disposing);
				}
			}
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_PeerName", m_peerName);
			info.AddValue("_NickName", m_nickname);
			info.AddValue("_DisplayName", m_displayName);
			if (m_emailAddress != null)
			{
				info.AddValue("_EmailAddress", m_emailAddress.ToString());
			}
			info.AddValue("_SubscribeAllowed", m_subscribeAllowed);
			info.AddValue("_Credentials", m_credentials.RawData);
			info.AddValue("_ContactXml", m_contactXml);
			info.AddValue("_JustCreated", m_justCreated);
		}

		internal void TracePeerContact()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerContact");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tPeerName: {0}", PeerName);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tNickname: {0}", Nickname);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tEmailAddress: {0}", EmailAddress.ToString());
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tSubscribeAllowed: {0}", SubscribeAllowed);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tCredentials FriendlyName: {0}", Credentials.FriendlyName);
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tCredentials raw data:");
				Logging.DumpData(Logging.P2PTraceSource, TraceEventType.Verbose, Logging.P2PTraceSource.MaxDataSize, Credentials.RawData, 0, Credentials.RawData.Length);
			}
			else
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tCredentials raw data length {0}", Credentials.RawData.Length);
			}
		}
	}
}
