#define TRACE
using System.ComponentModel;
using System.Diagnostics;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerEndPoint : IDisposable, IEquatable<PeerEndPoint>, ISerializable
	{
		private string m_endPointName;

		private IPEndPoint m_endPoint;

		private ISynchronizeInvoke m_synchronizingObject;

		private object m_lockNameChangedEvent;

		private RegisteredWaitHandle m_regNameChangedWaitHandle;

		private AutoResetEvent m_nameChangedEvent;

		private SafeCollabEvent m_safeNameChangedEvent;

		private bool m_Disposed;

		public string Name
		{
			get
			{
				return m_endPointName;
			}
			set
			{
				m_endPointName = value;
			}
		}

		public IPEndPoint EndPoint
		{
			get
			{
				return m_endPoint;
			}
			set
			{
				if (m_endPoint != null && m_endPoint.AddressFamily != AddressFamily.InterNetworkV6)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerEndPoint Endpoint set parameter is not IPv6.");
					throw new PeerToPeerException(SR.GetString("Collab_EndPointNotIPv6Error"));
				}
				m_endPoint = value;
			}
		}

		[Description("SynchronizingObject")]
		[Browsable(false)]
		[DefaultValue(null)]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_synchronizingObject;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_synchronizingObject = value;
			}
		}

		private object LockNameChangedEvent
		{
			get
			{
				if (m_lockNameChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockNameChangedEvent, value, null);
				}
				return m_lockNameChangedEvent;
			}
		}

		private event EventHandler<NameChangedEventArgs> m_nameChanged;

		public event EventHandler<NameChangedEventArgs> NameChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				CollaborationHelperFunctions.Initialize();
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddNameChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				CollaborationHelperFunctions.Initialize();
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemoveNameChanged(value);
			}
		}

		public PeerEndPoint()
		{
		}

		public PeerEndPoint(IPEndPoint endPoint)
			: this(endPoint, null)
		{
		}

		public PeerEndPoint(IPEndPoint endPoint, string endPointName)
		{
			if (endPoint == null)
			{
				throw new ArgumentNullException("endPoint");
			}
			if (m_endPoint != null && m_endPoint.AddressFamily != AddressFamily.InterNetworkV6)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerEndPoint Endpoint set parameter is not IPv6.");
				throw new ArgumentException("endPoint", SR.GetString("Collab_EndPointNotIPv6Error"));
			}
			m_endPoint = endPoint;
			m_endPointName = endPointName;
		}

		protected PeerEndPoint(SerializationInfo serializationInfo, StreamingContext streamingContext)
		{
			m_endPointName = serializationInfo.GetString("_EndPointName");
			m_endPoint = (IPEndPoint)serializationInfo.GetValue("_EndPoint", typeof(IPEndPoint));
		}

		[SecurityCritical]
		private void AddNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddNameChanged().");
			lock (LockNameChangedEvent)
			{
				if (this.m_nameChanged == null)
				{
					m_nameChangedEvent = new AutoResetEvent(initialState: false);
					m_regNameChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_nameChangedEvent, NameChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.EndPointChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_nameChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safeNameChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_NameChangedRegFailed"), num);
					}
				}
				this.m_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Combine(this.m_nameChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddNameChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() called.");
			lock (LockNameChangedEvent)
			{
				this.m_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Remove(this.m_nameChanged, callback);
				if (this.m_nameChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref m_regNameChangedWaitHandle, ref m_safeNameChangedEvent, ref m_nameChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean NameChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() successful.");
		}

		[SecurityCritical]
		private void NameChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "NameChangedCallback() called.");
			if (m_Disposed)
			{
				return;
			}
			while (true)
			{
				bool flag = true;
				NameChangedEventArgs nameChangedEventArgs = null;
				try
				{
					lock (LockNameChangedEvent)
					{
						if (m_safeNameChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safeNameChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetNameChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointChanged)
						{
							PEER_EVENT_ENDPOINT_CHANGED_DATA endpointChangedData = pEER_COLLAB_EVENT_DATA.endpointChangedData;
							PeerEndPoint peerEndPoint = null;
							if (endpointChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(endpointChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							if (peerEndPoint != null && Equals(peerEndPoint))
							{
								PeerContact peerContact = null;
								if (endpointChangedData.pContact != IntPtr.Zero)
								{
									PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(endpointChangedData.pContact, typeof(PEER_CONTACT));
									peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
								}
								nameChangedEventArgs = new NameChangedEventArgs(peerEndPoint, peerContact, peerEndPoint.Name);
							}
						}
						goto IL_01d8;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_01d8:
				if (nameChangedEventArgs != null)
				{
					OnNameChanged(nameChangedEventArgs);
					Name = nameChangedEventArgs.PeerEndPoint.Name;
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving NameChangedCallback().");
		}

		protected void OnNameChanged(NameChangedEventArgs nameChangedArgs)
		{
			EventHandler<NameChangedEventArgs> nameChanged = this.m_nameChanged;
			if (nameChanged != null)
			{
				if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
				{
					SynchronizingObject.BeginInvoke(nameChanged, new object[2]
					{
						this,
						nameChangedArgs
					});
				}
				else
				{
					nameChanged(this, nameChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the name changed event callback.");
			}
		}

		public bool Equals(PeerEndPoint other)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return other?.EndPoint.Equals(EndPoint) ?? false;
		}

		public override bool Equals(object obj)
		{
			return (obj as PeerEndPoint)?.EndPoint.Equals(EndPoint) ?? false;
		}

		public new static bool Equals(object objA, object objB)
		{
			PeerEndPoint peerEndPoint = objA as PeerEndPoint;
			PeerEndPoint peerEndPoint2 = objB as PeerEndPoint;
			if (peerEndPoint != null && peerEndPoint2 != null)
			{
				return peerEndPoint.EndPoint.Equals(peerEndPoint2.EndPoint);
			}
			return false;
		}

		public override int GetHashCode()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return EndPoint.GetHashCode();
		}

		public override string ToString()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return m_endPointName;
		}

		[SecurityCritical]
		public void Dispose()
		{
			Dispose(disposing: true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		protected virtual void Dispose(bool disposing)
		{
			if (!m_Disposed)
			{
				CollaborationHelperFunctions.CleanEventVars(ref m_regNameChangedWaitHandle, ref m_safeNameChangedEvent, ref m_nameChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean NameChanged variables successful.");
			}
			m_Disposed = true;
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_EndPointName", m_endPointName);
			info.AddValue("_EndPoint", m_endPoint);
		}

		internal void TracePeerEndPoint()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerEndPoint");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tEndPoint: {0}", (EndPoint != null) ? EndPoint.ToString() : null);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tDescription: {0}", Name);
		}
	}
}
