#define TRACE
using System.ComponentModel;
using System.Diagnostics;
using System.Net.Mail;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerNearMe : Peer, IEquatable<PeerNearMe>, ISerializable
	{
		private string m_nickname;

		private Guid m_id;

		private AsyncOperation m_refreshDataAsyncOp;

		private object m_refreshDataAsyncOpLock;

		private SendOrPostCallback OnRefreshDataCompletedDelegate;

		private static object s_lockPNMChangedEvent;

		private static RegisteredWaitHandle s_registeredPNMWaitHandle;

		private static AutoResetEvent s_peerNearMeChangedEvent;

		private static SafeCollabEvent s_safePeerNearMeChangedEvent;

		private bool m_Disposed;

		public string Nickname
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_nickname;
			}
			internal set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_nickname = value;
			}
		}

		internal Guid Id
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_id;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_id = value;
			}
		}

		private object RefreshDataAsyncOpLock
		{
			get
			{
				if (m_refreshDataAsyncOpLock == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_refreshDataAsyncOpLock, value, null);
				}
				return m_refreshDataAsyncOpLock;
			}
		}

		private static object LockPNMChangedEvent
		{
			get
			{
				if (s_lockPNMChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref s_lockPNMChangedEvent, value, null);
				}
				return s_lockPNMChangedEvent;
			}
		}

		private event EventHandler<RefreshDataCompletedEventArgs> m_refreshDataCompleted;

		public event EventHandler<RefreshDataCompletedEventArgs> RefreshDataCompleted
		{
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_refreshDataCompleted = (EventHandler<RefreshDataCompletedEventArgs>)Delegate.Combine(this.m_refreshDataCompleted, value);
			}
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_refreshDataCompleted = (EventHandler<RefreshDataCompletedEventArgs>)Delegate.Remove(this.m_refreshDataCompleted, value);
			}
		}

		private static event EventHandler<PeerNearMeChangedEventArgs> s_peerNearMeChanged;

		public static event EventHandler<PeerNearMeChangedEventArgs> PeerNearMeChanged
		{
			[SecurityCritical]
			add
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				AddPeerNearMeChanged(value);
			}
			[SecurityCritical]
			remove
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				RemovePeerNearMeChanged(value);
			}
		}

		[SecurityCritical]
		static PeerNearMe()
		{
			CollaborationHelperFunctions.Initialize();
		}

		public PeerNearMe()
		{
			OnRefreshDataCompletedDelegate = RefreshDataCompletedWaitCallback;
		}

		protected PeerNearMe(SerializationInfo serializationInfo, StreamingContext streamingContext)
			: base(serializationInfo, streamingContext)
		{
			m_id = (Guid)serializationInfo.GetValue("_Id", typeof(Guid));
			m_nickname = serializationInfo.GetString("_NickName");
			OnRefreshDataCompletedDelegate = RefreshDataCompletedWaitCallback;
		}

		public PeerContact AddToContactManager()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddToContactManager.");
			PeerContact peerContact = null;
			try
			{
				peerContact = PeerCollaboration.ContactManager.CreateContact(this);
				PeerCollaboration.ContactManager.AddContact(peerContact);
			}
			catch (Exception ex)
			{
				throw new PeerToPeerException(SR.GetString("Collab_AddToContactMgrFailed"), (ex.InnerException != null) ? ex.InnerException : ex);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving AddToContactManager.");
			return peerContact;
		}

		public PeerContact AddToContactManager(string displayName, string nickname, MailAddress emailAddress)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddToContactManager with Display name: {0} Nickname: {1} and Email Address: {2}", displayName, nickname, emailAddress);
			PeerContact peerContact = null;
			try
			{
				peerContact = PeerCollaboration.ContactManager.CreateContact(this);
				PeerCollaboration.ContactManager.AddContact(peerContact);
			}
			catch (Exception ex)
			{
				throw new PeerToPeerException(SR.GetString("Collab_AddToContactMgrFailed"), (ex.InnerException != null) ? ex.InnerException : ex);
			}
			peerContact.DisplayName = displayName;
			peerContact.Nickname = nickname;
			peerContact.EmailAddress = emailAddress;
			try
			{
				PeerCollaboration.ContactManager.UpdateContact(peerContact);
			}
			catch (Exception ex)
			{
				throw new PeerToPeerException(SR.GetString("Collab_AddToContactMgrFailed") + " " + SR.GetString("Collab_AddToContactMgrFailedUpdate"), (ex.InnerException != null) ? ex.InnerException : ex);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving AddToContactManager.");
			return peerContact;
		}

		[SecurityCritical]
		public static PeerNearMe CreateFromPeerEndPoint(PeerEndPoint peerEndPoint)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering CreateFromPeerEndPoint.");
			CollaborationHelperFunctions.Initialize();
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			PeerNearMeCollection peersNearMe = PeerCollaboration.GetPeersNearMe();
			PeerNearMe peerNearMe = null;
			foreach (PeerNearMe item in peersNearMe)
			{
				PeerEndPointCollection peerEndPoints = item.PeerEndPoints;
				if (peerEndPoints != null && peerEndPoints.Count != 0 && peerEndPoints[0].Equals(peerEndPoint))
				{
					peerNearMe = item;
				}
			}
			if (peerNearMe == null)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EndPointNotAPeerNearMe"));
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving CreateFromPeerEndPoint.");
			return peerNearMe;
		}

		internal override void RefreshIfNeeded()
		{
			RefreshData();
		}

		[SecurityCritical]
		public void RefreshData()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			InternalRefreshData(false);
		}

		[SecurityCritical]
		protected internal void InternalRefreshData(object state)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "InternalRefreshEndpointData called.");
			int num = 0;
			bool flag = (bool)state;
			Exception ex = null;
			AutoResetEvent autoResetEvent = new AutoResetEvent(initialState: false);
			PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
			pEventRegistrations.eventType = PeerCollabEventType.RequestStatusChanged;
			pEventRegistrations.pInstance = IntPtr.Zero;
			num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(autoResetEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out SafeCollabEvent phPeerEvent);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
				ex = PeerToPeerException.CreateFromHr(SR.GetString("Collab_ReqStatusChangedRegFailed"), num);
				if (!flag)
				{
					throw ex;
				}
			}
			PeerEndPointCollection peerEndPoints = PeerEndPoints;
			if (peerEndPoints.Count == 0)
			{
				return;
			}
			try
			{
				InternalRefreshData(peerEndPoints[0]);
			}
			catch (Exception ex2)
			{
				if (!flag)
				{
					throw;
				}
				ex = ex2;
			}
			while (ex == null)
			{
				autoResetEvent.WaitOne();
				num = UnsafeCollabNativeMethods.PeerCollabGetEventData(phPeerEvent, out SafeCollabData ppEventData);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
					ex = PeerToPeerException.CreateFromHr(SR.GetString("Collab_RefreshDataFailed"), num);
					if (!flag)
					{
						throw ex;
					}
					break;
				}
				PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
				if (pEER_COLLAB_EVENT_DATA.eventType != PeerCollabEventType.RequestStatusChanged)
				{
					continue;
				}
				PEER_EVENT_REQUEST_STATUS_CHANGED_DATA requestStatusChangedData = pEER_COLLAB_EVENT_DATA.requestStatusChangedData;
				PeerEndPoint peerEndPoint = null;
				if (requestStatusChangedData.pEndPoint != IntPtr.Zero)
				{
					PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(requestStatusChangedData.pEndPoint, typeof(PEER_ENDPOINT));
					peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
				}
				if (requestStatusChangedData.hrChange < 0)
				{
					ex = PeerToPeerException.CreateFromHr(SR.GetString("Collab_RefreshDataFailed"), requestStatusChangedData.hrChange);
				}
				if (ex != null && !flag)
				{
					throw ex;
				}
				if (!PeerEndPoints[0].Equals(peerEndPoint))
				{
					continue;
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Found endpoint match in Request status changed.");
				if (flag)
				{
					RefreshDataCompletedEventArgs args = new RefreshDataCompletedEventArgs(peerEndPoint, null, cancelled: false, m_refreshDataAsyncOp.UserSuppliedState);
					if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Firing RefreshDataCompleted event with folloding peer endpoint.");
						peerEndPoint.TracePeerEndPoint();
					}
					PrepareToRaiseRefreshDataCompletedEvent(m_refreshDataAsyncOp, args);
				}
				break;
			}
			if (ex != null)
			{
				RefreshDataCompletedEventArgs args = new RefreshDataCompletedEventArgs(null, ex, cancelled: false, m_refreshDataAsyncOp.UserSuppliedState);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Firing RefreshDataCompleted event with exception {0}.", ex);
				PrepareToRaiseRefreshDataCompletedEvent(m_refreshDataAsyncOp, args);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving InternalRefreshEndpointData.");
		}

		[SecurityCritical]
		internal static void InternalRefreshData(PeerEndPoint peerEndPoint)
		{
			PEER_ENDPOINT pEER_ENDPOINT = default(PEER_ENDPOINT);
			pEER_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gCHandle = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
			pEER_ENDPOINT.pwzEndpointName = gCHandle.AddrOfPinnedObject();
			GCHandle gCHandle2 = GCHandle.Alloc(pEER_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gCHandle2.AddrOfPinnedObject();
			int num;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabRefreshEndpointData(pcEndpoint);
			}
			finally
			{
				if (gCHandle.IsAllocated)
				{
					gCHandle.Free();
				}
				if (gCHandle2.IsAllocated)
				{
					gCHandle2.Free();
				}
			}
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRefreshEndpointData returned with errorcode {0}", num);
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_RefreshDataFailed"), num);
			}
		}

		[SecurityCritical]
		public void RefreshDataAsync(object userToken)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentNullException("userToken");
			}
			lock (RefreshDataAsyncOpLock)
			{
				if (m_refreshDataAsyncOp != null)
				{
					throw new PeerToPeerException(SR.GetString("Collab_DuplicateRefreshAsync"));
				}
				m_refreshDataAsyncOp = AsyncOperationManager.CreateOperation(userToken);
			}
			ThreadPool.QueueUserWorkItem(InternalRefreshData, true);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving RefreshDataAsync().");
		}

		protected void OnRefreshDataCompleted(RefreshDataCompletedEventArgs e)
		{
			EventHandler<RefreshDataCompletedEventArgs> refreshDataCompleted = this.m_refreshDataCompleted;
			if (refreshDataCompleted != null)
			{
				refreshDataCompleted(this, e);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the refresh endpoint completed event callback.");
			}
		}

		private void RefreshDataCompletedWaitCallback(object operationState)
		{
			m_refreshDataAsyncOp = null;
			OnRefreshDataCompleted((RefreshDataCompletedEventArgs)operationState);
		}

		internal void PrepareToRaiseRefreshDataCompletedEvent(AsyncOperation asyncOP, RefreshDataCompletedEventArgs args)
		{
			asyncOP.PostOperationCompleted(OnRefreshDataCompletedDelegate, args);
		}

		[SecurityCritical]
		private static void AddPeerNearMeChanged(EventHandler<PeerNearMeChangedEventArgs> cb)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddPeerNearMeChanged().");
			lock (LockPNMChangedEvent)
			{
				if (PeerNearMe.s_peerNearMeChanged == null)
				{
					s_peerNearMeChangedEvent = new AutoResetEvent(initialState: false);
					s_registeredPNMWaitHandle = ThreadPool.RegisterWaitForSingleObject(s_peerNearMeChangedEvent, PeerNearMeChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.PeopleNearMeChanged;
					pEventRegistrations.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(s_peerNearMeChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out s_safePeerNearMeChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_PeerNearMeChangedRegFailed"), num);
					}
				}
				PeerNearMe.s_peerNearMeChanged = (EventHandler<PeerNearMeChangedEventArgs>)Delegate.Combine(PeerNearMe.s_peerNearMeChanged, cb);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddPeerNearMeChanged() successful.");
		}

		[SecurityCritical]
		private static void RemovePeerNearMeChanged(EventHandler<PeerNearMeChangedEventArgs> cb)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePeerNearMeChanged() called.");
			lock (LockPNMChangedEvent)
			{
				PeerNearMe.s_peerNearMeChanged = (EventHandler<PeerNearMeChangedEventArgs>)Delegate.Remove(PeerNearMe.s_peerNearMeChanged, cb);
				if (PeerNearMe.s_peerNearMeChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref s_registeredPNMWaitHandle, ref s_safePeerNearMeChangedEvent, ref s_peerNearMeChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean PeerNearMeChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePeerNearMeChanged() successful.");
		}

		[SecurityCritical]
		private static void PeerNearMeChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerNearMeChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				PeerNearMeChangedEventArgs peerNearMeChangedEventArgs = null;
				try
				{
					lock (LockPNMChangedEvent)
					{
						if (s_safePeerNearMeChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(s_safePeerNearMeChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPeerNearMeChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.PeopleNearMeChanged)
						{
							PEER_EVENT_PEOPLE_NEAR_ME_CHANGED_DATA peopleNearMeChangedData = pEER_COLLAB_EVENT_DATA.peopleNearMeChangedData;
							PeerNearMe peerNearMe = null;
							if (peopleNearMeChangedData.pPeopleNearMe != IntPtr.Zero)
							{
								PEER_PEOPLE_NEAR_ME ppnm = (PEER_PEOPLE_NEAR_ME)Marshal.PtrToStructure(peopleNearMeChangedData.pPeopleNearMe, typeof(PEER_PEOPLE_NEAR_ME));
								peerNearMe = CollaborationHelperFunctions.PEER_PEOPLE_NEAR_METoPeerNearMe(ppnm);
							}
							peerNearMeChangedEventArgs = new PeerNearMeChangedEventArgs(peerNearMe, peopleNearMeChangedData.changeType);
						}
						goto IL_015e;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_015e:
				EventHandler<PeerNearMeChangedEventArgs> eventHandler = PeerNearMe.s_peerNearMeChanged;
				if (peerNearMeChangedEventArgs != null && eventHandler != null)
				{
					eventHandler(null, peerNearMeChangedEventArgs);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the peer near me changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving PeerNearMeChangedCallback().");
		}

		[SecurityCritical]
		public override PeerInvitationResponse Invite(PeerApplication applicationToInvite, string message, byte[] invitationData)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			PeerEndPointCollection peerEndPoints = PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			PeerEndPoint peerEndPoint = PeerEndPoints[0];
			PeerInvitationResponse peerInvitationResponse = Peer.InternalInviteEndPoint(applicationToInvite.Id, message, invitationData, peerEndPoint, null);
			CollaborationHelperFunctions.ThrowIfInvitationResponseInvalid(peerInvitationResponse);
			return peerInvitationResponse;
		}

		[SecurityCritical]
		public override PeerInvitationResponse Invite()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			PeerEndPointCollection peerEndPoints = PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			PeerEndPoint peerEndPoint = PeerEndPoints[0];
			PeerInvitationResponse peerInvitationResponse = Peer.InternalInviteEndPoint(currentApplicationGuid, null, null, peerEndPoint, null);
			CollaborationHelperFunctions.ThrowIfInvitationResponseInvalid(peerInvitationResponse);
			return peerInvitationResponse;
		}

		[SecurityCritical]
		public override void InviteAsync(object userToken)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			PeerEndPointCollection peerEndPoints = PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			InternalInviteAsync(currentApplicationGuid, null, null, PeerEndPoints, null, userToken);
		}

		[SecurityCritical]
		public override void InviteAsync(PeerApplication applicationToInvite, string message, byte[] invitationData, object userToken)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			PeerEndPointCollection peerEndPoints = PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			InternalInviteAsync(applicationToInvite.Id, message, invitationData, peerEndPoints, null, userToken);
		}

		public bool Equals(PeerNearMe other)
		{
			if (other == null)
			{
				throw new ArgumentNullException("other");
			}
			return other.Id.Equals(Id);
		}

		public override bool Equals(object obj)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PeerNearMe peerNearMe = obj as PeerNearMe;
			if (peerNearMe != null)
			{
				return object.Equals(peerNearMe.Id, Id);
			}
			return false;
		}

		public new static bool Equals(object objA, object objB)
		{
			PeerNearMe peerNearMe = objA as PeerNearMe;
			PeerNearMe peerNearMe2 = objB as PeerNearMe;
			if (peerNearMe != null && peerNearMe2 != null)
			{
				return object.Equals(peerNearMe.Id, peerNearMe2.Id);
			}
			return false;
		}

		public override int GetHashCode()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return Id.GetHashCode();
		}

		public override string ToString()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return Nickname;
		}

		protected override void Dispose(bool disposing)
		{
			if (!m_Disposed)
			{
				try
				{
					m_Disposed = true;
				}
				finally
				{
					base.Dispose(disposing);
				}
			}
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
			info.AddValue("_Id", m_id);
			info.AddValue("_NickName", m_nickname);
		}

		internal void TracePeerNearMe()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerNearMe");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tNickname: {0}", Nickname);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tID: {0}", Id);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tNumber of Endpoints: {0}", PeerEndPoints.Count);
			if (!Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose))
			{
				return;
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tEndPoints:");
			foreach (PeerEndPoint peerEndPoint in PeerEndPoints)
			{
				peerEndPoint.TracePeerEndPoint();
			}
		}
	}
}
