#define TRACE
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerObject : IDisposable, IEquatable<PeerObject>, ISerializable
	{
		private const int c_16K = 16384;

		private Guid m_id;

		private byte[] m_data;

		private PeerScope m_peerScope;

		private ISynchronizeInvoke m_synchronizingObject;

		private object m_lockObjChangedEvent;

		private RegisteredWaitHandle m_regObjChangedWaitHandle;

		private AutoResetEvent m_objChangedEvent;

		private SafeCollabEvent m_safeObjChangedEvent;

		private bool m_Disposed;

		public Guid Id
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_id;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_id = value;
			}
		}

		public byte[] Data
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_data;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				if (value != null && value.Length > 16384)
				{
					throw new ArgumentException(SR.GetString("Collab_ObjectDataSizeFailed"));
				}
				m_data = value;
			}
		}

		public PeerScope PeerScope
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_peerScope;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_peerScope = value;
			}
		}

		[Browsable(false)]
		[Description("SynchronizingObject")]
		[DefaultValue(null)]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_synchronizingObject;
			}
			set
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				m_synchronizingObject = value;
			}
		}

		private object LockObjChangedEvent
		{
			get
			{
				if (m_lockObjChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref m_lockObjChangedEvent, value, null);
				}
				return m_lockObjChangedEvent;
			}
		}

		private event EventHandler<ObjectChangedEventArgs> m_objectChanged;

		public event EventHandler<ObjectChangedEventArgs> ObjectChanged
		{
			[SecurityCritical]
			add
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				AddObjectChangedEvent(value);
			}
			[SecurityCritical]
			remove
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				RemoveObjectChangedEvent(value);
			}
		}

		[SecurityCritical]
		static PeerObject()
		{
			CollaborationHelperFunctions.Initialize();
		}

		public PeerObject()
		{
			m_id = Guid.NewGuid();
		}

		public PeerObject(Guid Id, byte[] data, PeerScope peerScope)
		{
			if (data != null && data.Length > 16384)
			{
				throw new ArgumentException(SR.GetString("Collab_ObjectDataSizeFailed"), "data");
			}
			m_id = Id;
			m_peerScope = peerScope;
			m_data = data;
		}

		protected PeerObject(SerializationInfo serializationInfo, StreamingContext streamingContext)
		{
			m_id = (Guid)serializationInfo.GetValue("_Id", typeof(Guid));
			m_data = (byte[])serializationInfo.GetValue("_Data", typeof(byte[]));
			m_peerScope = (PeerScope)serializationInfo.GetInt32("_Scope");
		}

		[SecurityCritical]
		private void AddObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChanged() called.");
			lock (LockObjChangedEvent)
			{
				if (this.m_objectChanged == null)
				{
					if (m_id.Equals(Guid.Empty))
					{
						throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
					}
					m_objChangedEvent = new AutoResetEvent(initialState: false);
					m_regObjChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_objChangedEvent, ObjectChangedCallback, null, -1, executeOnlyOnce: false);
					PEER_COLLAB_EVENT_REGISTRATION pEventRegistrations = default(PEER_COLLAB_EVENT_REGISTRATION);
					pEventRegistrations.eventType = PeerCollabEventType.EndPointObjectChanged;
					GUID gUID = CollaborationHelperFunctions.ConvertGuidToGUID(m_id);
					GCHandle gCHandle = GCHandle.Alloc(gUID, GCHandleType.Pinned);
					pEventRegistrations.pInstance = gCHandle.AddrOfPinnedObject();
					try
					{
						int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(m_objChangedEvent.SafeWaitHandle, 1u, ref pEventRegistrations, out m_safeObjChangedEvent);
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", num);
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectChangedRegFailed"), num);
						}
					}
					finally
					{
						if (gCHandle.IsAllocated)
						{
							gCHandle.Free();
						}
					}
				}
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Combine(this.m_objectChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChanged() called.");
			lock (LockObjChangedEvent)
			{
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Remove(this.m_objectChanged, callback);
				if (this.m_objectChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref m_regObjChangedWaitHandle, ref m_safeObjChangedEvent, ref m_objChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChangedEvent variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChanged() successful.");
		}

		protected virtual void OnObjectChanged(ObjectChangedEventArgs objChangedArgs)
		{
			EventHandler<ObjectChangedEventArgs> objectChanged = this.m_objectChanged;
			if (objectChanged != null)
			{
				if (SynchronizingObject != null && SynchronizingObject.InvokeRequired)
				{
					SynchronizingObject.BeginInvoke(objectChanged, new object[2]
					{
						this,
						objChangedArgs
					});
				}
				else
				{
					objectChanged(this, objChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the object changed event callback.");
			}
		}

		[SecurityCritical]
		private void ObjectChangedCallback(object state, bool timedOut)
		{
			SafeCollabData ppEventData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ObjectChangedCallback() called.");
			while (true)
			{
				bool flag = true;
				ObjectChangedEventArgs objectChangedEventArgs = null;
				try
				{
					lock (LockObjChangedEvent)
					{
						if (m_safeObjChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(m_safeObjChangedEvent, out ppEventData);
					}
					switch (num)
					{
					case 6488066:
						break;
					default:
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", num);
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectChangedDataFailed"), num);
					case 0:
					{
						PEER_COLLAB_EVENT_DATA pEER_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(ppEventData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (pEER_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointObjectChanged)
						{
							PEER_EVENT_OBJECT_CHANGED_DATA objectChangedData = pEER_COLLAB_EVENT_DATA.objectChangedData;
							PEER_OBJECT po = (PEER_OBJECT)Marshal.PtrToStructure(objectChangedData.pObject, typeof(PEER_OBJECT));
							PeerObject peerObject = CollaborationHelperFunctions.ConvertPEER_OBJECTToPeerObject(po);
							if (object.Equals(m_id, peerObject.Id))
							{
								PeerContact peerContact = null;
								PeerEndPoint peerEndPoint = null;
								if (objectChangedData.pContact != IntPtr.Zero)
								{
									PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(objectChangedData.pContact, typeof(PEER_CONTACT));
									peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
								}
								if (objectChangedData.pEndPoint != IntPtr.Zero)
								{
									PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(objectChangedData.pEndPoint, typeof(PEER_ENDPOINT));
									peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
								}
								objectChangedEventArgs = new ObjectChangedEventArgs(peerEndPoint, peerContact, objectChangedData.changeType, peerObject);
							}
						}
						goto IL_01fc;
					}
					}
				}
				finally
				{
					ppEventData?.Dispose();
				}
				break;
				IL_01fc:
				if (objectChangedEventArgs != null)
				{
					OnObjectChanged(objectChangedEventArgs);
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ObjectChangedCallback().");
		}

		public bool Equals(PeerObject other)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return other?.Id.Equals(Id) ?? false;
		}

		public override bool Equals(object obj)
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return (obj as PeerObject)?.Id.Equals(Id) ?? false;
		}

		public new static bool Equals(object objA, object objB)
		{
			PeerObject peerObject = objA as PeerObject;
			PeerObject peerObject2 = objB as PeerObject;
			if (peerObject != null && peerObject2 != null)
			{
				return object.Equals(peerObject.Id, peerObject2.Id);
			}
			return false;
		}

		public override int GetHashCode()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return Id.GetHashCode();
		}

		public override string ToString()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			return Id.ToString();
		}

		[SecurityCritical]
		public void Dispose()
		{
			Dispose(disposing: true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		protected virtual void Dispose(bool disposing)
		{
			if (!m_Disposed)
			{
				CollaborationHelperFunctions.CleanEventVars(ref m_regObjChangedWaitHandle, ref m_safeObjChangedEvent, ref m_objChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChangedEvent variables successful.");
				m_Disposed = true;
			}
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_Id", m_id);
			info.AddValue("_Data", m_data);
			info.AddValue("_Scope", m_peerScope);
		}

		internal void TracePeerObject()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerObject");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tGuid: {0}", Id);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tPeerScope: {0}", PeerScope);
			if (Data != null)
			{
				if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose))
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tObject data:");
					Logging.DumpData(Logging.P2PTraceSource, TraceEventType.Verbose, Logging.P2PTraceSource.MaxDataSize, Data, 0, Data.Length);
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tObject data length {0}", Data.Length);
				}
			}
		}
	}
}
