#define TRACE
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Text;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public class Cloud : ISerializable, IEquatable<Cloud>
	{
		private const string PEER_PNRP_ALL_LINK_CLOUDS = "PEER_PNRP_ALL_LINKS";

		private string m_CloudName;

		private PnrpScope m_PnrpScope;

		private int m_ScopeId;

		public static readonly Cloud Available;

		public static readonly Cloud AllLinkLocal;

		public string Name
		{
			get
			{
				if (this == AllLinkLocal || this == Available)
				{
					return null;
				}
				return m_CloudName;
			}
		}

		internal string InternalName
		{
			get
			{
				if (this == AllLinkLocal)
				{
					return "PEER_PNRP_ALL_LINKS";
				}
				if (this == Available)
				{
					return null;
				}
				return m_CloudName;
			}
		}

		public PnrpScope Scope => m_PnrpScope;

		public int ScopeId => m_ScopeId;

		public static Cloud Global
		{
			get
			{
				CloudCollection clouds = null;
				Cloud cloud = null;
				GetCloudOrClouds(null, bGlobalCloudOnly: true, out clouds, out cloud);
				return cloud;
			}
		}

		static Cloud()
		{
			Available = new Cloud("AllAvailable", PnrpScope.All, -1);
			AllLinkLocal = new Cloud("AllLinkLocal", PnrpScope.LinkLocal, -1);
			if (!PeerToPeerOSHelper.SupportsP2P)
			{
				throw new PlatformNotSupportedException(SR.GetString("P2P_NotAvailable"));
			}
		}

		internal Cloud(string name, PnrpScope pnrpScope, int scopeId)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Creating cloud with Name: {0}, PnrpScope: {1}, ScopeID: {2}", name, pnrpScope, scopeId);
			m_CloudName = name;
			m_PnrpScope = pnrpScope;
			m_ScopeId = scopeId;
		}

		public static Cloud GetCloudByName(string cloudName)
		{
			if (cloudName == null || cloudName.Length == 0)
			{
				throw new ArgumentException(SR.GetString("Pnrp_CloudNameCantBeNull"), "cloudName");
			}
			CloudCollection clouds = null;
			Cloud cloud = null;
			GetCloudOrClouds(cloudName, bGlobalCloudOnly: false, out clouds, out cloud);
			return cloud;
		}

		public static CloudCollection GetAvailableClouds()
		{
			CloudCollection clouds = null;
			Cloud cloud = null;
			GetCloudOrClouds(null, bGlobalCloudOnly: false, out clouds, out cloud);
			return clouds;
		}

		[SecurityTreatAsSafe]
		[SecurityCritical]
		private static void GetCloudOrClouds(string cloudName, bool bGlobalCloudOnly, out CloudCollection clouds, out Cloud cloud)
		{
			cloud = null;
			clouds = null;
			Logging.Enter(Logging.P2PTraceSource, "Cloud::GetCloudOrClouds()");
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			int num = 0;
			uint pNumClouds = 0u;
			SafePeerData pArrayOfClouds = null;
			if (cloudName == null)
			{
				clouds = new CloudCollection();
			}
			try
			{
				UnsafeP2PNativeMethods.PnrpStartup();
				num = UnsafeP2PNativeMethods.PeerPnrpGetCloudInfo(out pNumClouds, out pArrayOfClouds);
				if (num != 0)
				{
					throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotEnumerateClouds"), num);
				}
				if (pNumClouds != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Number of clouds returned {0}", pNumClouds);
					IntPtr intPtr = pArrayOfClouds.DangerousGetHandle();
					for (ulong num2 = 0uL; num2 < pNumClouds; num2++)
					{
						PEER_PNRP_CLOUD_INFO pEER_PNRP_CLOUD_INFO = (PEER_PNRP_CLOUD_INFO)Marshal.PtrToStructure(intPtr, typeof(PEER_PNRP_CLOUD_INFO));
						string text = Marshal.PtrToStringUni(pEER_PNRP_CLOUD_INFO.pwzCloudName);
						intPtr = (IntPtr)((long)intPtr + Marshal.SizeOf(typeof(PEER_PNRP_CLOUD_INFO)));
						Cloud cloud2 = new Cloud(text, (PnrpScope)pEER_PNRP_CLOUD_INFO.dwScope, (int)pEER_PNRP_CLOUD_INFO.dwScopeId);
						if (cloudName == null && !bGlobalCloudOnly)
						{
							clouds.Add(cloud2);
							continue;
						}
						if (cloudName != null && cloudName == text)
						{
							cloud = cloud2;
							break;
						}
						if (bGlobalCloudOnly && cloud2.Scope == PnrpScope.Global)
						{
							cloud = cloud2;
							break;
						}
					}
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Warning, 0, "No Clouds returned from the native call");
				}
			}
			finally
			{
				pArrayOfClouds?.Dispose();
			}
			if (cloudName != null && cloud == null)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "The specific cloud name {0} asked for is not found", cloudName);
			}
			Logging.Leave(Logging.P2PTraceSource, "Cloud::GetCloudOrClouds()");
		}

		public bool Equals(Cloud other)
		{
			if (other == null)
			{
				return false;
			}
			return other.Name == Name && other.Scope == Scope && other.ScopeId == ScopeId;
		}

		public override bool Equals(object obj)
		{
			if (obj == null)
			{
				return false;
			}
			Cloud cloud = obj as Cloud;
			if (cloud == null)
			{
				return false;
			}
			return Equals(cloud);
		}

		public override int GetHashCode()
		{
			return m_CloudName.GetHashCode();
		}

		public override string ToString()
		{
			StringBuilder stringBuilder = new StringBuilder();
			stringBuilder.Append("Cloud Name:");
			stringBuilder.Append(Name);
			stringBuilder.Append(" Scope:");
			stringBuilder.Append(Scope);
			stringBuilder.Append(" ScopeId:");
			stringBuilder.Append(ScopeId);
			return stringBuilder.ToString();
		}

		protected Cloud(SerializationInfo info, StreamingContext context)
		{
			m_CloudName = info.GetString("_CloudName");
			m_PnrpScope = (PnrpScope)info.GetValue("_CloudScope", typeof(PnrpScope));
			m_ScopeId = info.GetInt32("_CloudScopeId");
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_CloudName", Name);
			info.AddValue("_CloudScope", m_PnrpScope);
			info.AddValue("_CloudScopeId", m_ScopeId);
		}
	}
}
