#define TRACE
using System.Diagnostics;
using System.Globalization;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Text;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public class PeerName : ISerializable, IEquatable<PeerName>
	{
		private const int PEER_MAX_CLASSIFIER_LENGTH = 149;

		private const int SECURE_AUTHORITY_LENGTH = 40;

		private const int INSECURE_AUTHORITY_LENGTH = 1;

		private const int PEER_MAX_PEERNAME_LENGTH = 190;

		private const string PEERNAME_UNSECURED_AUTHORITY = "0";

		private string m_PeerName;

		private string m_Authority;

		private string m_Classifier;

		private string m_PeerHostName;

		public string Authority => m_Authority;

		public string Classifier => m_Classifier;

		public string PeerHostName
		{
			[SecurityCritical]
			get
			{
				if (m_PeerHostName == null)
				{
					SafePeerData peerHostName = null;
					try
					{
						int num = UnsafeP2PNativeMethods.PeerNameToPeerHostName(m_PeerName, out peerHostName);
						if (num < 0)
						{
							throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotGetPeerHostNameFromPeerName"), num);
						}
						m_PeerHostName = peerHostName.UnicodeString;
					}
					finally
					{
						peerHostName?.Dispose();
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerHostName created for PeerName PeerHostName {0} PeerName {1}", m_PeerHostName, this);
				return m_PeerHostName;
			}
		}

		public bool IsSecured => m_Authority != "0";

		static PeerName()
		{
			if (!PeerToPeerOSHelper.SupportsP2P)
			{
				throw new PlatformNotSupportedException(SR.GetString("P2P_NotAvailable"));
			}
		}

		private PeerName(string peerName, string authority, string classifier)
		{
			m_PeerName = peerName;
			m_Classifier = classifier;
			m_Authority = authority;
		}

		public PeerName(string remotePeerName)
		{
			if (remotePeerName == null)
			{
				throw new ArgumentNullException("remotePeerName");
			}
			if (!StrongParsePeerName(remotePeerName, out string authority, out string classifier))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerName string {0} failed the check for a valid peer name", remotePeerName);
				throw new ArgumentException(SR.GetString("Pnrp_InvalidPeerName"), "remotePeerName");
			}
			if (classifier != null)
			{
				m_PeerName = authority + "." + classifier;
			}
			else
			{
				m_PeerName = authority;
			}
			m_Authority = authority;
			m_Classifier = classifier;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerName instance created - PeerName {0} Authority {1} Classfier {2}", m_PeerName, m_Authority, (m_Classifier == null) ? "null" : m_Classifier);
		}

		[SecurityCritical]
		public PeerName(string classifier, PeerNameType peerNameType)
		{
			if ((classifier == null || classifier.Length == 0) && peerNameType == PeerNameType.Unsecured)
			{
				throw new ArgumentNullException("classifier");
			}
			if (classifier != null && classifier.Length > 149)
			{
				throw new ArgumentException(SR.GetString("Pnrp_InvalidClassifier"), "classifier");
			}
			if (classifier != null && classifier.Length > 0)
			{
				classifier = classifier.Normalize(NormalizationForm.FormC);
			}
			SafePeerData peerName = null;
			SafePeerData defaultIdentity = null;
			try
			{
				if (peerNameType == PeerNameType.Unsecured)
				{
					int num = UnsafeP2PNativeMethods.PeerCreatePeerName(null, classifier, out peerName);
					if (num != 0)
					{
						throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotCreateUnsecuredPeerName"), num);
					}
					m_Authority = "0";
				}
				else
				{
					int num = UnsafeP2PNativeMethods.PeerIdentityGetDefault(out defaultIdentity);
					if (num != 0)
					{
						throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotGetDefaultIdentity"), num);
					}
					m_Authority = defaultIdentity.UnicodeString;
					num = UnsafeP2PNativeMethods.PeerCreatePeerName(m_Authority, classifier, out peerName);
					if (num != 0)
					{
						throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotCreateSecuredPeerName"), num);
					}
				}
				m_PeerName = peerName.UnicodeString;
				m_Classifier = classifier;
			}
			finally
			{
				peerName?.Dispose();
				defaultIdentity?.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerName instance created - PeerName {0} Authority {1} Classfier {2}", m_PeerName, m_Authority, m_Classifier);
		}

		[SecurityCritical]
		public static PeerName CreateFromPeerHostName(string peerHostName)
		{
			if (peerHostName == null)
			{
				throw new ArgumentNullException("peerHostName");
			}
			if (peerHostName.Length == 0)
			{
				throw new ArgumentException(SR.GetString("Pnrp_InvalidPeerHostName"), "peerHostName");
			}
			SafePeerData peerName = null;
			string text = null;
			try
			{
				int num = UnsafeP2PNativeMethods.PeerHostNameToPeerName(peerHostName, out peerName);
				if (num != 0)
				{
					throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotGetPeerNameFromPeerHostName"), num);
				}
				text = peerName.UnicodeString;
			}
			finally
			{
				peerName?.Dispose();
			}
			WeakParsePeerName(text, out string authority, out string classifier);
			PeerName result = new PeerName(text, authority, classifier);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerName created from PeerHostName - PeerHostName {0} to PeerName PeerName {1} Authority {2} Classfier {3}", peerHostName, text, authority, classifier);
			return result;
		}

		[SecurityCritical]
		public static PeerName CreateRelativePeerName(PeerName peerName, string classifier)
		{
			if (peerName == null)
			{
				throw new ArgumentNullException("peerName", SR.GetString("Pnrp_PeerNameCantBeNull"));
			}
			if (!peerName.IsSecured && (classifier == null || classifier.Length == 0))
			{
				throw new ArgumentException(SR.GetString("Pnrp_InvalidClassifier"), "classifier");
			}
			if (classifier != null && classifier.Length > 149)
			{
				throw new ArgumentException(SR.GetString("Pnrp_InvalidClassifier"), "classifier");
			}
			if (classifier != null && classifier.Length > 0)
			{
				classifier = classifier.Normalize(NormalizationForm.FormC);
			}
			SafePeerData peerName2 = null;
			string peerName3 = null;
			try
			{
				int num = UnsafeP2PNativeMethods.PeerCreatePeerName(peerName.IsSecured ? peerName.m_PeerName : null, classifier, out peerName2);
				if (num != 0)
				{
					throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotCreateRelativePeerName"), num);
				}
				peerName3 = peerName2.UnicodeString;
			}
			finally
			{
				peerName2?.Dispose();
			}
			WeakParsePeerName(peerName3, out string authority, out string classifier2);
			PeerName peerName4 = new PeerName(peerName3, authority, classifier2);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "A new PeerName created from existing PeerName with a new classfier. Existing PeerName {0} Classifier {1} New PeerName {2}", peerName, classifier, peerName4);
			return peerName4;
		}

		public override string ToString()
		{
			return m_PeerName;
		}

		public bool Equals(PeerName other)
		{
			if (other == null)
			{
				return false;
			}
			return string.Compare(other.m_PeerName, m_PeerName, StringComparison.Ordinal) == 0;
		}

		public override bool Equals(object obj)
		{
			if (obj == null)
			{
				return false;
			}
			PeerName peerName = obj as PeerName;
			if (peerName == null)
			{
				return false;
			}
			return Equals(peerName);
		}

		public override int GetHashCode()
		{
			return m_PeerName.GetHashCode();
		}

		private static bool StrongParsePeerName(string peerName, out string authority, out string classifier)
		{
			authority = null;
			classifier = null;
			if (peerName == null || peerName.Length == 0 || peerName.Length > 190)
			{
				return false;
			}
			if (peerName.Length < 3)
			{
				return false;
			}
			string text = null;
			int num = peerName.IndexOf('.');
			if (num == 0)
			{
				return false;
			}
			text = ((num < 0) ? peerName : ((num != peerName.Length - 1) ? peerName.Substring(0, num) : peerName.Substring(0, peerName.Length - 1)));
			if (text.Length != 40 && text.Length != 1)
			{
				return false;
			}
			if (text.Length == 1 && text != "0")
			{
				return false;
			}
			if (text.Length == 40)
			{
				string text2 = text;
				foreach (char character in text2)
				{
					if (!IsHexDigit(character))
					{
						return false;
					}
				}
			}
			string text3 = null;
			if (num != peerName.Length - 1 && num > 0)
			{
				text3 = peerName.Substring(num + 1);
			}
			if (text3 != null && text3.Length > 149)
			{
				return false;
			}
			authority = text.ToLower(CultureInfo.InvariantCulture);
			classifier = text3;
			return true;
		}

		private static bool IsHexDigit(char character)
		{
			return (character >= '0' && character <= '9') || (character >= 'A' && character <= 'F') || (character >= 'a' && character <= 'f');
		}

		private static void WeakParsePeerName(string peerName, out string authority, out string classifier)
		{
			authority = null;
			classifier = null;
			int num = peerName.IndexOf('.');
			if (num < 0)
			{
				authority = peerName;
			}
			else if (num == peerName.Length - 1)
			{
				authority = peerName.Substring(0, peerName.Length - 1);
			}
			else
			{
				authority = peerName.Substring(0, num);
			}
			if (num != peerName.Length - 1 && num > 0)
			{
				classifier = peerName.Substring(num + 1);
			}
			authority = authority.ToLower(CultureInfo.InvariantCulture);
		}

		protected PeerName(SerializationInfo info, StreamingContext context)
		{
			if (info == null)
			{
				throw new ArgumentNullException("info");
			}
			m_PeerName = info.GetString("_PeerName");
			m_Authority = info.GetString("_Authority");
			m_Classifier = info.GetString("_Classifier");
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_PeerName", m_PeerName);
			info.AddValue("_Authority", m_Authority);
			info.AddValue("_Classifier", m_Classifier);
		}
	}
}
