#define TRACE
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public class PeerNameRegistration : IDisposable, ISerializable
	{
		private const uint PEER_PNRP_AUTO_ADDRESSES = uint.MaxValue;

		private PeerNameRecord m_PeerNameRecord = new PeerNameRecord();

		private int m_Port;

		private Cloud m_Cloud;

		private bool m_IsRegistered;

		private SafePeerNameUnregister m_RegistrationHandle;

		private PeerName m_RegisteredPeerName;

		private bool m_UseAutoEndPointSelection = true;

		private bool m_Disposed;

		internal PeerNameRecord PeerNameRecord
		{
			get
			{
				if (m_Disposed)
				{
					throw new ObjectDisposedException(GetType().FullName);
				}
				return m_PeerNameRecord;
			}
		}

		public int Port
		{
			get
			{
				return m_Port;
			}
			set
			{
				if (value < 0 || value > 65535)
				{
					throw new ArgumentOutOfRangeException("port", SR.GetString("Pnrp_PortOutOfRange"));
				}
				m_Port = value;
			}
		}

		public PeerName PeerName
		{
			get
			{
				return m_PeerNameRecord.PeerName;
			}
			set
			{
				m_PeerNameRecord.PeerName = value;
			}
		}

		public IPEndPointCollection EndPointCollection => m_PeerNameRecord.EndPointCollection;

		public Cloud Cloud
		{
			get
			{
				return m_Cloud;
			}
			set
			{
				m_Cloud = value;
			}
		}

		public string Comment
		{
			get
			{
				return m_PeerNameRecord.Comment;
			}
			set
			{
				m_PeerNameRecord.Comment = value;
			}
		}

		public byte[] Data
		{
			get
			{
				return m_PeerNameRecord.Data;
			}
			set
			{
				m_PeerNameRecord.Data = value;
			}
		}

		public bool UseAutoEndPointSelection
		{
			get
			{
				return m_UseAutoEndPointSelection;
			}
			set
			{
				m_UseAutoEndPointSelection = value;
			}
		}

		static PeerNameRegistration()
		{
			if (!PeerToPeerOSHelper.SupportsP2P)
			{
				throw new PlatformNotSupportedException(SR.GetString("P2P_NotAvailable"));
			}
		}

		public PeerNameRegistration()
		{
		}

		public PeerNameRegistration(PeerName name, int port)
			: this(name, port, null)
		{
		}

		public PeerNameRegistration(PeerName name, int port, Cloud cloud)
		{
			if (name == null)
			{
				throw new ArgumentNullException("name");
			}
			if (cloud == null)
			{
				cloud = Cloud.Available;
			}
			if (port < 0 || port > 65535)
			{
				throw new ArgumentOutOfRangeException("port", SR.GetString("Pnrp_PortOutOfRange"));
			}
			m_PeerNameRecord.PeerName = name;
			m_Port = port;
			m_Cloud = cloud;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Created a PeerNameRegistration with PeerName {0}, Port {1}, Cloud {2} - Proceeding to register", name, port, cloud);
		}

		public bool IsRegistered()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			return m_IsRegistered;
		}

		[SecurityCritical]
		public void Start()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Proceeding to register through the Register method()");
			InternalRegister();
		}

		[SecurityCritical]
		private unsafe void InternalRegister()
		{
			if (m_IsRegistered)
			{
				throw new PeerToPeerException(SR.GetString("Pnrp_UseUpdateInsteadOfRegister"));
			}
			if (m_PeerNameRecord.PeerName == null)
			{
				throw new ArgumentNullException("PeerName");
			}
			if (!m_UseAutoEndPointSelection && EndPointCollection.Count == 0 && (Data == null || Data.Length <= 0))
			{
				throw new PeerToPeerException(SR.GetString("Pnrp_BlobOrEndpointListNeeded"));
			}
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			UnsafeP2PNativeMethods.PnrpStartup();
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "InternalRegister() is called with the following Info");
				m_PeerNameRecord.TracePeerNameRecord();
			}
			PEER_PNRP_REGISTRATION_INFO registrationInfo = default(PEER_PNRP_REGISTRATION_INFO);
			GCHandle gCHandle;
			if (m_PeerNameRecord.Data != null)
			{
				registrationInfo.payLoad.cbPayload = (uint)m_PeerNameRecord.Data.Length;
				gCHandle = GCHandle.Alloc(m_PeerNameRecord.Data, GCHandleType.Pinned);
				registrationInfo.payLoad.pbPayload = gCHandle.AddrOfPinnedObject();
			}
			else
			{
				gCHandle = default(GCHandle);
			}
			if (m_PeerNameRecord.Comment != null && m_PeerNameRecord.Comment.Length > 0)
			{
				registrationInfo.pwszComment = m_PeerNameRecord.Comment;
			}
			if (m_Cloud != null)
			{
				registrationInfo.pwszCloudName = m_Cloud.InternalName;
			}
			try
			{
				if (m_PeerNameRecord.EndPointCollection.Count == 0)
				{
					registrationInfo.wport = (ushort)m_Port;
					if (m_UseAutoEndPointSelection)
					{
						registrationInfo.cAddresses = uint.MaxValue;
					}
					int num = UnsafeP2PNativeMethods.PeerPnrpRegister(m_PeerNameRecord.PeerName.ToString(), ref registrationInfo, out m_RegistrationHandle);
					if (num != 0)
					{
						throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotRegisterPeerName"), num);
					}
				}
				else
				{
					int count = m_PeerNameRecord.EndPointCollection.Count;
					int cb = count * Marshal.SizeOf(typeof(IntPtr));
					IntPtr intPtr = Marshal.AllocHGlobal(cb);
					GCHandle[] array = new GCHandle[count];
					try
					{
						IntPtr* ptr = (IntPtr*)(void*)intPtr;
						for (int i = 0; i < m_PeerNameRecord.EndPointCollection.Count; i++)
						{
							byte[] value = SystemNetHelpers.SOCKADDRFromIPEndPoint(m_PeerNameRecord.EndPointCollection[i]);
							ref GCHandle reference = ref array[i];
							reference = GCHandle.Alloc(value, GCHandleType.Pinned);
							IntPtr intPtr2 = ptr[i] = array[i].AddrOfPinnedObject();
						}
						registrationInfo.ArrayOfSOCKADDRIN6Pointers = intPtr;
						registrationInfo.cAddresses = (uint)count;
						int num = UnsafeP2PNativeMethods.PeerPnrpRegister(m_PeerNameRecord.PeerName.ToString(), ref registrationInfo, out m_RegistrationHandle);
						if (num != 0)
						{
							throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotRegisterPeerName"), num);
						}
					}
					finally
					{
						if (intPtr != IntPtr.Zero)
						{
							Marshal.FreeHGlobal(intPtr);
						}
						for (int i = 0; i < array.Length; i++)
						{
							if (array[i].IsAllocated)
							{
								array[i].Free();
							}
						}
					}
				}
			}
			finally
			{
				if (gCHandle.IsAllocated)
				{
					gCHandle.Free();
				}
			}
			m_RegisteredPeerName = m_PeerNameRecord.PeerName;
			m_IsRegistered = true;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Registration is successful. The handle is {0}", m_RegistrationHandle.DangerousGetHandle());
		}

		[SecurityCritical]
		public unsafe void Update()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			if (!IsRegistered())
			{
				throw new InvalidOperationException(SR.GetString("Pnrp_CallRegisterBeforeUpdate"));
			}
			if (m_PeerNameRecord.PeerName == null)
			{
				throw new ArgumentNullException(SR.GetString("Pnrp_InvalidPeerName"));
			}
			if (!m_RegisteredPeerName.Equals(m_PeerNameRecord.PeerName))
			{
				throw new InvalidOperationException(SR.GetString("Pnrp_CantChangePeerNameAfterRegistration"));
			}
			if (!m_UseAutoEndPointSelection && (EndPointCollection.Count == 0 || Data == null || Data.Length <= 0))
			{
				throw new PeerToPeerException(SR.GetString("Pnrp_BlobOrEndpointListNeeded"));
			}
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Update() is called with the following Info");
				m_PeerNameRecord.TracePeerNameRecord();
			}
			PEER_PNRP_REGISTRATION_INFO registrationInfo = default(PEER_PNRP_REGISTRATION_INFO);
			if (m_PeerNameRecord.Data != null)
			{
				registrationInfo.payLoad.cbPayload = (uint)m_PeerNameRecord.Data.Length;
				GCHandle gCHandle = GCHandle.Alloc(m_PeerNameRecord.Data, GCHandleType.Pinned);
				registrationInfo.payLoad.pbPayload = gCHandle.AddrOfPinnedObject();
				gCHandle.Free();
			}
			if (m_PeerNameRecord.Comment != null && m_PeerNameRecord.Comment.Length > 0)
			{
				registrationInfo.pwszComment = m_PeerNameRecord.Comment;
			}
			registrationInfo.pwszCloudName = null;
			if (m_Cloud != null)
			{
				registrationInfo.pwszCloudName = m_Cloud.InternalName;
			}
			if (m_PeerNameRecord.EndPointCollection.Count == 0)
			{
				registrationInfo.wport = (ushort)m_Port;
				registrationInfo.cAddresses = uint.MaxValue;
				int num = UnsafeP2PNativeMethods.PeerPnrpUpdateRegistration(m_RegistrationHandle, ref registrationInfo);
				if (num != 0)
				{
					throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotRegisterPeerName"), num);
				}
				return;
			}
			int count = m_PeerNameRecord.EndPointCollection.Count;
			int cb = count * Marshal.SizeOf(typeof(IntPtr));
			IntPtr intPtr = Marshal.AllocHGlobal(cb);
			GCHandle[] array = new GCHandle[count];
			try
			{
				IntPtr* ptr = (IntPtr*)(void*)intPtr;
				for (int i = 0; i < m_PeerNameRecord.EndPointCollection.Count; i++)
				{
					byte[] value = SystemNetHelpers.SOCKADDRFromIPEndPoint(m_PeerNameRecord.EndPointCollection[i]);
					ref GCHandle reference = ref array[i];
					reference = GCHandle.Alloc(value, GCHandleType.Pinned);
					IntPtr intPtr2 = ptr[i] = array[i].AddrOfPinnedObject();
				}
				registrationInfo.ArrayOfSOCKADDRIN6Pointers = intPtr;
				registrationInfo.cAddresses = (uint)count;
				int num = UnsafeP2PNativeMethods.PeerPnrpUpdateRegistration(m_RegistrationHandle, ref registrationInfo);
				if (num != 0)
				{
					throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotRegisterPeerName"), num);
				}
			}
			finally
			{
				if (intPtr != IntPtr.Zero)
				{
					Marshal.FreeHGlobal(intPtr);
				}
				for (int i = 0; i < array.Length; i++)
				{
					if (array[i].IsAllocated)
					{
						array[i].Free();
					}
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Update of existing registration is successful. The handle is {0}", m_RegistrationHandle.DangerousGetHandle());
		}

		[SecurityCritical]
		public void Stop()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			if (m_RegistrationHandle.IsInvalid || m_RegistrationHandle.IsClosed)
			{
				throw new InvalidOperationException(SR.GetString("Pnrp_NoRegistrationFound"));
			}
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			m_RegistrationHandle.Dispose();
			m_PeerNameRecord = new PeerNameRecord();
			m_RegisteredPeerName = null;
			m_IsRegistered = false;
		}

		public void Dispose()
		{
			Dispose(disposing: true);
			GC.SuppressFinalize(this);
		}

		protected virtual void Dispose(bool disposing)
		{
			if (!m_Disposed)
			{
				try
				{
					Stop();
				}
				catch (ObjectDisposedException)
				{
				}
				catch (InvalidOperationException)
				{
				}
			}
			m_Disposed = true;
		}

		protected PeerNameRegistration(SerializationInfo info, StreamingContext context)
		{
			m_Port = info.GetInt32("_Port");
			m_UseAutoEndPointSelection = info.GetBoolean("_UseAutoEndPointSelection");
			m_Cloud = (info.GetValue("_Cloud", typeof(Cloud)) as Cloud);
			m_PeerNameRecord = (info.GetValue("_PeerNameRecord", typeof(PeerNameRecord)) as PeerNameRecord);
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_Port", m_Port);
			info.AddValue("_UseAutoEndPointSelection", m_UseAutoEndPointSelection);
			info.AddValue("_Cloud", m_Cloud);
			info.AddValue("_PeerNameRecord", m_PeerNameRecord);
		}
	}
}
