#define TRACE
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer
{
	public class PeerNameResolver
	{
		private SendOrPostCallback OnResolveProgressChangedDelegate;

		private SendOrPostCallback OnResolveCompletedDelegate;

		private object m_PeerNameResolverHelperListLock = new object();

		private Dictionary<object, PeerNameResolverHelper> m_PeerNameResolverHelperList = new Dictionary<object, PeerNameResolverHelper>();

		private static int s_TraceEventId;

		private static int NewTraceEventId
		{
			get
			{
				Interlocked.CompareExchange(ref s_TraceEventId, 0, int.MaxValue);
				Interlocked.Increment(ref s_TraceEventId);
				return s_TraceEventId;
			}
		}

		private event EventHandler<ResolveProgressChangedEventArgs> m_ResolveProgressChanged;

		public event EventHandler<ResolveProgressChangedEventArgs> ResolveProgressChanged
		{
			add
			{
				PnrpPermission.UnrestrictedPnrpPermission.Demand();
				this.m_ResolveProgressChanged = (EventHandler<ResolveProgressChangedEventArgs>)Delegate.Combine(this.m_ResolveProgressChanged, value);
			}
			remove
			{
				PnrpPermission.UnrestrictedPnrpPermission.Demand();
				this.m_ResolveProgressChanged = (EventHandler<ResolveProgressChangedEventArgs>)Delegate.Remove(this.m_ResolveProgressChanged, value);
			}
		}

		private event EventHandler<ResolveCompletedEventArgs> m_ResolveCompleted;

		public event EventHandler<ResolveCompletedEventArgs> ResolveCompleted
		{
			add
			{
				PnrpPermission.UnrestrictedPnrpPermission.Demand();
				this.m_ResolveCompleted = (EventHandler<ResolveCompletedEventArgs>)Delegate.Combine(this.m_ResolveCompleted, value);
			}
			remove
			{
				PnrpPermission.UnrestrictedPnrpPermission.Demand();
				this.m_ResolveCompleted = (EventHandler<ResolveCompletedEventArgs>)Delegate.Remove(this.m_ResolveCompleted, value);
			}
		}

		static PeerNameResolver()
		{
			if (!PeerToPeerOSHelper.SupportsP2P)
			{
				throw new PlatformNotSupportedException(SR.GetString("P2P_NotAvailable"));
			}
		}

		public PeerNameResolver()
		{
			OnResolveProgressChangedDelegate = ResolveProgressChangedWaitCallback;
			OnResolveCompletedDelegate = ResolveCompletedWaitCallback;
		}

		public PeerNameRecordCollection Resolve(PeerName peerName)
		{
			return Resolve(peerName, Cloud.Available, int.MaxValue);
		}

		public PeerNameRecordCollection Resolve(PeerName peerName, Cloud cloud)
		{
			return Resolve(peerName, cloud, int.MaxValue);
		}

		public PeerNameRecordCollection Resolve(PeerName peerName, int maxRecords)
		{
			return Resolve(peerName, Cloud.Available, maxRecords);
		}

		[SecurityCritical]
		public unsafe PeerNameRecordCollection Resolve(PeerName peerName, Cloud cloud, int maxRecords)
		{
			if (peerName == null)
			{
				throw new ArgumentNullException(SR.GetString("Pnrp_PeerNameCantBeNull"), "peerName");
			}
			if (maxRecords <= 0)
			{
				throw new ArgumentOutOfRangeException("maxRecords", SR.GetString("Pnrp_MaxRecordsParameterMustBeGreaterThanZero"));
			}
			if (cloud == null)
			{
				cloud = Cloud.Available;
			}
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			UnsafeP2PNativeMethods.PnrpStartup();
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Sync Resolve called with PeerName: {0}, Cloud: {1}, MaxRecords {2}", peerName, cloud, maxRecords);
			string internalName = cloud.InternalName;
			uint pcEndPoints = (uint)maxRecords;
			SafePeerData pEndPoints;
			int num = UnsafeP2PNativeMethods.PeerPnrpResolve(peerName.ToString(), internalName, ref pcEndPoints, out pEndPoints);
			if (num != 0)
			{
				throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotStartNameResolution"), num);
			}
			PeerNameRecordCollection peerNameRecordCollection = new PeerNameRecordCollection();
			if (pcEndPoints != 0)
			{
				try
				{
					IntPtr value = pEndPoints.DangerousGetHandle();
					PEER_PNRP_ENDPOINT_INFO* ptr = (PEER_PNRP_ENDPOINT_INFO*)(void*)value;
					for (int i = 0; i < pcEndPoints; i++)
					{
						PeerNameRecord peerNameRecord = new PeerNameRecord();
						PEER_PNRP_ENDPOINT_INFO* ptr2 = ptr + i;
						peerNameRecord.PeerName = new PeerName(Marshal.PtrToStringUni(ptr2->pwszPeerName));
						string text = Marshal.PtrToStringUni(ptr2->pwszComment);
						if (text != null && text.Length > 0)
						{
							peerNameRecord.Comment = text;
						}
						if (ptr2->payLoad.cbPayload != 0)
						{
							peerNameRecord.Data = new byte[ptr2->payLoad.cbPayload];
							Marshal.Copy(ptr2->payLoad.pbPayload, peerNameRecord.Data, 0, (int)ptr2->payLoad.cbPayload);
						}
						IntPtr intPtr = ptr2->ArrayOfSOCKADDRIN6Pointers;
						for (uint num2 = 0u; num2 < ptr2->cAddresses; num2++)
						{
							IntPtr source = Marshal.ReadIntPtr(intPtr);
							byte[] array = new byte[2];
							Marshal.Copy(source, array, 0, 2);
							int num3 = 0;
							num3 = array[0] + (array[1] << 8);
							byte[] array2 = new byte[(num3 == 2) ? 16 : 28];
							Marshal.Copy(source, array2, 0, array2.Length);
							IPEndPoint item = SystemNetHelpers.IPEndPointFromSOCKADDRBuffer(array2);
							peerNameRecord.EndPointCollection.Add(item);
							intPtr = (IntPtr)((long)intPtr + Marshal.SizeOf(typeof(IntPtr)));
						}
						peerNameRecord.TracePeerNameRecord();
						peerNameRecordCollection.Add(peerNameRecord);
					}
				}
				finally
				{
					pEndPoints.Dispose();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Sync Resolve returnig with PeerNameRecord count :{0}", peerNameRecordCollection.Count);
			return peerNameRecordCollection;
		}

		[HostProtection(SecurityAction.LinkDemand, ExternalThreading = true)]
		public void ResolveAsync(PeerName peerName, object userState)
		{
			ResolveAsync(peerName, Cloud.Available, int.MaxValue, userState);
		}

		[HostProtection(SecurityAction.LinkDemand, ExternalThreading = true)]
		public void ResolveAsync(PeerName peerName, Cloud cloud, object userState)
		{
			ResolveAsync(peerName, cloud, int.MaxValue, userState);
		}

		[HostProtection(SecurityAction.LinkDemand, ExternalThreading = true)]
		public void ResolveAsync(PeerName peerName, int maxRecords, object userState)
		{
			ResolveAsync(peerName, Cloud.Available, maxRecords, userState);
		}

		[SecurityCritical]
		[HostProtection(SecurityAction.LinkDemand, ExternalThreading = true)]
		public void ResolveAsync(PeerName peerName, Cloud cloud, int maxRecords, object userState)
		{
			if (peerName == null)
			{
				throw new ArgumentNullException(SR.GetString("Pnrp_PeerNameCantBeNull"), "peerName");
			}
			if (cloud == null)
			{
				cloud = Cloud.Available;
			}
			if (maxRecords <= 0)
			{
				throw new ArgumentOutOfRangeException("maxRecords", SR.GetString("Pnrp_MaxRecordsParameterMustBeGreaterThanZero"));
			}
			if (this.m_ResolveCompleted == null)
			{
				throw new PeerToPeerException(SR.GetString("Pnrp_AtleastOneEvenHandlerNeeded"));
			}
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			UnsafeP2PNativeMethods.PnrpStartup();
			if (userState == null)
			{
				throw new ArgumentNullException(SR.GetString("NullUserToken"), "userState");
			}
			PeerNameResolverHelper peerNameResolverHelper = null;
			int newTraceEventId = NewTraceEventId;
			lock (m_PeerNameResolverHelperListLock)
			{
				if (m_PeerNameResolverHelperList.ContainsKey(userState))
				{
					throw new ArgumentException(SR.GetString("DuplicateUserToken"));
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, newTraceEventId, "PeerNameResolverHelper is being created with TraceEventId {0}", newTraceEventId);
				peerNameResolverHelper = new PeerNameResolverHelper(peerName, cloud, maxRecords, userState, this, newTraceEventId);
				m_PeerNameResolverHelperList[userState] = peerNameResolverHelper;
			}
			try
			{
				peerNameResolverHelper.StartAsyncResolve();
			}
			catch
			{
				lock (m_PeerNameResolverHelperListLock)
				{
					m_PeerNameResolverHelperList.Remove(userState);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, newTraceEventId, "Removing userState token from pending list {0}", userState.GetHashCode());
				}
				throw;
			}
		}

		protected void OnResolveProgressChanged(ResolveProgressChangedEventArgs e)
		{
			if (this.m_ResolveProgressChanged != null)
			{
				this.m_ResolveProgressChanged(this, e);
			}
		}

		private void ResolveProgressChangedWaitCallback(object operationState)
		{
			OnResolveProgressChanged((ResolveProgressChangedEventArgs)operationState);
		}

		internal void PrepareToRaiseProgressChangedEvent(AsyncOperation asyncOP, ResolveProgressChangedEventArgs args)
		{
			asyncOP.Post(OnResolveProgressChangedDelegate, args);
		}

		protected void OnResolveCompleted(ResolveCompletedEventArgs e)
		{
			if (this.m_ResolveCompleted != null)
			{
				this.m_ResolveCompleted(this, e);
			}
		}

		private void ResolveCompletedWaitCallback(object operationState)
		{
			OnResolveCompleted((ResolveCompletedEventArgs)operationState);
		}

		internal void PrepareToRaiseCompletedEvent(AsyncOperation asyncOP, ResolveCompletedEventArgs args)
		{
			asyncOP.PostOperationCompleted(OnResolveCompletedDelegate, args);
			lock (m_PeerNameResolverHelperListLock)
			{
				PeerNameResolverHelper peerNameResolverHelper = m_PeerNameResolverHelperList[args.UserState];
				if (peerNameResolverHelper == null)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Critical, 0, "userState for which we are about to call Completed event does not exist in the pending async list");
					return;
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, peerNameResolverHelper.TraceEventId, "userState {0} is being removed from the pending async list", args.UserState.GetHashCode());
				m_PeerNameResolverHelperList.Remove(args.UserState);
			}
		}

		[SecurityCritical]
		public void ResolveAsyncCancel(object userState)
		{
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			if (userState == null)
			{
				return;
			}
			PeerNameResolverHelper value;
			lock (m_PeerNameResolverHelperListLock)
			{
				if (!m_PeerNameResolverHelperList.TryGetValue(userState, out value))
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Warning, 0, "ResolveAsyncCancel called with a userState token that is not in the pending async list - returning");
					return;
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, value.TraceEventId, "Proceeding to cancel the pending async");
			value.CancelAsync();
		}
	}
}
