#define TRACE
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security;
using System.Threading;

namespace System.Net.PeerToPeer
{
	internal class PeerNameResolverHelper : IDisposable
	{
		private const uint FACILITY_P2P = 99u;

		private const uint NO_MORE_RECORDS = 16387u;

		private const int PEER_E_NO_MORE = -2140979197;

		internal object m_userState;

		internal SafePeerNameEndResolve m_SafePeerNameEndResolve;

		internal AutoResetEvent m_EndPointInfoAvailableEvent = new AutoResetEvent(initialState: false);

		internal RegisteredWaitHandle m_RegisteredWaitHandle;

		internal PeerName m_PeerName;

		internal Cloud m_Cloud;

		internal int m_MaxRecords;

		internal bool m_Disposed;

		internal bool m_CompletedOrException;

		internal bool m_Cancelled;

		private PeerNameRecordCollection m_PeerNameRecordCollection = new PeerNameRecordCollection();

		private AsyncOperation m_AsyncOp;

		private WeakReference m_PeerNameResolverWeakReference;

		private object m_Lock = new object();

		private int m_TraceEventId;

		internal int TraceEventId => m_TraceEventId;

		internal PeerNameResolverHelper(PeerName peerName, Cloud cloud, int MaxRecords, object userState, PeerNameResolver parent, int NewTraceEventId)
		{
			m_userState = userState;
			m_PeerName = peerName;
			m_Cloud = cloud;
			m_MaxRecords = MaxRecords;
			m_PeerNameResolverWeakReference = new WeakReference(parent);
			m_TraceEventId = NewTraceEventId;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "New PeerNameResolverHelper created with TraceEventID {0}", m_TraceEventId);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "\tPeerName: {0}, Cloud: {1}, MaxRecords: {2}, userState {3}, ParentReference {4}", m_PeerName, m_Cloud, m_MaxRecords, userState.GetHashCode(), m_PeerNameResolverWeakReference.Target.GetHashCode());
		}

		[SecurityCritical]
		internal void StartAsyncResolve()
		{
			if (m_Disposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
			m_RegisteredWaitHandle = ThreadPool.RegisterWaitForSingleObject(m_EndPointInfoAvailableEvent, EndPointInfoAvailableCallback, null, -1, executeOnlyOnce: false);
			int num = UnsafeP2PNativeMethods.PeerPnrpStartResolve(m_PeerName.ToString(), m_Cloud.InternalName, (uint)m_MaxRecords, m_EndPointInfoAvailableEvent.SafeWaitHandle, out m_SafePeerNameEndResolve);
			if (num != 0)
			{
				if (!m_SafePeerNameEndResolve.IsInvalid && !m_SafePeerNameEndResolve.IsClosed)
				{
					m_SafePeerNameEndResolve.Dispose();
				}
				m_RegisteredWaitHandle.Unregister(null);
				m_RegisteredWaitHandle = null;
				PeerToPeerException ex = PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotStartNameResolution"), num);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, m_TraceEventId, "Exception occurred while starting async resolve");
				throw ex;
			}
			m_AsyncOp = AsyncOperationManager.CreateOperation(m_userState);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Successfully started the async resolve. The native handle is {0}", m_SafePeerNameEndResolve.DangerousGetHandle());
		}

		[SecurityCritical]
		public unsafe void EndPointInfoAvailableCallback(object state, bool timedOut)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "EndPointInfoAvailableCallback called");
			PeerNameRecord peerNameRecord = null;
			int num = 0;
			PeerNameResolver peerNameResolver = null;
			if (m_Cancelled)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Detected that the async operation is already canceled  - before entering the lock");
				return;
			}
			lock (m_Lock)
			{
				if (m_Cancelled)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Detected that the async operation is already canceled - after entering the lock");
					return;
				}
				num = UnsafeP2PNativeMethods.PeerPnrpGetEndpoint(m_SafePeerNameEndResolve.DangerousGetHandle(), out SafePeerData pEndPoint);
				if (num == 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Proceeding to retrieve the endpoint information from incremental resolve");
					try
					{
						PEER_PNRP_ENDPOINT_INFO* ptr = (PEER_PNRP_ENDPOINT_INFO*)(void*)pEndPoint.DangerousGetHandle();
						peerNameRecord = new PeerNameRecord();
						peerNameRecord.PeerName = new PeerName(Marshal.PtrToStringUni(ptr->pwszPeerName));
						string text = Marshal.PtrToStringUni(ptr->pwszComment);
						if (text != null && text.Length > 0)
						{
							peerNameRecord.Comment = text;
						}
						if (ptr->payLoad.cbPayload != 0)
						{
							peerNameRecord.Data = new byte[ptr->payLoad.cbPayload];
							Marshal.Copy(ptr->payLoad.pbPayload, peerNameRecord.Data, 0, (int)ptr->payLoad.cbPayload);
						}
						IntPtr intPtr = ptr->ArrayOfSOCKADDRIN6Pointers;
						for (uint num2 = 0u; num2 < ptr->cAddresses; num2++)
						{
							IntPtr source = Marshal.ReadIntPtr(intPtr);
							byte[] array = new byte[2];
							Marshal.Copy(source, array, 0, 2);
							int num3 = 0;
							num3 = array[0] + (array[1] << 8);
							byte[] array2 = new byte[(num3 == 2) ? 16 : 28];
							Marshal.Copy(source, array2, 0, array2.Length);
							IPEndPoint item = SystemNetHelpers.IPEndPointFromSOCKADDRBuffer(array2);
							peerNameRecord.EndPointCollection.Add(item);
							intPtr = (IntPtr)((long)intPtr + Marshal.SizeOf(typeof(IntPtr)));
						}
					}
					finally
					{
						pEndPoint.Dispose();
					}
					peerNameRecord.TracePeerNameRecord();
					m_PeerNameRecordCollection.Add(peerNameRecord);
					ResolveProgressChangedEventArgs args = new ResolveProgressChangedEventArgs(peerNameRecord, m_AsyncOp.UserSuppliedState);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Proceeding to call progress changed event callback");
					(m_PeerNameResolverWeakReference.Target as PeerNameResolver)?.PrepareToRaiseProgressChangedEvent(m_AsyncOp, args);
					return;
				}
				if (num == -2140979197)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Native API returned that there are no more records - resolve completed successfully");
				}
				m_CompletedOrException = true;
				m_SafePeerNameEndResolve.Dispose();
			}
			ResolveCompletedEventArgs args2;
			if (num == -2140979197)
			{
				args2 = new ResolveCompletedEventArgs(m_PeerNameRecordCollection, null, canceled: false, m_AsyncOp.UserSuppliedState);
			}
			else
			{
				PeerToPeerException error = PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_ExceptionWhileResolvingAPeerName"), num);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Exception occurred when the native API is called to harvest an incremental resolve notification");
				args2 = new ResolveCompletedEventArgs(null, error, canceled: false, m_AsyncOp.UserSuppliedState);
			}
			peerNameResolver = (m_PeerNameResolverWeakReference.Target as PeerNameResolver);
			if (peerNameResolver != null)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "Proceeding to call the ResolveCompleted callback");
				peerNameResolver.PrepareToRaiseCompletedEvent(m_AsyncOp, args2);
			}
		}

		[SecurityCritical]
		public void ContineCancelCallback(object state)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "ContineCancelCallback called");
			try
			{
				if (m_CompletedOrException)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "ContinueCancelCallback detected (before acquiring lock) that another thread has already called completed event - so returning without calling cancel");
					return;
				}
				lock (m_Lock)
				{
					if (m_Cancelled)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "ContinueCancelCallback detected (after acquiring lock) that cancel has already been called");
						return;
					}
					if (m_CompletedOrException)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "ContinueCancelCallback detected (after acquiring lock) that another thread has already called completed event - so returning without calling cancel");
						return;
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, m_TraceEventId, "ContinueCancelCallback is proceeding to close the handle and call the Completed callback with Cancelled = true");
					m_Cancelled = true;
					m_SafePeerNameEndResolve.Dispose();
				}
				PeerNameResolver peerNameResolver = m_PeerNameResolverWeakReference.Target as PeerNameResolver;
				if (peerNameResolver != null)
				{
					ResolveCompletedEventArgs args = new ResolveCompletedEventArgs(null, null, canceled: true, m_AsyncOp.UserSuppliedState);
					peerNameResolver.PrepareToRaiseCompletedEvent(m_AsyncOp, args);
				}
			}
			catch
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Critical, m_TraceEventId, "Exception while cancelling the call ");
				throw;
			}
		}

		[SecurityCritical]
		public void CancelAsync()
		{
			ThreadPool.QueueUserWorkItem(ContineCancelCallback);
		}

		[SecurityCritical]
		public void Dispose()
		{
			Dispose(disposing: true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		public void Dispose(bool disposing)
		{
			if (!m_Disposed)
			{
				if (!m_SafePeerNameEndResolve.IsInvalid)
				{
					m_SafePeerNameEndResolve.Dispose();
				}
				if (m_RegisteredWaitHandle != null)
				{
					m_RegisteredWaitHandle.Unregister(null);
				}
				m_RegisteredWaitHandle = null;
				m_EndPointInfoAvailableEvent.Close();
			}
			m_Disposed = true;
		}
	}
}
