using System.Security;
using System.Security.Permissions;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public sealed class PnrpPermission : CodeAccessPermission, IUnrestrictedPermission
	{
		private bool m_noRestriction;

		internal static readonly PnrpPermission UnrestrictedPnrpPermission = new PnrpPermission(PermissionState.Unrestricted);

		public PnrpPermission(PermissionState state)
		{
			m_noRestriction = (state == PermissionState.Unrestricted);
		}

		internal PnrpPermission(bool free)
		{
			m_noRestriction = free;
		}

		public bool IsUnrestricted()
		{
			return m_noRestriction;
		}

		public override IPermission Copy()
		{
			return new PnrpPermission(m_noRestriction);
		}

		public override IPermission Union(IPermission target)
		{
			if (target == null)
			{
				return Copy();
			}
			PnrpPermission pnrpPermission = target as PnrpPermission;
			if (pnrpPermission == null)
			{
				throw new ArgumentException(SR.GetString("PnrpPermission_CantUnionWithNonPnrpPermission"), "target");
			}
			return new PnrpPermission(m_noRestriction || pnrpPermission.m_noRestriction);
		}

		public override IPermission Intersect(IPermission target)
		{
			if (target == null)
			{
				return null;
			}
			PnrpPermission pnrpPermission = target as PnrpPermission;
			if (pnrpPermission == null)
			{
				throw new ArgumentException(SR.GetString("PnrpPermission_CantIntersectWithNonPnrpPermission"), "target");
			}
			if (m_noRestriction && pnrpPermission.m_noRestriction)
			{
				return new PnrpPermission(free: true);
			}
			return null;
		}

		public override bool IsSubsetOf(IPermission target)
		{
			if (target == null)
			{
				return !m_noRestriction;
			}
			PnrpPermission pnrpPermission = target as PnrpPermission;
			if (pnrpPermission == null)
			{
				throw new ArgumentException(SR.GetString("PnrpPermission_TargetNotAPnrpPermission"), "target");
			}
			return !m_noRestriction || pnrpPermission.m_noRestriction;
		}

		public override void FromXml(SecurityElement e)
		{
			if (e == null)
			{
				throw new ArgumentNullException(SR.GetString("InvalidSecurityElem"));
			}
			if (!e.Tag.Equals("IPermission"))
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElem"), "securityElement");
			}
			string text = e.Attribute("class");
			if (text == null)
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElem"), "securityElement");
			}
			if (text.IndexOf(GetType().FullName, StringComparison.Ordinal) < 0)
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElem"), "securityElement");
			}
			string text2 = e.Attribute("Unrestricted");
			m_noRestriction = (text2 != null && 0 == string.Compare(text2, "true", StringComparison.OrdinalIgnoreCase));
		}

		public override SecurityElement ToXml()
		{
			SecurityElement securityElement = new SecurityElement("IPermission");
			securityElement.AddAttribute("class", GetType().FullName + ", " + GetType().Module.Assembly.FullName.Replace('"', '\''));
			securityElement.AddAttribute("version", "1");
			if (m_noRestriction)
			{
				securityElement.AddAttribute("Unrestricted", "true");
			}
			return securityElement;
		}
	}
}
