#define TRACE
using System.Diagnostics;
using System.Globalization;
using System.Security;
using System.Text;

namespace System.Net
{
	internal static class Logging
	{
		private static P2PTraceSource s_P2PTraceSource;

		internal static P2PTraceSource P2PTraceSource => s_P2PTraceSource;

		[SecurityCritical]
		static Logging()
		{
			s_P2PTraceSource = new P2PTraceSource();
			if (s_P2PTraceSource.Switch.ShouldTrace(TraceEventType.Critical))
			{
				AppDomain currentDomain = AppDomain.CurrentDomain;
				currentDomain.ProcessExit += ProcessExitEventHandler;
				currentDomain.DomainUnload += DomainUnloadEventHandler;
			}
		}

		[SecurityCritical]
		private static void DomainUnloadEventHandler(object sender, EventArgs e)
		{
			Close();
		}

		[SecurityCritical]
		private static void ProcessExitEventHandler(object sender, EventArgs e)
		{
			Close();
		}

		[SecurityCritical]
		private static void Close()
		{
			s_P2PTraceSource.Close();
		}

		internal static void Enter(TraceSource source, string method)
		{
			if (source.Switch.ShouldTrace(TraceEventType.Verbose))
			{
				source.TraceEvent(TraceEventType.Verbose, 0, "Entering --> " + method);
			}
		}

		internal static void Leave(TraceSource source, string message)
		{
			if (source.Switch.ShouldTrace(TraceEventType.Verbose))
			{
				source.TraceEvent(TraceEventType.Verbose, 0, "Leaving <-- " + message);
			}
		}

		internal static void DumpData(TraceSource source, TraceEventType eventType, int maxDataSize, byte[] buffer, int offset, int length)
		{
			if (buffer == null || buffer.Length == 0 || offset > buffer.Length)
			{
				return;
			}
			if (length > maxDataSize)
			{
				source.TraceEvent(eventType, 0, "dumping {0} of {1} bytes", maxDataSize, length);
				length = maxDataSize;
			}
			if (length < 0 || length > buffer.Length - offset)
			{
				length = buffer.Length - offset;
			}
			do
			{
				int num = Math.Min(length, 16);
				StringBuilder stringBuilder = new StringBuilder();
				stringBuilder.Append(string.Format(CultureInfo.CurrentCulture, "{0:X8} : ", offset));
				for (int i = 0; i < num; i++)
				{
					stringBuilder.Append(string.Format(CultureInfo.CurrentCulture, "{0:X2}", buffer[offset + i]) + ((i == 7) ? '-' : ' '));
				}
				for (int i = num; i < 16; i++)
				{
					stringBuilder.Append("   ");
				}
				stringBuilder.Append(": ");
				for (int i = 0; i < num; i++)
				{
					stringBuilder.Append((char)((buffer[offset + i] < 32 || buffer[offset + i] > 126) ? 46 : buffer[offset + i]));
				}
				source.TraceEvent(eventType, 0, stringBuilder.ToString());
				offset += num;
				length -= num;
			}
			while (length > 0);
		}
	}
}
