﻿using System;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Text;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public class Cloud : ISerializable, IEquatable<Cloud>
	{
		static Cloud()
		{
			if (!PeerToPeerOSHelper.SupportsP2P)
			{
				throw new PlatformNotSupportedException(SR.GetString("P2P_NotAvailable"));
			}
		}

		internal Cloud(string name, PnrpScope pnrpScope, int scopeId)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Creating cloud with Name: {0}, PnrpScope: {1}, ScopeID: {2}", new object[]
			{
				name,
				pnrpScope,
				scopeId
			});
			this.m_CloudName = name;
			this.m_PnrpScope = pnrpScope;
			this.m_ScopeId = scopeId;
		}

		public string Name
		{
			get
			{
				string result;
				if (this == Cloud.AllLinkLocal || this == Cloud.Available)
				{
					result = null;
				}
				else
				{
					result = this.m_CloudName;
				}
				return result;
			}
		}

		internal string InternalName
		{
			get
			{
				string result;
				if (this == Cloud.AllLinkLocal)
				{
					result = "PEER_PNRP_ALL_LINKS";
				}
				else if (this == Cloud.Available)
				{
					result = null;
				}
				else
				{
					result = this.m_CloudName;
				}
				return result;
			}
		}

		public PnrpScope Scope
		{
			get
			{
				return this.m_PnrpScope;
			}
		}

		public int ScopeId
		{
			get
			{
				return this.m_ScopeId;
			}
		}

		public static Cloud Global
		{
			get
			{
				CloudCollection cloudCollection = null;
				Cloud result = null;
				Cloud.GetCloudOrClouds(null, true, out cloudCollection, out result);
				return result;
			}
		}

		public static Cloud GetCloudByName(string cloudName)
		{
			if (cloudName == null || cloudName.Length == 0)
			{
				throw new ArgumentException(SR.GetString("Pnrp_CloudNameCantBeNull"), "cloudName");
			}
			CloudCollection cloudCollection = null;
			Cloud result = null;
			Cloud.GetCloudOrClouds(cloudName, false, out cloudCollection, out result);
			return result;
		}

		public static CloudCollection GetAvailableClouds()
		{
			CloudCollection result = null;
			Cloud cloud = null;
			Cloud.GetCloudOrClouds(null, false, out result, out cloud);
			return result;
		}

		[SecurityTreatAsSafe]
		[SecurityCritical]
		private static void GetCloudOrClouds(string cloudName, bool bGlobalCloudOnly, out CloudCollection clouds, out Cloud cloud)
		{
			cloud = null;
			clouds = null;
			Logging.Enter(Logging.P2PTraceSource, "Cloud::GetCloudOrClouds()");
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			uint num = 0U;
			SafePeerData safePeerData = null;
			if (cloudName == null)
			{
				clouds = new CloudCollection();
			}
			try
			{
				UnsafeP2PNativeMethods.PnrpStartup();
				int num2 = UnsafeP2PNativeMethods.PeerPnrpGetCloudInfo(out num, out safePeerData);
				if (num2 != 0)
				{
					throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotEnumerateClouds"), num2);
				}
				if (num != 0U)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Number of clouds returned {0}", new object[]
					{
						num
					});
					IntPtr intPtr = safePeerData.DangerousGetHandle();
					for (ulong num3 = 0UL; num3 < (ulong)num; num3 += 1UL)
					{
						PEER_PNRP_CLOUD_INFO peer_PNRP_CLOUD_INFO = (PEER_PNRP_CLOUD_INFO)Marshal.PtrToStructure(intPtr, typeof(PEER_PNRP_CLOUD_INFO));
						string text = Marshal.PtrToStringUni(peer_PNRP_CLOUD_INFO.pwzCloudName);
						intPtr = (IntPtr)((long)intPtr + (long)Marshal.SizeOf(typeof(PEER_PNRP_CLOUD_INFO)));
						Cloud cloud2 = new Cloud(text, (PnrpScope)peer_PNRP_CLOUD_INFO.dwScope, (int)peer_PNRP_CLOUD_INFO.dwScopeId);
						if (cloudName == null && !bGlobalCloudOnly)
						{
							clouds.Add(cloud2);
						}
						else
						{
							if (cloudName != null && cloudName == text)
							{
								cloud = cloud2;
								break;
							}
							if (bGlobalCloudOnly && cloud2.Scope == PnrpScope.Global)
							{
								cloud = cloud2;
								break;
							}
						}
					}
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Warning, 0, "No Clouds returned from the native call");
				}
			}
			finally
			{
				if (safePeerData != null)
				{
					safePeerData.Dispose();
				}
			}
			if (cloudName != null && cloud == null)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "The specific cloud name {0} asked for is not found", new object[]
				{
					cloudName
				});
			}
			Logging.Leave(Logging.P2PTraceSource, "Cloud::GetCloudOrClouds()");
		}

		public bool Equals(Cloud other)
		{
			return other != null && (other.Name == this.Name && other.Scope == this.Scope) && other.ScopeId == this.ScopeId;
		}

		public override bool Equals(object obj)
		{
			bool result;
			if (obj == null)
			{
				result = false;
			}
			else
			{
				Cloud cloud = obj as Cloud;
				result = (cloud != null && this.Equals(cloud));
			}
			return result;
		}

		public override int GetHashCode()
		{
			return this.m_CloudName.GetHashCode();
		}

		public override string ToString()
		{
			StringBuilder stringBuilder = new StringBuilder();
			stringBuilder.Append("Cloud Name:");
			stringBuilder.Append(this.Name);
			stringBuilder.Append(" Scope:");
			stringBuilder.Append(this.Scope);
			stringBuilder.Append(" ScopeId:");
			stringBuilder.Append(this.ScopeId);
			return stringBuilder.ToString();
		}

		protected Cloud(SerializationInfo info, StreamingContext context)
		{
			this.m_CloudName = info.GetString("_CloudName");
			this.m_PnrpScope = (PnrpScope)info.GetValue("_CloudScope", typeof(PnrpScope));
			this.m_ScopeId = info.GetInt32("_CloudScopeId");
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_CloudName", this.Name);
			info.AddValue("_CloudScope", this.m_PnrpScope);
			info.AddValue("_CloudScopeId", this.m_ScopeId);
		}

		private const string PEER_PNRP_ALL_LINK_CLOUDS = "PEER_PNRP_ALL_LINKS";

		private string m_CloudName;

		private PnrpScope m_PnrpScope;

		private int m_ScopeId;

		public static readonly Cloud Available = new Cloud("AllAvailable", PnrpScope.All, -1);

		public static readonly Cloud AllLinkLocal = new Cloud("AllLinkLocal", PnrpScope.LinkLocal, -1);
	}
}
