﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	public sealed class ContactManager : IDisposable
	{
		[SecurityCritical]
		internal ContactManager()
		{
			CollaborationHelperFunctions.Initialize();
			this.OnCreateContactCompletedDelegate = new SendOrPostCallback(this.CreateContactCompletedWaitCallback);
		}

		public static PeerContact LocalContact
		{
			[SecurityCritical]
			get
			{
				SafeCollabData safeCollabData = null;
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Get MyContact called.");
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				MyContact result;
				try
				{
					int num = UnsafeCollabNativeMethods.PeerCollabGetContact(null, out safeCollabData);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetContact returned with errorcode {0}", new object[]
						{
							num
						});
						return null;
					}
					PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_CONTACT));
					result = (MyContact)CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc, true);
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Get MyContact successful.");
				return result;
			}
		}

		[Description("SynchronizingObject")]
		[DefaultValue(null)]
		[Browsable(false)]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_synchronizingObject;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_synchronizingObject = value;
			}
		}

		private event EventHandler<SubscriptionListChangedEventArgs> m_subscriptionListChanged;

		public event EventHandler<SubscriptionListChangedEventArgs> SubscriptionListChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddSubscriptionListChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemoveSubscriptionListChanged(value);
			}
		}

		private object LockSubLstChangedEvent
		{
			get
			{
				if (this.m_lockSubLstChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockSubLstChangedEvent, value, null);
				}
				return this.m_lockSubLstChangedEvent;
			}
		}

		[SecurityCritical]
		private void AddSubscriptionListChanged(EventHandler<SubscriptionListChangedEventArgs> callback)
		{
			lock (this.LockSubLstChangedEvent)
			{
				if (this.m_subscriptionListChanged == null)
				{
					this.m_subLstChangedEvent = new AutoResetEvent(false);
					this.m_regSubLstChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_subLstChangedEvent, new WaitOrTimerCallback(this.SubListChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.WatchListChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_subLstChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeSubLstChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_SubListChangedRegFailed"), num);
					}
				}
				this.m_subscriptionListChanged = (EventHandler<SubscriptionListChangedEventArgs>)Delegate.Combine(this.m_subscriptionListChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddSubscriptionListChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveSubscriptionListChanged(EventHandler<SubscriptionListChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveSubscriptionListChanged() called.");
			lock (this.LockSubLstChangedEvent)
			{
				this.m_subscriptionListChanged = (EventHandler<SubscriptionListChangedEventArgs>)Delegate.Remove(this.m_subscriptionListChanged, callback);
				if (this.m_subscriptionListChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref this.m_regSubLstChangedWaitHandle, ref this.m_safeSubLstChangedEvent, ref this.m_subLstChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean SubscriptionListChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveSubscriptionListChanged() successful.");
		}

		[SecurityCritical]
		private void SubListChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "SubListChangedCallback() called.");
			for (;;)
			{
				SubscriptionListChangedEventArgs subscriptionListChangedEventArgs = null;
				try
				{
					lock (this.LockSubLstChangedEvent)
					{
						if (this.m_safeSubLstChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeSubLstChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetSubListChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.WatchListChanged)
					{
						PEER_EVENT_WATCHLIST_CHANGED_DATA watchListChangedData = peer_COLLAB_EVENT_DATA.watchListChangedData;
						PeerContact peerContact = null;
						if (watchListChangedData.pContact != IntPtr.Zero)
						{
							PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(watchListChangedData.pContact, typeof(PEER_CONTACT));
							peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
						}
						subscriptionListChangedEventArgs = new SubscriptionListChangedEventArgs(null, peerContact, watchListChangedData.changeType);
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				EventHandler<SubscriptionListChangedEventArgs> subscriptionListChanged = this.m_subscriptionListChanged;
				if (subscriptionListChangedEventArgs != null && subscriptionListChanged != null)
				{
					if (this.SynchronizingObject != null && this.SynchronizingObject.InvokeRequired)
					{
						this.SynchronizingObject.BeginInvoke(subscriptionListChanged, new object[]
						{
							this,
							subscriptionListChangedEventArgs
						});
					}
					else
					{
						subscriptionListChanged(this, subscriptionListChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the subscription list changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving SubListChangedCallback().");
		}

		private event EventHandler<NameChangedEventArgs> m_nameChanged;

		public event EventHandler<NameChangedEventArgs> NameChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddNameChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemoveNameChanged(value);
			}
		}

		private object LockNameChangedEvent
		{
			get
			{
				if (this.m_lockNameChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockNameChangedEvent, value, null);
				}
				return this.m_lockNameChangedEvent;
			}
		}

		[SecurityCritical]
		private void AddNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			lock (this.LockNameChangedEvent)
			{
				if (this.m_nameChanged == null)
				{
					this.m_nameChangedEvent = new AutoResetEvent(false);
					this.m_regNameChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_nameChangedEvent, new WaitOrTimerCallback(this.NameChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.EndPointChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_nameChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeNameChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_NameChangedRegFailed"), num);
					}
				}
				this.m_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Combine(this.m_nameChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddNameChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() called.");
			lock (this.LockNameChangedEvent)
			{
				this.m_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Remove(this.m_nameChanged, callback);
				if (this.m_nameChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref this.m_regNameChangedWaitHandle, ref this.m_safeNameChangedEvent, ref this.m_nameChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean NameChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() successful.");
		}

		[SecurityCritical]
		private void NameChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "NameChangedCallback() called.");
			for (;;)
			{
				NameChangedEventArgs nameChangedEventArgs = null;
				try
				{
					lock (this.LockNameChangedEvent)
					{
						if (this.m_safeNameChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeNameChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetNameChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointChanged)
					{
						PEER_EVENT_ENDPOINT_CHANGED_DATA endpointChangedData = peer_COLLAB_EVENT_DATA.endpointChangedData;
						PeerContact peerContact = null;
						PeerEndPoint peerEndPoint = null;
						string name = null;
						if (endpointChangedData.pContact != IntPtr.Zero)
						{
							PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(endpointChangedData.pContact, typeof(PEER_CONTACT));
							peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
						}
						if (endpointChangedData.pEndPoint != IntPtr.Zero)
						{
							PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(endpointChangedData.pEndPoint, typeof(PEER_ENDPOINT));
							peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							name = peerEndPoint.Name;
						}
						nameChangedEventArgs = new NameChangedEventArgs(peerEndPoint, peerContact, name);
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				EventHandler<NameChangedEventArgs> nameChanged = this.m_nameChanged;
				if (nameChangedEventArgs != null && nameChanged != null)
				{
					if (this.SynchronizingObject != null && this.SynchronizingObject.InvokeRequired)
					{
						this.SynchronizingObject.BeginInvoke(nameChanged, new object[]
						{
							this,
							nameChangedEventArgs
						});
					}
					else
					{
						nameChanged(this, nameChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the name changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving NameChangedCallback().");
		}

		private event EventHandler<PresenceChangedEventArgs> m_presenceChanged;

		public event EventHandler<PresenceChangedEventArgs> PresenceChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddPresenceChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemovePresenceChanged(value);
			}
		}

		private object LockPresenceChangedEvent
		{
			get
			{
				if (this.m_lockPresenceChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockPresenceChangedEvent, value, null);
				}
				return this.m_lockPresenceChangedEvent;
			}
		}

		[SecurityCritical]
		private void AddPresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			lock (this.LockPresenceChangedEvent)
			{
				if (this.m_presenceChanged == null)
				{
					this.m_presenceChangedEvent = new AutoResetEvent(false);
					this.m_regPresenceChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_presenceChangedEvent, new WaitOrTimerCallback(this.PresenceChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.EndPointPresenceChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_presenceChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safePresenceChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_PresenceChangedRegFailed"), num);
					}
				}
				this.m_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Combine(this.m_presenceChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddPresenceChanged() successful.");
		}

		[SecurityCritical]
		private void RemovePresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() called.");
			lock (this.LockPresenceChangedEvent)
			{
				this.m_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Remove(this.m_presenceChanged, callback);
				if (this.m_presenceChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref this.m_regPresenceChangedWaitHandle, ref this.m_safePresenceChangedEvent, ref this.m_presenceChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean PresenceChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() successful.");
		}

		[SecurityCritical]
		private void PresenceChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PresenceChangedCallback() called.");
			for (;;)
			{
				PresenceChangedEventArgs presenceChangedEventArgs = null;
				try
				{
					lock (this.LockPresenceChangedEvent)
					{
						if (this.m_safePresenceChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safePresenceChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPresenceChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointPresenceChanged)
					{
						PEER_EVENT_PRESENCE_CHANGED_DATA presenceChangedData = peer_COLLAB_EVENT_DATA.presenceChangedData;
						PeerPresenceInfo peerPresenceInfo = null;
						if (presenceChangedData.pPresenceInfo != IntPtr.Zero)
						{
							PEER_PRESENCE_INFO peer_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(presenceChangedData.pPresenceInfo, typeof(PEER_PRESENCE_INFO));
							peerPresenceInfo = new PeerPresenceInfo();
							peerPresenceInfo.PresenceStatus = peer_PRESENCE_INFO.status;
							peerPresenceInfo.DescriptiveText = peer_PRESENCE_INFO.descText;
						}
						PeerContact peerContact = null;
						PeerEndPoint peerEndPoint = null;
						if (presenceChangedData.pContact != IntPtr.Zero)
						{
							PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(presenceChangedData.pContact, typeof(PEER_CONTACT));
							peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
						}
						if (presenceChangedData.pEndPoint != IntPtr.Zero)
						{
							PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(presenceChangedData.pEndPoint, typeof(PEER_ENDPOINT));
							peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
						}
						presenceChangedEventArgs = new PresenceChangedEventArgs(peerEndPoint, peerContact, presenceChangedData.changeType, peerPresenceInfo);
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				EventHandler<PresenceChangedEventArgs> presenceChanged = this.m_presenceChanged;
				if (presenceChangedEventArgs != null && presenceChanged != null)
				{
					if (this.SynchronizingObject != null && this.SynchronizingObject.InvokeRequired)
					{
						this.SynchronizingObject.BeginInvoke(presenceChanged, new object[]
						{
							this,
							presenceChangedEventArgs
						});
					}
					else
					{
						presenceChanged(this, presenceChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the presence changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving PresenceChangedCallback().");
		}

		private event EventHandler<ApplicationChangedEventArgs> m_applicationChanged;

		public event EventHandler<ApplicationChangedEventArgs> ApplicationChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddApplicationChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemoveApplicationChanged(value);
			}
		}

		private object LockAppChangedEvent
		{
			get
			{
				if (this.m_lockAppChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockAppChangedEvent, value, null);
				}
				return this.m_lockAppChangedEvent;
			}
		}

		[SecurityCritical]
		private void AddApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() called.");
			lock (this.LockAppChangedEvent)
			{
				if (this.m_applicationChanged == null)
				{
					this.m_appChangedEvent = new AutoResetEvent(false);
					this.m_regAppChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_appChangedEvent, new WaitOrTimerCallback(this.ApplicationChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.EndPointApplicationChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_appChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeAppChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ApplicationChangedRegFailed"), num);
					}
				}
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Combine(this.m_applicationChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() called.");
			lock (this.LockAppChangedEvent)
			{
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Remove(this.m_applicationChanged, callback);
				if (this.m_applicationChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref this.m_regAppChangedWaitHandle, ref this.m_safeAppChangedEvent, ref this.m_appChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() successful.");
		}

		[SecurityCritical]
		private void ApplicationChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ApplicationChangedCallback() called.");
			for (;;)
			{
				ApplicationChangedEventArgs applicationChangedEventArgs = null;
				try
				{
					lock (this.LockAppChangedEvent)
					{
						if (this.m_safeAppChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeAppChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetApplicationChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointApplicationChanged)
					{
						PEER_EVENT_APPLICATION_CHANGED_DATA applicationChangedData = peer_COLLAB_EVENT_DATA.applicationChangedData;
						PEER_APPLICATION pa = (PEER_APPLICATION)Marshal.PtrToStructure(applicationChangedData.pApplication, typeof(PEER_APPLICATION));
						PeerApplication peerApplication = CollaborationHelperFunctions.ConvertPEER_APPLICATIONToPeerApplication(pa);
						PeerContact peerContact = null;
						PeerEndPoint peerEndPoint = null;
						if (applicationChangedData.pContact != IntPtr.Zero)
						{
							PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(applicationChangedData.pContact, typeof(PEER_CONTACT));
							peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
						}
						if (applicationChangedData.pEndPoint != IntPtr.Zero)
						{
							PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(applicationChangedData.pEndPoint, typeof(PEER_ENDPOINT));
							peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
						}
						applicationChangedEventArgs = new ApplicationChangedEventArgs(peerEndPoint, peerContact, applicationChangedData.changeType, peerApplication);
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				EventHandler<ApplicationChangedEventArgs> applicationChanged = this.m_applicationChanged;
				if (applicationChangedEventArgs != null && applicationChanged != null)
				{
					if (this.SynchronizingObject != null && this.SynchronizingObject.InvokeRequired)
					{
						this.SynchronizingObject.BeginInvoke(applicationChanged, new object[]
						{
							this,
							applicationChangedEventArgs
						});
					}
					else
					{
						applicationChanged(this, applicationChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the application changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ApplicationChangedCallback().");
		}

		private event EventHandler<ObjectChangedEventArgs> m_objectChanged;

		public event EventHandler<ObjectChangedEventArgs> ObjectChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddObjectChangedEvent(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemoveObjectChangedEvent(value);
			}
		}

		private object LockObjChangedEvent
		{
			get
			{
				if (this.m_lockObjChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockObjChangedEvent, value, null);
				}
				return this.m_lockObjChangedEvent;
			}
		}

		[SecurityCritical]
		private void AddObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChangedEvent() called.");
			lock (this.LockObjChangedEvent)
			{
				if (this.m_objectChanged == null)
				{
					this.m_objChangedEvent = new AutoResetEvent(false);
					this.m_regObjChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_objChangedEvent, new WaitOrTimerCallback(this.ObjectChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.EndPointObjectChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_objChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeObjChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectChangedRegFailed"), num);
					}
				}
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Combine(this.m_objectChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() called.");
			lock (this.LockObjChangedEvent)
			{
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Remove(this.m_objectChanged, callback);
				if (this.m_objectChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref this.m_regObjChangedWaitHandle, ref this.m_safeObjChangedEvent, ref this.m_objChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() successful.");
		}

		[SecurityCritical]
		private void ObjectChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ObjectChangedCallback() called.");
			for (;;)
			{
				ObjectChangedEventArgs objectChangedEventArgs = null;
				try
				{
					lock (this.LockObjChangedEvent)
					{
						if (this.m_safeObjChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeObjChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointObjectChanged)
					{
						PEER_EVENT_OBJECT_CHANGED_DATA objectChangedData = peer_COLLAB_EVENT_DATA.objectChangedData;
						PEER_OBJECT po = (PEER_OBJECT)Marshal.PtrToStructure(objectChangedData.pObject, typeof(PEER_OBJECT));
						PeerObject peerObject = CollaborationHelperFunctions.ConvertPEER_OBJECTToPeerObject(po);
						PeerContact peerContact = null;
						PeerEndPoint peerEndPoint = null;
						if (objectChangedData.pContact != IntPtr.Zero)
						{
							PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(objectChangedData.pContact, typeof(PEER_CONTACT));
							peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
						}
						if (objectChangedData.pEndPoint != IntPtr.Zero)
						{
							PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(objectChangedData.pEndPoint, typeof(PEER_ENDPOINT));
							peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
						}
						objectChangedEventArgs = new ObjectChangedEventArgs(peerEndPoint, peerContact, objectChangedData.changeType, peerObject);
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				EventHandler<ObjectChangedEventArgs> objectChanged = this.m_objectChanged;
				if (objectChangedEventArgs != null && objectChanged != null)
				{
					if (this.SynchronizingObject != null && this.SynchronizingObject.InvokeRequired)
					{
						this.SynchronizingObject.BeginInvoke(objectChanged, new object[]
						{
							this,
							objectChangedEventArgs
						});
					}
					else
					{
						objectChanged(this, objectChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the object changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ObjectChangedCallback().");
		}

		[SecurityCritical]
		public unsafe PeerContactCollection GetContacts()
		{
			PeerContactCollection peerContactCollection = new PeerContactCollection();
			SafeCollabEnum safeCollabEnum = null;
			uint num = 0U;
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetContacts()");
			try
			{
				int num2 = UnsafeCollabNativeMethods.PeerCollabEnumContacts(out safeCollabEnum);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumContacts returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetContactsFailed"), num2);
				}
				num2 = UnsafeCollabNativeMethods.PeerGetItemCount(safeCollabEnum, ref num);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetContactsFailed"), num2);
				}
				if (num == 0U)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "No contacts found. \nLeaving GetContacts()");
					return peerContactCollection;
				}
				SafeCollabData safeCollabData = null;
				try
				{
					num2 = UnsafeCollabNativeMethods.PeerGetNextItem(safeCollabEnum, ref num, out safeCollabData);
					if (num2 != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetNextItem returned with errorcode {0}", new object[]
						{
							num2
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetContactsFailed"), num2);
					}
					IntPtr value = safeCollabData.DangerousGetHandle();
					IntPtr* ptr = (IntPtr*)((void*)value);
					for (ulong num3 = 0UL; num3 < (ulong)num; num3 += 1UL)
					{
						IntPtr ptr2 = ptr[num3 * (ulong)((long)sizeof(IntPtr)) / (ulong)sizeof(IntPtr)];
						PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(ptr2, typeof(PEER_CONTACT));
						PeerContact item = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
						peerContactCollection.Add(item);
					}
				}
				finally
				{
					safeCollabData.Dispose();
				}
			}
			finally
			{
				safeCollabEnum.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Returning collections with {0} contacts. \nLeaving GetContacts()");
			return peerContactCollection;
		}

		[SecurityCritical]
		public PeerContact GetContact(PeerName peerName)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerName == null)
			{
				throw new ArgumentNullException("peerName");
			}
			SafeCollabData safeCollabData = null;
			PeerContact result = null;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetContact() with peername {0}", new object[]
			{
				peerName.ToString()
			});
			try
			{
				int num = UnsafeCollabNativeMethods.PeerCollabGetContact(peerName.ToString(), out safeCollabData);
				if (num == -2140971007)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Contact not found in Contact Manager");
					throw new PeerToPeerException(SR.GetString("Collab_ContactNotFound"));
				}
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetContact returned with errorcode {0}", new object[]
					{
						num
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetContactFailed"), num);
				}
				if (!safeCollabData.DangerousGetHandle().Equals(IntPtr.Zero))
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Found contact.");
					PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_CONTACT));
					result = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "No contact found.");
				}
			}
			finally
			{
				safeCollabData.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving GetContact()");
			return result;
		}

		[SecurityCritical]
		public PeerContact CreateContact(PeerNearMe peerNearMe)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerNearMe == null)
			{
				throw new ArgumentNullException("peerNearMe");
			}
			if (peerNearMe.PeerEndPoints == null || peerNearMe.PeerEndPoints.Count == 0 || peerNearMe.PeerEndPoints[0].EndPoint == null)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering CreateContact() with peernearme", new object[]
			{
				peerNearMe.ToString()
			});
			PeerEndPoint peerEndPoint = peerNearMe.PeerEndPoints[0];
			PEER_ENDPOINT peer_ENDPOINT = default(PEER_ENDPOINT);
			peer_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gchandle = default(GCHandle);
			if (peerEndPoint.Name != null)
			{
				gchandle = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
				peer_ENDPOINT.pwzEndpointName = gchandle.AddrOfPinnedObject();
			}
			GCHandle gchandle2 = GCHandle.Alloc(peer_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gchandle2.AddrOfPinnedObject();
			string text = null;
			peerNearMe.RefreshData();
			int num = UnsafeCollabNativeMethods.PeerCollabQueryContactData(pcEndpoint, ref text);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetContact returned with errorcode {0}", new object[]
				{
					num
				});
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_CreateContactFailed"), num);
			}
			SafeCollabData safeCollabData = null;
			PeerContact peerContact = null;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabParseContact(text, out safeCollabData);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetContact returned with errorcode {0}", new object[]
					{
						num
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_CreateContactFailed"), num);
				}
				PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_CONTACT));
				peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
				peerContact.JustCreated = true;
				peerContact.ContactXml = text;
			}
			finally
			{
				if (safeCollabData != null)
				{
					safeCollabData.Dispose();
				}
				gchandle.Free();
				gchandle2.Free();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving CreateContact().");
			return peerContact;
		}

		private object CreateContactAsyncListLock
		{
			get
			{
				if (this.m_createContactAsyncListLock == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_createContactAsyncListLock, value, null);
				}
				return this.m_createContactAsyncListLock;
			}
		}

		private event EventHandler<CreateContactCompletedEventArgs> m_createContactCompleted;

		public event EventHandler<CreateContactCompletedEventArgs> CreateContactCompleted
		{
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_createContactCompleted = (EventHandler<CreateContactCompletedEventArgs>)Delegate.Combine(this.m_createContactCompleted, value);
			}
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_createContactCompleted = (EventHandler<CreateContactCompletedEventArgs>)Delegate.Remove(this.m_createContactCompleted, value);
			}
		}

		public void CreateContactAsync(PeerNearMe peerNearMe, object userToken)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerNearMe == null)
			{
				throw new ArgumentNullException("PeerNearMe");
			}
			if (peerNearMe.PeerEndPoints == null || peerNearMe.PeerEndPoints.Count == 0 || peerNearMe.PeerEndPoints[0].EndPoint == null)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering CreateContactAsync() with token {0} and following peernearme", new object[]
				{
					userToken
				});
				peerNearMe.TracePeerNearMe();
			}
			lock (this.CreateContactAsyncListLock)
			{
				if (this.m_createContactAsyncList.ContainsKey(userToken))
				{
					throw new ArgumentException(SR.GetString("DuplicateUserToken"));
				}
				AsyncOperation value = AsyncOperationManager.CreateOperation(userToken);
				this.m_createContactAsyncList[userToken] = value;
			}
			ThreadPool.QueueUserWorkItem(new WaitCallback(this.CreateContactAsyncHelper), new ContactManager.CreateContactAsyncState(peerNearMe, userToken));
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving CreateContactAsync().");
		}

		private void CreateContactAsyncHelper(object state)
		{
			Exception ex = null;
			PeerContact peerContact = null;
			ContactManager.CreateContactAsyncState createContactAsyncState = state as ContactManager.CreateContactAsyncState;
			PeerNearMe peerNearMe = createContactAsyncState.PeerNearMe;
			object userToken = createContactAsyncState.UserToken;
			try
			{
				peerContact = this.CreateContact(peerNearMe);
			}
			catch (ObjectDisposedException ex2)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "CreateContactAsyncHelper caught error {0}", new object[]
				{
					ex2.Message
				});
				ex = ex2;
			}
			catch (PeerToPeerException ex3)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "CreateContactAsyncHelper caught error {0}", new object[]
				{
					ex3.Message
				});
				ex = ex3;
			}
			CreateContactCompletedEventArgs args;
			if (ex == null)
			{
				args = new CreateContactCompletedEventArgs(peerContact, null, false, userToken);
			}
			else
			{
				args = new CreateContactCompletedEventArgs(peerContact, ex, false, userToken);
			}
			this.PrepareToRaiseCreateContactCompletedEvent(this.m_createContactAsyncList[userToken], args);
		}

		private void OnCreateContactCompleted(CreateContactCompletedEventArgs e)
		{
			EventHandler<CreateContactCompletedEventArgs> createContactCompleted = this.m_createContactCompleted;
			if (createContactCompleted != null)
			{
				createContactCompleted(this, e);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the create contact completed event callback.");
			}
		}

		private void CreateContactCompletedWaitCallback(object operationState)
		{
			CreateContactCompletedEventArgs createContactCompletedEventArgs = (CreateContactCompletedEventArgs)operationState;
			this.m_createContactAsyncList.Remove(createContactCompletedEventArgs.UserState);
			this.OnCreateContactCompleted(createContactCompletedEventArgs);
		}

		internal void PrepareToRaiseCreateContactCompletedEvent(AsyncOperation asyncOP, CreateContactCompletedEventArgs args)
		{
			asyncOP.PostOperationCompleted(this.OnCreateContactCompletedDelegate, args);
		}

		[SecurityCritical]
		public void AddContact(PeerContact peerContact)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerContact == null)
			{
				throw new ArgumentNullException("peerContact");
			}
			string pwzContactData = null;
			if (peerContact.ContactXml == null)
			{
				try
				{
					pwzContactData = peerContact.ToXml();
				}
				catch (PeerToPeerException ex)
				{
					throw new PeerToPeerException(SR.GetString("Collab_AddContactFailedNoXml"), ex.InnerException);
				}
			}
			else
			{
				pwzContactData = peerContact.ContactXml;
			}
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddContact() with following peercontact");
				peerContact.TracePeerContact();
			}
			SafeCollabData safeCollabData = null;
			int num = 0;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabAddContact(pwzContactData, out safeCollabData);
			}
			finally
			{
				if (safeCollabData != null)
				{
					safeCollabData.Dispose();
				}
			}
			if (num == -2147024713)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabAddContact returned with errorcode {0}. Contact already exists.", new object[]
				{
					num
				});
				throw new ArgumentException(SR.GetString("Collab_AddContactFailed") + " " + SR.GetString("Collab_ContactExists"));
			}
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabAddContact returned with errorcode {0}", new object[]
				{
					num
				});
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_AddContactFailed"), num);
			}
			peerContact.JustCreated = false;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving AddContact() successfully.");
		}

		public void DeleteContact(PeerContact peerContact)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering DeleteContact().");
			if (peerContact == null)
			{
				throw new ArgumentNullException("peerContact");
			}
			this.DeleteContact(peerContact.PeerName);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving DeleteContact() successfully.");
		}

		[SecurityCritical]
		public void DeleteContact(PeerName peerName)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering DeleteContact().");
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerName == null)
			{
				throw new ArgumentNullException("peerName");
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Peername is {0}", new object[]
			{
				peerName.ToString()
			});
			int num = UnsafeCollabNativeMethods.PeerCollabDeleteContact(peerName.ToString());
			if (num == -2140971007)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Contact not found in Contact Manager");
				throw new ArgumentException(SR.GetString("Collab_ContactNotFound"), "peerName");
			}
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabDeleteContact returned with errorcode {0}", new object[]
				{
					num
				});
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_DeleteContactFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving DeleteContact() successfully.");
		}

		[SecurityCritical]
		public void UpdateContact(PeerContact peerContact)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerContact == null)
			{
				throw new ArgumentNullException("peerContact");
			}
			if (peerContact.PeerName == null)
			{
				throw new ArgumentException(SR.GetString("Collab_NoPeerNameInContact"));
			}
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering UpdateContact() with following peercontact");
				peerContact.TracePeerContact();
			}
			SafeCollabMemory safeCollabMemory = null;
			int num = 0;
			try
			{
				PEER_CONTACT peer_CONTACT = CollaborationHelperFunctions.ConvertPeerContactToPEER_CONTACT(peerContact, ref safeCollabMemory);
				num = UnsafeCollabNativeMethods.PeerCollabUpdateContact(ref peer_CONTACT);
			}
			finally
			{
				if (safeCollabMemory != null)
				{
					safeCollabMemory.Dispose();
				}
			}
			if (num == -2140971007)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Contact not found in Contact Manager");
				throw new ArgumentException(SR.GetString("Collab_ContactNotFound"), "peerContact");
			}
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabUpdateContact returned with errorcode {0}", new object[]
				{
					num
				});
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_UpdateContactFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving UpdateContact() successfully.");
		}

		[SecurityCritical]
		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		private void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				CollaborationHelperFunctions.CleanEventVars(ref this.m_regSubLstChangedWaitHandle, ref this.m_safeSubLstChangedEvent, ref this.m_subLstChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean SubscriptionListChanged variables successful.");
				CollaborationHelperFunctions.CleanEventVars(ref this.m_regNameChangedWaitHandle, ref this.m_safeNameChangedEvent, ref this.m_nameChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean NameChanged variables successful.");
				CollaborationHelperFunctions.CleanEventVars(ref this.m_regPresenceChangedWaitHandle, ref this.m_safePresenceChangedEvent, ref this.m_presenceChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean PresenceChanged variables successful.");
				CollaborationHelperFunctions.CleanEventVars(ref this.m_regAppChangedWaitHandle, ref this.m_safeAppChangedEvent, ref this.m_appChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChanged variables successful.");
				CollaborationHelperFunctions.CleanEventVars(ref this.m_regObjChangedWaitHandle, ref this.m_safeObjChangedEvent, ref this.m_objChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChanged variables successful.");
				this.m_Disposed = true;
			}
		}

		private ISynchronizeInvoke m_synchronizingObject;

		private object m_lockSubLstChangedEvent;

		private RegisteredWaitHandle m_regSubLstChangedWaitHandle;

		private AutoResetEvent m_subLstChangedEvent;

		private SafeCollabEvent m_safeSubLstChangedEvent;

		private object m_lockNameChangedEvent;

		private RegisteredWaitHandle m_regNameChangedWaitHandle;

		private SafeCollabEvent m_safeNameChangedEvent;

		private object m_lockPresenceChangedEvent;

		private RegisteredWaitHandle m_regPresenceChangedWaitHandle;

		private SafeCollabEvent m_safePresenceChangedEvent;

		private object m_lockAppChangedEvent;

		private RegisteredWaitHandle m_regAppChangedWaitHandle;

		private AutoResetEvent m_appChangedEvent;

		private SafeCollabEvent m_safeAppChangedEvent;

		private object m_lockObjChangedEvent;

		private RegisteredWaitHandle m_regObjChangedWaitHandle;

		private AutoResetEvent m_objChangedEvent;

		private SafeCollabEvent m_safeObjChangedEvent;

		private SendOrPostCallback OnCreateContactCompletedDelegate;

		private object m_createContactAsyncListLock;

		private Dictionary<object, AsyncOperation> m_createContactAsyncList = new Dictionary<object, AsyncOperation>();

		private bool m_Disposed;

		private class CreateContactAsyncState
		{
			internal CreateContactAsyncState(PeerNearMe peerNearMe, object userToken)
			{
				this.m_peerNearMe = peerNearMe;
				this.m_userToken = userToken;
			}

			internal PeerNearMe PeerNearMe
			{
				get
				{
					return this.m_peerNearMe;
				}
			}

			internal object UserToken
			{
				get
				{
					return this.m_userToken;
				}
			}

			private PeerNearMe m_peerNearMe;

			private object m_userToken;
		}
	}
}
