﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	internal class InviteAsyncHelper : IDisposable
	{
		internal InviteAsyncHelper(PeerContact peerContact, Peer parentPeer, PeerEndPointCollection peerEndPoints, Guid applicationId, string message, byte[] inviteData, object userState, int NewTraceEventId)
		{
			this.m_userState = userState;
			this.m_peerContact = peerContact;
			this.m_applicationId = applicationId;
			this.m_message = message;
			this.m_inviteData = inviteData;
			this.m_peerEndPoints = peerEndPoints;
			this.m_TraceEventId = NewTraceEventId;
			this.m_peerWeakReference = new WeakReference(parentPeer);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "New InviteAsyncHelper created with TraceEventID {0}", new object[]
			{
				this.m_TraceEventId
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "\tPeerContact: {0}, App Guid: {1}, userState {2}, ParentReference {3}", new object[]
			{
				(this.m_peerContact != null) ? this.m_peerContact.ToString() : "null",
				applicationId.ToString(),
				userState.GetHashCode(),
				this.m_peerWeakReference.Target.GetHashCode()
			});
		}

		[SecurityCritical]
		internal void InviteAsync()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "InviteAsync called");
			this.m_RegisteredWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_InviteEvent, new WaitOrTimerCallback(this.InviteCallback), null, -1, false);
			PEER_INVITATION pi = default(PEER_INVITATION);
			pi.applicationId = CollaborationHelperFunctions.ConvertGuidToGUID(this.m_applicationId);
			pi.pwzMessage = this.m_message;
			pi.applicationData.cbData = (uint)((this.m_inviteData != null) ? this.m_inviteData.Length : 0);
			if (this.m_inviteData != null && this.m_inviteData.Length > 0)
			{
				SafeCollabMemory safeCollabMemory = new SafeCollabMemory(this.m_inviteData.Length);
				pi.applicationData.pbData = safeCollabMemory.DangerousGetHandle();
				Marshal.Copy(this.m_inviteData, 0, pi.applicationData.pbData, this.m_inviteData.Length);
			}
			else
			{
				pi.applicationData.pbData = IntPtr.Zero;
			}
			foreach (PeerEndPoint peerEndPoint in this.m_peerEndPoints)
			{
				try
				{
					this.InviteAsyncEndPoint(peerEndPoint, pi);
				}
				catch (PeerToPeerException)
				{
					if (!this.m_SafeCollabInvite.IsInvalid && !this.m_SafeCollabInvite.IsClosed)
					{
						this.m_SafeCollabInvite.Dispose();
					}
					this.m_RegisteredWaitHandle.Unregister(null);
					this.m_RegisteredWaitHandle = null;
					throw;
				}
			}
			this.m_AsyncOp = AsyncOperationManager.CreateOperation(this.m_userState);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Leaving InviteAsync.");
		}

		[SecurityCritical]
		internal void InviteAsyncEndPoint(PeerEndPoint peerEndPoint, PEER_INVITATION pi)
		{
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "InviteAsyncEndPoint() is called with the following Info");
				peerEndPoint.TracePeerEndPoint();
			}
			PEER_ENDPOINT peer_ENDPOINT = default(PEER_ENDPOINT);
			peer_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gchandle = default(GCHandle);
			if (peerEndPoint.Name != null)
			{
				gchandle = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
				peer_ENDPOINT.pwzEndpointName = gchandle.AddrOfPinnedObject();
			}
			GCHandle gchandle2 = GCHandle.Alloc(peer_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gchandle2.AddrOfPinnedObject();
			int num;
			try
			{
				if (this.m_peerContact != null)
				{
					SafeCollabMemory safeCollabMemory = null;
					PEER_CONTACT peer_CONTACT = CollaborationHelperFunctions.ConvertPeerContactToPEER_CONTACT(this.m_peerContact, ref safeCollabMemory);
					try
					{
						num = UnsafeCollabNativeMethods.PeerCollabAsyncInviteContact(ref peer_CONTACT, pcEndpoint, ref pi, this.m_InviteEvent.SafeWaitHandle, out this.m_SafeCollabInvite);
					}
					finally
					{
						if (safeCollabMemory != null)
						{
							safeCollabMemory.Dispose();
						}
					}
				}
				else
				{
					num = UnsafeCollabNativeMethods.PeerCollabAsyncInviteEndpoint(pcEndpoint, ref pi, this.m_InviteEvent.SafeWaitHandle, out this.m_SafeCollabInvite);
				}
			}
			finally
			{
				if (gchandle.IsAllocated)
				{
					gchandle.Free();
				}
				if (gchandle2.IsAllocated)
				{
					gchandle2.Free();
				}
			}
			if (num != 0)
			{
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_AsyncInviteFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Leaving InviteAsyncEndPoint.");
		}

		[SecurityCritical]
		internal void InviteCallback(object state, bool timedOut)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Entering InviteCallback.");
			SafeCollabData safeCollabData = null;
			int num = 0;
			InviteCompletedEventArgs inviteCompletedEventArgs = null;
			bool flag = false;
			PEER_INVITATION_RESPONSE peer_INVITATION_RESPONSE = default(PEER_INVITATION_RESPONSE);
			try
			{
				lock (this.m_Lock)
				{
					if (this.m_Cancelled || this.m_Completed)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Invite cancelled({0}) or completed({1}). Returning without doing anything.", new object[]
						{
							this.m_Cancelled,
							this.m_Completed
						});
						return;
					}
					num = UnsafeCollabNativeMethods.PeerCollabGetInvitationResponse(this.m_SafeCollabInvite, out safeCollabData);
				}
				if (num != 0 && num != -2140966907)
				{
					this.m_latestException = PeerToPeerException.CreateFromHr(SR.GetString("Collab_AsyncInviteException"), num);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Got an exception {0}. Storing it in latest exception.", new object[]
					{
						this.m_latestException
					});
				}
				else
				{
					peer_INVITATION_RESPONSE = (PEER_INVITATION_RESPONSE)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_INVITATION_RESPONSE));
					lock (this.m_responses)
					{
						this.m_responses.Add(peer_INVITATION_RESPONSE.action);
					}
					if (peer_INVITATION_RESPONSE.action == PeerInvitationResponseType.Accepted)
					{
						inviteCompletedEventArgs = new InviteCompletedEventArgs(new PeerInvitationResponse(peer_INVITATION_RESPONSE.action), null, false, this.m_AsyncOp.UserSuppliedState);
						flag = true;
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Found an accepted. About to fire callback.");
						this.m_RegisteredWaitHandle.Unregister(null);
					}
				}
				Interlocked.Increment(ref this.m_numberOfResponses);
				if (!flag && this.m_numberOfResponses == this.m_peerEndPoints.Count && !this.m_aboutToFireCallback)
				{
					lock (this.m_aboutToFireCallbackLock)
					{
						if (this.m_aboutToFireCallback)
						{
							return;
						}
						this.m_aboutToFireCallback = true;
						bool flag2 = false;
						bool flag3 = false;
						flag = true;
						foreach (PeerInvitationResponseType peerInvitationResponseType in this.m_responses)
						{
							if (peerInvitationResponseType == PeerInvitationResponseType.Expired)
							{
								flag3 = true;
							}
							else if (peerInvitationResponseType == PeerInvitationResponseType.Declined)
							{
								flag2 = true;
								break;
							}
						}
						if (flag2)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Got a declined invite response.");
							inviteCompletedEventArgs = new InviteCompletedEventArgs(new PeerInvitationResponse(PeerInvitationResponseType.Declined), null, false, this.m_AsyncOp.UserSuppliedState);
						}
						else if (flag3)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Got an expired invite response but no accepted or declined.");
							inviteCompletedEventArgs = new InviteCompletedEventArgs(new PeerInvitationResponse(PeerInvitationResponseType.Expired), null, false, this.m_AsyncOp.UserSuppliedState);
						}
						else
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Got all error responses");
							inviteCompletedEventArgs = new InviteCompletedEventArgs(null, (this.m_latestException != null) ? this.m_latestException : new PeerToPeerException("InviteAsync failure."), false, this.m_AsyncOp.UserSuppliedState);
						}
					}
				}
				if (flag)
				{
					Peer peer = this.m_peerWeakReference.Target as Peer;
					if (!this.m_Completed && peer != null)
					{
						lock (this.m_Lock)
						{
							if (!this.m_Completed && !this.m_Cancelled)
							{
								this.m_Completed = true;
								Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Firing callback with response type {0}.", new object[]
								{
									inviteCompletedEventArgs.InviteResponse
								});
								peer.PrepareToRaiseInviteCompletedEvent(this.m_AsyncOp, inviteCompletedEventArgs);
							}
						}
					}
				}
			}
			finally
			{
				if (safeCollabData != null)
				{
					safeCollabData.Dispose();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Leaving InviteCallback.");
		}

		[SecurityCritical]
		public void ContinueCancelCallback(object state)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Entering ContineCancelCallback.");
			try
			{
				lock (this.m_Lock)
				{
					if (this.m_Completed)
					{
						return;
					}
					this.m_Cancelled = true;
					int num = UnsafeCollabNativeMethods.PeerCollabCancelInvitation(this.m_SafeCollabInvite);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabCancelInvitation returned with errorcode {0}", new object[]
						{
							num
						});
					}
					this.m_SafeCollabInvite.Dispose();
				}
				Peer peer = this.m_peerWeakReference.Target as Peer;
				if (peer != null)
				{
					InviteCompletedEventArgs args = new InviteCompletedEventArgs(null, null, true, this.m_AsyncOp.UserSuppliedState);
					peer.PrepareToRaiseInviteCompletedEvent(this.m_AsyncOp, args);
				}
			}
			catch (ObjectDisposedException ex)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Critical, 0, "Exception while cancelling the call {0}", new object[]
				{
					ex
				});
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Leaving ContineCancelCallback.");
		}

		[SecurityCritical]
		public void CancelAsync(object state)
		{
			ThreadPool.QueueUserWorkItem(new WaitCallback(this.ContinueCancelCallback), state);
		}

		[SecurityCritical]
		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		public void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				if (!this.m_SafeCollabInvite.IsInvalid)
				{
					this.m_SafeCollabInvite.Dispose();
				}
				if (this.m_RegisteredWaitHandle != null)
				{
					this.m_RegisteredWaitHandle.Unregister(null);
					this.m_RegisteredWaitHandle = null;
				}
				if (this.m_InviteEvent != null)
				{
					this.m_InviteEvent.Close();
				}
			}
			this.m_Disposed = true;
		}

		public override int GetHashCode()
		{
			return this.m_TraceEventId;
		}

		internal int TraceEventId
		{
			get
			{
				return this.m_TraceEventId;
			}
		}

		internal object m_userState;

		internal SafeCollabInvite m_SafeCollabInvite;

		internal AutoResetEvent m_InviteEvent = new AutoResetEvent(false);

		internal RegisteredWaitHandle m_RegisteredWaitHandle;

		internal bool m_Disposed;

		internal bool m_Cancelled;

		private AsyncOperation m_AsyncOp;

		private WeakReference m_peerWeakReference;

		private object m_Lock = new object();

		private int m_TraceEventId;

		private Exception m_latestException;

		private Collection<PeerInvitationResponseType> m_responses = new Collection<PeerInvitationResponseType>();

		private bool m_Completed;

		private bool m_aboutToFireCallback;

		private object m_aboutToFireCallbackLock = new object();

		private int m_numberOfResponses;

		private PeerContact m_peerContact;

		private PeerEndPointCollection m_peerEndPoints;

		private Guid m_applicationId;

		private string m_message;

		private byte[] m_inviteData;
	}
}
