﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	internal class MyContact : PeerContact, ISerializable
	{
		[SecurityCritical]
		internal MyContact()
		{
		}

		[SecurityCritical]
		internal MyContact(SerializationInfo serializationInfo, StreamingContext streamingContext) : base(serializationInfo, streamingContext)
		{
		}

		internal override bool InternalIsSubscribed()
		{
			return true;
		}

		internal override SubscriptionType InternalSubscribeAllowedGet()
		{
			return SubscriptionType.Allowed;
		}

		internal override void InternalSubscribeAllowedSet(SubscriptionType value)
		{
			throw new PeerToPeerException(SR.GetString("Collab_SubscribeLocalContactFailed"));
		}

		private event EventHandler<PresenceChangedEventArgs> m_presenceChanged;

		[SecurityCritical]
		internal override void AddPresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			lock (base.LockPresenceChangedEvent)
			{
				if (this.m_presenceChanged == null)
				{
					base.PresenceChangedEvent = new AutoResetEvent(false);
					base.PresenceChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(base.PresenceChangedEvent, new WaitOrTimerCallback(this.PresenceChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.MyPresenceChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(base.PresenceChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safePresenceChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_PresenceChangedRegFailed"), num);
					}
				}
				this.m_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Combine(this.m_presenceChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddMyPresenceChanged() successful.");
		}

		[SecurityCritical]
		internal override void RemovePresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() called.");
			lock (base.LockPresenceChangedEvent)
			{
				this.m_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Remove(this.m_presenceChanged, callback);
				if (this.m_presenceChanged == null)
				{
					base.CleanContactPresenceEventVars();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() successful.");
		}

		protected override void OnPresenceChanged(PresenceChangedEventArgs presenceChangedArgs)
		{
			EventHandler<PresenceChangedEventArgs> presenceChanged = this.m_presenceChanged;
			if (presenceChanged != null)
			{
				presenceChanged(this, presenceChangedArgs);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the presence changed event callback.");
			}
		}

		[SecurityCritical]
		internal override void PresenceChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PresenceChangedCallback() called.");
			if (!this.m_Disposed)
			{
				for (;;)
				{
					PresenceChangedEventArgs presenceChangedEventArgs = null;
					try
					{
						lock (base.LockPresenceChangedEvent)
						{
							if (this.m_safePresenceChangedEvent.IsInvalid)
							{
								return;
							}
							num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safePresenceChangedEvent, out safeCollabData);
						}
						if (num == 6488066)
						{
							break;
						}
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
							{
								num
							});
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPresenceChangedDataFailed"), num);
						}
						PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyPresenceChanged)
						{
							PEER_EVENT_PRESENCE_CHANGED_DATA presenceChangedData = peer_COLLAB_EVENT_DATA.presenceChangedData;
							PeerPresenceInfo peerPresenceInfo = null;
							if (presenceChangedData.pPresenceInfo != IntPtr.Zero)
							{
								PEER_PRESENCE_INFO peer_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(presenceChangedData.pPresenceInfo, typeof(PEER_PRESENCE_INFO));
								peerPresenceInfo = new PeerPresenceInfo();
								peerPresenceInfo.PresenceStatus = peer_PRESENCE_INFO.status;
								peerPresenceInfo.DescriptiveText = peer_PRESENCE_INFO.descText;
							}
							PeerEndPoint peerEndPoint = null;
							if (presenceChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(presenceChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							presenceChangedEventArgs = new PresenceChangedEventArgs(peerEndPoint, null, presenceChangedData.changeType, peerPresenceInfo);
						}
					}
					finally
					{
						if (safeCollabData != null)
						{
							safeCollabData.Dispose();
						}
					}
					if (presenceChangedEventArgs != null)
					{
						this.OnPresenceChanged(presenceChangedEventArgs);
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving PresenceChangedCallback().");
			}
		}

		private event EventHandler<ApplicationChangedEventArgs> m_applicationChanged;

		[SecurityCritical]
		internal override void AddApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			lock (base.LockAppChangedEvent)
			{
				if (this.m_applicationChanged == null)
				{
					base.AppChangedEvent = new AutoResetEvent(false);
					base.AppChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(base.AppChangedEvent, new WaitOrTimerCallback(this.ApplicationChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.MyApplicationChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(base.AppChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeAppChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ApplicationChangedRegFailed"), num);
					}
				}
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Combine(this.m_applicationChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() successful.");
		}

		[SecurityCritical]
		internal override void RemoveApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() called.");
			lock (base.LockAppChangedEvent)
			{
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Remove(this.m_applicationChanged, callback);
				if (this.m_applicationChanged == null)
				{
					base.CleanContactObjEventVars();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() successful.");
		}

		protected override void OnApplicationChanged(ApplicationChangedEventArgs appChangedArgs)
		{
			EventHandler<ApplicationChangedEventArgs> applicationChanged = this.m_applicationChanged;
			if (applicationChanged != null)
			{
				applicationChanged(this, appChangedArgs);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the application changed event callback.");
			}
		}

		[SecurityCritical]
		internal override void ApplicationChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ApplicationChangedCallback() called.");
			if (!this.m_Disposed)
			{
				for (;;)
				{
					ApplicationChangedEventArgs applicationChangedEventArgs = null;
					try
					{
						lock (base.LockAppChangedEvent)
						{
							if (this.m_safeAppChangedEvent.IsInvalid)
							{
								return;
							}
							num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeAppChangedEvent, out safeCollabData);
						}
						if (num == 6488066)
						{
							break;
						}
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
							{
								num
							});
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetApplicationChangedDataFailed"), num);
						}
						PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyApplicationChanged)
						{
							PEER_EVENT_APPLICATION_CHANGED_DATA applicationChangedData = peer_COLLAB_EVENT_DATA.applicationChangedData;
							PEER_APPLICATION pa = (PEER_APPLICATION)Marshal.PtrToStructure(applicationChangedData.pApplication, typeof(PEER_APPLICATION));
							PeerApplication peerApplication = CollaborationHelperFunctions.ConvertPEER_APPLICATIONToPeerApplication(pa);
							PeerEndPoint peerEndPoint = null;
							if (applicationChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(applicationChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							applicationChangedEventArgs = new ApplicationChangedEventArgs(peerEndPoint, null, applicationChangedData.changeType, peerApplication);
						}
					}
					finally
					{
						if (safeCollabData != null)
						{
							safeCollabData.Dispose();
						}
					}
					if (applicationChangedEventArgs != null)
					{
						this.OnApplicationChanged(applicationChangedEventArgs);
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ApplicationChangedCallback().");
			}
		}

		private event EventHandler<ObjectChangedEventArgs> m_objectChanged;

		[SecurityCritical]
		internal override void AddObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			lock (base.LockObjChangedEvent)
			{
				if (this.m_objectChanged == null)
				{
					base.ObjChangedEvent = new AutoResetEvent(false);
					base.ObjChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(base.ObjChangedEvent, new WaitOrTimerCallback(this.ObjectChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.MyObjectChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(base.ObjChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeObjChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectChangedRegFailed"), num);
					}
				}
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Combine(this.m_objectChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChanged() successful.");
		}

		[SecurityCritical]
		internal override void RemoveObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() called.");
			lock (base.LockObjChangedEvent)
			{
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Remove(this.m_objectChanged, callback);
				if (this.m_objectChanged == null)
				{
					base.CleanContactObjEventVars();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() successful.");
		}

		protected override void OnObjectChanged(ObjectChangedEventArgs objChangedArgs)
		{
			EventHandler<ObjectChangedEventArgs> objectChanged = this.m_objectChanged;
			if (objectChanged != null)
			{
				if (base.SynchronizingObject != null && base.SynchronizingObject.InvokeRequired)
				{
					base.SynchronizingObject.BeginInvoke(objectChanged, new object[]
					{
						this,
						objChangedArgs
					});
				}
				else
				{
					objectChanged(this, objChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the object changed event callback.");
			}
		}

		[SecurityCritical]
		internal override void ObjectChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ObjectChangedCallback() called.");
			if (!this.m_Disposed)
			{
				for (;;)
				{
					ObjectChangedEventArgs objectChangedEventArgs = null;
					try
					{
						lock (base.LockObjChangedEvent)
						{
							if (this.m_safeObjChangedEvent.IsInvalid)
							{
								return;
							}
							num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeObjChangedEvent, out safeCollabData);
						}
						if (num == 6488066)
						{
							break;
						}
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
							{
								num
							});
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectChangedDataFailed"), num);
						}
						PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyObjectChanged)
						{
							PEER_EVENT_OBJECT_CHANGED_DATA objectChangedData = peer_COLLAB_EVENT_DATA.objectChangedData;
							PEER_OBJECT po = (PEER_OBJECT)Marshal.PtrToStructure(objectChangedData.pObject, typeof(PEER_OBJECT));
							PeerObject peerObject = CollaborationHelperFunctions.ConvertPEER_OBJECTToPeerObject(po);
							PeerEndPoint peerEndPoint = null;
							if (objectChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(objectChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							objectChangedEventArgs = new ObjectChangedEventArgs(peerEndPoint, null, objectChangedData.changeType, peerObject);
						}
					}
					finally
					{
						if (safeCollabData != null)
						{
							safeCollabData.Dispose();
						}
					}
					if (objectChangedEventArgs != null)
					{
						this.OnObjectChanged(objectChangedEventArgs);
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ObjectChangedCallback().");
			}
		}

		public override void Subscribe()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
		}

		public override void SubscribeAsync(object userToken)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			if (userToken == null)
			{
				throw new ArgumentNullException("userToken");
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering SubscribeAsync() with user token {0}.", new object[]
			{
				userToken
			});
			lock (base.AsyncLock)
			{
				if (base.AsyncOp != null)
				{
					throw new PeerToPeerException(SR.GetString("Collab_DuplicateSubscribeAsync"));
				}
				base.AsyncOp = AsyncOperationManager.CreateOperation(userToken);
			}
			base.PrepareToRaiseSubscribeCompletedEvent(base.AsyncOp, new SubscribeCompletedEventArgs(null, this, null, false, userToken));
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving SubscribeAsync().");
		}

		public override void Unsubscribe()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			throw new PeerToPeerException(SR.GetString("Collab_UnsubscribeLocalContactFail"));
		}

		[SecurityCritical]
		protected override void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				try
				{
					this.m_Disposed = true;
				}
				finally
				{
					base.Dispose(disposing);
				}
			}
		}

		private bool m_Disposed;
	}
}
