﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Text;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public abstract class Peer : IDisposable, IEquatable<Peer>, ISerializable
	{
		public virtual PeerEndPointCollection PeerEndPoints
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_peerEndPoints;
			}
		}

		public bool IsOnline
		{
			get
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Get Isonline called.");
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				bool flag = false;
				foreach (PeerEndPoint peerEndPoint in this.PeerEndPoints)
				{
					PeerPresenceInfo peerPresenceInfo = null;
					try
					{
						peerPresenceInfo = this.GetPresenceInfo(peerEndPoint);
					}
					catch (Exception ex)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "Exception thrown {0}", new object[]
						{
							ex.Message
						});
					}
					if (peerPresenceInfo != null && peerPresenceInfo.PresenceStatus == PeerPresenceStatus.Online)
					{
						flag = true;
						break;
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving Isonline called with {0}.", new object[]
				{
					flag
				});
				return flag;
			}
		}

		[DefaultValue(null)]
		[Description("SynchronizingObject")]
		[Browsable(false)]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_synchronizingObject;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_synchronizingObject = value;
			}
		}

		internal static Guid CurrentApplicationGuid
		{
			[SecurityCritical]
			get
			{
				Guid empty = Guid.Empty;
				string b = Path.Combine(Environment.CurrentDirectory, Process.GetCurrentProcess().ProcessName + ".exe");
				string b2 = null;
				string[] commandLineArgs = Environment.GetCommandLineArgs();
				int num = commandLineArgs.Length;
				if (num > 1)
				{
					StringBuilder stringBuilder = new StringBuilder();
					for (int i = 1; i < num; i++)
					{
						stringBuilder.Append(commandLineArgs[i]);
						if (i != num - 1)
						{
							stringBuilder.Append(' ');
						}
					}
					b2 = stringBuilder.ToString();
				}
				PeerApplicationCollection localRegisteredApplications = PeerCollaboration.GetLocalRegisteredApplications();
				foreach (PeerApplication peerApplication in localRegisteredApplications)
				{
					if (peerApplication.CommandLineArgs == b2 && peerApplication.Path == b)
					{
						return peerApplication.Id;
					}
				}
				return empty;
			}
		}

		internal Peer()
		{
			this.OnInviteCompletedDelegate = new SendOrPostCallback(this.InviteCompletedWaitCallback);
		}

		protected Peer(SerializationInfo serializationInfo, StreamingContext streamingContext) : this()
		{
			this.m_peerEndPoints = (PeerEndPointCollection)serializationInfo.GetValue("_PeerEndPoints", typeof(PeerEndPointCollection));
		}

		[SecurityCritical]
		public PeerPresenceInfo GetPresenceInfo(PeerEndPoint peerEndPoint)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "GetPresenceInfo()called.");
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new ArgumentException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Getting presence for the following endpoint.");
				peerEndPoint.TracePeerEndPoint();
			}
			SafeCollabData safeCollabData = null;
			PeerPresenceInfo peerPresenceInfo = null;
			PEER_ENDPOINT peer_ENDPOINT = default(PEER_ENDPOINT);
			peer_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gchandle = default(GCHandle);
			if (peerEndPoint.Name != null)
			{
				gchandle = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
				peer_ENDPOINT.pwzEndpointName = gchandle.AddrOfPinnedObject();
			}
			GCHandle gchandle2 = GCHandle.Alloc(peer_ENDPOINT, GCHandleType.Pinned);
			IntPtr endpoint = gchandle2.AddrOfPinnedObject();
			this.RefreshIfNeeded();
			try
			{
				int num = UnsafeCollabNativeMethods.PeerCollabGetPresenceInfo(endpoint, out safeCollabData);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetPresenceInfo returned with errorcode {0}", new object[]
					{
						num
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPresenceFailed"), num);
				}
				IntPtr ptr = safeCollabData.DangerousGetHandle();
				PEER_PRESENCE_INFO peer_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(ptr, typeof(PEER_PRESENCE_INFO));
				peerPresenceInfo = new PeerPresenceInfo();
				peerPresenceInfo.PresenceStatus = peer_PRESENCE_INFO.status;
				peerPresenceInfo.DescriptiveText = peer_PRESENCE_INFO.descText;
			}
			finally
			{
				if (gchandle.IsAllocated)
				{
					gchandle.Free();
				}
				if (gchandle2.IsAllocated)
				{
					gchandle2.Free();
				}
				if (safeCollabData != null)
				{
					safeCollabData.Dispose();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving GetPresenceInfo().");
			return peerPresenceInfo;
		}

		[SecurityCritical]
		public PeerObjectCollection GetObjects()
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			return this.InternalGetAllObjects(Guid.Empty, false);
		}

		[SecurityCritical]
		public PeerObjectCollection GetObjects(Guid objectId)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			return this.InternalGetAllObjects(objectId, true);
		}

		internal abstract void RefreshIfNeeded();

		[SecurityCritical]
		private PeerObjectCollection InternalGetAllObjects(Guid objectId, bool guidSupplied)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering InternalGetAllObjects() with ObjectId {0}.", new object[]
			{
				objectId
			});
			Dictionary<Guid, PeerObject> dictionary = new Dictionary<Guid, PeerObject>();
			this.RefreshIfNeeded();
			PeerObjectCollection peerObjectCollection;
			foreach (PeerEndPoint peerEndPoint in this.PeerEndPoints)
			{
				peerObjectCollection = Peer.InternalGetObjects(objectId, guidSupplied, peerEndPoint);
				if (guidSupplied && peerObjectCollection.Count != 0)
				{
					return peerObjectCollection;
				}
				foreach (PeerObject peerObject in peerObjectCollection)
				{
					dictionary[peerObject.Id] = peerObject;
				}
			}
			Dictionary<Guid, PeerObject>.ValueCollection values = dictionary.Values;
			peerObjectCollection = new PeerObjectCollection();
			foreach (PeerObject peerObject in values)
			{
				peerObjectCollection.Add(peerObject);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving InternalGetAllObjects(). Returning collection with {0} objects.", new object[]
			{
				peerObjectCollection.Count
			});
			return peerObjectCollection;
		}

		[SecurityCritical]
		internal unsafe static PeerObjectCollection InternalGetObjects(Guid objectId, bool guidSupplied, PeerEndPoint peerEndPoint)
		{
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering InternalGetObjects() with the following PeerEndPoint");
				peerEndPoint.TracePeerEndPoint();
			}
			PeerObjectCollection peerObjectCollection = new PeerObjectCollection();
			SafeCollabEnum safeCollabEnum = null;
			uint num = 0U;
			GCHandle gchandle = default(GCHandle);
			IntPtr pObjectId = IntPtr.Zero;
			if (guidSupplied)
			{
				GUID guid = CollaborationHelperFunctions.ConvertGuidToGUID(objectId);
				gchandle = GCHandle.Alloc(guid, GCHandleType.Pinned);
				pObjectId = gchandle.AddrOfPinnedObject();
			}
			PEER_ENDPOINT peer_ENDPOINT = default(PEER_ENDPOINT);
			peer_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gchandle2 = default(GCHandle);
			if (peerEndPoint.Name != null)
			{
				gchandle2 = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
				peer_ENDPOINT.pwzEndpointName = gchandle2.AddrOfPinnedObject();
			}
			GCHandle gchandle3 = GCHandle.Alloc(peer_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gchandle3.AddrOfPinnedObject();
			try
			{
				int num2 = UnsafeCollabNativeMethods.PeerCollabEnumObjects(pcEndpoint, pObjectId, out safeCollabEnum);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumObjects returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectsFailed"), num2);
				}
				num2 = UnsafeCollabNativeMethods.PeerGetItemCount(safeCollabEnum, ref num);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectsFailed"), num2);
				}
				if (num == 0U)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "No PeerObjects found.");
					return peerObjectCollection;
				}
				SafeCollabData safeCollabData;
				num2 = UnsafeCollabNativeMethods.PeerGetNextItem(safeCollabEnum, ref num, out safeCollabData);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetNextItem returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectsFailed"), num2);
				}
				IntPtr value = safeCollabData.DangerousGetHandle();
				IntPtr* ptr = (IntPtr*)((void*)value);
				for (ulong num3 = 0UL; num3 < (ulong)num; num3 += 1UL)
				{
					PEER_OBJECT* ptr2 = (PEER_OBJECT*)((void*)ptr[num3 * (ulong)((long)sizeof(IntPtr)) / (ulong)sizeof(IntPtr)]);
					byte[] array = null;
					if (ptr2->data.cbData != 0U)
					{
						array = new byte[ptr2->data.cbData];
						Marshal.Copy(ptr2->data.pbData, array, 0, (int)ptr2->data.cbData);
					}
					PeerObject peerObject = new PeerObject(CollaborationHelperFunctions.ConvertGUIDToGuid(ptr2->guid), array, (PeerScope)ptr2->dwPublicationScope);
					if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Retrieved following Object");
						peerObject.TracePeerObject();
					}
					peerObjectCollection.Add(peerObject);
				}
			}
			finally
			{
				if (gchandle.IsAllocated)
				{
					gchandle.Free();
				}
				if (gchandle2.IsAllocated)
				{
					gchandle2.Free();
				}
				if (gchandle3.IsAllocated)
				{
					gchandle3.Free();
				}
				if (safeCollabEnum != null)
				{
					safeCollabEnum.Dispose();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving InternalGetApplications(). Returning collection with {0} objects.", new object[]
			{
				peerObjectCollection.Count
			});
			return peerObjectCollection;
		}

		public abstract PeerInvitationResponse Invite();

		public abstract PeerInvitationResponse Invite(PeerApplication applicationToInvite, string message, byte[] invitationData);

		[SecurityCritical]
		internal static PeerInvitationResponse InternalInviteEndPoint(Guid applicationToInviteGuid, string message, byte[] invitationData, PeerEndPoint peerEndPoint, PeerContact peerContact)
		{
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering InternalInviteEndPoint() with the following information.");
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Invitation Message: ", new object[]
				{
					message
				});
				if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose) && invitationData != null)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tInvitation data:");
					Logging.DumpData(Logging.P2PTraceSource, TraceEventType.Verbose, Logging.P2PTraceSource.MaxDataSize, invitationData, 0, invitationData.Length);
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Invitation Data length: ", new object[]
					{
						(invitationData != null) ? invitationData.Length : 0
					});
				}
				if (peerEndPoint != null)
				{
					peerEndPoint.TracePeerEndPoint();
				}
			}
			PEER_INVITATION peer_INVITATION = default(PEER_INVITATION);
			peer_INVITATION.applicationId = CollaborationHelperFunctions.ConvertGuidToGUID(applicationToInviteGuid);
			peer_INVITATION.pwzMessage = message;
			peer_INVITATION.applicationData.cbData = (uint)((invitationData != null) ? invitationData.Length : 0);
			if (invitationData != null && invitationData.Length > 0)
			{
				SafeCollabMemory safeCollabMemory = new SafeCollabMemory(invitationData.Length);
				peer_INVITATION.applicationData.pbData = safeCollabMemory.DangerousGetHandle();
				Marshal.Copy(invitationData, 0, peer_INVITATION.applicationData.pbData, invitationData.Length);
			}
			else
			{
				peer_INVITATION.applicationData.pbData = IntPtr.Zero;
			}
			PEER_ENDPOINT peer_ENDPOINT = default(PEER_ENDPOINT);
			peer_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gchandle = default(GCHandle);
			if (peerEndPoint.Name != null)
			{
				gchandle = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
				peer_ENDPOINT.pwzEndpointName = gchandle.AddrOfPinnedObject();
			}
			GCHandle gchandle2 = GCHandle.Alloc(peer_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gchandle2.AddrOfPinnedObject();
			SafeCollabData safeCollabData = null;
			PeerInvitationResponse peerInvitationResponse = null;
			try
			{
				int num;
				if (peerContact != null)
				{
					SafeCollabMemory safeCollabMemory2 = null;
					PEER_CONTACT peer_CONTACT = CollaborationHelperFunctions.ConvertPeerContactToPEER_CONTACT(peerContact, ref safeCollabMemory2);
					try
					{
						num = UnsafeCollabNativeMethods.PeerCollabInviteContact(ref peer_CONTACT, pcEndpoint, ref peer_INVITATION, out safeCollabData);
					}
					finally
					{
						if (safeCollabMemory2 != null)
						{
							safeCollabMemory2.Dispose();
						}
					}
				}
				else
				{
					num = UnsafeCollabNativeMethods.PeerCollabInviteEndpoint(pcEndpoint, ref peer_INVITATION, out safeCollabData);
				}
				if (num != 0)
				{
					if (num == -2140966907 || num == -2147023436)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Timed out. Leaving InternalInviteEndPoint() with InvitationResponseType expired.");
						return new PeerInvitationResponse(PeerInvitationResponseType.Expired);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, ((peerContact != null) ? "PeerCollabInviteContact" : "PeerCollabInviteEndpoint") + " returned with errorcode {0}", new object[]
					{
						num
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_InviteFailed"), num);
				}
				else if (!safeCollabData.IsInvalid)
				{
					peerInvitationResponse = new PeerInvitationResponse(((PEER_INVITATION_RESPONSE)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_INVITATION_RESPONSE))).action);
				}
			}
			finally
			{
				if (safeCollabData != null)
				{
					safeCollabData.Dispose();
				}
				if (gchandle.IsAllocated)
				{
					gchandle.Free();
				}
				if (gchandle2.IsAllocated)
				{
					gchandle2.Free();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving InternalInviteEndPoint() with InvitationResponse {0}.", new object[]
			{
				peerInvitationResponse
			});
			return peerInvitationResponse;
		}

		private event EventHandler<InviteCompletedEventArgs> m_inviteCompleted;

		public event EventHandler<InviteCompletedEventArgs> InviteCompleted
		{
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_inviteCompleted = (EventHandler<InviteCompletedEventArgs>)Delegate.Combine(this.m_inviteCompleted, value);
			}
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_inviteCompleted = (EventHandler<InviteCompletedEventArgs>)Delegate.Remove(this.m_inviteCompleted, value);
			}
		}

		public abstract void InviteAsync(object userToken);

		public abstract void InviteAsync(PeerApplication applicationToInvite, string message, byte[] invitationData, object userToken);

		[SecurityCritical]
		internal void InternalInviteAsync(Guid applicationToInviteGuid, string message, byte[] invitationData, PeerEndPointCollection peerEndPoints, PeerContact peerContact, object userToken)
		{
			InviteAsyncHelper inviteAsyncHelper = null;
			int newTraceEventId = Peer.NewTraceEventId;
			lock (this.m_inviteAsyncHelperList)
			{
				if (this.m_inviteAsyncHelperList.ContainsKey(userToken))
				{
					throw new ArgumentException(SR.GetString("DuplicateUserToken"));
				}
				inviteAsyncHelper = new InviteAsyncHelper(peerContact, this, peerEndPoints, applicationToInviteGuid, message, invitationData, userToken, newTraceEventId);
				this.m_inviteAsyncHelperList[userToken] = inviteAsyncHelper;
			}
			try
			{
				inviteAsyncHelper.InviteAsync();
			}
			catch
			{
				lock (this.m_inviteAsyncHelperList)
				{
					this.m_inviteAsyncHelperList.Remove(userToken);
				}
				throw;
			}
		}

		protected virtual void OnInviteCompleted(InviteCompletedEventArgs e)
		{
			EventHandler<InviteCompletedEventArgs> inviteCompleted = this.m_inviteCompleted;
			if (inviteCompleted != null)
			{
				inviteCompleted(this, e);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the invite completed event callback.");
			}
		}

		private void InviteCompletedWaitCallback(object operationState)
		{
			this.OnInviteCompleted((InviteCompletedEventArgs)operationState);
		}

		internal void PrepareToRaiseInviteCompletedEvent(AsyncOperation asyncOP, InviteCompletedEventArgs args)
		{
			lock (this.m_inviteAsyncHelperList)
			{
				InviteAsyncHelper inviteAsyncHelper = this.m_inviteAsyncHelperList[args.UserState];
				if (inviteAsyncHelper == null)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Critical, 0, "userState for which we are about to call Completed event does not exist in the pending async list");
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Critical, inviteAsyncHelper.TraceEventId, "userState {0} is being removed from the pending async list", new object[]
					{
						args.UserState.GetHashCode()
					});
					this.m_inviteAsyncHelperList.Remove(args.UserState);
				}
			}
			asyncOP.PostOperationCompleted(this.OnInviteCompletedDelegate, args);
		}

		[SecurityCritical]
		public void InviteAsyncCancel(object userToken)
		{
			if (userToken == null)
			{
				throw new ArgumentNullException("userToken");
			}
			InviteAsyncHelper inviteAsyncHelper;
			lock (this.m_inviteAsyncHelperList)
			{
				if (!this.m_inviteAsyncHelperList.TryGetValue(userToken, out inviteAsyncHelper))
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Warning, 0, "InviteAsyncCancel called with a userState token that is not in the pending async list - returning");
					return;
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, inviteAsyncHelper.TraceEventId, "Proceeding to cancel the pending async");
			inviteAsyncHelper.CancelAsync(userToken);
		}

		internal static int NewTraceEventId
		{
			get
			{
				Interlocked.CompareExchange(ref Peer.s_TraceEventId, 0, int.MaxValue);
				Interlocked.Increment(ref Peer.s_TraceEventId);
				return Peer.s_TraceEventId;
			}
		}

		public bool Equals(Peer other)
		{
			if (other != null)
			{
				if (other.PeerEndPoints != null)
				{
					return other.PeerEndPoints.Equals(this.PeerEndPoints);
				}
			}
			return false;
		}

		public override string ToString()
		{
			return this.PeerEndPoints.ToString();
		}

		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_PeerEndPoints", this.PeerEndPoints);
		}

		protected virtual void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				this.m_Disposed = true;
			}
		}

		private PeerEndPointCollection m_peerEndPoints = new PeerEndPointCollection();

		private ISynchronizeInvoke m_synchronizingObject;

		private SendOrPostCallback OnInviteCompletedDelegate;

		internal Dictionary<object, InviteAsyncHelper> m_inviteAsyncHelperList = new Dictionary<object, InviteAsyncHelper>();

		private static int s_TraceEventId = 1;

		private bool m_Disposed;
	}
}
