﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerApplication : IDisposable, IEquatable<PeerApplication>, ISerializable
	{
		[SecurityCritical]
		static PeerApplication()
		{
			CollaborationHelperFunctions.Initialize();
		}

		public PeerApplication()
		{
		}

		public PeerApplication(Guid id, string description, byte[] data, string path, string commandLineArgs, PeerScope peerScope)
		{
			if (data != null && data.Length > 16384)
			{
				throw new ArgumentException(SR.GetString("Collab_ApplicationDataSizeFailed"), "data");
			}
			this.m_id = id;
			this.m_description = description;
			this.m_data = data;
			this.m_path = path;
			this.m_commandLineArgs = commandLineArgs;
			this.m_peerScope = peerScope;
		}

		protected PeerApplication(SerializationInfo serializationInfo, StreamingContext streamingContext)
		{
			this.m_id = (Guid)serializationInfo.GetValue("_Id", typeof(Guid));
			this.m_data = (byte[])serializationInfo.GetValue("_Data", typeof(byte[]));
			this.m_description = serializationInfo.GetString("_Description");
			this.m_path = serializationInfo.GetString("_Path");
			this.m_commandLineArgs = serializationInfo.GetString("_CommandLineArgs");
			this.m_peerScope = (PeerScope)serializationInfo.GetInt32("_Scope");
		}

		public Guid Id
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_id;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_id = value;
			}
		}

		public byte[] Data
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_data;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				if (value != null && value.Length > 16384)
				{
					throw new ArgumentException(SR.GetString("Collab_ApplicationDataSizeFailed"));
				}
				this.m_data = value;
			}
		}

		public string Description
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_description;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_description = value;
			}
		}

		public string Path
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_path;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_path = value;
			}
		}

		public string CommandLineArgs
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_commandLineArgs;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_commandLineArgs = value;
			}
		}

		public PeerScope PeerScope
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_peerScope;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_peerScope = value;
			}
		}

		[DefaultValue(null)]
		[Browsable(false)]
		[Description("SynchronizingObject")]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_synchronizingObject;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_synchronizingObject = value;
			}
		}

		private event EventHandler<ApplicationChangedEventArgs> m_applicationChanged;

		public event EventHandler<ApplicationChangedEventArgs> ApplicationChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddApplicationChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemoveApplicationChanged(value);
			}
		}

		private object LockAppChangedEvent
		{
			get
			{
				if (this.m_lockAppChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockAppChangedEvent, value, null);
				}
				return this.m_lockAppChangedEvent;
			}
		}

		[SecurityCritical]
		private void AddApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() called.");
			lock (this.LockAppChangedEvent)
			{
				if (this.m_applicationChanged == null)
				{
					if (this.m_id.Equals(Guid.Empty))
					{
						throw new PeerToPeerException("No application guid defined");
					}
					this.m_appChangedEvent = new AutoResetEvent(false);
					this.m_regAppChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_appChangedEvent, new WaitOrTimerCallback(this.ApplicationChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.EndPointApplicationChanged;
					GUID guid = CollaborationHelperFunctions.ConvertGuidToGUID(this.m_id);
					GCHandle gchandle = GCHandle.Alloc(guid, GCHandleType.Pinned);
					peer_COLLAB_EVENT_REGISTRATION.pInstance = gchandle.AddrOfPinnedObject();
					try
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Registering event with App ID {0}", new object[]
						{
							this.m_id.ToString()
						});
						int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_appChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeAppChangedEvent);
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
							{
								num
							});
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ApplicationChangedRegFailed"), num);
						}
					}
					finally
					{
						if (gchandle.IsAllocated)
						{
							gchandle.Free();
						}
					}
				}
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Combine(this.m_applicationChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddApplicationChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() called.");
			lock (this.LockAppChangedEvent)
			{
				this.m_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Remove(this.m_applicationChanged, callback);
				if (this.m_applicationChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref this.m_regAppChangedWaitHandle, ref this.m_safeAppChangedEvent, ref this.m_appChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() successful.");
		}

		protected virtual void OnApplicationChanged(ApplicationChangedEventArgs appChangedArgs)
		{
			EventHandler<ApplicationChangedEventArgs> applicationChanged = this.m_applicationChanged;
			if (applicationChanged != null)
			{
				if (this.SynchronizingObject != null && this.SynchronizingObject.InvokeRequired)
				{
					this.SynchronizingObject.BeginInvoke(applicationChanged, new object[]
					{
						this,
						appChangedArgs
					});
				}
				else
				{
					applicationChanged(this, appChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the application changed event callback.");
			}
		}

		[SecurityCritical]
		private void ApplicationChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ApplicationChangedCallback() called.");
			if (!this.m_Disposed)
			{
				for (;;)
				{
					ApplicationChangedEventArgs applicationChangedEventArgs = null;
					try
					{
						lock (this.LockAppChangedEvent)
						{
							if (this.m_safeAppChangedEvent.IsInvalid)
							{
								return;
							}
							num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeAppChangedEvent, out safeCollabData);
						}
						if (num == 6488066)
						{
							break;
						}
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
							{
								num
							});
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetApplicationChangedDataFailed"), num);
						}
						PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointApplicationChanged)
						{
							PEER_EVENT_APPLICATION_CHANGED_DATA applicationChangedData = peer_COLLAB_EVENT_DATA.applicationChangedData;
							PEER_APPLICATION pa = (PEER_APPLICATION)Marshal.PtrToStructure(applicationChangedData.pApplication, typeof(PEER_APPLICATION));
							PeerApplication peerApplication = CollaborationHelperFunctions.ConvertPEER_APPLICATIONToPeerApplication(pa);
							if (object.Equals(this.m_id, peerApplication.Id))
							{
								PeerContact peerContact = null;
								PeerEndPoint peerEndPoint = null;
								if (applicationChangedData.pContact != IntPtr.Zero)
								{
									PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(applicationChangedData.pContact, typeof(PEER_CONTACT));
									peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
								}
								if (applicationChangedData.pEndPoint != IntPtr.Zero)
								{
									PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(applicationChangedData.pEndPoint, typeof(PEER_ENDPOINT));
									peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
								}
								applicationChangedEventArgs = new ApplicationChangedEventArgs(peerEndPoint, peerContact, applicationChangedData.changeType, peerApplication);
							}
						}
					}
					finally
					{
						if (safeCollabData != null)
						{
							safeCollabData.Dispose();
						}
					}
					if (applicationChangedEventArgs != null)
					{
						this.OnApplicationChanged(applicationChangedEventArgs);
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ApplicationChangedCallback().");
			}
		}

		public bool Equals(PeerApplication other)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return other != null && object.Equals(other.Id, this.m_id);
		}

		public override bool Equals(object obj)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerApplication peerApplication = obj as PeerApplication;
			return peerApplication != null && object.Equals(peerApplication.Id, this.Id);
		}

		public new static bool Equals(object objA, object objB)
		{
			PeerApplication peerApplication = objA as PeerApplication;
			PeerApplication peerApplication2 = objB as PeerApplication;
			return peerApplication != null && peerApplication2 != null && object.Equals(peerApplication.Id, peerApplication2.Id);
		}

		public override int GetHashCode()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return this.Id.GetHashCode();
		}

		public override string ToString()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return this.Id.ToString() + " " + this.Description;
		}

		[SecurityCritical]
		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		protected virtual void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				CollaborationHelperFunctions.CleanEventVars(ref this.m_regAppChangedWaitHandle, ref this.m_safeAppChangedEvent, ref this.m_appChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChanged variables successful.");
				this.m_Disposed = true;
			}
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_Id", this.m_id);
			info.AddValue("_Data", this.m_data);
			info.AddValue("_Description", this.m_description);
			info.AddValue("_Path", this.m_path);
			info.AddValue("_CommandLineArgs", this.m_commandLineArgs);
			info.AddValue("_Scope", this.m_peerScope);
		}

		internal void TracePeerApplication()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerApplication");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tGuid: {0}", new object[]
			{
				this.Id
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tDescription: {0}", new object[]
			{
				this.Description
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tPath: {0}", new object[]
			{
				this.Path
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tCommandLineArgs: {0}", new object[]
			{
				this.CommandLineArgs
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tPeerScope: {0}", new object[]
			{
				this.PeerScope
			});
			if (this.Data != null)
			{
				if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose))
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tApplication data:");
					Logging.DumpData(Logging.P2PTraceSource, TraceEventType.Verbose, Logging.P2PTraceSource.MaxDataSize, this.Data, 0, this.Data.Length);
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tApplication data length {0}", new object[]
					{
						this.Data.Length
					});
				}
			}
		}

		private const int c_16K = 16384;

		private Guid m_id;

		private byte[] m_data;

		private string m_description;

		private string m_path;

		private string m_commandLineArgs;

		private PeerScope m_peerScope;

		private ISynchronizeInvoke m_synchronizingObject;

		private object m_lockAppChangedEvent;

		private RegisteredWaitHandle m_regAppChangedWaitHandle;

		private AutoResetEvent m_appChangedEvent;

		private SafeCollabEvent m_safeAppChangedEvent;

		private bool m_Disposed;
	}
}
