﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	public static class PeerCollaboration
	{
		public static PeerPresenceInfo LocalPresenceInfo
		{
			[SecurityCritical]
			get
			{
				SafeCollabData safeCollabData = null;
				PeerPresenceInfo peerPresenceInfo = null;
				try
				{
					CollaborationHelperFunctions.Initialize();
					int num = UnsafeCollabNativeMethods.PeerCollabGetPresenceInfo(IntPtr.Zero, out safeCollabData);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetPresenceInfo returned with errorcode {0}", new object[]
						{
							num
						});
						return null;
					}
					IntPtr ptr = safeCollabData.DangerousGetHandle();
					PEER_PRESENCE_INFO peer_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(ptr, typeof(PEER_PRESENCE_INFO));
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Retrieved a PeerPresenceInfo with PresenceStatus {0}, Desc {1}", new object[]
					{
						peer_PRESENCE_INFO.status,
						peer_PRESENCE_INFO.descText
					});
					peerPresenceInfo = new PeerPresenceInfo();
					peerPresenceInfo.PresenceStatus = peer_PRESENCE_INFO.status;
					peerPresenceInfo.DescriptiveText = peer_PRESENCE_INFO.descText;
				}
				catch (Exception ex)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetPresenceInfo had exception {0}", new object[]
					{
						ex.Message
					});
					throw;
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				return peerPresenceInfo;
			}
			[SecurityCritical]
			set
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				if (value == null)
				{
					throw new ArgumentNullException("value");
				}
				if (value.PresenceStatus == PeerPresenceStatus.Offline)
				{
					throw new PeerToPeerException(SR.GetString("Collab_SetPresenceOffline"));
				}
				PEER_PRESENCE_INFO peer_PRESENCE_INFO;
				peer_PRESENCE_INFO.status = value.PresenceStatus;
				peer_PRESENCE_INFO.descText = value.DescriptiveText;
				int num = UnsafeCollabNativeMethods.PeerCollabSetPresenceInfo(ref peer_PRESENCE_INFO);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSetPresenceInfo returned with errorcode {0}", new object[]
					{
						num
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_SetLocalPresenceFailed"), num);
				}
			}
		}

		public static string LocalEndPointName
		{
			[SecurityCritical]
			get
			{
				string text = null;
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "LocalEndPointName get called.");
				try
				{
					int num = UnsafeCollabNativeMethods.PeerCollabGetEndpointName(ref text);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEndpointName returned with errorcode {0}", new object[]
						{
							num
						});
					}
				}
				catch (Exception ex)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "LocalEndPointName threw exception {0}", new object[]
					{
						ex.ToString()
					});
					throw;
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "LocalEndPointName get returning {0}.", new object[]
				{
					text
				});
				return text;
			}
			[SecurityCritical]
			set
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "LocalEndPointName set called with {0}.", new object[]
				{
					value
				});
				int num = UnsafeCollabNativeMethods.PeerCollabSetEndpointName(value);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSetEndpointName returned with errorcode {0}", new object[]
					{
						num
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_SetLocalEndPointNameFailed"), num);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving LocalEndPointName set.");
			}
		}

		public static PeerScope SignInScope
		{
			[SecurityCritical]
			get
			{
				PeerScope result = PeerScope.None;
				CollaborationHelperFunctions.Initialize();
				int num = UnsafeCollabNativeMethods.PeerCollabGetSigninOptions(ref result);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetSigninOptions returned with errorcode {0}", new object[]
					{
						num
					});
				}
				return result;
			}
		}

		internal static object ContactManagerSyncObject
		{
			get
			{
				if (PeerCollaboration.s_contactManagerSyncObject == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref PeerCollaboration.s_contactManagerSyncObject, value, null);
				}
				return PeerCollaboration.s_contactManagerSyncObject;
			}
		}

		public static ContactManager ContactManager
		{
			[SecurityCritical]
			get
			{
				if (PeerCollaboration.s_contactManager == null)
				{
					lock (PeerCollaboration.ContactManagerSyncObject)
					{
						if (PeerCollaboration.s_contactManager == null)
						{
							PeerCollaboration.s_contactManager = new ContactManager();
						}
					}
				}
				return PeerCollaboration.s_contactManager;
			}
		}

		public static PeerApplicationLaunchInfo ApplicationLaunchInfo
		{
			[SecurityCritical]
			get
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetApplicationLaunchInfo.");
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerApplicationLaunchInfo peerApplicationLaunchInfo = null;
				SafeCollabData safeCollabData = null;
				int num = UnsafeCollabNativeMethods.PeerCollabGetAppLaunchInfo(out safeCollabData);
				PeerApplicationLaunchInfo result;
				if (num == -2147023728)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "No Application launch info available.");
					result = peerApplicationLaunchInfo;
				}
				else if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetAppLaunchInfo returned with errorcode {0}", new object[]
					{
						num
					});
					result = peerApplicationLaunchInfo;
				}
				else
				{
					try
					{
						PEER_APP_LAUNCH_INFO peer_APP_LAUNCH_INFO = (PEER_APP_LAUNCH_INFO)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_APP_LAUNCH_INFO));
						peerApplicationLaunchInfo = new PeerApplicationLaunchInfo();
						if (peer_APP_LAUNCH_INFO.pContact != IntPtr.Zero)
						{
							PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(peer_APP_LAUNCH_INFO.pContact, typeof(PEER_CONTACT));
							peerApplicationLaunchInfo.PeerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
						}
						PEER_INVITATION peer_INVITATION = (PEER_INVITATION)Marshal.PtrToStructure(peer_APP_LAUNCH_INFO.pInvitation, typeof(PEER_INVITATION));
						peerApplicationLaunchInfo.PeerApplication = new PeerApplication(CollaborationHelperFunctions.ConvertGUIDToGuid(peer_INVITATION.applicationId), null, null, null, null, PeerScope.None);
						peerApplicationLaunchInfo.Message = peer_INVITATION.pwzMessage;
						byte[] array = null;
						if (peer_INVITATION.applicationData.cbData != 0U)
						{
							array = new byte[peer_INVITATION.applicationData.cbData];
							Marshal.Copy(peer_INVITATION.applicationData.pbData, array, 0, (int)peer_INVITATION.applicationData.cbData);
						}
						peerApplicationLaunchInfo.Data = array;
						PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(peer_APP_LAUNCH_INFO.pEndpoint, typeof(PEER_ENDPOINT));
						peerApplicationLaunchInfo.PeerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "GetApplicationLaunchInfo successful.");
					}
					catch (Exception ex)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "GetApplicationLaunchInfo threw error {0}.", new object[]
						{
							ex.ToString()
						});
						throw;
					}
					finally
					{
						if (safeCollabData != null)
						{
							safeCollabData.Dispose();
						}
					}
					result = peerApplicationLaunchInfo;
				}
				return result;
			}
		}

		[Browsable(false)]
		[DefaultValue(null)]
		[Description("SynchronizingObject")]
		public static ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				return PeerCollaboration.s_synchronizingObject;
			}
			set
			{
				PeerCollaboration.s_synchronizingObject = value;
			}
		}

		[SecurityCritical]
		public static void SignIn(PeerScope peerScope)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering SignIn with peerscope {0}.", new object[]
			{
				peerScope
			});
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerScope < PeerScope.None || peerScope > PeerScope.All)
			{
				throw new ArgumentOutOfRangeException("peerScope");
			}
			if (peerScope == PeerScope.None)
			{
				throw new ArgumentException(SR.GetString("Collab_SignInWithNone"), "peerScope");
			}
			CollaborationHelperFunctions.Initialize();
			int num = UnsafeCollabNativeMethods.PeerCollabSignin(IntPtr.Zero, peerScope);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSignin returned with errorcode {0}", new object[]
				{
					num
				});
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_SignInFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Signed into {0}.", new object[]
			{
				peerScope
			});
		}

		[SecurityCritical]
		public static void SignOut(PeerScope peerScope)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering SignOut with peerscope {0}.", new object[]
			{
				peerScope
			});
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			CollaborationHelperFunctions.Initialize();
			if (peerScope < PeerScope.None || peerScope > PeerScope.All)
			{
				throw new ArgumentOutOfRangeException("peerScope");
			}
			if (peerScope != PeerScope.None)
			{
				int num = UnsafeCollabNativeMethods.PeerCollabSignout(peerScope);
				if (num != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSignout returned with errorcode {0}", new object[]
					{
						num
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_SignOutFailed"), num);
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Signed out of {0}.", new object[]
			{
				peerScope
			});
		}

		[SecurityCritical]
		public unsafe static PeerNearMeCollection GetPeersNearMe()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetPeersNearMeGetPeersNearMe.");
			CollaborationHelperFunctions.Initialize();
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			PeerNearMeCollection peerNearMeCollection = new PeerNearMeCollection();
			SafeCollabEnum safeCollabEnum = null;
			uint num = 0U;
			try
			{
				int num2 = UnsafeCollabNativeMethods.PeerCollabEnumPeopleNearMe(out safeCollabEnum);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumPeopleNearMe returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPeersNearMeFailed"), num2);
				}
				num2 = UnsafeCollabNativeMethods.PeerGetItemCount(safeCollabEnum, ref num);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPeersNearMeFailed"), num2);
				}
				if (num == 0U)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "No peers found. \nLeaving GetPeersNearMe()");
					return peerNearMeCollection;
				}
				SafeCollabData safeCollabData;
				num2 = UnsafeCollabNativeMethods.PeerGetNextItem(safeCollabEnum, ref num, out safeCollabData);
				IntPtr value = safeCollabData.DangerousGetHandle();
				IntPtr* ptr = (IntPtr*)((void*)value);
				for (ulong num3 = 0UL; num3 < (ulong)num; num3 += 1UL)
				{
					IntPtr ptr2 = ptr[num3 * (ulong)((long)sizeof(IntPtr)) / (ulong)sizeof(IntPtr)];
					PEER_PEOPLE_NEAR_ME peer_PEOPLE_NEAR_ME = (PEER_PEOPLE_NEAR_ME)Marshal.PtrToStructure(ptr2, typeof(PEER_PEOPLE_NEAR_ME));
					PeerNearMe peerNearMe = new PeerNearMe();
					peerNearMe.Id = CollaborationHelperFunctions.ConvertGUIDToGuid(peer_PEOPLE_NEAR_ME.id);
					peerNearMe.Nickname = Marshal.PtrToStringUni(peer_PEOPLE_NEAR_ME.pwzNickname);
					PEER_ENDPOINT endpoint = peer_PEOPLE_NEAR_ME.endpoint;
					peerNearMe.PeerEndPoints.Add(CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(endpoint));
					peerNearMeCollection.Add(peerNearMe);
				}
			}
			finally
			{
				if (safeCollabEnum != null)
				{
					safeCollabEnum.Dispose();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving GetPeersNearMeGetPeersNearMe with {0} peers.", new object[]
			{
				peerNearMeCollection.Count
			});
			return peerNearMeCollection;
		}

		[SecurityCritical]
		public static void RegisterApplication(PeerApplication application, PeerApplicationRegistrationType type)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (application == null)
			{
				throw new ArgumentNullException("application");
			}
			if (application.Path == null)
			{
				throw new ArgumentException(SR.GetString("Collab_AppRegNoPathError"));
			}
			if (type < PeerApplicationRegistrationType.CurrentUser || type > PeerApplicationRegistrationType.AllUsers)
			{
				throw new ArgumentOutOfRangeException("type");
			}
			CollaborationHelperFunctions.Initialize();
			int num = 0;
			PEER_APPLICATION_REGISTRATION_INFO peer_APPLICATION_REGISTRATION_INFO = default(PEER_APPLICATION_REGISTRATION_INFO);
			peer_APPLICATION_REGISTRATION_INFO.application.guid = CollaborationHelperFunctions.ConvertGuidToGUID(application.Id);
			peer_APPLICATION_REGISTRATION_INFO.pwzApplicationArguments = application.CommandLineArgs;
			peer_APPLICATION_REGISTRATION_INFO.pwzApplicationToLaunch = application.Path;
			peer_APPLICATION_REGISTRATION_INFO.dwPublicationScope = (uint)application.PeerScope;
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RegisterApplication() is called with the following Info");
				application.TracePeerApplication();
			}
			SafeCollabMemory safeCollabMemory = null;
			peer_APPLICATION_REGISTRATION_INFO.application.data.cbData = (uint)((application.Data != null) ? application.Data.Length : 0);
			GCHandle gchandle = default(GCHandle);
			try
			{
				if (application.Data != null && application.Data.Length > 0)
				{
					safeCollabMemory = new SafeCollabMemory(application.Data.Length);
					peer_APPLICATION_REGISTRATION_INFO.application.data.pbData = safeCollabMemory.DangerousGetHandle();
					Marshal.Copy(application.Data, 0, peer_APPLICATION_REGISTRATION_INFO.application.data.pbData, application.Data.Length);
				}
				else
				{
					peer_APPLICATION_REGISTRATION_INFO.application.data.pbData = IntPtr.Zero;
				}
				gchandle = GCHandle.Alloc(application.Description, GCHandleType.Pinned);
				peer_APPLICATION_REGISTRATION_INFO.application.pwzDescription = gchandle.AddrOfPinnedObject();
				num = UnsafeCollabNativeMethods.PeerCollabRegisterApplication(ref peer_APPLICATION_REGISTRATION_INFO, type);
			}
			finally
			{
				if (gchandle.IsAllocated)
				{
					gchandle.Free();
				}
				if (safeCollabMemory != null)
				{
					safeCollabMemory.Dispose();
				}
			}
			if (num == -2147024713)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterApplication returned with errorcode {0}. Application already registered.", new object[]
				{
					num
				});
				throw new ArgumentException(SR.GetString("Collab_AppRegFailed") + " " + SR.GetString("Collab_AppExists"));
			}
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterApplication returned with errorcode {0}", new object[]
				{
					num
				});
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_AppRegFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "RegisterApplication successful");
		}

		[SecurityCritical]
		public static void UnregisterApplication(PeerApplication application, PeerApplicationRegistrationType type)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (application == null)
			{
				throw new ArgumentNullException("application");
			}
			if (application.Id.Equals(Guid.Empty))
			{
				throw new ArgumentException(SR.GetString("Collab_EmptyGuidError"));
			}
			if (type < PeerApplicationRegistrationType.CurrentUser || type > PeerApplicationRegistrationType.AllUsers)
			{
				throw new ArgumentOutOfRangeException("type");
			}
			CollaborationHelperFunctions.Initialize();
			GUID guid = CollaborationHelperFunctions.ConvertGuidToGUID(application.Id);
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "UnregisterApplication() is called with the following Info");
				application.TracePeerApplication();
			}
			int num = UnsafeCollabNativeMethods.PeerCollabUnregisterApplication(ref guid, type);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabUnregisterApplication returned with errorcode {0}", new object[]
				{
					num
				});
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_AppUnregFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "UnregisterApplication successful");
		}

		public static PeerApplicationCollection GetLocalRegisteredApplications()
		{
			PeerApplicationCollection localRegisteredApplications = PeerCollaboration.GetLocalRegisteredApplications(PeerApplicationRegistrationType.AllUsers);
			PeerApplicationCollection localRegisteredApplications2 = PeerCollaboration.GetLocalRegisteredApplications(PeerApplicationRegistrationType.CurrentUser);
			PeerApplicationCollection result;
			if (localRegisteredApplications == null || localRegisteredApplications.Count == 0)
			{
				result = localRegisteredApplications2;
			}
			else if (localRegisteredApplications2 == null || localRegisteredApplications2.Count == 0)
			{
				result = localRegisteredApplications;
			}
			else
			{
				foreach (PeerApplication peerApplication in localRegisteredApplications2)
				{
					PeerApplication peerApplication2 = null;
					foreach (PeerApplication peerApplication3 in localRegisteredApplications)
					{
						if (peerApplication3.Id.Equals(peerApplication.Id))
						{
							peerApplication2 = peerApplication3;
						}
					}
					if (peerApplication2 != null)
					{
						localRegisteredApplications.Remove(peerApplication2);
					}
				}
				foreach (PeerApplication peerApplication3 in localRegisteredApplications)
				{
					localRegisteredApplications2.Add(peerApplication3);
				}
				result = localRegisteredApplications2;
			}
			return result;
		}

		[SecurityCritical]
		public unsafe static PeerApplicationCollection GetLocalRegisteredApplications(PeerApplicationRegistrationType type)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetLocalRegisteredApplications.");
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (type < PeerApplicationRegistrationType.CurrentUser || type > PeerApplicationRegistrationType.AllUsers)
			{
				throw new ArgumentOutOfRangeException("type");
			}
			CollaborationHelperFunctions.Initialize();
			PeerApplicationCollection peerApplicationCollection = new PeerApplicationCollection();
			SafeCollabEnum safeCollabEnum = new SafeCollabEnum();
			uint num = 0U;
			try
			{
				int num2 = UnsafeCollabNativeMethods.PeerCollabEnumApplications(IntPtr.Zero, IntPtr.Zero, out safeCollabEnum);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumApplications returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalAppsFailed"), num2);
				}
				num2 = UnsafeCollabNativeMethods.PeerGetItemCount(safeCollabEnum, ref num);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalAppsFailed"), num2);
				}
				if (num == 0U)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "No local registered PeerApplications found.");
					return peerApplicationCollection;
				}
				SafeCollabData safeCollabData = null;
				try
				{
					num2 = UnsafeCollabNativeMethods.PeerGetNextItem(safeCollabEnum, ref num, out safeCollabData);
					if (num2 != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetNextItem returned with errorcode {0}", new object[]
						{
							num2
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalAppsFailed"), num2);
					}
					IntPtr value = safeCollabData.DangerousGetHandle();
					IntPtr* ptr = (IntPtr*)((void*)value);
					for (ulong num3 = 0UL; num3 < (ulong)num; num3 += 1UL)
					{
						PEER_APPLICATION* ptr2 = (PEER_APPLICATION*)((void*)ptr[num3 * (ulong)((long)sizeof(IntPtr)) / (ulong)sizeof(IntPtr)]);
						string description = Marshal.PtrToStringUni(ptr2->pwzDescription);
						byte[] array = null;
						if (ptr2->data.cbData != 0U)
						{
							array = new byte[ptr2->data.cbData];
							Marshal.Copy(ptr2->data.pbData, array, 0, (int)ptr2->data.cbData);
						}
						PeerApplication item = new PeerApplication(CollaborationHelperFunctions.ConvertGUIDToGuid(ptr2->guid), description, array, null, null, PeerScope.None);
						peerApplicationCollection.Add(item);
					}
				}
				finally
				{
					safeCollabData.Dispose();
				}
			}
			finally
			{
				safeCollabEnum.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Got all local registered Applications. Start filtering");
			PeerApplicationCollection peerApplicationCollection2 = new PeerApplicationCollection();
			foreach (PeerApplication peerApplication in peerApplicationCollection)
			{
				GUID guid = CollaborationHelperFunctions.ConvertGuidToGUID(peerApplication.Id);
				SafeCollabData safeCollabData2 = new SafeCollabData();
				try
				{
					int num2 = UnsafeCollabNativeMethods.PeerCollabGetApplicationRegistrationInfo(ref guid, type, out safeCollabData2);
					if (num2 != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetApplicationRegistrationInfo returned with errorcode {0}", new object[]
						{
							num2
						});
					}
					if (!safeCollabData2.IsInvalid)
					{
						PEER_APPLICATION_REGISTRATION_INFO peer_APPLICATION_REGISTRATION_INFO = (PEER_APPLICATION_REGISTRATION_INFO)Marshal.PtrToStructure(safeCollabData2.DangerousGetHandle(), typeof(PEER_APPLICATION_REGISTRATION_INFO));
						peerApplication.Path = peer_APPLICATION_REGISTRATION_INFO.pwzApplicationToLaunch;
						peerApplication.CommandLineArgs = peer_APPLICATION_REGISTRATION_INFO.pwzApplicationArguments;
						peerApplication.PeerScope = (PeerScope)peer_APPLICATION_REGISTRATION_INFO.dwPublicationScope;
						peerApplicationCollection2.Add(peerApplication);
					}
				}
				finally
				{
					safeCollabData2.Dispose();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Filtering successful. Returning collection with {0} applications", new object[]
			{
				peerApplicationCollection2.Count
			});
			return peerApplicationCollection2;
		}

		[SecurityCritical]
		public static void SetObject(PeerObject peerObject)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerObject == null)
			{
				throw new ArgumentNullException("peerObject");
			}
			CollaborationHelperFunctions.Initialize();
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "SetObject() is called with the following info");
				peerObject.TracePeerObject();
			}
			PEER_OBJECT peer_OBJECT = default(PEER_OBJECT);
			int num = 0;
			peer_OBJECT.guid = CollaborationHelperFunctions.ConvertGuidToGUID(peerObject.Id);
			peer_OBJECT.dwPublicationScope = (uint)peerObject.PeerScope;
			SafeCollabMemory safeCollabMemory = null;
			try
			{
				if (peerObject.Data != null && peerObject.Data.Length > 0)
				{
					safeCollabMemory = new SafeCollabMemory(peerObject.Data.Length);
					peer_OBJECT.data.pbData = safeCollabMemory.DangerousGetHandle();
					peer_OBJECT.data.cbData = (uint)peerObject.Data.Length;
					Marshal.Copy(peerObject.Data, 0, peer_OBJECT.data.pbData, peerObject.Data.Length);
				}
				else
				{
					peer_OBJECT.data.pbData = IntPtr.Zero;
					peer_OBJECT.data.cbData = 0U;
				}
				num = UnsafeCollabNativeMethods.PeerCollabSetObject(ref peer_OBJECT);
			}
			finally
			{
				if (safeCollabMemory != null)
				{
					safeCollabMemory.Dispose();
				}
			}
			if (num == -2147024713)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSetObject returned with errorcode {0}. Object already set.", new object[]
				{
					num
				});
				throw new ArgumentException(SR.GetString("Collab_ObjectSetFailed") + " " + SR.GetString("Collab_ObjectExists"));
			}
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabSetObject returned with errorcode {0}", new object[]
				{
					num
				});
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectSetFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Set Object successful.");
		}

		[SecurityCritical]
		public static void DeleteObject(PeerObject peerObject)
		{
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (peerObject == null)
			{
				throw new ArgumentNullException("peerObject");
			}
			CollaborationHelperFunctions.Initialize();
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "DeleteObject() is called with the following info");
				peerObject.TracePeerObject();
			}
			GUID guid = CollaborationHelperFunctions.ConvertGuidToGUID(peerObject.Id);
			int num = UnsafeCollabNativeMethods.PeerCollabDeleteObject(ref guid);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabDeleteObject returned with errorcode {0}", new object[]
				{
					num
				});
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectDeleteFailed"), num);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Delete Object successful.");
		}

		[SecurityCritical]
		public unsafe static PeerObjectCollection GetLocalSetObjects()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering GetLocalSetObjects.");
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			CollaborationHelperFunctions.Initialize();
			PeerObjectCollection peerObjectCollection = new PeerObjectCollection();
			SafeCollabEnum safeCollabEnum = null;
			uint num = 0U;
			try
			{
				int num2 = UnsafeCollabNativeMethods.PeerCollabEnumObjects(IntPtr.Zero, IntPtr.Zero, out safeCollabEnum);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabEnumObjects returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalObjectsFailed"), num2);
				}
				num2 = UnsafeCollabNativeMethods.PeerGetItemCount(safeCollabEnum, ref num);
				if (num2 != 0)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetItemCount returned with errorcode {0}", new object[]
					{
						num2
					});
					throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalObjectsFailed"), num2);
				}
				if (num == 0U)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "No local PeerObjects found.");
					return peerObjectCollection;
				}
				SafeCollabData safeCollabData = null;
				try
				{
					num2 = UnsafeCollabNativeMethods.PeerGetNextItem(safeCollabEnum, ref num, out safeCollabData);
					if (num2 != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerGetNextItem returned with errorcode {0}", new object[]
						{
							num2
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetLocalObjectsFailed"), num2);
					}
					IntPtr value = safeCollabData.DangerousGetHandle();
					IntPtr* ptr = (IntPtr*)((void*)value);
					for (ulong num3 = 0UL; num3 < (ulong)num; num3 += 1UL)
					{
						PEER_OBJECT* ptr2 = (PEER_OBJECT*)((void*)ptr[num3 * (ulong)((long)sizeof(IntPtr)) / (ulong)sizeof(IntPtr)]);
						byte[] array = null;
						if (ptr2->data.cbData != 0U)
						{
							array = new byte[ptr2->data.cbData];
							Marshal.Copy(ptr2->data.pbData, array, 0, (int)ptr2->data.cbData);
						}
						PeerObject item = new PeerObject(CollaborationHelperFunctions.ConvertGUIDToGuid(ptr2->guid), array, (PeerScope)ptr2->dwPublicationScope);
						peerObjectCollection.Add(item);
					}
				}
				finally
				{
					safeCollabData.Dispose();
				}
			}
			finally
			{
				safeCollabEnum.Dispose();
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Returning collection with {0} objects", new object[]
			{
				peerObjectCollection.Count
			});
			return peerObjectCollection;
		}

		private static event EventHandler<NameChangedEventArgs> s_nameChanged;

		public static event EventHandler<NameChangedEventArgs> LocalNameChanged
		{
			[SecurityCritical]
			add
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerCollaboration.AddNameChanged(value);
			}
			[SecurityCritical]
			remove
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerCollaboration.RemoveNameChanged(value);
			}
		}

		private static object LockNameChangedEvent
		{
			get
			{
				if (PeerCollaboration.s_lockNameChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref PeerCollaboration.s_lockNameChangedEvent, value, null);
				}
				return PeerCollaboration.s_lockNameChangedEvent;
			}
		}

		[SecurityCritical]
		private static void AddNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddNameChanged().");
			lock (PeerCollaboration.LockNameChangedEvent)
			{
				if (PeerCollaboration.s_nameChanged == null)
				{
					PeerCollaboration.s_nameChangedEvent = new AutoResetEvent(false);
					PeerCollaboration.s_regNameChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(PeerCollaboration.s_nameChangedEvent, new WaitOrTimerCallback(PeerCollaboration.NameChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.MyEndPointChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(PeerCollaboration.s_nameChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out PeerCollaboration.s_safeNameChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_NameChangedRegFailed"), num);
					}
				}
				PeerCollaboration.s_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Combine(PeerCollaboration.s_nameChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddNameChanged() successful.");
		}

		[SecurityCritical]
		private static void RemoveNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() called.");
			lock (PeerCollaboration.LockNameChangedEvent)
			{
				PeerCollaboration.s_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Remove(PeerCollaboration.s_nameChanged, callback);
				if (PeerCollaboration.s_nameChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref PeerCollaboration.s_regNameChangedWaitHandle, ref PeerCollaboration.s_safeNameChangedEvent, ref PeerCollaboration.s_nameChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean NameChangedEvent variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() successful.");
		}

		[SecurityCritical]
		private static void NameChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "NameChangedCallback() called.");
			for (;;)
			{
				NameChangedEventArgs nameChangedEventArgs = null;
				try
				{
					lock (PeerCollaboration.LockNameChangedEvent)
					{
						if (PeerCollaboration.s_safeNameChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(PeerCollaboration.s_safeNameChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetNameChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyEndPointChanged)
					{
						if (peer_COLLAB_EVENT_DATA.endpointChangedData.pEndPoint != IntPtr.Zero)
						{
							return;
						}
						string localEndPointName = PeerCollaboration.LocalEndPointName;
						nameChangedEventArgs = new NameChangedEventArgs(null, null, localEndPointName);
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				EventHandler<NameChangedEventArgs> eventHandler = PeerCollaboration.s_nameChanged;
				if (nameChangedEventArgs != null && eventHandler != null)
				{
					if (PeerCollaboration.SynchronizingObject != null && PeerCollaboration.SynchronizingObject.InvokeRequired)
					{
						PeerCollaboration.SynchronizingObject.BeginInvoke(eventHandler, new object[]
						{
							null,
							nameChangedEventArgs
						});
					}
					else
					{
						eventHandler(null, nameChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the name changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving NameChangedCallback().");
		}

		private static event EventHandler<PresenceChangedEventArgs> s_presenceChanged;

		public static event EventHandler<PresenceChangedEventArgs> LocalPresenceChanged
		{
			[SecurityCritical]
			add
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerCollaboration.AddPresenceChanged(value);
			}
			[SecurityCritical]
			remove
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerCollaboration.RemovePresenceChanged(value);
			}
		}

		private static object LockPresenceChangedEvent
		{
			get
			{
				if (PeerCollaboration.s_lockPresenceChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref PeerCollaboration.s_lockPresenceChangedEvent, value, null);
				}
				return PeerCollaboration.s_lockPresenceChangedEvent;
			}
		}

		[SecurityCritical]
		private static void AddPresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			CollaborationHelperFunctions.AddMyPresenceChanged(callback, ref PeerCollaboration.s_presenceChanged, PeerCollaboration.LockPresenceChangedEvent, ref PeerCollaboration.s_regPresenceChangedWaitHandle, ref PeerCollaboration.s_presenceChangedEvent, ref PeerCollaboration.s_safePresenceChangedEvent, new WaitOrTimerCallback(PeerCollaboration.PresenceChangedCallback));
		}

		[SecurityCritical]
		private static void RemovePresenceChanged(EventHandler<PresenceChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() called.");
			lock (PeerCollaboration.LockPresenceChangedEvent)
			{
				PeerCollaboration.s_presenceChanged = (EventHandler<PresenceChangedEventArgs>)Delegate.Remove(PeerCollaboration.s_presenceChanged, callback);
				if (PeerCollaboration.s_presenceChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref PeerCollaboration.s_regPresenceChangedWaitHandle, ref PeerCollaboration.s_safePresenceChangedEvent, ref PeerCollaboration.s_presenceChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean PresenceChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePresenceChanged() successful.");
		}

		[SecurityCritical]
		private static void PresenceChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PresenceChangedCallback() called.");
			for (;;)
			{
				PresenceChangedEventArgs presenceChangedEventArgs = null;
				try
				{
					lock (PeerCollaboration.LockPresenceChangedEvent)
					{
						if (PeerCollaboration.s_safePresenceChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(PeerCollaboration.s_safePresenceChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPresenceChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyPresenceChanged)
					{
						PEER_EVENT_PRESENCE_CHANGED_DATA presenceChangedData = peer_COLLAB_EVENT_DATA.presenceChangedData;
						PeerPresenceInfo peerPresenceInfo = null;
						if (presenceChangedData.pPresenceInfo != IntPtr.Zero)
						{
							PEER_PRESENCE_INFO peer_PRESENCE_INFO = (PEER_PRESENCE_INFO)Marshal.PtrToStructure(presenceChangedData.pPresenceInfo, typeof(PEER_PRESENCE_INFO));
							peerPresenceInfo = new PeerPresenceInfo();
							peerPresenceInfo.PresenceStatus = peer_PRESENCE_INFO.status;
							peerPresenceInfo.DescriptiveText = peer_PRESENCE_INFO.descText;
						}
						if (presenceChangedData.pEndPoint != IntPtr.Zero)
						{
							return;
						}
						presenceChangedEventArgs = new PresenceChangedEventArgs(null, null, presenceChangedData.changeType, peerPresenceInfo);
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				EventHandler<PresenceChangedEventArgs> eventHandler = PeerCollaboration.s_presenceChanged;
				if (presenceChangedEventArgs != null && eventHandler != null)
				{
					if (PeerCollaboration.SynchronizingObject != null && PeerCollaboration.SynchronizingObject.InvokeRequired)
					{
						PeerCollaboration.SynchronizingObject.BeginInvoke(eventHandler, new object[]
						{
							null,
							presenceChangedEventArgs
						});
					}
					else
					{
						eventHandler(null, presenceChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the presence changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving PresenceChangedCallback().");
		}

		public static event EventHandler<ObjectChangedEventArgs> LocalObjectChanged
		{
			[SecurityCritical]
			add
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerCollaboration.AddObjectChangedEvent(value);
			}
			[SecurityCritical]
			remove
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerCollaboration.RemoveObjectChangedEvent(value);
			}
		}

		private static object LockObjChangedEvent
		{
			get
			{
				if (PeerCollaboration.s_lockObjChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref PeerCollaboration.s_lockObjChangedEvent, value, null);
				}
				return PeerCollaboration.s_lockObjChangedEvent;
			}
		}

		[SecurityCritical]
		private static void AddObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			CollaborationHelperFunctions.AddMyObjectChanged(callback, ref PeerCollaboration.s_objectChanged, PeerCollaboration.LockObjChangedEvent, ref PeerCollaboration.s_regObjChangedWaitHandle, ref PeerCollaboration.s_objChangedEvent, ref PeerCollaboration.s_safeObjChangedEvent, new WaitOrTimerCallback(PeerCollaboration.ObjectChangedCallback));
		}

		[SecurityCritical]
		private static void RemoveObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() called.");
			lock (PeerCollaboration.LockObjChangedEvent)
			{
				PeerCollaboration.s_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Remove(PeerCollaboration.s_objectChanged, callback);
				if (PeerCollaboration.s_objectChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref PeerCollaboration.s_regObjChangedWaitHandle, ref PeerCollaboration.s_safeObjChangedEvent, ref PeerCollaboration.s_objChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChangedEvent variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChangedEvent() successful.");
		}

		[SecurityCritical]
		private static void ObjectChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ObjectChangedCallback() called.");
			for (;;)
			{
				ObjectChangedEventArgs objectChangedEventArgs = null;
				try
				{
					lock (PeerCollaboration.LockObjChangedEvent)
					{
						if (PeerCollaboration.s_safeObjChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(PeerCollaboration.s_safeObjChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyObjectChanged)
					{
						PEER_EVENT_OBJECT_CHANGED_DATA objectChangedData = peer_COLLAB_EVENT_DATA.objectChangedData;
						PEER_OBJECT po = (PEER_OBJECT)Marshal.PtrToStructure(objectChangedData.pObject, typeof(PEER_OBJECT));
						PeerObject peerObject = CollaborationHelperFunctions.ConvertPEER_OBJECTToPeerObject(po);
						PeerEndPoint peerEndPoint = null;
						if (objectChangedData.pEndPoint != IntPtr.Zero)
						{
							PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(objectChangedData.pEndPoint, typeof(PEER_ENDPOINT));
							peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
						}
						objectChangedEventArgs = new ObjectChangedEventArgs(peerEndPoint, null, objectChangedData.changeType, peerObject);
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				EventHandler<ObjectChangedEventArgs> eventHandler = PeerCollaboration.s_objectChanged;
				if (objectChangedEventArgs != null && eventHandler != null)
				{
					if (PeerCollaboration.SynchronizingObject != null && PeerCollaboration.SynchronizingObject.InvokeRequired)
					{
						PeerCollaboration.SynchronizingObject.BeginInvoke(eventHandler, new object[]
						{
							null,
							objectChangedEventArgs
						});
					}
					else
					{
						eventHandler(null, objectChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the object changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ObjectChangedCallback().");
		}

		private static event EventHandler<ApplicationChangedEventArgs> s_applicationChanged;

		public static event EventHandler<ApplicationChangedEventArgs> LocalApplicationChanged
		{
			[SecurityCritical]
			add
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerCollaboration.AddApplicationChanged(value);
			}
			[SecurityCritical]
			remove
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerCollaboration.RemoveApplicationChanged(value);
			}
		}

		private static object LockAppChangedEvent
		{
			get
			{
				if (PeerCollaboration.s_lockAppChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref PeerCollaboration.s_lockAppChangedEvent, value, null);
				}
				return PeerCollaboration.s_lockAppChangedEvent;
			}
		}

		[SecurityCritical]
		private static void AddApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			CollaborationHelperFunctions.AddMyApplicationChanged(callback, ref PeerCollaboration.s_applicationChanged, PeerCollaboration.LockAppChangedEvent, ref PeerCollaboration.s_regAppChangedWaitHandle, ref PeerCollaboration.s_appChangedEvent, ref PeerCollaboration.s_safeAppChangedEvent, new WaitOrTimerCallback(PeerCollaboration.ApplicationChangedCallback));
		}

		[SecurityCritical]
		private static void RemoveApplicationChanged(EventHandler<ApplicationChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() called.");
			lock (PeerCollaboration.LockAppChangedEvent)
			{
				PeerCollaboration.s_applicationChanged = (EventHandler<ApplicationChangedEventArgs>)Delegate.Remove(PeerCollaboration.s_applicationChanged, callback);
				if (PeerCollaboration.s_applicationChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref PeerCollaboration.s_regAppChangedWaitHandle, ref PeerCollaboration.s_safeAppChangedEvent, ref PeerCollaboration.s_appChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ApplicationChangEvent variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveApplicationChanged() successful.");
		}

		[SecurityCritical]
		private static void ApplicationChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ApplicationChangedCallback() called.");
			for (;;)
			{
				ApplicationChangedEventArgs applicationChangedEventArgs = null;
				try
				{
					lock (PeerCollaboration.LockAppChangedEvent)
					{
						if (PeerCollaboration.s_safeAppChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(PeerCollaboration.s_safeAppChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetApplicationChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.MyApplicationChanged)
					{
						PEER_EVENT_APPLICATION_CHANGED_DATA applicationChangedData = peer_COLLAB_EVENT_DATA.applicationChangedData;
						PEER_APPLICATION pa = (PEER_APPLICATION)Marshal.PtrToStructure(applicationChangedData.pApplication, typeof(PEER_APPLICATION));
						PeerApplication peerApplication = CollaborationHelperFunctions.ConvertPEER_APPLICATIONToPeerApplication(pa);
						if (applicationChangedData.pEndPoint != IntPtr.Zero)
						{
							return;
						}
						applicationChangedEventArgs = new ApplicationChangedEventArgs(null, null, applicationChangedData.changeType, peerApplication);
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				EventHandler<ApplicationChangedEventArgs> eventHandler = PeerCollaboration.s_applicationChanged;
				if (applicationChangedEventArgs != null && eventHandler != null)
				{
					if (PeerCollaboration.SynchronizingObject != null && PeerCollaboration.SynchronizingObject.InvokeRequired)
					{
						PeerCollaboration.SynchronizingObject.BeginInvoke(eventHandler, new object[]
						{
							null,
							applicationChangedEventArgs
						});
					}
					else
					{
						eventHandler(null, applicationChangedEventArgs);
					}
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the application changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ApplicationChangedCallback().");
		}

		private static object s_contactManagerSyncObject;

		private static ContactManager s_contactManager;

		private static ISynchronizeInvoke s_synchronizingObject;

		private static object s_lockNameChangedEvent;

		private static RegisteredWaitHandle s_regNameChangedWaitHandle;

		private static SafeCollabEvent s_safeNameChangedEvent;

		private static object s_lockPresenceChangedEvent;

		private static RegisteredWaitHandle s_regPresenceChangedWaitHandle;

		private static SafeCollabEvent s_safePresenceChangedEvent;

		private static EventHandler<ObjectChangedEventArgs> s_objectChanged;

		private static object s_lockObjChangedEvent;

		private static RegisteredWaitHandle s_regObjChangedWaitHandle;

		private static AutoResetEvent s_objChangedEvent;

		private static SafeCollabEvent s_safeObjChangedEvent;

		private static object s_lockAppChangedEvent;

		private static RegisteredWaitHandle s_regAppChangedWaitHandle;

		private static AutoResetEvent s_appChangedEvent;

		private static SafeCollabEvent s_safeAppChangedEvent;
	}
}
