﻿using System;
using System.Security;
using System.Security.Permissions;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public sealed class PeerCollaborationPermission : CodeAccessPermission, IUnrestrictedPermission
	{
		public PeerCollaborationPermission(PermissionState state)
		{
			this.m_noRestriction = (state == PermissionState.Unrestricted);
		}

		internal PeerCollaborationPermission(bool free)
		{
			this.m_noRestriction = free;
		}

		public bool IsUnrestricted()
		{
			return this.m_noRestriction;
		}

		public override IPermission Copy()
		{
			IPermission result;
			if (this.m_noRestriction)
			{
				result = new PeerCollaborationPermission(true);
			}
			else
			{
				result = new PeerCollaborationPermission(false);
			}
			return result;
		}

		public override IPermission Union(IPermission target)
		{
			IPermission result;
			if (target == null)
			{
				result = this.Copy();
			}
			else
			{
				PeerCollaborationPermission peerCollaborationPermission = target as PeerCollaborationPermission;
				if (peerCollaborationPermission == null)
				{
					throw new ArgumentException(SR.GetString("Collab_PermissionUnionError"), "target");
				}
				result = new PeerCollaborationPermission(this.m_noRestriction || peerCollaborationPermission.m_noRestriction);
			}
			return result;
		}

		public override IPermission Intersect(IPermission target)
		{
			IPermission result;
			if (target == null)
			{
				result = null;
			}
			else
			{
				PeerCollaborationPermission peerCollaborationPermission = target as PeerCollaborationPermission;
				if (peerCollaborationPermission == null)
				{
					throw new ArgumentException(SR.GetString("Collab_PermissionIntersectError"), "target");
				}
				if (this.m_noRestriction && peerCollaborationPermission.m_noRestriction)
				{
					result = new PeerCollaborationPermission(true);
				}
				else
				{
					result = null;
				}
			}
			return result;
		}

		public override bool IsSubsetOf(IPermission target)
		{
			bool result;
			if (target == null)
			{
				result = !this.m_noRestriction;
			}
			else
			{
				PeerCollaborationPermission peerCollaborationPermission = target as PeerCollaborationPermission;
				if (peerCollaborationPermission == null)
				{
					throw new ArgumentException(SR.GetString("Collab_BadPermissionTarget"), "target");
				}
				result = (!this.m_noRestriction || peerCollaborationPermission.m_noRestriction);
			}
			return result;
		}

		public override void FromXml(SecurityElement e)
		{
			if (e == null)
			{
				throw new ArgumentNullException("e");
			}
			if (!e.Tag.Equals("IPermission"))
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElem"), "e");
			}
			string text = e.Attribute("class");
			if (text == null)
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElemNoClass"), "e");
			}
			if (text.IndexOf(base.GetType().FullName, StringComparison.Ordinal) < 0)
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElemNoType"), "e");
			}
			string text2 = e.Attribute("Unrestricted");
			this.m_noRestriction = (text2 != null && 0 == string.Compare(text2, "true", StringComparison.OrdinalIgnoreCase));
		}

		public override SecurityElement ToXml()
		{
			SecurityElement securityElement = new SecurityElement("IPermission");
			securityElement.AddAttribute("class", base.GetType().FullName + ", " + base.GetType().Module.Assembly.FullName.Replace('"', '\''));
			securityElement.AddAttribute("version", "1");
			if (this.m_noRestriction)
			{
				securityElement.AddAttribute("Unrestricted", "true");
			}
			return securityElement;
		}

		private bool m_noRestriction;

		internal static readonly PeerCollaborationPermission UnrestrictedPeerCollaborationPermission = new PeerCollaborationPermission(PermissionState.Unrestricted);
	}
}
