﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerEndPoint : IDisposable, IEquatable<PeerEndPoint>, ISerializable
	{
		public PeerEndPoint()
		{
		}

		public PeerEndPoint(IPEndPoint endPoint) : this(endPoint, null)
		{
		}

		public PeerEndPoint(IPEndPoint endPoint, string endPointName)
		{
			if (endPoint == null)
			{
				throw new ArgumentNullException("endPoint");
			}
			if (this.m_endPoint != null && this.m_endPoint.AddressFamily != AddressFamily.InterNetworkV6)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerEndPoint Endpoint set parameter is not IPv6.");
				throw new ArgumentException("endPoint", SR.GetString("Collab_EndPointNotIPv6Error"));
			}
			this.m_endPoint = endPoint;
			this.m_endPointName = endPointName;
		}

		protected PeerEndPoint(SerializationInfo serializationInfo, StreamingContext streamingContext)
		{
			this.m_endPointName = serializationInfo.GetString("_EndPointName");
			this.m_endPoint = (IPEndPoint)serializationInfo.GetValue("_EndPoint", typeof(IPEndPoint));
		}

		public string Name
		{
			get
			{
				return this.m_endPointName;
			}
			set
			{
				this.m_endPointName = value;
			}
		}

		public IPEndPoint EndPoint
		{
			get
			{
				return this.m_endPoint;
			}
			set
			{
				if (this.m_endPoint != null && this.m_endPoint.AddressFamily != AddressFamily.InterNetworkV6)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerEndPoint Endpoint set parameter is not IPv6.");
					throw new PeerToPeerException(SR.GetString("Collab_EndPointNotIPv6Error"));
				}
				this.m_endPoint = value;
			}
		}

		[Description("SynchronizingObject")]
		[Browsable(false)]
		[DefaultValue(null)]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_synchronizingObject;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_synchronizingObject = value;
			}
		}

		private event EventHandler<NameChangedEventArgs> m_nameChanged;

		public event EventHandler<NameChangedEventArgs> NameChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				CollaborationHelperFunctions.Initialize();
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddNameChanged(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				CollaborationHelperFunctions.Initialize();
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemoveNameChanged(value);
			}
		}

		private object LockNameChangedEvent
		{
			get
			{
				if (this.m_lockNameChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockNameChangedEvent, value, null);
				}
				return this.m_lockNameChangedEvent;
			}
		}

		[SecurityCritical]
		private void AddNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddNameChanged().");
			lock (this.LockNameChangedEvent)
			{
				if (this.m_nameChanged == null)
				{
					this.m_nameChangedEvent = new AutoResetEvent(false);
					this.m_regNameChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_nameChangedEvent, new WaitOrTimerCallback(this.NameChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.EndPointChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_nameChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeNameChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_NameChangedRegFailed"), num);
					}
				}
				this.m_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Combine(this.m_nameChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddNameChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveNameChanged(EventHandler<NameChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() called.");
			lock (this.LockNameChangedEvent)
			{
				this.m_nameChanged = (EventHandler<NameChangedEventArgs>)Delegate.Remove(this.m_nameChanged, callback);
				if (this.m_nameChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref this.m_regNameChangedWaitHandle, ref this.m_safeNameChangedEvent, ref this.m_nameChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean NameChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveNameChanged() successful.");
		}

		[SecurityCritical]
		private void NameChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "NameChangedCallback() called.");
			if (!this.m_Disposed)
			{
				for (;;)
				{
					NameChangedEventArgs nameChangedEventArgs = null;
					try
					{
						lock (this.LockNameChangedEvent)
						{
							if (this.m_safeNameChangedEvent.IsInvalid)
							{
								return;
							}
							num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeNameChangedEvent, out safeCollabData);
						}
						if (num == 6488066)
						{
							break;
						}
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
							{
								num
							});
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetNameChangedDataFailed"), num);
						}
						PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointChanged)
						{
							PEER_EVENT_ENDPOINT_CHANGED_DATA endpointChangedData = peer_COLLAB_EVENT_DATA.endpointChangedData;
							PeerEndPoint peerEndPoint = null;
							if (endpointChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(endpointChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							if (peerEndPoint != null && this.Equals(peerEndPoint))
							{
								PeerContact peerContact = null;
								if (endpointChangedData.pContact != IntPtr.Zero)
								{
									PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(endpointChangedData.pContact, typeof(PEER_CONTACT));
									peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
								}
								nameChangedEventArgs = new NameChangedEventArgs(peerEndPoint, peerContact, peerEndPoint.Name);
							}
						}
					}
					finally
					{
						if (safeCollabData != null)
						{
							safeCollabData.Dispose();
						}
					}
					if (nameChangedEventArgs != null)
					{
						this.OnNameChanged(nameChangedEventArgs);
						this.Name = nameChangedEventArgs.PeerEndPoint.Name;
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving NameChangedCallback().");
			}
		}

		protected void OnNameChanged(NameChangedEventArgs nameChangedArgs)
		{
			EventHandler<NameChangedEventArgs> nameChanged = this.m_nameChanged;
			if (nameChanged != null)
			{
				if (this.SynchronizingObject != null && this.SynchronizingObject.InvokeRequired)
				{
					this.SynchronizingObject.BeginInvoke(nameChanged, new object[]
					{
						this,
						nameChangedArgs
					});
				}
				else
				{
					nameChanged(this, nameChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the name changed event callback.");
			}
		}

		public bool Equals(PeerEndPoint other)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return other != null && other.EndPoint.Equals(this.EndPoint);
		}

		public override bool Equals(object obj)
		{
			PeerEndPoint peerEndPoint = obj as PeerEndPoint;
			return peerEndPoint != null && peerEndPoint.EndPoint.Equals(this.EndPoint);
		}

		public new static bool Equals(object objA, object objB)
		{
			PeerEndPoint peerEndPoint = objA as PeerEndPoint;
			PeerEndPoint peerEndPoint2 = objB as PeerEndPoint;
			return peerEndPoint != null && peerEndPoint2 != null && peerEndPoint.EndPoint.Equals(peerEndPoint2.EndPoint);
		}

		public override int GetHashCode()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return this.EndPoint.GetHashCode();
		}

		public override string ToString()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return this.m_endPointName;
		}

		[SecurityCritical]
		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		protected virtual void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				CollaborationHelperFunctions.CleanEventVars(ref this.m_regNameChangedWaitHandle, ref this.m_safeNameChangedEvent, ref this.m_nameChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean NameChanged variables successful.");
			}
			this.m_Disposed = true;
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_EndPointName", this.m_endPointName);
			info.AddValue("_EndPoint", this.m_endPoint);
		}

		internal void TracePeerEndPoint()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerEndPoint");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tEndPoint: {0}", new object[]
			{
				(this.EndPoint != null) ? this.EndPoint.ToString() : null
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tDescription: {0}", new object[]
			{
				this.Name
			});
		}

		private string m_endPointName;

		private IPEndPoint m_endPoint;

		private ISynchronizeInvoke m_synchronizingObject;

		private object m_lockNameChangedEvent;

		private RegisteredWaitHandle m_regNameChangedWaitHandle;

		private SafeCollabEvent m_safeNameChangedEvent;

		private bool m_Disposed;
	}
}
