﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Net.Mail;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerNearMe : Peer, IEquatable<PeerNearMe>, ISerializable
	{
		[SecurityCritical]
		static PeerNearMe()
		{
			CollaborationHelperFunctions.Initialize();
		}

		public PeerNearMe()
		{
			this.OnRefreshDataCompletedDelegate = new SendOrPostCallback(this.RefreshDataCompletedWaitCallback);
		}

		protected PeerNearMe(SerializationInfo serializationInfo, StreamingContext streamingContext) : base(serializationInfo, streamingContext)
		{
			this.m_id = (Guid)serializationInfo.GetValue("_Id", typeof(Guid));
			this.m_nickname = serializationInfo.GetString("_NickName");
			this.OnRefreshDataCompletedDelegate = new SendOrPostCallback(this.RefreshDataCompletedWaitCallback);
		}

		public string Nickname
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_nickname;
			}
			internal set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_nickname = value;
			}
		}

		internal Guid Id
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_id;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_id = value;
			}
		}

		public PeerContact AddToContactManager()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddToContactManager.");
			PeerContact peerContact = null;
			try
			{
				peerContact = PeerCollaboration.ContactManager.CreateContact(this);
				PeerCollaboration.ContactManager.AddContact(peerContact);
			}
			catch (Exception ex)
			{
				throw new PeerToPeerException(SR.GetString("Collab_AddToContactMgrFailed"), (ex.InnerException != null) ? ex.InnerException : ex);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving AddToContactManager.");
			return peerContact;
		}

		public PeerContact AddToContactManager(string displayName, string nickname, MailAddress emailAddress)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddToContactManager with Display name: {0} Nickname: {1} and Email Address: {2}", new object[]
			{
				displayName,
				nickname,
				emailAddress
			});
			PeerContact peerContact = null;
			try
			{
				peerContact = PeerCollaboration.ContactManager.CreateContact(this);
				PeerCollaboration.ContactManager.AddContact(peerContact);
			}
			catch (Exception ex)
			{
				throw new PeerToPeerException(SR.GetString("Collab_AddToContactMgrFailed"), (ex.InnerException != null) ? ex.InnerException : ex);
			}
			peerContact.DisplayName = displayName;
			peerContact.Nickname = nickname;
			peerContact.EmailAddress = emailAddress;
			try
			{
				PeerCollaboration.ContactManager.UpdateContact(peerContact);
			}
			catch (Exception ex)
			{
				throw new PeerToPeerException(SR.GetString("Collab_AddToContactMgrFailed") + " " + SR.GetString("Collab_AddToContactMgrFailedUpdate"), (ex.InnerException != null) ? ex.InnerException : ex);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving AddToContactManager.");
			return peerContact;
		}

		[SecurityCritical]
		public static PeerNearMe CreateFromPeerEndPoint(PeerEndPoint peerEndPoint)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering CreateFromPeerEndPoint.");
			CollaborationHelperFunctions.Initialize();
			if (peerEndPoint == null)
			{
				throw new ArgumentNullException("peerEndPoint");
			}
			if (peerEndPoint.EndPoint == null)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndPointInPeerEndPoint"));
			}
			PeerNearMeCollection peersNearMe = PeerCollaboration.GetPeersNearMe();
			PeerNearMe peerNearMe = null;
			foreach (PeerNearMe peerNearMe2 in peersNearMe)
			{
				PeerEndPointCollection peerEndPoints = peerNearMe2.PeerEndPoints;
				if (peerEndPoints != null && peerEndPoints.Count != 0 && peerEndPoints[0].Equals(peerEndPoint))
				{
					peerNearMe = peerNearMe2;
				}
			}
			if (peerNearMe == null)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EndPointNotAPeerNearMe"));
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving CreateFromPeerEndPoint.");
			return peerNearMe;
		}

		internal override void RefreshIfNeeded()
		{
			this.RefreshData();
		}

		[SecurityCritical]
		public void RefreshData()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			this.InternalRefreshData(false);
		}

		[SecurityCritical]
		protected internal void InternalRefreshData(object state)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "InternalRefreshEndpointData called.");
			bool flag = (bool)state;
			Exception ex = null;
			AutoResetEvent autoResetEvent = new AutoResetEvent(false);
			PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
			peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.RequestStatusChanged;
			peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
			SafeCollabEvent hPeerEvent;
			int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(autoResetEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out hPeerEvent);
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
				{
					num
				});
				ex = PeerToPeerException.CreateFromHr(SR.GetString("Collab_ReqStatusChangedRegFailed"), num);
				if (!flag)
				{
					throw ex;
				}
			}
			PeerEndPointCollection peerEndPoints = this.PeerEndPoints;
			if (peerEndPoints.Count != 0)
			{
				try
				{
					PeerNearMe.InternalRefreshData(peerEndPoints[0]);
				}
				catch (Exception ex2)
				{
					if (!flag)
					{
						throw;
					}
					ex = ex2;
				}
				while (ex == null)
				{
					autoResetEvent.WaitOne();
					SafeCollabData safeCollabData;
					num = UnsafeCollabNativeMethods.PeerCollabGetEventData(hPeerEvent, out safeCollabData);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						ex = PeerToPeerException.CreateFromHr(SR.GetString("Collab_RefreshDataFailed"), num);
						if (!flag)
						{
							throw ex;
						}
						break;
					}
					else
					{
						PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
						if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.RequestStatusChanged)
						{
							PEER_EVENT_REQUEST_STATUS_CHANGED_DATA requestStatusChangedData = peer_COLLAB_EVENT_DATA.requestStatusChangedData;
							PeerEndPoint peerEndPoint = null;
							if (requestStatusChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(requestStatusChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							if (requestStatusChangedData.hrChange < 0)
							{
								ex = PeerToPeerException.CreateFromHr(SR.GetString("Collab_RefreshDataFailed"), requestStatusChangedData.hrChange);
							}
							if (ex != null)
							{
								if (!flag)
								{
									throw ex;
								}
							}
							if (this.PeerEndPoints[0].Equals(peerEndPoint))
							{
								Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Found endpoint match in Request status changed.");
								if (flag)
								{
									RefreshDataCompletedEventArgs args = new RefreshDataCompletedEventArgs(peerEndPoint, null, false, this.m_refreshDataAsyncOp.UserSuppliedState);
									if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
									{
										Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Firing RefreshDataCompleted event with folloding peer endpoint.");
										peerEndPoint.TracePeerEndPoint();
									}
									this.PrepareToRaiseRefreshDataCompletedEvent(this.m_refreshDataAsyncOp, args);
								}
								break;
							}
						}
					}
				}
				if (ex != null)
				{
					RefreshDataCompletedEventArgs args = new RefreshDataCompletedEventArgs(null, ex, false, this.m_refreshDataAsyncOp.UserSuppliedState);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Firing RefreshDataCompleted event with exception {0}.", new object[]
					{
						ex
					});
					this.PrepareToRaiseRefreshDataCompletedEvent(this.m_refreshDataAsyncOp, args);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving InternalRefreshEndpointData.");
			}
		}

		[SecurityCritical]
		internal static void InternalRefreshData(PeerEndPoint peerEndPoint)
		{
			PEER_ENDPOINT peer_ENDPOINT = default(PEER_ENDPOINT);
			peer_ENDPOINT.peerAddress = CollaborationHelperFunctions.ConvertIPEndpointToPEER_ADDRESS(peerEndPoint.EndPoint);
			GCHandle gchandle = GCHandle.Alloc(peerEndPoint.Name, GCHandleType.Pinned);
			peer_ENDPOINT.pwzEndpointName = gchandle.AddrOfPinnedObject();
			GCHandle gchandle2 = GCHandle.Alloc(peer_ENDPOINT, GCHandleType.Pinned);
			IntPtr pcEndpoint = gchandle2.AddrOfPinnedObject();
			int num;
			try
			{
				num = UnsafeCollabNativeMethods.PeerCollabRefreshEndpointData(pcEndpoint);
			}
			finally
			{
				if (gchandle.IsAllocated)
				{
					gchandle.Free();
				}
				if (gchandle2.IsAllocated)
				{
					gchandle2.Free();
				}
			}
			if (num != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRefreshEndpointData returned with errorcode {0}", new object[]
				{
					num
				});
				throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_RefreshDataFailed"), num);
			}
		}

		private object RefreshDataAsyncOpLock
		{
			get
			{
				if (this.m_refreshDataAsyncOpLock == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_refreshDataAsyncOpLock, value, null);
				}
				return this.m_refreshDataAsyncOpLock;
			}
		}

		private event EventHandler<RefreshDataCompletedEventArgs> m_refreshDataCompleted;

		public event EventHandler<RefreshDataCompletedEventArgs> RefreshDataCompleted
		{
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_refreshDataCompleted = (EventHandler<RefreshDataCompletedEventArgs>)Delegate.Combine(this.m_refreshDataCompleted, value);
			}
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.m_refreshDataCompleted = (EventHandler<RefreshDataCompletedEventArgs>)Delegate.Remove(this.m_refreshDataCompleted, value);
			}
		}

		[SecurityCritical]
		public void RefreshDataAsync(object userToken)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentNullException("userToken");
			}
			lock (this.RefreshDataAsyncOpLock)
			{
				if (this.m_refreshDataAsyncOp != null)
				{
					throw new PeerToPeerException(SR.GetString("Collab_DuplicateRefreshAsync"));
				}
				this.m_refreshDataAsyncOp = AsyncOperationManager.CreateOperation(userToken);
			}
			ThreadPool.QueueUserWorkItem(new WaitCallback(this.InternalRefreshData), true);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving RefreshDataAsync().");
		}

		protected void OnRefreshDataCompleted(RefreshDataCompletedEventArgs e)
		{
			EventHandler<RefreshDataCompletedEventArgs> refreshDataCompleted = this.m_refreshDataCompleted;
			if (refreshDataCompleted != null)
			{
				refreshDataCompleted(this, e);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the refresh endpoint completed event callback.");
			}
		}

		private void RefreshDataCompletedWaitCallback(object operationState)
		{
			this.m_refreshDataAsyncOp = null;
			this.OnRefreshDataCompleted((RefreshDataCompletedEventArgs)operationState);
		}

		internal void PrepareToRaiseRefreshDataCompletedEvent(AsyncOperation asyncOP, RefreshDataCompletedEventArgs args)
		{
			asyncOP.PostOperationCompleted(this.OnRefreshDataCompletedDelegate, args);
		}

		private static event EventHandler<PeerNearMeChangedEventArgs> s_peerNearMeChanged;

		public static event EventHandler<PeerNearMeChangedEventArgs> PeerNearMeChanged
		{
			[SecurityCritical]
			add
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerNearMe.AddPeerNearMeChanged(value);
			}
			[SecurityCritical]
			remove
			{
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				CollaborationHelperFunctions.Initialize();
				PeerNearMe.RemovePeerNearMeChanged(value);
			}
		}

		private static object LockPNMChangedEvent
		{
			get
			{
				if (PeerNearMe.s_lockPNMChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref PeerNearMe.s_lockPNMChangedEvent, value, null);
				}
				return PeerNearMe.s_lockPNMChangedEvent;
			}
		}

		[SecurityCritical]
		private static void AddPeerNearMeChanged(EventHandler<PeerNearMeChangedEventArgs> cb)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Entering AddPeerNearMeChanged().");
			lock (PeerNearMe.LockPNMChangedEvent)
			{
				if (PeerNearMe.s_peerNearMeChanged == null)
				{
					PeerNearMe.s_peerNearMeChangedEvent = new AutoResetEvent(false);
					PeerNearMe.s_registeredPNMWaitHandle = ThreadPool.RegisterWaitForSingleObject(PeerNearMe.s_peerNearMeChangedEvent, new WaitOrTimerCallback(PeerNearMe.PeerNearMeChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.PeopleNearMeChanged;
					peer_COLLAB_EVENT_REGISTRATION.pInstance = IntPtr.Zero;
					int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(PeerNearMe.s_peerNearMeChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out PeerNearMe.s_safePeerNearMeChangedEvent);
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_PeerNearMeChangedRegFailed"), num);
					}
				}
				PeerNearMe.s_peerNearMeChanged = (EventHandler<PeerNearMeChangedEventArgs>)Delegate.Combine(PeerNearMe.s_peerNearMeChanged, cb);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddPeerNearMeChanged() successful.");
		}

		[SecurityCritical]
		private static void RemovePeerNearMeChanged(EventHandler<PeerNearMeChangedEventArgs> cb)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePeerNearMeChanged() called.");
			lock (PeerNearMe.LockPNMChangedEvent)
			{
				PeerNearMe.s_peerNearMeChanged = (EventHandler<PeerNearMeChangedEventArgs>)Delegate.Remove(PeerNearMe.s_peerNearMeChanged, cb);
				if (PeerNearMe.s_peerNearMeChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref PeerNearMe.s_registeredPNMWaitHandle, ref PeerNearMe.s_safePeerNearMeChangedEvent, ref PeerNearMe.s_peerNearMeChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean PeerNearMeChanged variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemovePeerNearMeChanged() successful.");
		}

		[SecurityCritical]
		private static void PeerNearMeChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerNearMeChangedCallback() called.");
			for (;;)
			{
				PeerNearMeChangedEventArgs peerNearMeChangedEventArgs = null;
				try
				{
					lock (PeerNearMe.LockPNMChangedEvent)
					{
						if (PeerNearMe.s_safePeerNearMeChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(PeerNearMe.s_safePeerNearMeChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetPeerNearMeChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.PeopleNearMeChanged)
					{
						PEER_EVENT_PEOPLE_NEAR_ME_CHANGED_DATA peopleNearMeChangedData = peer_COLLAB_EVENT_DATA.peopleNearMeChangedData;
						PeerNearMe peerNearMe = null;
						if (peopleNearMeChangedData.pPeopleNearMe != IntPtr.Zero)
						{
							PEER_PEOPLE_NEAR_ME ppnm = (PEER_PEOPLE_NEAR_ME)Marshal.PtrToStructure(peopleNearMeChangedData.pPeopleNearMe, typeof(PEER_PEOPLE_NEAR_ME));
							peerNearMe = CollaborationHelperFunctions.PEER_PEOPLE_NEAR_METoPeerNearMe(ppnm);
						}
						peerNearMeChangedEventArgs = new PeerNearMeChangedEventArgs(peerNearMe, peopleNearMeChangedData.changeType);
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				EventHandler<PeerNearMeChangedEventArgs> eventHandler = PeerNearMe.s_peerNearMeChanged;
				if (peerNearMeChangedEventArgs != null && eventHandler != null)
				{
					eventHandler(null, peerNearMeChangedEventArgs);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the peer near me changed event callback.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving PeerNearMeChangedCallback().");
		}

		[SecurityCritical]
		public override PeerInvitationResponse Invite(PeerApplication applicationToInvite, string message, byte[] invitationData)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			PeerEndPointCollection peerEndPoints = this.PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			PeerEndPoint peerEndPoint = this.PeerEndPoints[0];
			PeerInvitationResponse peerInvitationResponse = Peer.InternalInviteEndPoint(applicationToInvite.Id, message, invitationData, peerEndPoint, null);
			CollaborationHelperFunctions.ThrowIfInvitationResponseInvalid(peerInvitationResponse);
			return peerInvitationResponse;
		}

		[SecurityCritical]
		public override PeerInvitationResponse Invite()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			PeerEndPointCollection peerEndPoints = this.PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			PeerEndPoint peerEndPoint = this.PeerEndPoints[0];
			PeerInvitationResponse peerInvitationResponse = Peer.InternalInviteEndPoint(currentApplicationGuid, null, null, peerEndPoint, null);
			CollaborationHelperFunctions.ThrowIfInvitationResponseInvalid(peerInvitationResponse);
			return peerInvitationResponse;
		}

		[SecurityCritical]
		public override void InviteAsync(object userToken)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			Guid currentApplicationGuid = Peer.CurrentApplicationGuid;
			if (currentApplicationGuid.Equals(Guid.Empty))
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoGuidForCurrApp"));
			}
			PeerEndPointCollection peerEndPoints = this.PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			base.InternalInviteAsync(currentApplicationGuid, null, null, this.PeerEndPoints, null, userToken);
		}

		[SecurityCritical]
		public override void InviteAsync(PeerApplication applicationToInvite, string message, byte[] invitationData, object userToken)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
			if (applicationToInvite == null)
			{
				throw new ArgumentNullException("applicationToInvite");
			}
			if (applicationToInvite.Id == Guid.Empty)
			{
				throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
			}
			if (userToken == null)
			{
				throw new ArgumentException(SR.GetString("NullUserToken"));
			}
			PeerEndPointCollection peerEndPoints = this.PeerEndPoints;
			if (peerEndPoints == null || peerEndPoints.Count == 0)
			{
				throw new PeerToPeerException(SR.GetString("Collab_NoEndpointFound"));
			}
			base.InternalInviteAsync(applicationToInvite.Id, message, invitationData, peerEndPoints, null, userToken);
		}

		public bool Equals(PeerNearMe other)
		{
			if (other == null)
			{
				throw new ArgumentNullException("other");
			}
			return other.Id.Equals(this.Id);
		}

		public override bool Equals(object obj)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerNearMe peerNearMe = obj as PeerNearMe;
			return peerNearMe != null && object.Equals(peerNearMe.Id, this.Id);
		}

		public new static bool Equals(object objA, object objB)
		{
			PeerNearMe peerNearMe = objA as PeerNearMe;
			PeerNearMe peerNearMe2 = objB as PeerNearMe;
			return peerNearMe != null && peerNearMe2 != null && object.Equals(peerNearMe.Id, peerNearMe2.Id);
		}

		public override int GetHashCode()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return this.Id.GetHashCode();
		}

		public override string ToString()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return this.Nickname;
		}

		protected override void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				try
				{
					this.m_Disposed = true;
				}
				finally
				{
					base.Dispose(disposing);
				}
			}
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
			info.AddValue("_Id", this.m_id);
			info.AddValue("_NickName", this.m_nickname);
		}

		internal void TracePeerNearMe()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerNearMe");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tNickname: {0}", new object[]
			{
				this.Nickname
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tID: {0}", new object[]
			{
				this.Id
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tNumber of Endpoints: {0}", new object[]
			{
				this.PeerEndPoints.Count
			});
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tEndPoints:");
				foreach (PeerEndPoint peerEndPoint in this.PeerEndPoints)
				{
					peerEndPoint.TracePeerEndPoint();
				}
			}
		}

		private string m_nickname;

		private Guid m_id;

		private AsyncOperation m_refreshDataAsyncOp;

		private object m_refreshDataAsyncOpLock;

		private SendOrPostCallback OnRefreshDataCompletedDelegate;

		private static object s_lockPNMChangedEvent;

		private static RegisteredWaitHandle s_registeredPNMWaitHandle;

		private static SafeCollabEvent s_safePeerNearMeChangedEvent;

		private bool m_Disposed;
	}
}
