﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Threading;

namespace System.Net.PeerToPeer.Collaboration
{
	[Serializable]
	public class PeerObject : IDisposable, IEquatable<PeerObject>, ISerializable
	{
		[SecurityCritical]
		static PeerObject()
		{
			CollaborationHelperFunctions.Initialize();
		}

		public PeerObject()
		{
			this.m_id = Guid.NewGuid();
		}

		public PeerObject(Guid Id, byte[] data, PeerScope peerScope)
		{
			if (data != null && data.Length > 16384)
			{
				throw new ArgumentException(SR.GetString("Collab_ObjectDataSizeFailed"), "data");
			}
			this.m_id = Id;
			this.m_peerScope = peerScope;
			this.m_data = data;
		}

		protected PeerObject(SerializationInfo serializationInfo, StreamingContext streamingContext)
		{
			this.m_id = (Guid)serializationInfo.GetValue("_Id", typeof(Guid));
			this.m_data = (byte[])serializationInfo.GetValue("_Data", typeof(byte[]));
			this.m_peerScope = (PeerScope)serializationInfo.GetInt32("_Scope");
		}

		public Guid Id
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_id;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_id = value;
			}
		}

		public byte[] Data
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_data;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				if (value != null && value.Length > 16384)
				{
					throw new ArgumentException(SR.GetString("Collab_ObjectDataSizeFailed"));
				}
				this.m_data = value;
			}
		}

		public PeerScope PeerScope
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_peerScope;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_peerScope = value;
			}
		}

		[Browsable(false)]
		[Description("SynchronizingObject")]
		[DefaultValue(null)]
		public ISynchronizeInvoke SynchronizingObject
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_synchronizingObject;
			}
			set
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				this.m_synchronizingObject = value;
			}
		}

		private event EventHandler<ObjectChangedEventArgs> m_objectChanged;

		public event EventHandler<ObjectChangedEventArgs> ObjectChanged
		{
			[SecurityCritical]
			add
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.AddObjectChangedEvent(value);
			}
			[SecurityCritical]
			remove
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				PeerCollaborationPermission.UnrestrictedPeerCollaborationPermission.Demand();
				this.RemoveObjectChangedEvent(value);
			}
		}

		private object LockObjChangedEvent
		{
			get
			{
				if (this.m_lockObjChangedEvent == null)
				{
					object value = new object();
					Interlocked.CompareExchange(ref this.m_lockObjChangedEvent, value, null);
				}
				return this.m_lockObjChangedEvent;
			}
		}

		[SecurityCritical]
		private void AddObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChanged() called.");
			lock (this.LockObjChangedEvent)
			{
				if (this.m_objectChanged == null)
				{
					if (this.m_id.Equals(Guid.Empty))
					{
						throw new PeerToPeerException(SR.GetString("Collab_EmptyGuidError"));
					}
					this.m_objChangedEvent = new AutoResetEvent(false);
					this.m_regObjChangedWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_objChangedEvent, new WaitOrTimerCallback(this.ObjectChangedCallback), null, -1, false);
					PEER_COLLAB_EVENT_REGISTRATION peer_COLLAB_EVENT_REGISTRATION = default(PEER_COLLAB_EVENT_REGISTRATION);
					peer_COLLAB_EVENT_REGISTRATION.eventType = PeerCollabEventType.EndPointObjectChanged;
					GUID guid = CollaborationHelperFunctions.ConvertGuidToGUID(this.m_id);
					GCHandle gchandle = GCHandle.Alloc(guid, GCHandleType.Pinned);
					peer_COLLAB_EVENT_REGISTRATION.pInstance = gchandle.AddrOfPinnedObject();
					try
					{
						int num = UnsafeCollabNativeMethods.PeerCollabRegisterEvent(this.m_objChangedEvent.SafeWaitHandle, 1U, ref peer_COLLAB_EVENT_REGISTRATION, out this.m_safeObjChangedEvent);
						if (num != 0)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabRegisterEvent returned with errorcode {0}", new object[]
							{
								num
							});
							throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_ObjectChangedRegFailed"), num);
						}
					}
					finally
					{
						if (gchandle.IsAllocated)
						{
							gchandle.Free();
						}
					}
				}
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Combine(this.m_objectChanged, callback);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "AddObjectChanged() successful.");
		}

		[SecurityCritical]
		private void RemoveObjectChangedEvent(EventHandler<ObjectChangedEventArgs> callback)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChanged() called.");
			lock (this.LockObjChangedEvent)
			{
				this.m_objectChanged = (EventHandler<ObjectChangedEventArgs>)Delegate.Remove(this.m_objectChanged, callback);
				if (this.m_objectChanged == null)
				{
					CollaborationHelperFunctions.CleanEventVars(ref this.m_regObjChangedWaitHandle, ref this.m_safeObjChangedEvent, ref this.m_objChangedEvent);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChangedEvent variables successful.");
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "RemoveObjectChanged() successful.");
		}

		protected virtual void OnObjectChanged(ObjectChangedEventArgs objChangedArgs)
		{
			EventHandler<ObjectChangedEventArgs> objectChanged = this.m_objectChanged;
			if (objectChanged != null)
			{
				if (this.SynchronizingObject != null && this.SynchronizingObject.InvokeRequired)
				{
					this.SynchronizingObject.BeginInvoke(objectChanged, new object[]
					{
						this,
						objChangedArgs
					});
				}
				else
				{
					objectChanged(this, objChangedArgs);
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Fired the object changed event callback.");
			}
		}

		[SecurityCritical]
		private void ObjectChangedCallback(object state, bool timedOut)
		{
			SafeCollabData safeCollabData = null;
			int num = 0;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "ObjectChangedCallback() called.");
			for (;;)
			{
				ObjectChangedEventArgs objectChangedEventArgs = null;
				try
				{
					lock (this.LockObjChangedEvent)
					{
						if (this.m_safeObjChangedEvent.IsInvalid)
						{
							return;
						}
						num = UnsafeCollabNativeMethods.PeerCollabGetEventData(this.m_safeObjChangedEvent, out safeCollabData);
					}
					if (num == 6488066)
					{
						break;
					}
					if (num != 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, 0, "PeerCollabGetEventData returned with errorcode {0}", new object[]
						{
							num
						});
						throw PeerToPeerException.CreateFromHr(SR.GetString("Collab_GetObjectChangedDataFailed"), num);
					}
					PEER_COLLAB_EVENT_DATA peer_COLLAB_EVENT_DATA = (PEER_COLLAB_EVENT_DATA)Marshal.PtrToStructure(safeCollabData.DangerousGetHandle(), typeof(PEER_COLLAB_EVENT_DATA));
					if (peer_COLLAB_EVENT_DATA.eventType == PeerCollabEventType.EndPointObjectChanged)
					{
						PEER_EVENT_OBJECT_CHANGED_DATA objectChangedData = peer_COLLAB_EVENT_DATA.objectChangedData;
						PEER_OBJECT po = (PEER_OBJECT)Marshal.PtrToStructure(objectChangedData.pObject, typeof(PEER_OBJECT));
						PeerObject peerObject = CollaborationHelperFunctions.ConvertPEER_OBJECTToPeerObject(po);
						if (object.Equals(this.m_id, peerObject.Id))
						{
							PeerContact peerContact = null;
							PeerEndPoint peerEndPoint = null;
							if (objectChangedData.pContact != IntPtr.Zero)
							{
								PEER_CONTACT pc = (PEER_CONTACT)Marshal.PtrToStructure(objectChangedData.pContact, typeof(PEER_CONTACT));
								peerContact = CollaborationHelperFunctions.ConvertPEER_CONTACTToPeerContact(pc);
							}
							if (objectChangedData.pEndPoint != IntPtr.Zero)
							{
								PEER_ENDPOINT pe = (PEER_ENDPOINT)Marshal.PtrToStructure(objectChangedData.pEndPoint, typeof(PEER_ENDPOINT));
								peerEndPoint = CollaborationHelperFunctions.ConvertPEER_ENDPOINTToPeerEndPoint(pe);
							}
							objectChangedEventArgs = new ObjectChangedEventArgs(peerEndPoint, peerContact, objectChangedData.changeType, peerObject);
						}
					}
				}
				finally
				{
					if (safeCollabData != null)
					{
						safeCollabData.Dispose();
					}
				}
				if (objectChangedEventArgs != null)
				{
					this.OnObjectChanged(objectChangedEventArgs);
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Leaving ObjectChangedCallback().");
		}

		public bool Equals(PeerObject other)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return other != null && other.Id.Equals(this.Id);
		}

		public override bool Equals(object obj)
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PeerObject peerObject = obj as PeerObject;
			return peerObject != null && peerObject.Id.Equals(this.Id);
		}

		public new static bool Equals(object objA, object objB)
		{
			PeerObject peerObject = objA as PeerObject;
			PeerObject peerObject2 = objB as PeerObject;
			return peerObject != null && peerObject2 != null && object.Equals(peerObject.Id, peerObject2.Id);
		}

		public override int GetHashCode()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return this.Id.GetHashCode();
		}

		public override string ToString()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			return this.Id.ToString();
		}

		[SecurityCritical]
		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		protected virtual void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				CollaborationHelperFunctions.CleanEventVars(ref this.m_regObjChangedWaitHandle, ref this.m_safeObjChangedEvent, ref this.m_objChangedEvent);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Clean ObjectChangedEvent variables successful.");
				this.m_Disposed = true;
			}
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_Id", this.m_id);
			info.AddValue("_Data", this.m_data);
			info.AddValue("_Scope", this.m_peerScope);
		}

		internal void TracePeerObject()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerObject");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tGuid: {0}", new object[]
			{
				this.Id
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tPeerScope: {0}", new object[]
			{
				this.PeerScope
			});
			if (this.Data != null)
			{
				if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose))
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tObject data:");
					Logging.DumpData(Logging.P2PTraceSource, TraceEventType.Verbose, Logging.P2PTraceSource.MaxDataSize, this.Data, 0, this.Data.Length);
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tObject data length {0}", new object[]
					{
						this.Data.Length
					});
				}
			}
		}

		private const int c_16K = 16384;

		private Guid m_id;

		private byte[] m_data;

		private PeerScope m_peerScope;

		private ISynchronizeInvoke m_synchronizingObject;

		private object m_lockObjChangedEvent;

		private RegisteredWaitHandle m_regObjChangedWaitHandle;

		private AutoResetEvent m_objChangedEvent;

		private SafeCollabEvent m_safeObjChangedEvent;

		private bool m_Disposed;
	}
}
