﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Text;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public class PeerName : ISerializable, IEquatable<PeerName>
	{
		static PeerName()
		{
			if (!PeerToPeerOSHelper.SupportsP2P)
			{
				throw new PlatformNotSupportedException(SR.GetString("P2P_NotAvailable"));
			}
		}

		private PeerName(string peerName, string authority, string classifier)
		{
			this.m_PeerName = peerName;
			this.m_Classifier = classifier;
			this.m_Authority = authority;
		}

		public PeerName(string remotePeerName)
		{
			if (remotePeerName == null)
			{
				throw new ArgumentNullException("remotePeerName");
			}
			string text;
			string text2;
			if (!PeerName.StrongParsePeerName(remotePeerName, out text, out text2))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerName string {0} failed the check for a valid peer name", new object[]
				{
					remotePeerName
				});
				throw new ArgumentException(SR.GetString("Pnrp_InvalidPeerName"), "remotePeerName");
			}
			if (text2 != null)
			{
				this.m_PeerName = text + "." + text2;
			}
			else
			{
				this.m_PeerName = text;
			}
			this.m_Authority = text;
			this.m_Classifier = text2;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerName instance created - PeerName {0} Authority {1} Classfier {2}", new object[]
			{
				this.m_PeerName,
				this.m_Authority,
				(this.m_Classifier == null) ? "null" : this.m_Classifier
			});
		}

		[SecurityCritical]
		public PeerName(string classifier, PeerNameType peerNameType)
		{
			if ((classifier == null || classifier.Length == 0) && peerNameType == PeerNameType.Unsecured)
			{
				throw new ArgumentNullException("classifier");
			}
			if (classifier != null && classifier.Length > 149)
			{
				throw new ArgumentException(SR.GetString("Pnrp_InvalidClassifier"), "classifier");
			}
			if (classifier != null && classifier.Length > 0)
			{
				classifier = classifier.Normalize(NormalizationForm.FormC);
			}
			SafePeerData safePeerData = null;
			SafePeerData safePeerData2 = null;
			try
			{
				if (peerNameType == PeerNameType.Unsecured)
				{
					int num = UnsafeP2PNativeMethods.PeerCreatePeerName(null, classifier, out safePeerData);
					if (num != 0)
					{
						throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotCreateUnsecuredPeerName"), num);
					}
					this.m_Authority = "0";
				}
				else
				{
					int num = UnsafeP2PNativeMethods.PeerIdentityGetDefault(out safePeerData2);
					if (num != 0)
					{
						throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotGetDefaultIdentity"), num);
					}
					this.m_Authority = safePeerData2.UnicodeString;
					num = UnsafeP2PNativeMethods.PeerCreatePeerName(this.m_Authority, classifier, out safePeerData);
					if (num != 0)
					{
						throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotCreateSecuredPeerName"), num);
					}
				}
				this.m_PeerName = safePeerData.UnicodeString;
				this.m_Classifier = classifier;
			}
			finally
			{
				if (safePeerData != null)
				{
					safePeerData.Dispose();
				}
				if (safePeerData2 != null)
				{
					safePeerData2.Dispose();
				}
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerName instance created - PeerName {0} Authority {1} Classfier {2}", new object[]
			{
				this.m_PeerName,
				this.m_Authority,
				this.m_Classifier
			});
		}

		[SecurityCritical]
		public static PeerName CreateFromPeerHostName(string peerHostName)
		{
			if (peerHostName == null)
			{
				throw new ArgumentNullException("peerHostName");
			}
			if (peerHostName.Length == 0)
			{
				throw new ArgumentException(SR.GetString("Pnrp_InvalidPeerHostName"), "peerHostName");
			}
			SafePeerData safePeerData = null;
			string text = null;
			try
			{
				int num = UnsafeP2PNativeMethods.PeerHostNameToPeerName(peerHostName, out safePeerData);
				if (num != 0)
				{
					throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotGetPeerNameFromPeerHostName"), num);
				}
				text = safePeerData.UnicodeString;
			}
			finally
			{
				if (safePeerData != null)
				{
					safePeerData.Dispose();
				}
			}
			string text2;
			string text3;
			PeerName.WeakParsePeerName(text, out text2, out text3);
			PeerName result = new PeerName(text, text2, text3);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerName created from PeerHostName - PeerHostName {0} to PeerName PeerName {1} Authority {2} Classfier {3}", new object[]
			{
				peerHostName,
				text,
				text2,
				text3
			});
			return result;
		}

		[SecurityCritical]
		public static PeerName CreateRelativePeerName(PeerName peerName, string classifier)
		{
			if (peerName == null)
			{
				throw new ArgumentNullException("peerName", SR.GetString("Pnrp_PeerNameCantBeNull"));
			}
			if (!peerName.IsSecured && (classifier == null || classifier.Length == 0))
			{
				throw new ArgumentException(SR.GetString("Pnrp_InvalidClassifier"), "classifier");
			}
			if (classifier != null && classifier.Length > 149)
			{
				throw new ArgumentException(SR.GetString("Pnrp_InvalidClassifier"), "classifier");
			}
			if (classifier != null && classifier.Length > 0)
			{
				classifier = classifier.Normalize(NormalizationForm.FormC);
			}
			SafePeerData safePeerData = null;
			string peerName2 = null;
			try
			{
				int num = UnsafeP2PNativeMethods.PeerCreatePeerName(peerName.IsSecured ? peerName.m_PeerName : null, classifier, out safePeerData);
				if (num != 0)
				{
					throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotCreateRelativePeerName"), num);
				}
				peerName2 = safePeerData.UnicodeString;
			}
			finally
			{
				if (safePeerData != null)
				{
					safePeerData.Dispose();
				}
			}
			string authority;
			string classifier2;
			PeerName.WeakParsePeerName(peerName2, out authority, out classifier2);
			PeerName peerName3 = new PeerName(peerName2, authority, classifier2);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "A new PeerName created from existing PeerName with a new classfier. Existing PeerName {0} Classifier {1} New PeerName {2}", new object[]
			{
				peerName,
				classifier,
				peerName3
			});
			return peerName3;
		}

		public override string ToString()
		{
			return this.m_PeerName;
		}

		public bool Equals(PeerName other)
		{
			return other != null && string.Compare(other.m_PeerName, this.m_PeerName, StringComparison.Ordinal) == 0;
		}

		public override bool Equals(object obj)
		{
			bool result;
			if (obj == null)
			{
				result = false;
			}
			else
			{
				PeerName peerName = obj as PeerName;
				result = (peerName != null && this.Equals(peerName));
			}
			return result;
		}

		public override int GetHashCode()
		{
			return this.m_PeerName.GetHashCode();
		}

		public string Authority
		{
			get
			{
				return this.m_Authority;
			}
		}

		public string Classifier
		{
			get
			{
				return this.m_Classifier;
			}
		}

		public string PeerHostName
		{
			[SecurityCritical]
			get
			{
				if (this.m_PeerHostName == null)
				{
					SafePeerData safePeerData = null;
					try
					{
						int num = UnsafeP2PNativeMethods.PeerNameToPeerHostName(this.m_PeerName, out safePeerData);
						if (num < 0)
						{
							throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotGetPeerHostNameFromPeerName"), num);
						}
						this.m_PeerHostName = safePeerData.UnicodeString;
					}
					finally
					{
						if (safePeerData != null)
						{
							safePeerData.Dispose();
						}
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "PeerHostName created for PeerName PeerHostName {0} PeerName {1}", new object[]
				{
					this.m_PeerHostName,
					this
				});
				return this.m_PeerHostName;
			}
		}

		public bool IsSecured
		{
			get
			{
				return this.m_Authority != "0";
			}
		}

		private static bool StrongParsePeerName(string peerName, out string authority, out string classifier)
		{
			authority = null;
			classifier = null;
			bool result;
			if (peerName == null || peerName.Length == 0 || peerName.Length > 190)
			{
				result = false;
			}
			else if (peerName.Length < 3)
			{
				result = false;
			}
			else
			{
				int num = peerName.IndexOf('.');
				if (num == 0)
				{
					result = false;
				}
				else
				{
					string text;
					if (num < 0)
					{
						text = peerName;
					}
					else if (num == peerName.Length - 1)
					{
						text = peerName.Substring(0, peerName.Length - 1);
					}
					else
					{
						text = peerName.Substring(0, num);
					}
					if (text.Length != 40 && text.Length != 1)
					{
						result = false;
					}
					else if (text.Length == 1 && text != "0")
					{
						result = false;
					}
					else
					{
						if (text.Length == 40)
						{
							foreach (char character in text)
							{
								if (!PeerName.IsHexDigit(character))
								{
									return false;
								}
							}
						}
						string text3 = null;
						if (num != peerName.Length - 1 && num > 0)
						{
							text3 = peerName.Substring(num + 1);
						}
						if (text3 != null && text3.Length > 149)
						{
							result = false;
						}
						else
						{
							authority = text.ToLower(CultureInfo.InvariantCulture);
							classifier = text3;
							result = true;
						}
					}
				}
			}
			return result;
		}

		private static bool IsHexDigit(char character)
		{
			return (character >= '0' && character <= '9') || (character >= 'A' && character <= 'F') || (character >= 'a' && character <= 'f');
		}

		private static void WeakParsePeerName(string peerName, out string authority, out string classifier)
		{
			authority = null;
			classifier = null;
			int num = peerName.IndexOf('.');
			if (num < 0)
			{
				authority = peerName;
			}
			else if (num == peerName.Length - 1)
			{
				authority = peerName.Substring(0, peerName.Length - 1);
			}
			else
			{
				authority = peerName.Substring(0, num);
			}
			if (num != peerName.Length - 1 && num > 0)
			{
				classifier = peerName.Substring(num + 1);
			}
			authority = authority.ToLower(CultureInfo.InvariantCulture);
		}

		protected PeerName(SerializationInfo info, StreamingContext context)
		{
			if (info == null)
			{
				throw new ArgumentNullException("info");
			}
			this.m_PeerName = info.GetString("_PeerName");
			this.m_Authority = info.GetString("_Authority");
			this.m_Classifier = info.GetString("_Classifier");
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_PeerName", this.m_PeerName);
			info.AddValue("_Authority", this.m_Authority);
			info.AddValue("_Classifier", this.m_Classifier);
		}

		private const int PEER_MAX_CLASSIFIER_LENGTH = 149;

		private const int SECURE_AUTHORITY_LENGTH = 40;

		private const int INSECURE_AUTHORITY_LENGTH = 1;

		private const int PEER_MAX_PEERNAME_LENGTH = 190;

		private const string PEERNAME_UNSECURED_AUTHORITY = "0";

		private string m_PeerName;

		private string m_Authority;

		private string m_Classifier;

		private string m_PeerHostName;
	}
}
