﻿using System;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public class PeerNameRecord : ISerializable
	{
		protected PeerNameRecord(SerializationInfo info, StreamingContext context)
		{
			this.m_Comment = info.GetString("_Comment");
			this.m_Data = (info.GetValue("_Data", typeof(byte[])) as byte[]);
			this.m_EndPointCollection = (info.GetValue("_EndpointList", typeof(IPEndPointCollection)) as IPEndPointCollection);
			this.m_PeerName = (info.GetValue("_PeerName", typeof(PeerName)) as PeerName);
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_PeerName", this.m_PeerName);
			info.AddValue("_EndpointList", this.m_EndPointCollection);
			info.AddValue("_Comment", this.m_Comment);
			info.AddValue("_Data", this.m_Data);
		}

		public PeerNameRecord()
		{
		}

		public PeerName PeerName
		{
			get
			{
				return this.m_PeerName;
			}
			set
			{
				this.m_PeerName = value;
			}
		}

		public IPEndPointCollection EndPointCollection
		{
			get
			{
				return this.m_EndPointCollection;
			}
		}

		public string Comment
		{
			get
			{
				return this.m_Comment;
			}
			set
			{
				if (value == null)
				{
					throw new ArgumentNullException("Comment", SR.GetString("Pnrp_CommentCantBeNull"));
				}
				if (value.Length <= 0)
				{
					throw new ArgumentException(SR.GetString("Pnrp_CommentCantBeNull"), "Comment");
				}
				if (value.Length > 39)
				{
					throw new ArgumentException(SR.GetString("Pnrp_CommentMaxLengthExceeded", new object[]
					{
						39
					}));
				}
				this.m_Comment = value;
			}
		}

		public byte[] Data
		{
			get
			{
				return this.m_Data;
			}
			set
			{
				if (value == null)
				{
					throw new ArgumentNullException("Data", SR.GetString("Pnrp_DataCantBeNull"));
				}
				if (value.Length <= 0)
				{
					throw new ArgumentException(SR.GetString("Pnrp_DataCantBeNull"), "Data");
				}
				if (value.Length > 4096)
				{
					throw new ArgumentException(SR.GetString("Pnrp_DataLengthExceeded", new object[]
					{
						4096
					}));
				}
				this.m_Data = value;
			}
		}

		internal void TracePeerNameRecord()
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Contents of the PeerNameRecord");
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tPeerName: {0}", new object[]
			{
				this.PeerName
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tComment: {0}", new object[]
			{
				this.Comment
			});
			if (this.EndPointCollection != null && this.EndPointCollection.Count != 0)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tThe EndPointList is ");
				foreach (IPEndPoint ipendPoint in this.EndPointCollection)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\t\tIPEndPoint is {0}", new object[]
					{
						ipendPoint
					});
				}
			}
			else
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tThe EndPointList is empty or null");
			}
			if (this.Data != null)
			{
				if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Verbose))
				{
					Logging.DumpData(Logging.P2PTraceSource, TraceEventType.Verbose, Logging.P2PTraceSource.MaxDataSize, this.Data, 0, this.Data.Length);
				}
				else
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "\tCustom data length {0}", new object[]
					{
						this.Data.Length
					});
				}
			}
		}

		private const int MAX_COMMENT_SIZE = 39;

		private const int MAX_DATA_SIZE = 4096;

		private PeerName m_PeerName;

		private IPEndPointCollection m_EndPointCollection = new IPEndPointCollection();

		private string m_Comment;

		private byte[] m_Data;
	}
}
