﻿using System;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public class PeerNameRegistration : IDisposable, ISerializable
	{
		static PeerNameRegistration()
		{
			if (!PeerToPeerOSHelper.SupportsP2P)
			{
				throw new PlatformNotSupportedException(SR.GetString("P2P_NotAvailable"));
			}
		}

		public PeerNameRegistration()
		{
		}

		public PeerNameRegistration(PeerName name, int port) : this(name, port, null)
		{
		}

		public PeerNameRegistration(PeerName name, int port, Cloud cloud)
		{
			if (name == null)
			{
				throw new ArgumentNullException("name");
			}
			if (cloud == null)
			{
				cloud = Cloud.Available;
			}
			if (port < 0 || port > 65535)
			{
				throw new ArgumentOutOfRangeException("port", SR.GetString("Pnrp_PortOutOfRange"));
			}
			this.m_PeerNameRecord.PeerName = name;
			this.m_Port = port;
			this.m_Cloud = cloud;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Created a PeerNameRegistration with PeerName {0}, Port {1}, Cloud {2} - Proceeding to register", new object[]
			{
				name,
				port,
				cloud
			});
		}

		internal PeerNameRecord PeerNameRecord
		{
			get
			{
				if (this.m_Disposed)
				{
					throw new ObjectDisposedException(base.GetType().FullName);
				}
				return this.m_PeerNameRecord;
			}
		}

		public int Port
		{
			get
			{
				return this.m_Port;
			}
			set
			{
				if (value < 0 || value > 65535)
				{
					throw new ArgumentOutOfRangeException("port", SR.GetString("Pnrp_PortOutOfRange"));
				}
				this.m_Port = value;
			}
		}

		public PeerName PeerName
		{
			get
			{
				return this.m_PeerNameRecord.PeerName;
			}
			set
			{
				this.m_PeerNameRecord.PeerName = value;
			}
		}

		public IPEndPointCollection EndPointCollection
		{
			get
			{
				return this.m_PeerNameRecord.EndPointCollection;
			}
		}

		public Cloud Cloud
		{
			get
			{
				return this.m_Cloud;
			}
			set
			{
				this.m_Cloud = value;
			}
		}

		public string Comment
		{
			get
			{
				return this.m_PeerNameRecord.Comment;
			}
			set
			{
				this.m_PeerNameRecord.Comment = value;
			}
		}

		public byte[] Data
		{
			get
			{
				return this.m_PeerNameRecord.Data;
			}
			set
			{
				this.m_PeerNameRecord.Data = value;
			}
		}

		public bool UseAutoEndPointSelection
		{
			get
			{
				return this.m_UseAutoEndPointSelection;
			}
			set
			{
				this.m_UseAutoEndPointSelection = value;
			}
		}

		public bool IsRegistered()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			return this.m_IsRegistered;
		}

		[SecurityCritical]
		public void Start()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Proceeding to register through the Register method()");
			this.InternalRegister();
		}

		[SecurityCritical]
		private unsafe void InternalRegister()
		{
			if (this.m_IsRegistered)
			{
				throw new PeerToPeerException(SR.GetString("Pnrp_UseUpdateInsteadOfRegister"));
			}
			if (this.m_PeerNameRecord.PeerName == null)
			{
				throw new ArgumentNullException("PeerName");
			}
			if (!this.m_UseAutoEndPointSelection)
			{
				if (this.EndPointCollection.Count == 0 && (this.Data == null || this.Data.Length <= 0))
				{
					throw new PeerToPeerException(SR.GetString("Pnrp_BlobOrEndpointListNeeded"));
				}
			}
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			UnsafeP2PNativeMethods.PnrpStartup();
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "InternalRegister() is called with the following Info");
				this.m_PeerNameRecord.TracePeerNameRecord();
			}
			PEER_PNRP_REGISTRATION_INFO peer_PNRP_REGISTRATION_INFO = default(PEER_PNRP_REGISTRATION_INFO);
			GCHandle gchandle;
			if (this.m_PeerNameRecord.Data != null)
			{
				peer_PNRP_REGISTRATION_INFO.payLoad.cbPayload = (uint)this.m_PeerNameRecord.Data.Length;
				gchandle = GCHandle.Alloc(this.m_PeerNameRecord.Data, GCHandleType.Pinned);
				peer_PNRP_REGISTRATION_INFO.payLoad.pbPayload = gchandle.AddrOfPinnedObject();
			}
			else
			{
				gchandle = default(GCHandle);
			}
			if (this.m_PeerNameRecord.Comment != null && this.m_PeerNameRecord.Comment.Length > 0)
			{
				peer_PNRP_REGISTRATION_INFO.pwszComment = this.m_PeerNameRecord.Comment;
			}
			if (this.m_Cloud != null)
			{
				peer_PNRP_REGISTRATION_INFO.pwszCloudName = this.m_Cloud.InternalName;
			}
			try
			{
				if (this.m_PeerNameRecord.EndPointCollection.Count == 0)
				{
					peer_PNRP_REGISTRATION_INFO.wport = (ushort)this.m_Port;
					if (this.m_UseAutoEndPointSelection)
					{
						peer_PNRP_REGISTRATION_INFO.cAddresses = uint.MaxValue;
					}
					int num = UnsafeP2PNativeMethods.PeerPnrpRegister(this.m_PeerNameRecord.PeerName.ToString(), ref peer_PNRP_REGISTRATION_INFO, out this.m_RegistrationHandle);
					if (num != 0)
					{
						throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotRegisterPeerName"), num);
					}
				}
				else
				{
					int count = this.m_PeerNameRecord.EndPointCollection.Count;
					int cb = count * Marshal.SizeOf(typeof(IntPtr));
					IntPtr intPtr = Marshal.AllocHGlobal(cb);
					GCHandle[] array = new GCHandle[count];
					try
					{
						IntPtr* ptr = (IntPtr*)((void*)intPtr);
						for (int i = 0; i < this.m_PeerNameRecord.EndPointCollection.Count; i++)
						{
							byte[] value = SystemNetHelpers.SOCKADDRFromIPEndPoint(this.m_PeerNameRecord.EndPointCollection[i]);
							array[i] = GCHandle.Alloc(value, GCHandleType.Pinned);
							IntPtr intPtr2 = array[i].AddrOfPinnedObject();
							ptr[(IntPtr)i * (IntPtr)sizeof(IntPtr) / (IntPtr)sizeof(IntPtr)] = intPtr2;
						}
						peer_PNRP_REGISTRATION_INFO.ArrayOfSOCKADDRIN6Pointers = intPtr;
						peer_PNRP_REGISTRATION_INFO.cAddresses = (uint)count;
						int num = UnsafeP2PNativeMethods.PeerPnrpRegister(this.m_PeerNameRecord.PeerName.ToString(), ref peer_PNRP_REGISTRATION_INFO, out this.m_RegistrationHandle);
						if (num != 0)
						{
							throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotRegisterPeerName"), num);
						}
					}
					finally
					{
						if (intPtr != IntPtr.Zero)
						{
							Marshal.FreeHGlobal(intPtr);
						}
						for (int i = 0; i < array.Length; i++)
						{
							if (array[i].IsAllocated)
							{
								array[i].Free();
							}
						}
					}
				}
			}
			finally
			{
				if (gchandle.IsAllocated)
				{
					gchandle.Free();
				}
			}
			this.m_RegisteredPeerName = this.m_PeerNameRecord.PeerName;
			this.m_IsRegistered = true;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Registration is successful. The handle is {0}", new object[]
			{
				this.m_RegistrationHandle.DangerousGetHandle()
			});
		}

		[SecurityCritical]
		public unsafe void Update()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			if (!this.IsRegistered())
			{
				throw new InvalidOperationException(SR.GetString("Pnrp_CallRegisterBeforeUpdate"));
			}
			if (this.m_PeerNameRecord.PeerName == null)
			{
				throw new ArgumentNullException(SR.GetString("Pnrp_InvalidPeerName"));
			}
			if (!this.m_RegisteredPeerName.Equals(this.m_PeerNameRecord.PeerName))
			{
				throw new InvalidOperationException(SR.GetString("Pnrp_CantChangePeerNameAfterRegistration"));
			}
			if (!this.m_UseAutoEndPointSelection)
			{
				if (this.EndPointCollection.Count == 0 || this.Data == null || this.Data.Length <= 0)
				{
					throw new PeerToPeerException(SR.GetString("Pnrp_BlobOrEndpointListNeeded"));
				}
			}
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			if (Logging.P2PTraceSource.Switch.ShouldTrace(TraceEventType.Information))
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Update() is called with the following Info");
				this.m_PeerNameRecord.TracePeerNameRecord();
			}
			PEER_PNRP_REGISTRATION_INFO peer_PNRP_REGISTRATION_INFO = default(PEER_PNRP_REGISTRATION_INFO);
			if (this.m_PeerNameRecord.Data != null)
			{
				peer_PNRP_REGISTRATION_INFO.payLoad.cbPayload = (uint)this.m_PeerNameRecord.Data.Length;
				GCHandle gchandle = GCHandle.Alloc(this.m_PeerNameRecord.Data, GCHandleType.Pinned);
				peer_PNRP_REGISTRATION_INFO.payLoad.pbPayload = gchandle.AddrOfPinnedObject();
				gchandle.Free();
			}
			if (this.m_PeerNameRecord.Comment != null && this.m_PeerNameRecord.Comment.Length > 0)
			{
				peer_PNRP_REGISTRATION_INFO.pwszComment = this.m_PeerNameRecord.Comment;
			}
			peer_PNRP_REGISTRATION_INFO.pwszCloudName = null;
			if (this.m_Cloud != null)
			{
				peer_PNRP_REGISTRATION_INFO.pwszCloudName = this.m_Cloud.InternalName;
			}
			if (this.m_PeerNameRecord.EndPointCollection.Count == 0)
			{
				peer_PNRP_REGISTRATION_INFO.wport = (ushort)this.m_Port;
				peer_PNRP_REGISTRATION_INFO.cAddresses = uint.MaxValue;
				int num = UnsafeP2PNativeMethods.PeerPnrpUpdateRegistration(this.m_RegistrationHandle, ref peer_PNRP_REGISTRATION_INFO);
				if (num != 0)
				{
					throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotRegisterPeerName"), num);
				}
			}
			else
			{
				int count = this.m_PeerNameRecord.EndPointCollection.Count;
				int cb = count * Marshal.SizeOf(typeof(IntPtr));
				IntPtr intPtr = Marshal.AllocHGlobal(cb);
				GCHandle[] array = new GCHandle[count];
				try
				{
					IntPtr* ptr = (IntPtr*)((void*)intPtr);
					for (int i = 0; i < this.m_PeerNameRecord.EndPointCollection.Count; i++)
					{
						byte[] value = SystemNetHelpers.SOCKADDRFromIPEndPoint(this.m_PeerNameRecord.EndPointCollection[i]);
						array[i] = GCHandle.Alloc(value, GCHandleType.Pinned);
						IntPtr intPtr2 = array[i].AddrOfPinnedObject();
						ptr[(IntPtr)i * (IntPtr)sizeof(IntPtr) / (IntPtr)sizeof(IntPtr)] = intPtr2;
					}
					peer_PNRP_REGISTRATION_INFO.ArrayOfSOCKADDRIN6Pointers = intPtr;
					peer_PNRP_REGISTRATION_INFO.cAddresses = (uint)count;
					int num = UnsafeP2PNativeMethods.PeerPnrpUpdateRegistration(this.m_RegistrationHandle, ref peer_PNRP_REGISTRATION_INFO);
					if (num != 0)
					{
						throw PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotRegisterPeerName"), num);
					}
				}
				finally
				{
					if (intPtr != IntPtr.Zero)
					{
						Marshal.FreeHGlobal(intPtr);
					}
					for (int i = 0; i < array.Length; i++)
					{
						if (array[i].IsAllocated)
						{
							array[i].Free();
						}
					}
				}
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, 0, "Update of existing registration is successful. The handle is {0}", new object[]
				{
					this.m_RegistrationHandle.DangerousGetHandle()
				});
			}
		}

		[SecurityCritical]
		public void Stop()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			if (this.m_RegistrationHandle.IsInvalid || this.m_RegistrationHandle.IsClosed)
			{
				throw new InvalidOperationException(SR.GetString("Pnrp_NoRegistrationFound"));
			}
			PnrpPermission.UnrestrictedPnrpPermission.Demand();
			this.m_RegistrationHandle.Dispose();
			this.m_PeerNameRecord = new PeerNameRecord();
			this.m_RegisteredPeerName = null;
			this.m_IsRegistered = false;
		}

		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		protected virtual void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				try
				{
					this.Stop();
				}
				catch (ObjectDisposedException)
				{
				}
				catch (InvalidOperationException)
				{
				}
			}
			this.m_Disposed = true;
		}

		protected PeerNameRegistration(SerializationInfo info, StreamingContext context)
		{
			this.m_Port = info.GetInt32("_Port");
			this.m_UseAutoEndPointSelection = info.GetBoolean("_UseAutoEndPointSelection");
			this.m_Cloud = (info.GetValue("_Cloud", typeof(Cloud)) as Cloud);
			this.m_PeerNameRecord = (info.GetValue("_PeerNameRecord", typeof(PeerNameRecord)) as PeerNameRecord);
		}

		[SecurityCritical]
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter, SerializationFormatter = true)]
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
		{
			this.GetObjectData(info, context);
		}

		[SecurityPermission(SecurityAction.LinkDemand, SerializationFormatter = true)]
		protected virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_Port", this.m_Port);
			info.AddValue("_UseAutoEndPointSelection", this.m_UseAutoEndPointSelection);
			info.AddValue("_Cloud", this.m_Cloud);
			info.AddValue("_PeerNameRecord", this.m_PeerNameRecord);
		}

		private const uint PEER_PNRP_AUTO_ADDRESSES = 4294967295U;

		private PeerNameRecord m_PeerNameRecord = new PeerNameRecord();

		private int m_Port;

		private Cloud m_Cloud;

		private bool m_IsRegistered;

		private SafePeerNameUnregister m_RegistrationHandle;

		private PeerName m_RegisteredPeerName;

		private bool m_UseAutoEndPointSelection = true;

		private bool m_Disposed;
	}
}
