﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security;
using System.Threading;

namespace System.Net.PeerToPeer
{
	internal class PeerNameResolverHelper : IDisposable
	{
		internal PeerNameResolverHelper(PeerName peerName, Cloud cloud, int MaxRecords, object userState, PeerNameResolver parent, int NewTraceEventId)
		{
			this.m_userState = userState;
			this.m_PeerName = peerName;
			this.m_Cloud = cloud;
			this.m_MaxRecords = MaxRecords;
			this.m_PeerNameResolverWeakReference = new WeakReference(parent);
			this.m_TraceEventId = NewTraceEventId;
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "New PeerNameResolverHelper created with TraceEventID {0}", new object[]
			{
				this.m_TraceEventId
			});
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "\tPeerName: {0}, Cloud: {1}, MaxRecords: {2}, userState {3}, ParentReference {4}", new object[]
			{
				this.m_PeerName,
				this.m_Cloud,
				this.m_MaxRecords,
				userState.GetHashCode(),
				this.m_PeerNameResolverWeakReference.Target.GetHashCode()
			});
		}

		[SecurityCritical]
		internal void StartAsyncResolve()
		{
			if (this.m_Disposed)
			{
				throw new ObjectDisposedException(base.GetType().FullName);
			}
			this.m_RegisteredWaitHandle = ThreadPool.RegisterWaitForSingleObject(this.m_EndPointInfoAvailableEvent, new WaitOrTimerCallback(this.EndPointInfoAvailableCallback), null, -1, false);
			int num = UnsafeP2PNativeMethods.PeerPnrpStartResolve(this.m_PeerName.ToString(), this.m_Cloud.InternalName, (uint)this.m_MaxRecords, this.m_EndPointInfoAvailableEvent.SafeWaitHandle, out this.m_SafePeerNameEndResolve);
			if (num != 0)
			{
				if (!this.m_SafePeerNameEndResolve.IsInvalid && !this.m_SafePeerNameEndResolve.IsClosed)
				{
					this.m_SafePeerNameEndResolve.Dispose();
				}
				this.m_RegisteredWaitHandle.Unregister(null);
				this.m_RegisteredWaitHandle = null;
				PeerToPeerException ex = PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_CouldNotStartNameResolution"), num);
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Error, this.m_TraceEventId, "Exception occurred while starting async resolve");
				throw ex;
			}
			this.m_AsyncOp = AsyncOperationManager.CreateOperation(this.m_userState);
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Successfully started the async resolve. The native handle is {0}", new object[]
			{
				this.m_SafePeerNameEndResolve.DangerousGetHandle()
			});
		}

		[SecurityCritical]
		public unsafe void EndPointInfoAvailableCallback(object state, bool timedOut)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "EndPointInfoAvailableCallback called");
			PeerNameRecord peerNameRecord = null;
			int num = 0;
			if (this.m_Cancelled)
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Detected that the async operation is already canceled  - before entering the lock");
			}
			else
			{
				PeerNameResolver peerNameResolver;
				lock (this.m_Lock)
				{
					if (this.m_Cancelled)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Detected that the async operation is already canceled - after entering the lock");
						return;
					}
					SafePeerData safePeerData;
					num = UnsafeP2PNativeMethods.PeerPnrpGetEndpoint(this.m_SafePeerNameEndResolve.DangerousGetHandle(), out safePeerData);
					if (num == 0)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Proceeding to retrieve the endpoint information from incremental resolve");
						try
						{
							PEER_PNRP_ENDPOINT_INFO* ptr = (PEER_PNRP_ENDPOINT_INFO*)((void*)safePeerData.DangerousGetHandle());
							peerNameRecord = new PeerNameRecord();
							peerNameRecord.PeerName = new PeerName(Marshal.PtrToStringUni(ptr->pwszPeerName));
							string text = Marshal.PtrToStringUni(ptr->pwszComment);
							if (text != null && text.Length > 0)
							{
								peerNameRecord.Comment = text;
							}
							if (ptr->payLoad.cbPayload != 0U)
							{
								peerNameRecord.Data = new byte[ptr->payLoad.cbPayload];
								Marshal.Copy(ptr->payLoad.pbPayload, peerNameRecord.Data, 0, (int)ptr->payLoad.cbPayload);
							}
							IntPtr intPtr = ptr->ArrayOfSOCKADDRIN6Pointers;
							for (uint num2 = 0U; num2 < ptr->cAddresses; num2 += 1U)
							{
								IntPtr source = Marshal.ReadIntPtr(intPtr);
								byte[] array = new byte[2];
								Marshal.Copy(source, array, 0, 2);
								int num3 = (int)array[0] + ((int)array[1] << 8);
								byte[] array2 = new byte[(num3 == 2) ? 16 : 28];
								Marshal.Copy(source, array2, 0, array2.Length);
								IPEndPoint item = SystemNetHelpers.IPEndPointFromSOCKADDRBuffer(array2);
								peerNameRecord.EndPointCollection.Add(item);
								intPtr = (IntPtr)((long)intPtr + (long)Marshal.SizeOf(typeof(IntPtr)));
							}
						}
						finally
						{
							safePeerData.Dispose();
						}
						peerNameRecord.TracePeerNameRecord();
						this.m_PeerNameRecordCollection.Add(peerNameRecord);
						ResolveProgressChangedEventArgs args = new ResolveProgressChangedEventArgs(peerNameRecord, this.m_AsyncOp.UserSuppliedState);
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Proceeding to call progress changed event callback");
						peerNameResolver = (this.m_PeerNameResolverWeakReference.Target as PeerNameResolver);
						if (peerNameResolver != null)
						{
							peerNameResolver.PrepareToRaiseProgressChangedEvent(this.m_AsyncOp, args);
						}
						return;
					}
					if (num == -2140979197)
					{
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Native API returned that there are no more records - resolve completed successfully");
					}
					this.m_CompletedOrException = true;
					this.m_SafePeerNameEndResolve.Dispose();
				}
				ResolveCompletedEventArgs args2;
				if (num == -2140979197)
				{
					args2 = new ResolveCompletedEventArgs(this.m_PeerNameRecordCollection, null, false, this.m_AsyncOp.UserSuppliedState);
				}
				else
				{
					PeerToPeerException error = PeerToPeerException.CreateFromHr(SR.GetString("Pnrp_ExceptionWhileResolvingAPeerName"), num);
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Exception occurred when the native API is called to harvest an incremental resolve notification");
					args2 = new ResolveCompletedEventArgs(null, error, false, this.m_AsyncOp.UserSuppliedState);
				}
				peerNameResolver = (this.m_PeerNameResolverWeakReference.Target as PeerNameResolver);
				if (peerNameResolver != null)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "Proceeding to call the ResolveCompleted callback");
					peerNameResolver.PrepareToRaiseCompletedEvent(this.m_AsyncOp, args2);
				}
			}
		}

		[SecurityCritical]
		public void ContineCancelCallback(object state)
		{
			Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "ContineCancelCallback called");
			try
			{
				if (this.m_CompletedOrException)
				{
					Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "ContinueCancelCallback detected (before acquiring lock) that another thread has already called completed event - so returning without calling cancel");
				}
				else
				{
					lock (this.m_Lock)
					{
						if (this.m_Cancelled)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "ContinueCancelCallback detected (after acquiring lock) that cancel has already been called");
							return;
						}
						if (this.m_CompletedOrException)
						{
							Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "ContinueCancelCallback detected (after acquiring lock) that another thread has already called completed event - so returning without calling cancel");
							return;
						}
						Logging.P2PTraceSource.TraceEvent(TraceEventType.Information, this.m_TraceEventId, "ContinueCancelCallback is proceeding to close the handle and call the Completed callback with Cancelled = true");
						this.m_Cancelled = true;
						this.m_SafePeerNameEndResolve.Dispose();
					}
					PeerNameResolver peerNameResolver = this.m_PeerNameResolverWeakReference.Target as PeerNameResolver;
					if (peerNameResolver != null)
					{
						ResolveCompletedEventArgs args = new ResolveCompletedEventArgs(null, null, true, this.m_AsyncOp.UserSuppliedState);
						peerNameResolver.PrepareToRaiseCompletedEvent(this.m_AsyncOp, args);
					}
				}
			}
			catch
			{
				Logging.P2PTraceSource.TraceEvent(TraceEventType.Critical, this.m_TraceEventId, "Exception while cancelling the call ");
				throw;
			}
		}

		[SecurityCritical]
		public void CancelAsync()
		{
			ThreadPool.QueueUserWorkItem(new WaitCallback(this.ContineCancelCallback));
		}

		[SecurityCritical]
		public void Dispose()
		{
			this.Dispose(true);
			GC.SuppressFinalize(this);
		}

		[SecurityCritical]
		public void Dispose(bool disposing)
		{
			if (!this.m_Disposed)
			{
				if (!this.m_SafePeerNameEndResolve.IsInvalid)
				{
					this.m_SafePeerNameEndResolve.Dispose();
				}
				if (this.m_RegisteredWaitHandle != null)
				{
					this.m_RegisteredWaitHandle.Unregister(null);
				}
				this.m_RegisteredWaitHandle = null;
				this.m_EndPointInfoAvailableEvent.Close();
			}
			this.m_Disposed = true;
		}

		internal int TraceEventId
		{
			get
			{
				return this.m_TraceEventId;
			}
		}

		private const uint FACILITY_P2P = 99U;

		private const uint NO_MORE_RECORDS = 16387U;

		private const int PEER_E_NO_MORE = -2140979197;

		internal object m_userState;

		internal SafePeerNameEndResolve m_SafePeerNameEndResolve;

		internal AutoResetEvent m_EndPointInfoAvailableEvent = new AutoResetEvent(false);

		internal RegisteredWaitHandle m_RegisteredWaitHandle;

		internal PeerName m_PeerName;

		internal Cloud m_Cloud;

		internal int m_MaxRecords;

		internal bool m_Disposed;

		internal bool m_CompletedOrException;

		internal bool m_Cancelled;

		private PeerNameRecordCollection m_PeerNameRecordCollection = new PeerNameRecordCollection();

		private AsyncOperation m_AsyncOp;

		private WeakReference m_PeerNameResolverWeakReference;

		private object m_Lock = new object();

		private int m_TraceEventId;
	}
}
