﻿using System;
using System.Security;
using System.Security.Permissions;

namespace System.Net.PeerToPeer
{
	[Serializable]
	public sealed class PnrpPermission : CodeAccessPermission, IUnrestrictedPermission
	{
		public PnrpPermission(PermissionState state)
		{
			this.m_noRestriction = (state == PermissionState.Unrestricted);
		}

		internal PnrpPermission(bool free)
		{
			this.m_noRestriction = free;
		}

		public bool IsUnrestricted()
		{
			return this.m_noRestriction;
		}

		public override IPermission Copy()
		{
			return new PnrpPermission(this.m_noRestriction);
		}

		public override IPermission Union(IPermission target)
		{
			IPermission result;
			if (target == null)
			{
				result = this.Copy();
			}
			else
			{
				PnrpPermission pnrpPermission = target as PnrpPermission;
				if (pnrpPermission == null)
				{
					throw new ArgumentException(SR.GetString("PnrpPermission_CantUnionWithNonPnrpPermission"), "target");
				}
				result = new PnrpPermission(this.m_noRestriction || pnrpPermission.m_noRestriction);
			}
			return result;
		}

		public override IPermission Intersect(IPermission target)
		{
			IPermission result;
			if (target == null)
			{
				result = null;
			}
			else
			{
				PnrpPermission pnrpPermission = target as PnrpPermission;
				if (pnrpPermission == null)
				{
					throw new ArgumentException(SR.GetString("PnrpPermission_CantIntersectWithNonPnrpPermission"), "target");
				}
				if (this.m_noRestriction && pnrpPermission.m_noRestriction)
				{
					result = new PnrpPermission(true);
				}
				else
				{
					result = null;
				}
			}
			return result;
		}

		public override bool IsSubsetOf(IPermission target)
		{
			bool result;
			if (target == null)
			{
				result = !this.m_noRestriction;
			}
			else
			{
				PnrpPermission pnrpPermission = target as PnrpPermission;
				if (pnrpPermission == null)
				{
					throw new ArgumentException(SR.GetString("PnrpPermission_TargetNotAPnrpPermission"), "target");
				}
				result = (!this.m_noRestriction || pnrpPermission.m_noRestriction);
			}
			return result;
		}

		public override void FromXml(SecurityElement e)
		{
			if (e == null)
			{
				throw new ArgumentNullException(SR.GetString("InvalidSecurityElem"));
			}
			if (!e.Tag.Equals("IPermission"))
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElem"), "securityElement");
			}
			string text = e.Attribute("class");
			if (text == null)
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElem"), "securityElement");
			}
			if (text.IndexOf(base.GetType().FullName, StringComparison.Ordinal) < 0)
			{
				throw new ArgumentException(SR.GetString("InvalidSecurityElem"), "securityElement");
			}
			string text2 = e.Attribute("Unrestricted");
			this.m_noRestriction = (text2 != null && 0 == string.Compare(text2, "true", StringComparison.OrdinalIgnoreCase));
		}

		public override SecurityElement ToXml()
		{
			SecurityElement securityElement = new SecurityElement("IPermission");
			securityElement.AddAttribute("class", base.GetType().FullName + ", " + base.GetType().Module.Assembly.FullName.Replace('"', '\''));
			securityElement.AddAttribute("version", "1");
			if (this.m_noRestriction)
			{
				securityElement.AddAttribute("Unrestricted", "true");
			}
			return securityElement;
		}

		private bool m_noRestriction;

		internal static readonly PnrpPermission UnrestrictedPnrpPermission = new PnrpPermission(PermissionState.Unrestricted);
	}
}
